@@ -1,14 +1,31 @@
-/* radare2 - Copyleft 2011-2016 - pancake */
+/* radare2 - Copyleft 2011-2017 - pancake */
 
 #include <r_util.h>
 #include <r_socket.h>
 
+#if __UNIX__
+static void fwd(int sig) {
+	/* do nothing? send kill signal to remote process */
+}
+
+static void rarun2_tty() {
+	/* TODO: Implement in native code */
+	r_sys_cmd ("tty");
+	close(1);
+	dup2(2, 1);
+	signal (SIGINT, fwd);
+	for (;;) {
+		sleep (1);
+	}
+}
+#endif
+
 int main(int argc, char **argv) {
 	char *file;
 	RRunProfile *p;
 	int i, ret;
 	if (argc == 1 || !strcmp (argv[1], "-h")) {
-		eprintf ("Usage: rarun2 [-v] [script.rr2] [directive ..]\n");
+		eprintf ("Usage: rarun2 -v|-t|script.rr2 [directive ..]\n");
 		printf ("%s", r_run_help ());
 		return 1;
 	}
@@ -17,6 +34,15 @@ int main(int argc, char **argv) {
 		return 0;
 	}
 	file = argv[1];
+	if (!strcmp (file, "-t")) {
+#if __UNIX__
+		rarun2_tty ();
+		return 0;
+#else
+		eprintf ("Not supported\n");
+		return 1;
+#endif
+	}
 	if (*file && !strchr (file, '=')) {
 		p = r_run_new (file);
 	} else {
@@ -41,8 +67,9 @@ int main(int argc, char **argv) {
 			}
 		}
 	}
-	if (!p) return 1;
-
+	if (!p) {
+		return 1;
+	}
 	ret = r_run_config_env (p);
 	if (ret) {
 		printf("error while configuring the environment.\n");
