@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2017 - nibble, pancake */
+/* radare - LGPL - Copyright 2008-2018 - nibble, pancake */
 
 // TODO: rename to r_anal_meta_get() ??
 #if 0
@@ -24,19 +24,21 @@ Keys:
 
 #include <r_anal.h>
 #include <r_core.h>
-#include <r_print.h>
+#include <r_util.h>
 
-#define META_RANGE_BASE(x) ((x)>>12)
-#define META_RANGE_SIZE 0xfff
+#define META_RANGE_BASE(x) ((x)>>5)
 #undef DB
 #define DB a->sdb_meta
 
 static char *meta_inrange_get (RAnal *a, ut64 addr, int size) {
+	if (size <= 0) {
+		return NULL;
+	}
 	ut64 base = META_RANGE_BASE (addr);
-	ut64 base2 = META_RANGE_BASE (addr + size) + 1;
+	ut64 base2 = META_RANGE_BASE (addr + size - 1);
 	char *res = NULL;
 	// return string array of all the offsets where there are stuff
-	for (; base < base2; base += META_RANGE_SIZE) {
+	for (; base <= base2; base++) {
 		const char *key = sdb_fmt ("range.0x%"PFMT64x, base);
 		// char *r = sdb_array_get (DB, key, 0, 0);
 		char *r = sdb_get (DB, key, 0);
@@ -52,13 +54,15 @@ static char *meta_inrange_get (RAnal *a, ut64 addr, int size) {
 }
 
 static bool meta_inrange_add (RAnal *a, ut64 addr, int size) {
+	if (size <= 0) {
+		return false;
+	}
 	bool set = false;
-	char key[64];
 	ut64 base, base2;
 	base = META_RANGE_BASE (addr);
-	base2 = META_RANGE_BASE (addr + size) + 1;
-	for (; base < base2; base += META_RANGE_SIZE) {
-		snprintf (key, sizeof (key)-1, "range.0x%"PFMT64x, base);
+	base2 = META_RANGE_BASE (addr + size - 1);
+	for (; base <= base2; base++) {
+		const char *key = sdb_fmt ("range.0x%"PFMT64x, base);
 		if (sdb_array_add_num (DB, key, addr, 0)) {
 			set = true;
 		}
@@ -67,18 +71,18 @@ static bool meta_inrange_add (RAnal *a, ut64 addr, int size) {
 }
 
 static bool meta_inrange_del (RAnal *a, ut64 addr, int size) {
+	if (size <= 0) {
+		return false;
+	}
 	bool set = false;
-	char key[64];
 	ut64 base = META_RANGE_BASE (addr);
-	ut64 base2 = META_RANGE_BASE (addr + size) + 1;
-// TODO: optimize this thing?
-	for (; base < base2; base += META_RANGE_SIZE) {
-		snprintf (key, sizeof (key)-1, "range.0x%"PFMT64x, base);
+	ut64 base2 = META_RANGE_BASE (addr + size - 1);
+	for (; base <= base2; base++) {
+		const char *key = sdb_fmt ("range.0x%"PFMT64x, base);
 		if (sdb_array_remove_num (DB, key, addr, 0)) {
 			set = true;
 		}
 	}
-	//sdb_array_del (DB);
 	return set;
 }
 
@@ -128,6 +132,15 @@ R_API int r_meta_set_string(RAnal *a, int type, ut64 addr, const char *s) {
 	snprintf (val, sizeof (val)-1, "%d,%d,%s", (int)size, space_idx, e_str);
 	sdb_set (DB, key, val, 0);
 	free ((void*)e_str);
+
+	/* send event */
+	REventMeta rems = {
+		.type = type,
+		.addr = addr,
+		.string = s
+	};
+	r_event_send (a->ev, R_EVENT_META_SET, &rems);
+
 	return ret;
 }
 
@@ -222,7 +235,9 @@ R_API int r_meta_del(RAnal *a, int type, ut64 addr, ut64 size) {
 						"meta.%c.0x%"PFMT64x,
 						type, sdb_atoi (s));
 					sdb_unset (DB, key, 0);
-					if (!next) break;
+					if (!next) {
+						break;
+					}
 				}
 				free (dtr);
 			}
@@ -277,8 +292,7 @@ R_API int r_meta_del(RAnal *a, int type, ut64 addr, ut64 size) {
 }
 
 R_API int r_meta_var_comment_del(RAnal *a, int type, ut64 idx, ut64 addr) {
-	char *key;
-	key = r_str_newf ("meta.%c.0x%"PFMT64x"0x%"PFMT64x, type, addr, idx);
+	char *key = r_str_newf ("meta.%c.0x%"PFMT64x"0x%"PFMT64x, type, addr, idx);
 	sdb_unset (DB, key, 0);
 	return 0;
 }
@@ -294,7 +308,9 @@ R_API void r_meta_item_free(void *_item) {
 
 R_API RAnalMetaItem *r_meta_item_new(int type) {
 	RAnalMetaItem *mi = R_NEW0 (RAnalMetaItem);
-	if (mi) mi->type = type;
+	if (mi) {
+		mi->type = type;
+	}
 	return mi;
 }
 
@@ -346,8 +362,7 @@ R_API bool r_meta_deserialize_val(RAnalMetaItem *it, int type, ut64 from, const
 
 static int meta_add(RAnal *a, int type, int subtype, ut64 from, ut64 to, const char *str) {
 	int space_idx = a->meta_spaces.space_idx;
-	char *e_str, key[100], val[2048];
-	int exists;
+	char key[100], val[2048];
 	if (from > to) {
 		return false;
 	}
@@ -358,10 +373,10 @@ static int meta_add(RAnal *a, int type, int subtype, ut64 from, ut64 to, const c
 		return false;
 	}
 	/* set entry */
-	e_str = sdb_encode ((const void*)str, -1);
+	char *e_str = sdb_encode ((const void*)str, -1);
 	RAnalMetaItem mi = {from, to, (int)(to - from), type, subtype, e_str, space_idx};
 	meta_serialize (&mi, key, sizeof (key), val, sizeof (val));
-	exists = sdb_exists (DB, key);
+	bool exists = sdb_exists (DB, key);
 
 	sdb_set (DB, key, val, 0);
 	free (e_str);
@@ -374,10 +389,15 @@ static int meta_add(RAnal *a, int type, int subtype, ut64 from, ut64 to, const c
 	snprintf (key, sizeof (key) - 1, "meta.0x%"PFMT64x, from);
 	if (exists) {
 		const char *value = sdb_const_get (DB, key, 0);
-		int idx = sdb_array_indexof (DB, key, value, 0);
-		sdb_array_delete (DB, key, idx, 0);
+		if (value) {
+			int idx = sdb_array_indexof (DB, key, value, 0);
+			if (idx >= 0) {
+				sdb_array_delete (DB, key, idx, 0);
+			}
+		}
 	}
-	snprintf (val, sizeof (val)-1, "%c", type);
+	val[0] = type;
+	val[1] = '\0';
 	sdb_array_add (DB, key, val, 0);
 	meta_inrange_add (a, from, to - from);
 	return true;
@@ -391,7 +411,7 @@ R_API int r_meta_add_with_subtype(RAnal *a, int type, int subtype, ut64 from, ut
 	return meta_add (a, type, subtype, from, to, str);
 }
 
-R_API RAnalMetaItem *r_meta_find(RAnal *a, ut64 at, int type, int where) {
+static RAnalMetaItem *r_meta_find_(RAnal *a, ut64 at, int type, int where, int excl_type) {
 	const char *infos, *metas;
 	char key[100];
 	Sdb *s = a->sdb_meta;
@@ -414,6 +434,9 @@ R_API RAnalMetaItem *r_meta_find(RAnal *a, ut64 at, int type, int where) {
 		if (type != R_META_TYPE_ANY && type != *infos) {
 			continue;
 		}
+		if (excl_type && excl_type == *infos) {
+			continue;
+		}
 		snprintf (key, sizeof (key), "meta.%c.0x%" PFMT64x, *infos, at);
 		metas = sdb_const_get (s, key, 0);
 		if (metas) {
@@ -426,6 +449,15 @@ R_API RAnalMetaItem *r_meta_find(RAnal *a, ut64 at, int type, int where) {
 	return NULL;
 }
 
+// TODO should be named get imho
+R_API RAnalMetaItem *r_meta_find(RAnal *a, ut64 at, int type, int where) {
+	return r_meta_find_ (a, at, type, where, R_META_TYPE_NONE);
+}
+
+R_API RAnalMetaItem *r_meta_find_any_except(RAnal *a, ut64 at, int type, int where) {
+	return r_meta_find_ (a, at, R_META_TYPE_ANY, where, type);
+}
+
 R_API RAnalMetaItem *r_meta_find_in(RAnal *a, ut64 at, int type, int where) {
 	char *res = meta_inrange_get (a, at, 1);
 	if (!res) {
@@ -450,15 +482,18 @@ R_API RAnalMetaItem *r_meta_find_in(RAnal *a, ut64 at, int type, int where) {
 R_API const char *r_meta_type_to_string(int type) {
 	// XXX: use type as '%c'
 	switch (type) {
-	case R_META_TYPE_HIDE: return "Ch";
-	case R_META_TYPE_CODE: return "Cc";
 	case R_META_TYPE_DATA: return "Cd";
+	case R_META_TYPE_CODE: return "Cc";
 	case R_META_TYPE_STRING: return "Cs";
 	case R_META_TYPE_FORMAT: return "Cf";
 	case R_META_TYPE_MAGIC: return "Cm";
+	case R_META_TYPE_HIDE: return "Ch";
 	case R_META_TYPE_COMMENT: return "CCu";
+	case R_META_TYPE_RUN: return "Cr"; // not in C? help
+	case R_META_TYPE_HIGHLIGHT: return "CH"; // not in C?
+	case R_META_TYPE_VARTYPE: return "Ct";
 	}
-	return "(...)";
+	return "# unknown meta # ";
 }
 
 static bool isFirst = true;
@@ -495,6 +530,12 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full) {
 			pstr = str;
 		} else if (d->type == 's') {
 			pstr = str;
+		} else if (d->type == 't') {
+			// Sanitize (don't escape) Ct comments so we can see "char *", etc.
+			free (str);
+			str = strdup (d->str);
+			r_str_sanitize (str);
+			pstr = str;
 		} else if (d->type != 'C') {
 			r_name_filter (str, 0);
 			pstr = str;
@@ -513,7 +554,9 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full) {
 			} else {
 				a->cb_printf ("\"%s\"", str);
 			}
-			if (d->type == 's') {
+			if (d->type == 'd') {
+				a->cb_printf (", \"size\":%d",(int)d->size);
+			} else if (d->type == 's') {
 				const char *enc;
 				switch (d->subtype) {
 				case R_STRING_ENC_UTF8:
@@ -545,7 +588,9 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full) {
 				{
 				const char *type = r_meta_type_to_string (d->type);
 				char *s = sdb_encode ((const ut8*)pstr, -1);
-				if (!s) s = strdup (pstr);
+				if (!s) {
+					s = strdup (pstr);
+				}
 				if (rad) {
 					if (!strcmp (type, "CCu")) {
 						a->cb_printf ("%s base64:%s @ 0x%08"PFMT64x"\n",
@@ -632,6 +677,14 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full) {
 					}
 				}
 				break;
+			case 't': /* vartype */
+				if (rad) {
+					a->cb_printf ("%s %s @ 0x%08"PFMT64x"\n",
+						r_meta_type_to_string (d->type), pstr, d->from);
+				} else {
+					a->cb_printf ("0x%08"PFMT64x" %s\n", d->from, pstr);
+				}
+				break;
 			default:
 				if (rad) {
 					a->cb_printf ("%s %d 0x%08"PFMT64x" # %s\n",
@@ -647,8 +700,9 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, int rad, bool show_full) {
 			}
 			break;
 		}
-		if (str)
+		if (str) {
 			free (str);
+		}
 	}
 }
 
@@ -658,12 +712,8 @@ static int meta_print_item(void *user, const char *k, const char *v) {
 	if (!meta_deserialize (&it, k, v)) {
 		return 1;
 	}
-	int uirad = ui->rad;
-	if (ui->rad == 'f') {
-		if (!r_anal_fcn_in (ui->fcn, it.from)) {
-			goto beach;
-		}
-		ui->rad = 0;
+	if (ui->fcn && !r_anal_fcn_in (ui->fcn, it.from)) {
+		goto beach;
 	}
 	if (!it.str) {
 		it.str = strdup (""); // don't break in free
@@ -674,30 +724,69 @@ static int meta_print_item(void *user, const char *k, const char *v) {
 	r_meta_print (ui->anal, &it, ui->rad, true);
 beach:
 	free (it.str);
-	ui->rad = uirad;
 	return 1;
 }
 
+R_API void r_meta_list_offset(RAnal *a, ut64 addr, char input) {
+	const int types[] = {
+		R_META_TYPE_VARTYPE,
+		R_META_TYPE_HIGHLIGHT,
+		R_META_TYPE_RUN,
+		R_META_TYPE_COMMENT,
+		R_META_TYPE_HIDE,
+		R_META_TYPE_MAGIC,
+		R_META_TYPE_FORMAT,
+		R_META_TYPE_STRING,
+		R_META_TYPE_CODE,
+		R_META_TYPE_DATA,
+	};
+
+	char key[100];
+	int i;
+
+	for (i = 0; i < sizeof (types) / sizeof (types[0]); i ++) {
+		snprintf (key, sizeof (key)-1, "meta.%c.0x%"PFMT64x, types[i], addr);
+		const char *k = sdb_const_get (DB, key, 0);
+		if (!k) {
+			continue;
+		}
+
+		RAnalMetaUserItem ui = { a };
+
+		meta_print_item ((void *)&ui, key, k);
+	}
+}
+
 R_API int r_meta_list_cb(RAnal *a, int type, int rad, SdbForeachCallback cb, void *user, ut64 addr) {
-	RAnalFunction *fcn = (addr != UT64_MAX) ? r_anal_get_fcn_at (a, addr, 0) : NULL;
-	RAnalMetaUserItem ui = { a, type, rad, cb, user, 0, fcn};
-	SdbList *ls = sdb_foreach_list (DB, true);
-	SdbListIter *lsi;
-	SdbKv *kv;
 	if (rad == 'j') {
 		a->cb_printf ("[");
 	}
+
+	RAnalMetaUserItem ui = { a, type, rad, cb, user, 0, NULL };
+
+	if (addr != UT64_MAX) {
+		ui.fcn = r_anal_get_fcn_in (a, addr, 0);
+		if (!ui.fcn) {
+			goto beach;
+		}
+	}
+
+	SdbList *ls = sdb_foreach_list (DB, true);
+	SdbListIter *lsi;
+	SdbKv *kv;
 	isFirst = true; // TODO: kill global
 	ls_foreach (ls, lsi, kv) {
-		if (type == R_META_TYPE_ANY || (strlen (kv->key) > 5 && kv->key[5] == type)) {
+		if (type == R_META_TYPE_ANY || (strlen (sdbkv_key (kv)) > 5 && sdbkv_key (kv)[5] == type)) {
 			if (cb) {
-				cb ((void *)&ui, kv->key, kv->value);
+				cb ((void *)&ui, sdbkv_key (kv), sdbkv_value (kv));
 			} else {
-				meta_print_item ((void *)&ui, kv->key, kv->value);
+				meta_print_item ((void *)&ui, sdbkv_key (kv), sdbkv_value (kv));
 			}
 		}
 	}
 	ls_free (ls);
+
+beach:
 	if (rad == 'j') {
 		a->cb_printf ("]\n");
 	}
@@ -743,8 +832,9 @@ static int meta_unset_cb(void *user, const char *k, const char *v) {
 	RAnalMetaUserItem *ui = user;
 	RAnal *a = ui->anal;
 	RAnalMetaItem it = {0};
-	if (!strstr(k, ".0x"))
+	if (!strstr (k, ".0x")) {
 		return 1;
+	}
 	meta_deserialize (&it, k, v);
 	if (it.space != -1) {
 		it.space = -1;
@@ -768,8 +858,9 @@ static int meta_count_cb(void *user, const char *k, const char *v) {
 	RAnalMetaUserItem *ui = user;
 	myMetaUser *mu = ui->user;
 	RAnalMetaItem it = {0};
-	if (!strstr(k, ".0x"))
+	if (!strstr (k, ".0x")) {
 		return 1;
+	}
 	meta_deserialize (&it, k, v);
 	if (mu) {
 		if (it.space == mu->ctx) {
