@@ -88,9 +88,9 @@ static void map_cpu_memory (RAnal *anal, int entry, ut32 addr, ut32 size, bool f
 		}
 	} else {
 		// allocate memory for address space
-		char *mstr = r_str_newf ("malloc://%d", size);		
-		desc = anal->iob.open_at (anal->iob.io, mstr, R_IO_READ | R_IO_WRITE, 0, addr);		
-		r_str_free (mstr);
+		char *mstr = r_str_newf ("malloc://%d", size);
+		desc = anal->iob.open_at (anal->iob.io, mstr, R_PERM_RW, 0, addr);
+		free (mstr);
 		// set 8051 address space as name of mapped memory
 		if (desc && anal->iob.fd_get_name (anal->iob.io, desc->fd)) {
 			RList *maps = anal->iob.fd_get_map (anal->iob.io, desc->fd);
@@ -99,7 +99,7 @@ static void map_cpu_memory (RAnal *anal, int entry, ut32 addr, ut32 size, bool f
 			r_list_foreach (maps, iter, current_map) {
 				char *cmdstr = r_str_newf ("omni %d %s", current_map->id, mem_map[entry].name);
 				anal->coreb.cmd (anal->coreb.core, cmdstr);
-				r_str_free (cmdstr);
+				free (cmdstr);
 			}
 			r_list_free (maps);
 		}
@@ -231,9 +231,9 @@ static RI8051Reg registers[] = {
 #define ev_imm1 buf[1]
 #define ev_imm2 buf[2]
 #define ev_imm16 op->val
-#define ev_ri 1 & buf[0]
-#define ev_rix 1 & buf[0]
-#define ev_rn 7 & buf[0]
+#define ev_ri (1 & buf[0])
+#define ev_rix (1 & buf[0])
+#define ev_rn (7 & buf[0])
 #define ev_sp2 0
 #define ev_sp1 0
 
@@ -456,10 +456,10 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 		e ("c,1,&,!,"); cjmp;
 		break;
 	case 0x60: /* jz offset */
-		e ("a,0,==,"); cjmp;
+		e ("a,0,==,$z,"); cjmp;
 		break;
 	case 0x70: /* jnz offset */
-		e ("a,0,==,!,"); cjmp;
+		e ("a,0,==,$z,!,"); cjmp;
 		break;
 
 	case 0x11: case 0x31: case 0x51: case 0x71:
@@ -468,7 +468,7 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 		call;
 		break;
 	case 0x01: case 0x21: case 0x41: case 0x61:
-	case 0x81: case 0xA1: case 0xC1: case 0xE1: /* ajmp addr11 */	
+	case 0x81: case 0xA1: case 0xC1: case 0xE1: /* ajmp addr11 */
 	case 0x02: /* ljmp addr16 */
 	case 0x80: /* sjmp offset */
 		jmp;
@@ -552,7 +552,7 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 		break;
 	case 0x84: /* div ab */
 		// note: escape % if this becomes a format string
-		e ("b,0,==,ov,=,b,a,%,b,a,/=,b,=,0,c,=," flag_p);
+		e ("b,0,==,$z,ov,=,b,a,%,b,a,/=,b,=,0,c,=," flag_p);
 		break;
 	case 0x85: /* mov direct, direct */
 		xr (dir1); xw (dir2);
@@ -584,7 +584,7 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 		xi (dp, "++");
 		break;
 	case 0xA4: /* mul ab */
-		e ("8,a,b,*,DUP,a,=,>>,DUP,b,=,0,==,!,ov,=,0,c,=," flag_p);
+		e ("8,a,b,*,DUP,a,=,>>,DUP,b,=,0,==,$z,!,ov,=,0,c,=," flag_p);
 		break;
 	case 0xA5: /* "reserved" */
 		e ("0,trap");
@@ -633,7 +633,7 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 	case 0xC5: /* xch a, direct */
 		xr (a); e ("0,+,"); xr (dir1); xw (a); xw (dir1); e (flag_p);
 		break;
-	case 0xC6: case 0xC7: /* xch a, @Ri */ 
+	case 0xC6: case 0xC7: /* xch a, @Ri */
 		xr (a); e ("0,+,"); xr (ri); xw (a); xw (ri); e (flag_p);
 		break;
 	case 0xC8: case 0xC9: case 0xCA: case 0xCB:
@@ -654,10 +654,10 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 		// if (lower nibble > 9) or (AC == 1) add 6
 		// if (higher nibble > 9) or (C == 1) add 0x60
 		// carry |= carry caused by this operation
-		e ("a,0x0f,&,9,<,ac,|,?{,6,a,+=,$c7,c,|=,},a,0xf0,&,0x90,<,c,|,?{,0x60,a,+=,$c7,c,|=,}," flag_p);
+		e ("a,0x0f,&,9,==,$b4,ac,|,?{,6,a,+=,$c7,c,|=,},a,0xf0,&,0x90,==,$b8,c,|,?{,0x60,a,+=,$c7,c,|=,}," flag_p);
 		break;
 	case 0xD5: /* djnz direct, offset */
-		xi (dir1, "--"); xr (dir1); e ("0,==,!,"); cjmp;
+		xi (dir1, "--"); xr (dir1); e ("0,==,$z,!,"); cjmp;
 		break;
 	case 0xD6:
 	case 0xD7: /* xchd a, @Ri*/
@@ -667,7 +667,7 @@ static void analop_esil(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf) {
 		break;
 	case 0xD8: case 0xD9: case 0xDA: case 0xDB:
 	case 0xDC: case 0xDD: case 0xDE: case 0xDF: /* djnz Rn, offset */
-		xi (rn, "--"); xr (rn); e ("0,==,!,"); cjmp;
+		xi (rn, "--"); xr (rn); e ("0,==,$z,!,"); cjmp;
 		break;
 	case 0xE0: /* movx a, @dptr */
 		xr (dpx); xw (a); e (flag_p);
@@ -1058,7 +1058,7 @@ RAnalPlugin r_anal_plugin_8051 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_8051,
 	.version = R2_VERSION
