@@ -133,7 +133,7 @@ RAnalPlugin r_anal_plugin_cr16 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_cr16,
 	.version = R2_VERSION
