@@ -76,8 +76,6 @@ static void opex(RStrBuf *buf, csh handle, cs_insn *insn) {
 	r_strbuf_append (buf, "]}");
 }
 
-#define ZERO_FILL(x) memset (&x, 0, sizeof (x))
-
 static int parse_reg_name(RRegItem *reg, csh handle, cs_insn *insn, int reg_num) {
 	if (!reg) {
 		return -1;
@@ -147,22 +145,30 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 // XXX no arch->cpu ?!?! CS_MODE_MICRO, N64
 	op->delay = 0;
 	// replace this with the asm.features?
-	if (a->cpu && strstr (a->cpu, "68000"))
+	if (a->cpu && strstr (a->cpu, "68000")) {
 		mode |= CS_MODE_M68K_000;
-	if (a->cpu && strstr (a->cpu, "68010"))
+	}
+	if (a->cpu && strstr (a->cpu, "68010")) {
 		mode |= CS_MODE_M68K_010;
-	if (a->cpu && strstr (a->cpu, "68020"))
+	}
+	if (a->cpu && strstr (a->cpu, "68020")) {
 		mode |= CS_MODE_M68K_020;
-	if (a->cpu && strstr (a->cpu, "68030"))
+	}
+	if (a->cpu && strstr (a->cpu, "68030")) {
 		mode |= CS_MODE_M68K_030;
-	if (a->cpu && strstr (a->cpu, "68040"))
+	}
+	if (a->cpu && strstr (a->cpu, "68040")) {
 		mode |= CS_MODE_M68K_040;
-	if (a->cpu && strstr (a->cpu, "68060"))
+	}
+	if (a->cpu && strstr (a->cpu, "68060")) {
 		mode |= CS_MODE_M68K_060;
+	}
 	op->size = 4;
 	if (handle == 0) {
 		ret = cs_open (CS_ARCH_M68K, mode, &handle);
-		if (ret != CS_ERR_OK) goto fin;
+		if (ret != CS_ERR_OK) {
+			goto fin;
+		}
 		cs_option (handle, CS_OPT_DETAIL, CS_OPT_ON);
 	}
 	n = cs_disasm (handle, (ut8*)buf, len, addr, 1, &insn);
@@ -559,6 +565,8 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 		op->type = R_ANAL_OP_TYPE_NOP;
 		break;
 	case M68K_INS_NOT:
+		op->type = R_ANAL_OP_TYPE_NOT;
+		break;
 	case M68K_INS_OR:
 	case M68K_INS_ORI:
 		op->type = R_ANAL_OP_TYPE_OR;
@@ -580,15 +588,23 @@ static int analop(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *buf, int len) {
 	case M68K_INS_REMS:
 	case M68K_INS_REMU:
 	case M68K_INS_RESET:
+		break;
 	case M68K_INS_ROL:
+		op->type = R_ANAL_OP_TYPE_ROL;
+		break;
 	case M68K_INS_ROR:
+		op->type = R_ANAL_OP_TYPE_ROR;
+		break;
 	case M68K_INS_ROXL:
 	case M68K_INS_ROXR:
+		break;
 	case M68K_INS_RTD:
 	case M68K_INS_RTE:
 	case M68K_INS_RTM:
 	case M68K_INS_RTR:
 	case M68K_INS_RTS:
+		op->type = R_ANAL_OP_TYPE_RET;
+		break;
 	case M68K_INS_SBCD:
 	case M68K_INS_ST:
 	case M68K_INS_SF:
@@ -745,7 +761,7 @@ RAnalPlugin r_anal_plugin_m68k_cs = {
 #endif
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_m68k_cs,
 	.version = R2_VERSION
