@@ -25,7 +25,7 @@ RAnalPlugin r_anal_plugin_null = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_null,
 	.version = R2_VERSION
