@@ -215,7 +215,7 @@ RAnalPlugin r_anal_plugin_tms320c64x = {
 #endif
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_tms320c64x,
 	.version = R2_VERSION
