@@ -56,7 +56,7 @@ static void z80_op_size(const ut8 *data, int len, int *size, int *size_prefix) {
 }
 
 static int z80_anal_op(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len) {
-	int ilen;
+	int ilen = 0;
 	z80_op_size (data, len, &ilen, &op->nopcode);
 
 	memset (op, '\0', sizeof (RAnalOp));
@@ -427,7 +427,7 @@ RAnalPlugin r_anal_plugin_z80 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ANAL,
 	.data = &r_anal_plugin_z80,
 	.version = R2_VERSION
