@@ -117,10 +117,11 @@ R_API RList *r_anal_reflines_get(RAnal *anal, ut64 addr, const ut8 *buf, ut64 le
 			nlines--;
 		}
 		{
-			const RAnalMetaItem *mi = r_meta_find (anal, addr, R_META_TYPE_ANY, 0);
+			const RAnalMetaItem *mi = r_meta_find_any_except (anal, addr, R_META_TYPE_COMMENT, 0);
 			if (mi) {
 				ptr += mi->size;
 				addr += mi->size;
+				free (mi->str);
 				continue;
 			}
 		}
@@ -193,7 +194,9 @@ R_API RList *r_anal_reflines_get(RAnal *anal, ut64 addr, const ut8 *buf, ut64 le
 			if (min < 0) {
 				min = 0;
 			}
-			while (free_levels[++min] == 1);
+			while (free_levels[++min] == 1) {
+				;
+			}
 		} else {
 			free_levels[el->r->level - 1] = 0;
 			if (min > el->r->level - 1) {
@@ -296,7 +299,7 @@ R_API RList* r_anal_reflines_fcn_get(RAnal *anal, RAnalFunction *fcn, int nlines
 				RListIter *iter;
 				r_list_foreach (bb->switch_op->cases, iter, caseop) {
 					if (caseop) {
-						if (!linesout) {// && (op.jump > opc+len || op.jump < pc)) 
+						if (!linesout) {// && (op.jump > opc+len || op.jump < pc))
 							continue;
 						}
 						item = R_NEW0 (RAnalRefline);
@@ -321,8 +324,9 @@ R_API int r_anal_reflines_middle(RAnal *a, RList* /*<RAnalRefline>*/ list, ut64
 		RAnalRefline *ref;
 		RListIter *iter;
 		r_list_foreach (list, iter, ref) {
-			if ((ref->to > addr) && (ref->to < addr+len))
+			if ((ref->to > addr) && (ref->to < addr + len)) {
 				return true;
+			}
 		}
 	}
 	return false;
@@ -370,6 +374,21 @@ static void fill_level(RBuffer *b, int pos, char ch, RAnalRefline *r, int wide)
 	}
 }
 
+static inline bool refline_kept(RAnalRefline *ref, bool middle_after, ut64 addr) {
+	if (middle_after) {
+		if (ref->direction < 0) {
+			if (ref->from == addr) {
+				return false;
+			}
+		} else {
+			if (ref->to == addr) {
+				return false;
+			}
+		}
+	}
+	return true;
+}
+
 // TODO: move into another file
 // TODO: this is TOO SLOW. do not iterate over all reflines or gtfo
 R_API char* r_anal_reflines_str(void *_core, ut64 addr, int opts) {
@@ -383,6 +402,7 @@ R_API char* r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 	int dir = 0, wide = opts & R_ANAL_REFLINE_TYPE_WIDE;
 	int pos = -1, max_level = -1;
 	int middle = opts & R_ANAL_REFLINE_TYPE_MIDDLE;
+	int middle_after = opts & R_ANAL_REFLINE_TYPE_MIDDLE_AFTER;
 	char *str = NULL;
 
 	if (!c || !anal || !anal->reflines) {
@@ -398,7 +418,7 @@ R_API char* r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 			r_list_free (lvls);
 			return NULL;
 		}
-		if (in_refline (addr, ref)) {
+		if (in_refline (addr, ref) && refline_kept (ref, middle_after, addr)) {
 			r_list_add_sorted (lvls, (void *)ref, (RListComparator)cmp_by_ref_lvl);
 		}
 	}
@@ -410,7 +430,7 @@ R_API char* r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 			r_buf_free (b);
 			return NULL;
 		}
-		if (ref->from == addr || ref->to == addr) {
+		if ((ref->from == addr || ref->to == addr) && !middle_after) {
 			const char *corner = get_corner_char (ref, addr, middle);
 			const char ch = ref->from == addr ? '=' : '-';
 
