@@ -13,7 +13,9 @@ R_API RAnalValue *r_anal_value_new_from_string(const char *str) {
 
 R_API RAnalValue *r_anal_value_copy(RAnalValue *ov) {
 	RAnalValue *v = R_NEW0 (RAnalValue);
-	if (!v) return NULL;
+	if (!v) {
+		return NULL;
+	}
 	memcpy (v, ov, sizeof (RAnalValue));
 	// reference to reg and regdelta should be kept
 	return v;
@@ -34,13 +36,16 @@ R_API void r_anal_value_free(RAnalValue *value) {
 // mul*value+regbase+regidx+delta
 R_API ut64 r_anal_value_to_ut64(RAnal *anal, RAnalValue *val) {
 	ut64 num;
-	if (!val)
+	if (!val) {
 		return 0LL;
+	}
 	num = val->base + (val->delta*(val->mul?val->mul:1));
-	if (val->reg)
+	if (val->reg) {
 		num += r_reg_get_value (anal->reg, val->reg);
-	if (val->regdelta)
+	}
+	if (val->regdelta) {
 		num += r_reg_get_value (anal->reg, val->regdelta);
+	}
 	switch (val->memref) {
 	case 1:
 	case 2:
@@ -60,10 +65,13 @@ R_API int r_anal_value_set_ut64(RAnal *anal, RAnalValue *val, ut64 num) {
 			ut64 addr = r_anal_value_to_ut64 (anal, val);
 			r_mem_set_num (data, val->memref, num);
 			anal->iob.write_at (anal->iob.io, addr, data, val->memref);
-		} else eprintf ("No IO binded to r_anal\n");
+		} else {
+			eprintf ("No IO binded to r_anal\n");
+		}
 	} else {
-		if (val->reg)
+		if (val->reg) {
 			r_reg_set_value (anal->reg, val->reg, num);
+		}
 	}
 	return false;							//is this necessary
 }
@@ -73,26 +81,41 @@ R_API char *r_anal_value_to_string (RAnalValue *value) {
 	if (value) {
 		out = r_str_new ("");
 		if (!value->base && !value->reg) {
-			if (value->imm != -1LL)
-				out = r_str_concatf (out, "0x%"PFMT64x, value->imm);
-			else out = r_str_concat (out, "-1");
+			if (value->imm != -1LL) {
+				out = r_str_appendf (out, "0x%"PFMT64x, value->imm);
+			} else {
+				out = r_str_append (out, "-1");
+			}
 		} else {
 			if (value->memref) {
 				switch (value->memref) {
-				case 1: out = r_str_concat (out, "(char)"); break;
-				case 2: out = r_str_concat (out, "(short)"); break;
-				case 4: out = r_str_concat (out, "(word)"); break;
-				case 8: out = r_str_concat (out, "(dword)"); break;
+				case 1: out = r_str_append (out, "(char)"); break;
+				case 2: out = r_str_append (out, "(short)"); break;
+				case 4: out = r_str_append (out, "(word)"); break;
+				case 8: out = r_str_append (out, "(dword)"); break;
 				}
-				out = r_str_concat (out, "[");
+				out = r_str_append (out, "[");
+			}
+			if (value->mul) {
+				out = r_str_appendf (out, "%d*", value->mul);
+			}
+			if (value->reg) {
+				out = r_str_appendf (out, "%s", value->reg->name);
+			}
+			if (value->regdelta) {
+				out = r_str_appendf (out, "+%s", value->regdelta->name);
+			}
+			if (value->base != 0) {
+				out = r_str_appendf (out, "0x%" PFMT64x, value->base);
+			}
+			if (value->delta > 0) {
+				out = r_str_appendf (out, "+0x%" PFMT64x, value->delta);
+			} else if (value->delta < 0) {
+				out = r_str_appendf (out, "-0x%" PFMT64x, -value->delta);
+			}
+			if (value->memref) {
+				out = r_str_append (out, "]");
 			}
-			if (value->mul) out = r_str_concatf (out, "%d*", value->mul);
-			if (value->reg) out = r_str_concatf (out, "%s", value->reg->name);
-			if (value->regdelta) out = r_str_concatf (out, "+%s", value->regdelta->name);
-			if (value->base!=0) out = r_str_concatf (out, "0x%"PFMT64x, value->base);
-			if (value->delta>0) out = r_str_concatf (out, "+0x%"PFMT64x, value->delta);
-			else if (value->delta<0) out = r_str_concatf (out, "-0x%"PFMT64x, -value->delta);
-			if (value->memref) out = r_str_concat (out, "]");
 		}
 	}
 	return out;
