@@ -68,11 +68,9 @@ R_API bool r_anal_var_display(RAnal *anal, int delta, char kind, const char *typ
 	return true;
 }
 
-R_API bool r_anal_var_add(RAnal *a, ut64 addr, int scope, int delta, char kind, const char *type, int size,
-		bool isarg, const char *name) {
-	if (!a) {
-		return false;
-	}
+R_API bool r_anal_var_add(RAnal *a, ut64 addr, int scope, int delta, char kind, R_IFNULL("int") const char *type, int size, bool isarg, R_NONNULL const char *name) {
+	r_return_val_if_fail (a, false);
+	r_return_val_if_fail (name, false);
 	if (!kind) {
 		kind = R_ANAL_VAR_KIND_BPV;
 	}
@@ -194,6 +192,7 @@ R_API int r_anal_var_retype(RAnal *a, ut64 addr, int scope, int delta, char kind
 }
 
 R_API int r_anal_var_delete_all(RAnal *a, ut64 addr, const char kind) {
+	r_return_val_if_fail (a, 0);
 	RAnalFunction *fcn = r_anal_get_fcn_in (a, addr, 0);
 	if (fcn) {
 		RAnalVar *v;
@@ -388,10 +387,8 @@ R_API ut64 r_anal_var_addr(RAnal *a, RAnalFunction *fcn, const char *name) {
 #define R_ANAL_VAR_SDB_SIZE 2 /* number */
 #define R_ANAL_VAR_SDB_NAME 3 /* string */
 
-#define IS_NUMBER(x) ((x) >= '0' && (x) <= '9')
-
 R_API bool r_anal_var_check_name(const char *name) {
-	return !IS_NUMBER (*name) && strcspn (name, "., =/");
+	return !isdigit (*name) && strcspn (name, "., =/");
 }
 
 // afvn local_48 counter
@@ -442,7 +439,8 @@ R_API int r_anal_var_rename(RAnal *a, ut64 addr, int scope, char kind, const cha
 		}
 		if (!old_name) {
 			old_name = stored_name;
-		} else if (strcmp (stored_name, old_name)) {
+		}
+		if (strcmp (stored_name, old_name)) {
 			return 0;
 		}
 		sdb_unset (DB, key, 0);
@@ -453,10 +451,10 @@ R_API int r_anal_var_rename(RAnal *a, ut64 addr, int scope, char kind, const cha
 }
 
 // Used for linking reg based arg and local-var like "mov [local_8h], rsi"
-static void r_anal_var_link (RAnal *a, ut64 addr, RAnalVar *var) {
-	const char *inst_key = sdb_fmt ("inst.0x%"PFMT64x ".lvar", addr);
-	const char *var_def = sdb_fmt ("0x%"PFMT64x ",%c,0x%x,0x%x", var->addr,
-			var->kind, var->scope, var->delta);
+static void r_anal_var_link(RAnal *a, ut64 addr, RAnalVar *var) {
+	const char *inst_key = sdb_fmt ("inst.0x%" PFMT64x ".lvar", addr);
+	const char *var_def = sdb_fmt ("0x%" PFMT64x ",%c,0x%x,0x%x", var->addr,
+		var->kind, var->scope, var->delta);
 	sdb_set (DB, inst_key, var_def, 0);
 }
 
@@ -573,7 +571,7 @@ static void var_add_structure_fields_to_list(RAnal *a, RAnalVar *av, const char
 static char *get_varname(RAnal *a, RAnalFunction *fcn, char type, const char *pfx, int idx) {
 	char *varname = r_str_newf ("%s_%xh", pfx, idx);
 	int i = 2;
-	char v_kind;
+	char v_kind = 0;
 	int v_delta = 0;
 	while (1) {
 		char *name_key = sdb_fmt ("var.0x%"PFMT64x ".%d.%s", fcn->addr, 1, varname);
@@ -611,12 +609,12 @@ static const char *get_regname(RAnal *anal, RAnalValue *value) {
 }
 
 static void extract_arg(RAnal *anal, RAnalFunction *fcn, RAnalOp *op, const char *reg, const char *sign, char type) {
-	char sigstr[16] = {0};
+	char sigstr[16] = { 0 };
 	st64 ptr;
 	char *addr;
-	if (!anal || !fcn || !op) {
-		return;
-	}
+
+	r_return_if_fail (anal && fcn && op);
+
 	snprintf (sigstr, sizeof (sigstr), ",%s,%s", reg, sign);
 	const char *op_esil = r_strbuf_get (&op->esil);
 	if (!op_esil) {
@@ -670,11 +668,17 @@ beach:
 R_API void extract_rarg(RAnal *anal, RAnalOp *op, RAnalFunction *fcn, int *reg_set, int *count) {
 	const char *opsreg = NULL;
 	const char *opdreg = NULL;
-	int i, delta = 0;
+	int i, argc = 0;
 
-	if (!anal || !op || !fcn) {
+	r_return_if_fail (anal && op && fcn);
+
+	if (!fcn->cc) {
+		R_LOG_DEBUG ("No calling convention for function '%s' to extract register arguments\n", fcn->name);
 		return;
 	}
+
+	char *fname = fcn->name;
+	Sdb *TDB = anal->sdb_types;
 	int max_count = r_anal_cc_max_arg (anal, fcn->cc);
 	if (!max_count || (*count >= max_count)) {
 		return;
@@ -685,26 +689,45 @@ R_API void extract_rarg(RAnal *anal, RAnalOp *op, RAnalFunction *fcn, int *reg_s
 	if (op->dst) {
 		opdreg = get_regname (anal, op->dst);
 	}
-	if (opsreg) {
-		RRegItem *ri = r_reg_get (anal->reg, opsreg, -1);
-		if (ri) {
-			delta = ri->index;
+	if (fname) {
+		char *tmp = strchr (fname, '.');
+		if (tmp) {
+			fname = tmp + 1;
 		}
+		argc = r_type_func_args_count (TDB, fname);
 	}
 	for (i = 0; i < max_count; i++) {
 		const char *regname = r_anal_cc_arg (anal, fcn->cc, i + 1);
 		// reg_set enusres we only extract first-read argument reg
 		if (!reg_set [i] && regname) {
-			if (op->src[0] && opsreg && !strcmp (opsreg, regname)) {
-				char *vname = r_str_newf ("%s%d", "arg", i + 1);
-				r_anal_var_add (anal, fcn->addr, 1, delta, 'r', NULL, anal->bits / 8,
-						1, vname);
-				if (op->var && op->var->kind != 'r') {
+			bool cond = (op->type  == R_ANAL_OP_TYPE_CMP) && opdreg &&
+				!strcmp (opdreg, regname);
+			if ((op->src[0] && opsreg && !strcmp (opsreg, regname)) || cond) {
+				const char *vname = NULL;
+				char *type = NULL;
+				char *name = NULL;
+				int delta = 0;
+				RRegItem *ri = r_reg_get (anal->reg, regname, -1);
+				if (ri) {
+					delta = ri->index;
+				}
+				if ((i < argc) && fname) {
+					type = r_type_func_args_type (TDB, fname, i);
+					vname = r_type_func_args_name (TDB, fname, i);
+				}
+				if (!vname) {
+					name = r_str_newf ("%s%d", "arg", i + 1);
+					vname = name;
+				}
+				r_anal_var_add (anal, fcn->addr, 1, delta, R_ANAL_VAR_KIND_REG, type,
+						anal->bits / 8, 1, vname);
+				if (op->var && op->var->kind != R_ANAL_VAR_KIND_REG) {
 					r_anal_var_link (anal, op->addr, op->var);
 				}
-				r_anal_var_access (anal, fcn->addr, 'r', 1, delta, 0, op->addr);
+				r_anal_var_access (anal, fcn->addr, R_ANAL_VAR_KIND_REG, 1, delta, 0, op->addr);
 				r_meta_set_string (anal, R_META_TYPE_VARTYPE, op->addr, vname);
-				free (vname);
+				free (name);
+				free (type);
 			}
 			if (op->dst && opdreg && !strcmp (opdreg, regname)) {
 				reg_set [i] = 1;
@@ -715,11 +738,10 @@ R_API void extract_rarg(RAnal *anal, RAnalOp *op, RAnalFunction *fcn, int *reg_s
 }
 
 R_API void extract_vars(RAnal *anal, RAnalFunction *fcn, RAnalOp *op) {
-	if (!anal || !fcn || !op) {
-		return;
-	}
+	r_return_if_fail (anal && fcn && op);
+
 	const char *BP = anal->reg->name[R_REG_NAME_BP];
-	const char *SP =  anal->reg->name[R_REG_NAME_SP];
+	const char *SP = anal->reg->name[R_REG_NAME_SP];
 	extract_arg (anal, fcn, op, BP, "+", 'b');
 	extract_arg (anal, fcn, op, BP, "-", 'b');
 	extract_arg (anal, fcn, op, SP, "+", 's');
@@ -765,6 +787,7 @@ static RList *var_generate_list(RAnal *a, RAnalFunction *fcn, int kind, bool dyn
 					free (av);
 					continue;
 				}
+				av->addr = fcn->addr;
 				av->delta = delta;
 				av->kind = kind;
 				av->name = strdup (vt.name);
@@ -855,13 +878,13 @@ R_API void r_anal_var_list_show(RAnal *anal, RAnalFunction *fcn, int kind, int m
 						"\"kind\":\"arg\",\"type\":\"%s\",\"ref\":"
 						"{\"base\":\"%s\", \"offset\":%"PFMT64d "}}",
 						var->name, var->type, anal->reg->name[R_REG_NAME_BP],
-						var->delta);
+						(st64)var->delta);
 				} else {
 					anal->cb_printf ("{\"name\":\"%s\","
 						"\"kind\":\"var\",\"type\":\"%s\",\"ref\":"
-						"{\"base\":\"%s\", \"offset\":-%"PFMT64d "}}",
+						"{\"base\":\"%s\", \"offset\":%"PFMT64d "}}",
 						var->name, var->type, anal->reg->name[R_REG_NAME_BP],
-						-var->delta);
+						(st64)-R_ABS (var->delta));
 				}
 				break;
 			case R_ANAL_VAR_KIND_REG: {
@@ -872,7 +895,7 @@ R_API void r_anal_var_list_show(RAnal *anal, RAnalFunction *fcn, int kind, int m
 				}
 				anal->cb_printf ("{\"name\":\"%s\","
 					"\"kind\":\"reg\",\"type\":\"%s\",\"ref\":\"%s\"}",
-					var->name, var->type, i->name, anal->reg->name[var->delta]);
+					var->name, var->type, i->name);
 			}
 				break;
 			case R_ANAL_VAR_KIND_SPV:
@@ -887,7 +910,7 @@ R_API void r_anal_var_list_show(RAnal *anal, RAnalFunction *fcn, int kind, int m
 						"\"kind\":\"var\",\"type\":\"%s\",\"ref\":"
 						"{\"base\":\"%s\", \"offset\":-%"PFMT64d "}}",
 						var->name, var->type, anal->reg->name[R_REG_NAME_SP],
-						var->delta);
+						(st64)R_ABS(var->delta));
 				}
 				break;
 			}
@@ -938,7 +961,7 @@ R_API void r_anal_var_list_show(RAnal *anal, RAnalFunction *fcn, int kind, int m
 		}
 	}
 	if (mode == 'j') {
-		anal->cb_printf ("]\n");
+		anal->cb_printf ("]");
 	}
 	r_list_free (list);
 }
