@@ -70,7 +70,7 @@ static int get_mem_option(char *token) {
 	                         "xxx", "osh", "oshst", "oshld", NULL};
 	int i = 0;
 	while (options[i]) {
-		if (!strcasecmp (token, options[i])) {
+		if (!r_str_casecmp (token, options[i])) {
 			return 15 - i;
 		}
 		i++;
@@ -244,7 +244,7 @@ static ut32 cmp(ArmOp *op) {
 }
 
 
-static ut32 sturop(ArmOp *op, int k) {
+static ut32 regsluop(ArmOp *op, int k) {
 	ut32 data = UT32_MAX;
 
 	if (op->operands[1].reg_type & ARM_REG32) {
@@ -418,7 +418,13 @@ static ut32 branch(ArmOp *op, ut64 addr, int k) {
 	if (op->operands[0].type & ARM_CONSTANT) {
 		n = op->operands[0].immediate;
 		if (!(n & 0x3 || n > 0x7ffffff)) {
-			n -= addr;
+			if (n > addr) {
+				n -= addr;
+			} else {
+				n -= addr;
+				n = n & 0xfffffff;
+				k |= 3;
+			}
 			n = n >> 2;
 			int t = n >> 24;
 			int h = n >> 16;
@@ -737,6 +743,10 @@ static bool parseOperands(char* str, ArmOp *op) {
 		while (token[0] == ' ') {
 			token++;
 		}
+		if (operand >= MAX_OPERANDS) {
+			eprintf ("Too many operands\n");
+			return false;
+		}
 		op->operands[operand].type = ARM_NOTYPE;
 		op->operands[operand].reg_type = ARM_UNDEFINED;
 		op->operands[operand].shift = ARM_NO_SHIFT;
@@ -752,10 +762,11 @@ static bool parseOperands(char* str, ArmOp *op) {
 		} else if (!strncmp (token, "asr", 3)) {
 			op->operands[operand].shift = ARM_ASR;
 		}
-		if (op->operands[operand].shift != ARM_NO_SHIFT) {
+		if (strlen (token) > 4 && op->operands[operand].shift != ARM_NO_SHIFT) {
 			op->operands_count ++;
 			op->operands[operand].shift_amount = r_num_math (NULL, token + 4);
 			if (op->operands[operand].shift_amount > 63) {
+				free (t);
 				return false;
 			}
 			operand ++;
@@ -774,6 +785,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 			op->operands[operand].reg_type = ARM_REG64;
 			op->operands[operand].reg = r_num_math (NULL, token + 1);
 			if (op->operands[operand].reg > 31) {
+				free (t);
 				return false;
 			}
 			break;
@@ -783,6 +795,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 			op->operands[operand].reg_type = ARM_REG32;
 			op->operands[operand].reg = r_num_math (NULL, token + 1);
 			if (op->operands[operand].reg > 31) {
+				free (t);
 				return false;
 			}
 			break;
@@ -796,7 +809,7 @@ static bool parseOperands(char* str, ArmOp *op) {
 			if (token[1] == 'P' || token [1] == 'p') {
 				int i;
 				for (i = 0; msr_const[i].name; i++) {
-					if (!strncasecmp (token, msr_const[i].name, strlen (msr_const[i].name))) {
+					if (!r_str_ncasecmp (token, msr_const[i].name, strlen (msr_const[i].name))) {
 						op->operands[operand].sp_val = msr_const[i].val;
 						break;
 					}
@@ -914,7 +927,11 @@ bool arm64ass(const char *str, ut64 addr, ut32 *op) {
 		return *op != -1;
 	}
 	if (!strncmp (str, "stur", 4)) {
-		*op = sturop (&ops, 0x000000f8);
+		*op = regsluop (&ops, 0x000000f8);
+		return *op != -1;
+	}
+	if (!strncmp (str, "ldur", 4)) {
+		*op = regsluop (&ops, 0x000040f8);
 		return *op != -1;
 	}
 	if (!strncmp (str, "str", 3)) {
