@@ -258,8 +258,9 @@ char *get_reg_pair(ut32 idx) {
 	default: res = NULL;
 	}
 
-	if(res != NULL)
-		res = strdup(res);
+	if (res != NULL) {
+		res = strdup (res);
+	}
 
 	return res;
 }
@@ -322,8 +323,9 @@ char *get_reg_name_3(ut32 idx) {
 	default: res = NULL;
 	}
 
-	if (res != NULL)
-		res = strdup(res);
+	if (res != NULL) {
+		res = strdup (res);
+	}
 	return res;
 }
 
@@ -377,8 +379,9 @@ char *get_reg_name_2(ut32 idx) {
 	default: res = NULL;
 	}
 
-	if(res != NULL)
-		res = strdup(res);
+	if (res != NULL) {
+		res = strdup (res);
+	}
 
 	return res;
 }
@@ -631,8 +634,9 @@ char *get_reg_name_1(ut32 idx) {
 	default: res = NULL;
 	}
 
-	if(res != NULL)
-		res = strdup(res);
+	if (res != NULL) {
+		res = strdup (res);
+	}
 
 	return res;
 }
@@ -846,8 +850,9 @@ char *get_status_regs_and_bits(char *reg_arg, int reg_bit) {
         }
   }
 
-  if(res != NULL)
-	res = strdup(res);
+  if (res != NULL) {
+	  res = strdup (res);
+  }
 
   return res;
 }
