@@ -18,6 +18,7 @@ char *directives[] = {
 
 static RAsmPlugin *asm_static_plugins[] = { R_ASM_STATIC_PLUGINS };
 
+/* pseudo.c - private api */
 static int r_asm_pseudo_align(RAsmCode *acode, RAsmOp *op, char *input) {
 	acode->code_align = r_num_math (NULL, input);
 	return 0;
@@ -36,8 +37,8 @@ static int r_asm_pseudo_string(RAsmOp *op, char *input, int zero) {
 		input++;
 	}
 	len = r_str_unescape (input) + zero;
-	r_hex_bin2str ((ut8*)input, len, op->buf_hex);
-	strncpy ((char*)op->buf, input, R_ASM_BUFSIZE - 1);
+	r_hex_bin2str ((ut8*)input, len, r_strbuf_get (&op->buf_hex));
+	r_strbuf_set (&op->buf, input); // uh?
 	return len;
 }
 
@@ -62,9 +63,16 @@ static inline int r_asm_pseudo_org(RAsm *a, char *input) {
 	return 0;
 }
 
+// wtf isnt this the same as r_asm_op_set_hex() ??
 static inline int r_asm_pseudo_hex(RAsmOp *op, char *input) {
-	int len = r_hex_str2bin (input, op->buf);
-	strncpy (op->buf_hex, r_str_trim_head_tail (input), R_ASM_BUFSIZE-1);
+	// assume input fits inside op->buf
+	int len = r_hex_str2bin (input, (ut8*)r_strbuf_get (&op->buf));
+	r_asm_op_set_hex (op, r_str_trim_head_tail (input));
+	if (len < 0) {
+		len = -len;
+		len--;
+		eprintf ("Buffer truncated at %d because of the RAsmOp abuse.\nInput: %s\n", len, input);
+	}
 	return len;
 }
 
@@ -78,19 +86,20 @@ static inline int r_asm_pseudo_intN(RAsm *a, RAsmOp *op, char *input, int n) {
 		return 0;
 	}
 	// XXX honor endian here
+	ut8 *buf = (ut8*)r_strbuf_get (&op->buf);
 	if (n == 2) {
 		s = (short)s64;
-		r_write_ble16 (op->buf, s, a->big_endian);
+		r_write_ble16 (buf, s, a->big_endian);
 	} else if (n == 4) {
 		i = (int)s64;
-		r_write_ble32 (op->buf, i, a->big_endian);
+		r_write_ble32 (buf, i, a->big_endian);
 	} else if (n == 8) {
 		l = (long int)s64;
-		r_write_ble64 (op->buf, l, a->big_endian);
+		r_write_ble64 (buf, l, a->big_endian);
 	} else {
 		return 0;
 	}
-	r_hex_bin2str (op->buf, n, op->buf_hex);
+	r_hex_bin2str (buf, n, r_strbuf_get (&op->buf_hex));
 	return n;
 }
 
@@ -111,24 +120,31 @@ static inline int r_asm_pseudo_byte(RAsmOp *op, char *input) {
 	r_str_replace_char (input, ',', ' ');
 	len = r_str_word_count (input);
 	r_str_word_set0 (input);
+	ut8 *buf = malloc (len);
+	if (!buf) {
+		return 0;
+	}
 	for (i = 0; i < len; i++) {
 		const char *word = r_str_word_get0 (input, i);
 		int num = (int)r_num_math (NULL, word);
-		op->buf[i] = num;
+		buf[i] = num;
 	}
-	r_hex_bin2str (op->buf, len, op->buf_hex);
+	r_asm_op_set_buf (op, buf, len);
+	free (buf);
 	return len;
 }
 
 static inline int r_asm_pseudo_fill(RAsmOp *op, char *input) {
-	int i, repeat=0, size=0, value=0;
+	int i, repeat = 0, size=0, value=0;
 	sscanf (input, "%d,%d,%d", &repeat, &size, &value); // use r_num?
-	size *= repeat;
+	size *= (sizeof (value) * repeat);
 	if (size > 0) {
-		for (i = 0; i < size; i++) {
-			op->buf[i] = value;
+		ut8 *buf = malloc (size);
+		for (i = 0; i < size; i+= sizeof(value)) {
+			memcpy (&buf[i], &value, sizeof(value));
 		}
-		r_hex_bin2str (op->buf, size, op->buf_hex);
+		r_asm_op_set_buf (op, buf, size);
+		free (buf);
 	} else {
 		size = 0;
 	}
@@ -136,15 +152,13 @@ static inline int r_asm_pseudo_fill(RAsmOp *op, char *input) {
 }
 
 static inline int r_asm_pseudo_incbin(RAsmOp *op, char *input) {
-	int skip = 0;
-	int count = 0;
 	int bytes_read = 0;
 	r_str_replace_char (input, ',', ' ');
 	// int len = r_str_word_count (input);
 	r_str_word_set0 (input);
 	//const char *filename = r_str_word_get0 (input, 0);
-	skip = (int)r_num_math (NULL, r_str_word_get0 (input, 1));
-	count = (int)r_num_math (NULL,r_str_word_get0 (input, 2));
+	int skip = (int)r_num_math (NULL, r_str_word_get0 (input, 1));
+	int count = (int)r_num_math (NULL,r_str_word_get0 (input, 2));
 	char *content = r_file_slurp (input, &bytes_read);
 	if (skip > 0) {
 		skip = skip > bytes_read ? bytes_read : skip;
@@ -158,7 +172,6 @@ static inline int r_asm_pseudo_incbin(RAsmOp *op, char *input) {
 	r_buf_free (op->buf_inc);
 	op->buf_inc = r_buf_new_with_string (content + skip);
 	// Terminate the original buffer
-	op->buf_hex[0] = '\0';
 	free (content);
 	return count;
 }
@@ -211,8 +224,9 @@ R_API int r_asm_filter_input(RAsm *a, const char *f) {
 }
 
 R_API int r_asm_filter_output(RAsm *a, const char *f) {
-	if (!a->ofilter)
+	if (!a->ofilter) {
 		a->ofilter = r_parse_new ();
+	}
 	if (!r_parse_use (a->ofilter, f)) {
 		r_parse_free (a->ofilter);
 		a->ofilter = NULL;
@@ -233,7 +247,7 @@ R_API RAsm *r_asm_free(RAsm *a) {
 		r_syscall_free (a->syscall);
 		free (a->cpu);
 		sdb_free (a->pair);
-		ht_free (a->flags);
+		ht_pp_free (a->flags);
 		a->pair = NULL;
 		free (a);
 	}
@@ -326,13 +340,6 @@ R_API int r_asm_use(RAsm *a, const char *name) {
 	return false;
 }
 
-R_API int r_asm_set_subarch(RAsm *a, const char *name) {
-	if (a->cur && a->cur->set_subarch) {
-		return a->cur->set_subarch(a, name);
-	}
-	return false;
-}
-
 static int has_bits(RAsmPlugin *h, int bits) {
 	return (h && h->bits && (bits & h->bits));
 }
@@ -395,28 +402,31 @@ R_API int r_asm_set_pc(RAsm *a, ut64 pc) {
 	return true;
 }
 
+static bool isInvalid (RAsmOp *op) {
+	const char *buf_asm = r_strbuf_get (&op->buf_asm);
+	return (buf_asm && *buf_asm && !strcmp (buf_asm, "invalid"));
+}
+
 R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
-	int oplen, ret;
+	int ret;
 	if (!a || !buf || !op) {
 		return -1;
 	}
+	if (len < 1) {
+		return 0;
+	}
 	ret = op->payload = 0;
 	op->size = 4;
 	op->bitsize = 0;
-	if (len < 1) {
-		return 0;
-	}
-	op->buf_asm[0] = '\0';
+	r_asm_op_set_asm (op, "");
 	if (a->pcalign) {
 		const int mod = a->pc % a->pcalign;
 		if (mod) {
 			op->size = a->pcalign - mod;
-			strcpy (op->buf_asm, "unaligned");
-			*op->buf_hex = 0;
-			if ((op->size * 4) >= sizeof (op->buf_hex)) {
-				oplen = (sizeof (op->buf_hex) / 4) - 1;
-			}
-			r_hex_bin2str (buf, op->size, op->buf_hex);
+			r_strbuf_set (&op->buf_asm, "unaligned");
+			r_strbuf_set (&op->buf_hex, "");
+			// assume op->size after pcalign-mod will be small enough to not need checks here
+			r_hex_bin2str (buf, op->size, r_strbuf_get (&op->buf_hex));
 			return -1;
 		}
 	}
@@ -436,53 +446,48 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	if (ret < 0) {
 		ret = 0;
 	}
-	oplen = r_asm_op_get_size (op);
 	if (op->bitsize > 0) {
-		oplen = op->size = op->bitsize / 8;
+		op->size = op->bitsize / 8;
 		a->bitshift += op->bitsize % 8;
 		int count = a->bitshift / 8;
 		if (count > 0) {
-			oplen = op->size = op->size + count;
+			op->size = op->size + count;
 			a->bitshift %= 8;
 		}
-	} else {
-		oplen = op->size;
 	}
-	if (oplen < 1) {
-		oplen = 1;
-	}
-	if (!op->buf_asm[0] || op->size < 1 || !strcmp (op->buf_asm, "invalid")) {
+
+	if (op->size < 1 || isInvalid (op)) {
 		if (a->invhex) {
 			if (a->bits == 16) {
 				ut16 b = r_read_le16 (buf);
-				snprintf (op->buf_asm, sizeof (op->buf_asm), ".word 0x%04x", b);
+				r_strbuf_set (&op->buf_asm, sdb_fmt (".word 0x%04x", b));
 			} else {
 				ut32 b = r_read_le32 (buf);
-				snprintf (op->buf_asm, sizeof (op->buf_asm), ".dword 0x%08x", b);
+				r_strbuf_set (&op->buf_asm, sdb_fmt (".dword 0x%08x", b));
 			}
 			// TODO: something for 64bits too?
 		} else {
-			strcpy (op->buf_asm, "invalid");
+			r_strbuf_set (&op->buf_asm, "invalid");
 		}
 	}
 	if (a->ofilter) {
-		r_parse_parse (a->ofilter, op->buf_asm, op->buf_asm);
-	}
-	// XXX this is crap and needs proper fix
-	int dstlen = R_MIN ((R_ASM_BUFSIZE -1), oplen);
-	int cpylen = R_MIN (dstlen, len);
-	if (cpylen > 0) {
-		memcpy (op->buf, buf, cpylen);
-		const int addrbytes = a->user ? ((RCore *)a->user)->io->addrbytes : 1;
-		int len = R_MIN (addrbytes * oplen, (sizeof (op->buf_hex) - 1) / 2);
-		// len is the length in input buffer, not len of output, so we must / 2
-		r_hex_bin2str (op->buf, len, op->buf_hex);
+		char *buf_asm = r_strbuf_get (&op->buf_asm);
+		r_parse_parse (a->ofilter, buf_asm, buf_asm);
 	}
+	int opsz = (op->size > 0)? R_MAX (0, R_MIN (len, op->size)): 1;
+	r_asm_op_set_buf (op, buf, opsz);
 	return ret;
 }
 
 typedef int (*Ase)(RAsm *a, RAsmOp *op, const char *buf);
 
+static bool assemblerMatches(RAsm *a, RAsmPlugin *h) {
+	if (!a || !h->arch || !h->assemble || !has_bits (h, a->bits)) {
+		return false;
+	}
+	return (!strncmp (a->cur->arch, h->arch, strlen (a->cur->arch)));
+}
+
 static Ase findAssembler(RAsm *a, const char *kw) {
 	Ase ase = NULL;
 	RAsmPlugin *h;
@@ -491,11 +496,7 @@ static Ase findAssembler(RAsm *a, const char *kw) {
 		return a->acur->assemble;
 	}
 	r_list_foreach (a->plugins, iter, h) {
-		if (h->arch && h->assemble
-				&& has_bits (h, a->bits)
-				&& !strncmp (a->cur->arch,
-					h->arch,
-					strlen (a->cur->arch))) {
+		if (assemblerMatches (a, h)) {
 			if (kw) {
 				if (strstr (h->name, kw)) {
 					return h->assemble;
@@ -514,7 +515,9 @@ static char *replace_directives_for(char *str, char *token) {
 	char *q = str;
 	bool changes = false;
 	for (;;) {
-		if (q) p = strstr (q, token);
+		if (q) {
+			p = strstr (q, token);
+		}
 		if (p) {
 			char *nl = strchr (p, '\n');
 			if (nl) {
@@ -528,7 +531,9 @@ static char *replace_directives_for(char *str, char *token) {
 			q = nl;
 			changes = true;
 		} else {
-			if (q) r_strbuf_append (sb, q);
+			if (q) {
+				r_strbuf_append (sb, q);
+			}
 			break;
 		}
 	}
@@ -563,7 +568,6 @@ R_API void r_asm_list_directives() {
 R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	int ret = 0;
 	char *b = strdup (buf);
-
 	if (!b) {
 		return 0;
 	}
@@ -590,16 +594,21 @@ R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 			ret = ase (a, op, b);
 		}
 	}
+	// XXX delete this block, the ase thing should be setting asm, buf and hex
 	if (op && ret > 0) {
-		r_hex_bin2str (op->buf, ret, op->buf_hex);
-		op->size = ret;
-		op->buf_hex[ret*2] = 0;
-		strncpy (op->buf_asm, b, R_ASM_BUFSIZE - 1);
+		op->size = ret; // XXX shouldnt be necessary
+		r_asm_op_set_asm (op, b); // XXX ase should be updating this already, isnt?
+		ut8 *ophex = (ut8*)r_strbuf_get (&op->buf_hex);
+		if (!*ophex) {
+			ut8 *opbuf = (ut8*)r_strbuf_get (&op->buf);
+			r_asm_op_set_buf (op, opbuf, ret);
+		}
 	}
 	free (b);
 	return ret;
 }
 
+// TODO: Use RStrBuf api here pls
 R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 	RStrBuf *buf_asm;
 	RAsmCode *acode;
@@ -630,9 +639,10 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 			ret = 1;
 		}
 		if (a->ofilter) {
-			r_parse_parse (a->ofilter, op.buf_asm, op.buf_asm);
+			char *op_buf_asm = r_strbuf_get (&op.buf_asm);
+			r_parse_parse (a->ofilter, op_buf_asm, op_buf_asm);
 		}
-		r_strbuf_append (buf_asm, op.buf_asm);
+		r_strbuf_append (buf_asm, r_strbuf_get (&op.buf_asm));
 		r_strbuf_append (buf_asm, "\n");
 	}
 	acode->buf_asm = r_strbuf_drain (buf_asm);
@@ -670,18 +680,16 @@ R_API RAsmCode* r_asm_assemble_file(RAsm *a, const char *file) {
 	return ac;
 }
 
-static void flag_free_kv(HtKv *kv) {
+static void flag_free_kv(HtPPKv *kv) {
 	free (kv->key);
 	free (kv->value);
-	free (kv);
 }
 
-static char* dup_val(void *v) {
-	char *str = strdup ((char *)v);
-	return str;
+static void *dup_val(const void *v) {
+	return (void *)strdup ((char *)v);
 }
 
-R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
+R_API RAsmCode *r_asm_massemble(RAsm *a, const char *buf) {
 	int labels = 0, num, stage, ret, idx, ctr, i, j, linenum = 0;
 	char *lbuf = NULL, *ptr2, *ptr = NULL, *ptr_start = NULL;
 	char *tokens[R_ASM_BUFSIZE], buf_token[R_ASM_BUFSIZE];
@@ -692,8 +700,8 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 	if (!buf) {
 		return NULL;
 	}
-	ht_free (a->flags);
-	if (!(a->flags = ht_new (dup_val, flag_free_kv, NULL))) {
+	ht_pp_free (a->flags);
+	if (!(a->flags = ht_pp_new (dup_val, flag_free_kv, NULL))) {
 		return NULL;
 	}
 	if (!(acode = r_asm_code_new ())) {
@@ -707,12 +715,11 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 		return r_asm_code_free (acode);
 	}
 	*acode->buf_hex = 0;
-	if (!(acode->buf = malloc (64))) {
+	if (!(acode->buf = calloc (1, 64))) {
 		return r_asm_code_free (acode);
 	}
 	lbuf = strdup (buf);
 	acode->code_align = 0;
-	memset (&op, 0, sizeof (op));
 
 	/* consider ,, an alias for a newline */
 	lbuf = r_str_replace (lbuf, ",,", "\n", true);
@@ -781,7 +788,6 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 		r_asm_set_pc (a, pc);
 		for (idx = ret = i = j = 0, off = a->pc, acode->buf_hex[0] = '\0';
 				i <= ctr; i++, idx += ret) {
-			memset (buf_token, 0, R_ASM_BUFSIZE);
 			strncpy (buf_token, tokens[i], R_ASM_BUFSIZE - 1);
 			if (inComment) {
 				if (!strncmp (ptr_start, "*/", 2)) {
@@ -792,10 +798,16 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 			// XXX TODO remove arch-specific hacks
 			if (!strncmp (a->cur->arch, "avr", 3)) {
 				for (ptr_start = buf_token; *ptr_start &&
-					isavrseparator (*ptr_start); ptr_start++);
+							    isavrseparator (*ptr_start);
+					ptr_start++) {
+					;
+				}
 			} else {
 				for (ptr_start = buf_token; *ptr_start &&
-					IS_SEPARATOR (*ptr_start); ptr_start++);
+							    IS_SEPARATOR (*ptr_start);
+					ptr_start++) {
+					;
+				}
 			}
 			if (!strncmp (ptr_start, "/*", 2)) {
 				if (!strstr (ptr_start + 2, "*/")) {
@@ -828,14 +840,13 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 				if (is_a_label) {
 					//if (stage != 2) {
 					if (ptr_start[1] != 0 && ptr_start[1] != ' ') {
-						char food[64];
 						*ptr = 0;
 						if (acode->code_align) {
 							off += (acode->code_align - (off % acode->code_align));
 						}
-						snprintf (food, sizeof (food), "0x%"PFMT64x"", off);
-
-						ht_insert (a->flags, ptr_start, food);
+						char food[64];
+						snprintf (food, sizeof (food), "0x%"PFMT64x, off);
+						ht_pp_insert (a->flags, ptr_start, food);
 						// TODO: warning when redefined
 						r_asm_code_set_equ (acode, ptr_start, food);
 					}
@@ -879,42 +890,44 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 					r_asm_use (a, "arm");
 					r_asm_set_bits (a, 16);
 					ret = 0;
-				} else if (!strncmp (ptr, ".arch ", 6))
+				} else if (!strncmp (ptr, ".arch ", 6)) {
 					ret = r_asm_pseudo_arch (a, ptr+6);
-				else if (!strncmp (ptr, ".bits ", 6))
+				} else if (!strncmp (ptr, ".bits ", 6)) {
 					ret = r_asm_pseudo_bits (a, ptr+6);
-				else if (!strncmp (ptr, ".fill ", 6))
+				} else if (!strncmp (ptr, ".fill ", 6)) {
 					ret = r_asm_pseudo_fill (&op, ptr+6);
-				else if (!strncmp (ptr, ".kernel ", 8))
+				} else if (!strncmp (ptr, ".kernel ", 8)) {
 					r_syscall_setup (a->syscall, a->cur->arch, a->bits, asmcpu, ptr + 8);
-				else if (!strncmp (ptr, ".cpu ", 5))
+				} else if (!strncmp (ptr, ".cpu ", 5)) {
 					r_asm_set_cpu (a, ptr + 5);
-				else if (!strncmp (ptr, ".os ", 4))
+				} else if (!strncmp (ptr, ".os ", 4)) {
 					r_syscall_setup (a->syscall, a->cur->arch, a->bits, asmcpu, ptr + 4);
-				else if (!strncmp (ptr, ".hex ", 5))
-					ret = r_asm_pseudo_hex (&op, ptr+5);
-				else if ((!strncmp (ptr, ".int16 ", 7)) || !strncmp (ptr, ".short ", 7))
+				} else if (!strncmp (ptr, ".hex ", 5)) {
+					ret = r_asm_pseudo_hex (&op, ptr + 5);
+				} else if ((!strncmp (ptr, ".int16 ", 7)) || !strncmp (ptr, ".short ", 7)) {
 					ret = r_asm_pseudo_int16 (a, &op, ptr+7);
-				else if (!strncmp (ptr, ".int32 ", 7))
+				} else if (!strncmp (ptr, ".int32 ", 7)) {
 					ret = r_asm_pseudo_int32 (a, &op, ptr+7);
-				else if (!strncmp (ptr, ".int64 ", 7))
+				} else if (!strncmp (ptr, ".int64 ", 7)) {
 					ret = r_asm_pseudo_int64 (a, &op, ptr+7);
-				else if (!strncmp (ptr, ".size", 5))
+				} else if (!strncmp (ptr, ".size", 5)) {
 					ret = true; // do nothing, ignored
-				else if (!strncmp (ptr, ".section", 8))
+				} else if (!strncmp (ptr, ".section", 8)) {
 					ret = true; // do nothing, ignored
-				else if ((!strncmp (ptr, ".byte ", 6)) || (!strncmp (ptr, ".int8 ", 6)))
+				} else if ((!strncmp (ptr, ".byte ", 6)) || (!strncmp (ptr, ".int8 ", 6))) {
 					ret = r_asm_pseudo_byte (&op, ptr+6);
-				else if (!strncmp (ptr, ".glob", 5)) { // .global .globl
-				//	eprintf (".global directive not yet implemented\n");
+				} else if (!strncmp (ptr, ".glob", 5)) { // .global .globl
+									 //	eprintf (".global directive not yet implemented\n");
 					ret = 0;
 					continue;
 				} else if (!strncmp (ptr, ".equ ", 5)) {
 					ptr2 = strchr (ptr + 5, ',');
-					if (!ptr2)
+					if (!ptr2) {
 						ptr2 = strchr (ptr + 5, '=');
-					if (!ptr2)
+					}
+					if (!ptr2) {
 						ptr2 = strchr (ptr + 5, ' ');
+					}
 					if (ptr2) {
 						*ptr2 = '\0';
 						r_asm_code_set_equ (acode, ptr + 5, ptr2 + 1);
@@ -943,8 +956,8 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 					continue;
 				}
 				if (ret < 0) {
-					eprintf ("!!! Oops\n");
-					free(lbuf);
+					eprintf ("!!! Oops (%s)\n", ptr);
+					free (lbuf);
 					return r_asm_code_free (acode);
 				}
 			} else { /* Instruction */
@@ -970,26 +983,32 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 			if (stage == STAGES - 1) {
 				if (ret < 1) {
 					eprintf ("Cannot assemble '%s' at line %d\n", ptr_start, linenum);
-					free(lbuf);
+					free (lbuf);
 					return r_asm_code_free (acode);
 				}
 				acode->len = idx + ret;
 				char *newbuf = realloc (acode->buf, (idx + ret) * 2);
 				if (!newbuf) {
-					free(lbuf);
+					free (lbuf);
 					return r_asm_code_free (acode);
 				}
 				acode->buf = (ut8*)newbuf;
-				newbuf = realloc (acode->buf_hex, strlen (acode->buf_hex) + strlen (op.buf_hex) + r_buf_size (op.buf_inc) + 1);
+
+				const int buf_inc_size = op.buf_inc ? r_buf_size (op.buf_inc): 0;
+				int newlen = strlen (acode->buf_hex) + r_strbuf_length (&op.buf_hex)
+					+ buf_inc_size + 128;
+				newbuf = realloc (acode->buf_hex, newlen);
 				if (!newbuf) {
-					free(lbuf);
+					free (lbuf);
 					return r_asm_code_free (acode);
 				}
 				acode->buf_hex = newbuf;
-				memcpy (acode->buf + idx, op.buf, ret);
-				strcat (acode->buf_hex, op.buf_hex);
-				if (r_buf_size (op.buf_inc) > 1) {
-					if (strlen (acode->buf_hex) > 0) {
+				acode->buf = malloc (4095);
+				memcpy (acode->buf + idx, r_strbuf_get (&op.buf), r_strbuf_length (&op.buf)); // ret);
+				// XXX slow. use strbuf pls
+				strcat (acode->buf_hex, r_strbuf_get (&op.buf_hex));
+				if (op.buf_inc && r_buf_size (op.buf_inc) > 1) {
+					if (*acode->buf_hex) {
 						strcat (acode->buf_hex, "\n");
 					}
 					char *s = r_buf_free_to_string (op.buf_inc);
@@ -1003,34 +1022,12 @@ R_API RAsmCode* r_asm_massemble(RAsm *a, const char *buf) {
 		}
 	}
 	free (lbuf);
+	r_hex_str2bin (acode->buf_hex, acode->buf);
 	return acode;
 }
 
-R_API int r_asm_modify(RAsm *a, ut8 *buf, int field, ut64 val) {
-	if (a->cur && a->cur->modify) {
-		return a->cur->modify (a, buf, field, val);
-	}
-	return false;
-}
-
-R_API char *r_asm_op_get_hex(RAsmOp *op) {
-	return strdup (op->buf_hex);
-}
-
-R_API char *r_asm_op_get_asm(RAsmOp *op) {
-	return strdup (op->buf_asm);
-}
-
-R_API int r_asm_op_get_size(RAsmOp *op) {
-	int len;
-	if (!op) {
-		return 1;
-	}
-	len = op->size - op->payload;
-	if (len < 1) {
-		len = 1;
-	}
-	return len;
+R_API bool r_asm_modify(RAsm *a, ut8 *buf, int field, ut64 val) {
+	return (a->cur && a->cur->modify) ? a->cur->modify (a, buf, field, val): false;
 }
 
 R_API int r_asm_get_offset(RAsm *a, int type, int idx) { // link to rbin
@@ -1041,30 +1038,21 @@ R_API int r_asm_get_offset(RAsm *a, int type, int idx) { // link to rbin
 }
 
 R_API char *r_asm_describe(RAsm *a, const char* str) {
-	if (!a->pair) {
-		return NULL;
-	}
-	return sdb_get (a->pair, str, 0);
+	return (a && a->pair)? sdb_get (a->pair, str, 0): NULL;
 }
 
 R_API RList* r_asm_get_plugins(RAsm *a) {
 	return a->plugins;
 }
 
-/* new simplified API */
-
 R_API bool r_asm_set_arch(RAsm *a, const char *name, int bits) {
-	if (!r_asm_use (a, name)) {
-		return false;
-	}
-	return r_asm_set_bits (a, bits);
+	return r_asm_use (a, name)? r_asm_set_bits (a, bits): false;
 }
 
 /* to ease the use of the native bindings (not used in r2) */
 R_API char *r_asm_to_string(RAsm *a, ut64 addr, const ut8 *b, int l) {
-	RAsmCode *code;
 	r_asm_set_pc (a, addr);
-	code = r_asm_mdisassemble (a, b, l);
+	RAsmCode *code = r_asm_mdisassemble (a, b, l);
 	if (code) {
 		char *buf_asm = code->buf_asm;
 		code->buf_asm = NULL;
@@ -1130,6 +1118,9 @@ R_API int r_asm_mnemonics_byname(RAsm *a, const char *name) {
 
 R_API RAsmCode* r_asm_rasm_assemble(RAsm *a, const char *buf, bool use_spp) {
 	char *lbuf = strdup (buf);
+	if (!lbuf) {
+		return NULL;
+	}
 	RAsmCode *acode;
 	if (use_spp) {
 		Output out;
