@@ -1,22 +1,17 @@
+/* radare - LGPL - Copyright 2014-2018 - fedor.sakharov */
+
 #include <stdio.h>
 #include <string.h>
 #include <r_types.h>
 #include <r_lib.h>
 #include <r_asm.h>
-
 #include <cr16_disas.h>
 
-static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len)
-{
-	int ret = 1;
+static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	struct cr16_cmd cmd;
-
-	ret = cr16_decode_command (buf, &cmd);
-
-	snprintf(op->buf_asm, R_ASM_BUFSIZE, "%s %s", cmd.instr, cmd.operands);
-	op->size = ret;
-
-	return ret;
+	int ret = cr16_decode_command (buf, &cmd);
+	r_strbuf_set (&op->buf_asm, sdb_fmt ("%s %s", cmd.instr, cmd.operands));
+	return op->size = ret;
 }
 
 RAsmPlugin r_asm_plugin_cr16 = {
@@ -30,7 +25,7 @@ RAsmPlugin r_asm_plugin_cr16 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_cr16,
 	.version = R2_VERSION
