@@ -37,7 +37,7 @@ RAsmPlugin r_asm_plugin_x86_gas = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_x86_gas,
 	.version = R2_VERSION
