@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2012-2015 - pancake, condret */
+/* radare - LGPL - Copyright 2012-2018 - pancake, condret */
 
 // fork of asm_z80.c
 
@@ -11,9 +11,7 @@
 
 static int disassemble(RAsm *a, RAsmOp *r_op, const ut8 *buf, int len) {
 	int dlen = gbDisass(r_op,buf,len);
-	if(dlen<0) dlen=0;
-	r_op->size = dlen;
-	return dlen;
+	return r_op->size = R_MAX (0, dlen);
 }
 
 static int assemble(RAsm *a, RAsmOp *r_op, const char *buf) {
@@ -24,6 +22,7 @@ RAsmPlugin r_asm_plugin_gb = {
 	.name = "gb",
 	.desc = "GameBoy(TM) (z80-like)",
 	.arch = "z80",
+	.author = "condret",
 	.license = "LGPL3",
 	.bits = 16,
 	.endian = R_SYS_ENDIAN_LITTLE,
@@ -32,7 +31,7 @@ RAsmPlugin r_asm_plugin_gb = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_gb,
 	.version = R2_VERSION
