@@ -1,17 +1,14 @@
-/* radare - LGPL - Copyright 2012-2013 - Alexander */
+/* radare - LGPL - Copyright 2012-2018 - Alexander */
 
 #include <r_types.h>
 #include <r_util.h>
 #include <r_lib.h>
 #include <r_asm.h>
-
 #include "../arch/i8080/i8080dis.c"
 
-static int do_disassemble(RAsm *a, struct r_asm_op_t *op, const ut8 *buf, int len) {
-	int dlen = i8080_disasm (buf, op->buf_asm, len);
-	if (dlen<0) dlen = 0;
-	op->size = dlen;
-	return op->size;
+static int do_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
+	int dlen = i8080_disasm (buf, r_strbuf_get (&op->buf_asm), len);
+	return op->size = R_MAX (0, dlen);
 }
 
 RAsmPlugin r_asm_plugin_i8080 = {
@@ -25,7 +22,7 @@ RAsmPlugin r_asm_plugin_i8080 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_i8080,
 	.version = R2_VERSION
