@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - nibble, pancake */
+/* radare - LGPL - Copyright 2009-2018 - nibble, pancake */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -10,7 +10,6 @@
 #include "../../shlr/java/class.h"
 
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
-	//void *cp;
 	RBinJavaObj *obj = NULL;
 	RBin *bin = a->binb.bin;
 	RBinPlugin *plugin = bin && bin->cur && bin->cur->o ?
@@ -21,15 +20,16 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 			//eprintf("Handling: %s disasm.\n", b->cur.file);
 		}
 	}
-
-	op->size = r_java_disasm (obj, a->pc, buf, len,
-		op->buf_asm, sizeof (op->buf_asm));
+	char buf_asm[256];
+	op->size = r_java_disasm (obj, a->pc, buf, len, buf_asm, sizeof (buf_asm));
+	r_strbuf_set (&op->buf_asm, buf_asm);
 	return op->size;
 }
 
-static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
+static int assemble(RAsm *a, RAsmOp *op, const char *input) {
 	// TODO: get class info from bin if possible
-	return op->size = r_java_assemble (a->pc, op->buf, buf);
+	// XXX wrong usage of strbuf_get here
+	return op->size = r_java_assemble (a->pc, (ut8*)r_strbuf_get (&op->buf), input);
 }
 
 RAsmPlugin r_asm_plugin_java = {
@@ -44,7 +44,7 @@ RAsmPlugin r_asm_plugin_java = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_java,
 	.version = R2_VERSION
