@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2014 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2018 - pancake, nibble */
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -9,13 +9,13 @@
 #include <r_util.h>
 #include <r_asm.h>
 
-#include "dis-asm.h"
+#include "disas-asm.h"
 #include "opcode/mips.h"
 int mips_assemble(const char *str, ut64 pc, ut8 *out);
 
 static int mips_mode = 0;
 static unsigned long Offset = 0;
-static char *buf_global = NULL;
+static RStrBuf *buf_global = NULL;
 static unsigned char bytes[4];
 
 static int mips_buffer_read_memory (bfd_vma memaddr, bfd_byte *myaddr, unsigned int length, struct disassemble_info *info) {
@@ -31,36 +31,15 @@ static void memory_error_func(int status, bfd_vma memaddr, struct disassemble_in
 	//--
 }
 
-static void print_address(bfd_vma address, struct disassemble_info *info) {
-	char tmp[32];
-	if (!buf_global)
-		return;
-	sprintf (tmp, "0x%08"PFMT64x, (ut64)address);
-	strcat (buf_global, tmp);
-}
-
-static int buf_fprintf(void *stream, const char *format, ...) {
-	va_list ap;
-	char *tmp;
-	if (!buf_global || !format)
-		return false;
-	va_start (ap, format);
- 	tmp = malloc (strlen (format)+strlen (buf_global)+2);
-	if (!tmp) {
-		va_end (ap);
-		return false;
-	}
-	sprintf (tmp, "%s%s", buf_global, format);
-	vsprintf (buf_global, tmp, ap);
-	va_end (ap);
-	free (tmp);
-	return true;
-}
+DECLARE_GENERIC_PRINT_ADDRESS_FUNC()
+DECLARE_GENERIC_FPRINTF_FUNC()
 
 static int disassemble(struct r_asm_t *a, struct r_asm_op_t *op, const ut8 *buf, int len) {
 	static struct disassemble_info disasm_obj;
-	if (len<4) return -1;
-	buf_global = op->buf_asm;
+	if (len < 4) {
+		return -1;
+	}
+	buf_global = &op->buf_asm;
 	Offset = a->pc;
 	memcpy (bytes, buf, 4); // TODO handle thumb
 
@@ -72,33 +51,32 @@ static int disassemble(struct r_asm_t *a, struct r_asm_op_t *op, const ut8 *buf,
 	disasm_obj.read_memory_func = &mips_buffer_read_memory;
 	disasm_obj.symbol_at_address_func = &symbol_at_address;
 	disasm_obj.memory_error_func = &memory_error_func;
-	disasm_obj.print_address_func = &print_address;
+	disasm_obj.print_address_func = &generic_print_address_func;
 	disasm_obj.buffer_vma = Offset;
 	disasm_obj.buffer_length = 4;
 	disasm_obj.endian = !a->big_endian;
-	disasm_obj.fprintf_func = &buf_fprintf;
+	disasm_obj.fprintf_func = &generic_fprintf_func;
 	disasm_obj.stream = stdout;
 
-	op->buf_asm[0] = '\0';
-	if (disasm_obj.endian == BFD_ENDIAN_LITTLE) {
-		op->size = print_insn_little_mips ((bfd_vma)Offset, &disasm_obj);
-	} else {
-		op->size = print_insn_big_mips ((bfd_vma)Offset, &disasm_obj);
+	op->size = (disasm_obj.endian == BFD_ENDIAN_LITTLE)
+		? print_insn_little_mips ((bfd_vma)Offset, &disasm_obj)
+		: print_insn_big_mips ((bfd_vma)Offset, &disasm_obj);
+	if (op->size == -1) {
+		r_strbuf_set (&op->buf_asm, "(data)");
 	}
-	if (op->size == -1)
-		strncpy (op->buf_asm, " (data)", R_ASM_BUFSIZE);
 	return op->size;
 }
 
 static int assemble(RAsm *a, RAsmOp *op, const char *str) {
-	int ret = mips_assemble (str, a->pc, op->buf);
+	ut8 *opbuf = (ut8*)r_strbuf_get (&op->buf);
+	int ret = mips_assemble (str, a->pc, opbuf);
 	if (a->big_endian) {
-		ut8 tmp = op->buf[0];
-		op->buf[0] = op->buf[3];
-		op->buf[3] = tmp;
-		tmp = op->buf[1];
-		op->buf[1] = op->buf[2];
-		op->buf[2] = tmp;
+		ut8 tmp = opbuf[0];
+		opbuf[0] = opbuf[3];
+		opbuf[3] = tmp;
+		tmp = opbuf[1];
+		opbuf[1] = opbuf[2];
+		opbuf[2] = tmp;
 	}
 	return ret;
 }
@@ -107,7 +85,7 @@ RAsmPlugin r_asm_plugin_mips_gnu = {
 	.name = "mips.gnu",
 	.arch = "mips",
 	.license = "GPL3",
-	.bits = 32|64,
+	.bits = 32 | 64,
 	.endian = R_SYS_ENDIAN_LITTLE | R_SYS_ENDIAN_BIG,
 	.desc = "MIPS CPU",
 	.disassemble = &disassemble,
@@ -115,7 +93,7 @@ RAsmPlugin r_asm_plugin_mips_gnu = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_mips_gnu,
 	.version = R2_VERSION
