@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2014-2015 - pancake */
+/* radare2 - LGPL - Copyright 2014-2018 - pancake */
 
 #include <r_asm.h>
 #include <r_lib.h>
@@ -24,10 +24,12 @@ static int decompile_vle(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	}
 	if (!vle_init (&handle, buf, len) && (instr = vle_next (&handle))) {
 		op->size = instr->size;
-		vle_snprint (op->buf_asm, R_ASM_BUFSIZE, a->pc, instr);
+		char buf_asm[64];
+		vle_snprint (buf_asm, sizeof (buf_asm), a->pc, instr);
+		r_asm_op_set_asm (op, buf_asm);
 		vle_free (instr);
 	} else {
-		strcpy (op->buf_asm, "invalid");
+		r_asm_op_set_asm (op, "invalid");
 		op->size = 2;
 		return -1;
 	}
@@ -41,12 +43,13 @@ static int decompile_ps(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	}
 	op->size = 4;
 	const ut32 data = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
-	if (libps_decode (data, &instr)) {
-		libps_snprint (op->buf_asm, R_ASM_BUFSIZE, a->pc, &instr);
-	} else {
-		strcpy (op->buf_asm, "invalid");
+	if (libps_decode (data, &instr) < 1) {
+		r_asm_op_set_asm (op, "invalid");
 		return -1;
 	}
+	char buf_asm[64];
+	libps_snprint (buf_asm, sizeof (buf_asm), a->pc, &instr);
+	r_asm_op_set_asm (op, buf_asm);
 	return op->size;
 }
 
@@ -90,27 +93,27 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		}
 	}
 	op->size = 4;
-	op->buf_asm[0] = 0;
 	cs_option (handle, CS_OPT_DETAIL, CS_OPT_OFF);
-
 	n = cs_disasm (handle, (const ut8*) buf, len, off, 1, &insn);
 	op->size = 4;
 	if (n > 0 && insn->size > 0) {
-		snprintf (op->buf_asm, R_ASM_BUFSIZE, "%s%s%s",
-				insn->mnemonic, insn->op_str[0] ? " " : "",
-				insn->op_str);
+		const char *opstr = sdb_fmt ("%s%s%s", insn->mnemonic,
+			insn->op_str[0] ? " " : "", insn->op_str);
+		r_asm_op_set_asm (op, opstr);
 		cs_free (insn, n);
 		return op->size;
 	}
-	//op->size = -1;
+	r_asm_op_set_asm (op, "invalid");
+	op->size = 4;
 	cs_free (insn, n);
-	return 4;
+	return op->size;
 }
 
 RAsmPlugin r_asm_plugin_ppc_cs = {
 	.name = "ppc",
 	.desc = "Capstone PowerPC disassembler",
 	.license = "BSD",
+	.author = "pancake",
 	.arch = "ppc",
 	.cpus = "ppc,vle,ps",
 	.bits = 32 | 64,
@@ -120,7 +123,7 @@ RAsmPlugin r_asm_plugin_ppc_cs = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_ppc_cs,
 	.version = R2_VERSION
