@@ -11,7 +11,7 @@
 #include <mybfd.h>
 
 static unsigned long Offset = 0;
-static char *buf_global = NULL;
+static RStrBuf *buf_global = NULL;
 static unsigned char bytes[4];
 
 static int sparc_buffer_read_memory (bfd_vma memaddr, bfd_byte *myaddr, unsigned int length, struct disassemble_info *info) {
@@ -27,59 +27,41 @@ static void memory_error_func(int status, bfd_vma memaddr, struct disassemble_in
 	//--
 }
 
-static void print_address(bfd_vma address, struct disassemble_info *info) {
-	char tmp[32];
-	if (!buf_global)
-		return;
-	sprintf (tmp, "0x%08"PFMT64x"", (ut64)address);
-	strcat (buf_global, tmp);
-}
-
-static int buf_fprintf(void *stream, const char *format, ...) {
-	va_list ap;
-	char tmp[1024];
-	if (!buf_global)
-		return 0;
-	va_start (ap, format);
-	vsnprintf (tmp, sizeof (tmp), format, ap);
-	strcat (buf_global, tmp);
-	va_end (ap);
-	return 0;
-}
+DECLARE_GENERIC_PRINT_ADDRESS_FUNC()
+DECLARE_GENERIC_FPRINTF_FUNC()
 
 static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	static struct disassemble_info disasm_obj;
 	if (len < 4) {
 		return -1;
 	}
-	buf_global = op->buf_asm;
+	buf_global = &op->buf_asm;
 	Offset = a->pc;
 	// disasm inverted
 	r_mem_swapendian (bytes, buf, 4); // TODO handle thumb
 
+	r_strbuf_set (&op->buf_asm, "");
 	/* prepare disassembler */
 	memset (&disasm_obj,'\0', sizeof (struct disassemble_info));
 	disasm_obj.buffer = bytes;
 	disasm_obj.read_memory_func = &sparc_buffer_read_memory;
 	disasm_obj.symbol_at_address_func = &symbol_at_address;
 	disasm_obj.memory_error_func = &memory_error_func;
-	disasm_obj.print_address_func = &print_address;
+	disasm_obj.print_address_func = &generic_print_address_func;
 	disasm_obj.endian = a->big_endian;
-	disasm_obj.fprintf_func = &buf_fprintf;
+	disasm_obj.fprintf_func = &generic_fprintf_func;
 	disasm_obj.stream = stdout;
 	disasm_obj.mach = ((a->bits == 64)
 			   ? bfd_mach_sparc_v9b
 			   : 0);
 
-	op->buf_asm[0] = '\0';
 	op->size = print_insn_sparc ((bfd_vma)Offset, &disasm_obj);
 
-	if (!strncmp (op->buf_asm, "unknown", 7)) {
-		strncpy (op->buf_asm, "invalid", R_ASM_BUFSIZE);
+	if (!strncmp (r_strbuf_get (&op->buf_asm), "unknown", 7)) {
+		r_asm_op_set_asm (op, "invalid");
 	}
-
 	if (op->size == -1) {
-		strncpy (op->buf_asm, " (data)", R_ASM_BUFSIZE);
+		r_asm_op_set_asm (op, "(data)");
 	}
 	return op->size;
 }
@@ -95,7 +77,7 @@ RAsmPlugin r_asm_plugin_sparc_gnu = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_sparc_gnu,
 	.version = R2_VERSION
