@@ -19,8 +19,7 @@ static int disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 	}
 	int ret = v850_decode_command (buf, len, &cmd);
 	if (ret > 0) {
-		snprintf (op->buf_asm, R_ASM_BUFSIZE, "%s %s",
-			  cmd.instr, cmd.operands);
+		r_asm_op_set_asm (op, sdb_fmt ("%s %s", cmd.instr, cmd.operands));
 	}
 	return op->size = ret;
 }
@@ -36,7 +35,7 @@ RAsmPlugin r_asm_plugin_v850 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_ASM,
 	.data = &r_asm_plugin_v850,
 	.version = R2_VERSION
