@@ -1,134 +1,128 @@
 /* radare - LGPL - Copyright 2015 - pancake */
 
 #include <r_bin.h>
+#include "i/private.h"
 
-static void hashify(char *s, ut64 vaddr) {
-	if (!s) return;
+static char *hashify(char *s, ut64 vaddr) {
+	r_return_val_if_fail (s, NULL);
+
+	char *os = s;
 	while (*s) {
 		if (!IS_PRINTABLE (*s)) {
 			if (vaddr && vaddr != UT64_MAX) {
-				sprintf (s, "_%" PFMT64d, vaddr);
-			} else {
-				ut32 hash = sdb_hash (s);
-				sprintf (s, "%x", hash);
+				free (os);
+				return r_str_newf ("_%" PFMT64d, vaddr);
 			}
-			return;
+			ut32 hash = sdb_hash (s);
+			free (os);
+			return r_str_newf ("%x", hash);
 		}
 		s++;
 	}
+	return os;
 }
 
-// TODO: optimize this api:
-// - bin plugins should call r_bin_filter_name() before appending
-R_API void r_bin_filter_name(Sdb *db, ut64 vaddr, char *name, int maxlen) {
-	const char *uname;
-	ut32 vhash, hash;
-	int count;
-	if (!db || !name) return;
-	uname = sdb_fmt ("%" PFMT64x ".%s", vaddr, name);
-	vhash = sdb_hash (uname); // vaddr hash - unique
-	hash = sdb_hash (name);   // name hash - if dupped and not in unique hash must insert
-	count = sdb_num_inc (db, sdb_fmt ("%x", hash), 1, 0);
+// - name should be allocated on the heap
+R_API char *r_bin_filter_name(RBinFile *bf, Sdb *db, ut64 vaddr, char *name) {
+	r_return_val_if_fail (db && name, NULL);
+
+	char *resname = name;
+	const char *uname = sdb_fmt ("%" PFMT64x ".%s", vaddr, resname);
+	ut32 vhash = sdb_hash (uname); // vaddr hash - unique
+	ut32 hash = sdb_hash (resname); // name hash - if dupped and not in unique hash must insert
+	int count = sdb_num_inc (db, sdb_fmt ("%x", hash), 1, 0);
+
 	if (sdb_exists (db, sdb_fmt ("%x", vhash))) {
 		// TODO: symbol is dupped, so symbol can be removed!
-		return;
+		return resname;
 	}
 	sdb_num_set (db, sdb_fmt ("%x", vhash), 1, 0);
 	if (vaddr) {
-		hashify (name, vaddr);
+		char *p = hashify (resname, vaddr);
+		if (p) {
+			resname = p;
+		}
 	}
 	if (count > 1) {
-		int namelen = strlen (name);
-		if (namelen > maxlen) name[maxlen] = 0;
-		strcat (name, sdb_fmt ("_%d", count - 1));
+		char *p = r_str_appendf (resname, "_%d", count - 1);
+		if (p) {
+			resname = p;
+		}
+
 		// two symbols at different addresses and same name wtf
 		//	eprintf ("Symbol '%s' dupped!\n", sym->name);
 	}
+	return resname;
 }
 
-R_API void r_bin_filter_sym(Sdb *db, ut64 vaddr, RBinSymbol *sym) {
-	if (!db || !sym) {
+R_API void r_bin_filter_sym(RBinFile *bf, Sdb *db, ut64 vaddr, RBinSymbol *sym) {
+	if (!db || !sym || !sym->name) {
 		return;
 	}
 	char *name = sym->name;
-	if (!name) {
-		return;
+	// if (!strncmp (sym->name, "imp.", 4)) {
+	// demangle symbol name depending on the language specs if any
+	if (bf && bf->o && bf->o->lang) {
+		const char *lang = r_bin_lang_tostring (bf->o->lang);
+		char *dn = r_bin_demangle (bf, lang, sym->name, sym->vaddr);
+		if (dn && *dn) {
+			sym->dname = dn;
+			// XXX this is wrong but is required for this test to pass
+			// pmb:new pancake$ bin/r2r.js db/formats/mangling/swift
+			sym->name = dn;
+			// extract class information from demangled symbol name
+			char *p = strchr (dn, '.');
+			if (p) {
+				if (IS_UPPER (*dn)) {
+					sym->classname = strdup (dn);
+					sym->classname[p - dn] = 0;
+				} else if (IS_UPPER (p[1])) {
+					sym->classname = strdup (p + 1);
+					p = strchr (sym->classname, '.');
+					if (p) {
+						*p = 0;
+					}
+				}
+			}
+		}
 	}
+
+	// XXX this is very slow, must be optimized
 	const char *uname = sdb_fmt ("%" PFMT64x ".%s", vaddr, name);
 	ut32 vhash = sdb_hash (uname); // vaddr hash - unique
-	ut32 hash = sdb_hash (name);   // name hash - if dupped and not in unique hash must insert
+	ut32 hash = sdb_hash (name); // name hash - if dupped and not in unique hash must insert
 	int count = sdb_num_inc (db, sdb_fmt ("%x", hash), 1, 0);
 	if (sdb_exists (db, sdb_fmt ("%x", vhash))) {
 		// TODO: symbol is dupped, so symbol can be removed!
 		return;
 	}
 	sdb_num_set (db, sdb_fmt ("%x", vhash), 1, 0);
-	if (vaddr) {
-		//hashify (name, vaddr);
-	}
 	sym->dup_count = count - 1;
 }
 
-R_API void r_bin_filter_symbols(RList *list) {
-	RListIter *iter;
-	RBinSymbol *sym;
-	const int maxlen = sizeof (sym->name) - 8;
+R_API void r_bin_filter_symbols(RBinFile *bf, RList *list) {
 	Sdb *db = sdb_new0 ();
-	if (!db) {
-		return;
-	}
-	if (maxlen > 0) {
+	if (db) {
+		RListIter *iter;
+		RBinSymbol *sym;
 		r_list_foreach (list, iter, sym) {
-			if (sym && sym->name) {
-				r_bin_filter_name (db, sym->vaddr, sym->name, maxlen);
-			}
-		}
-	} else {
-		r_list_foreach (list, iter, sym) {
-			if (sym && sym->name) {
-				r_bin_filter_sym (db, sym->vaddr, sym);
+			if (sym && sym->name && *sym->name) {
+				r_bin_filter_sym (bf, db, sym->vaddr, sym);
 			}
 		}
+		sdb_free (db);
 	}
-	sdb_free (db);
 }
 
-R_API void r_bin_filter_sections(RList *list) {
+R_API void r_bin_filter_sections(RBinFile *bf, RList *list) {
 	RBinSection *sec;
-	const int maxlen = 256;
 	Sdb *db = sdb_new0 ();
 	RListIter *iter;
-	if (maxlen > 0) {
-		r_list_foreach (list, iter, sec) {
-			r_bin_filter_name (db, sec->vaddr, sec->name, maxlen);
+	r_list_foreach (list, iter, sec) {
+		char *p = r_bin_filter_name (bf, db, sec->vaddr, sec->name);
+		if (p) {
+			sec->name = p;
 		}
-	} else {
-		eprintf ("SectionName is not dynamic\n");
-	}
-	sdb_free (db);
-}
-
-R_API void r_bin_filter_classes(RList *list) {
-	int namepad_len;
-	char *namepad;
-	Sdb *db = sdb_new0 ();
-	RListIter *iter, *iter2;
-	RBinClass *cls;
-	RBinSymbol *sym;
-	r_list_foreach (list, iter, cls) {
-		if (!cls->name) continue;
-		namepad_len = strlen (cls->name) + 32;
-		namepad = calloc (1, namepad_len + 1);
-		if (namepad) {
-			strcpy (namepad, cls->name);
-			r_bin_filter_name (db, cls->index, namepad, namepad_len);
-			free (cls->name);
-			cls->name = namepad;
-			r_list_foreach (cls->methods, iter2, sym) {
-				if (sym->name)
-					r_bin_filter_sym (db, sym->vaddr, sym);
-			}
-		} else eprintf ("Cannot alloc %d byte(s)\n", namepad_len);
 	}
 	sdb_free (db);
 }
@@ -195,11 +189,11 @@ R_API bool r_bin_strpurge(RBin *bin, const char *str, ut64 refaddr) {
 			char *range_sep;
 			ut64 addr, from, to;
 			for (i = 0, ptr = addrs; i < splits; i++, ptr += strlen (ptr) + 1) {
-				bool bang = false;
 				if (!strcmp (ptr, "true") && false_positive (str)) {
 					purge = true;
 					continue;
 				}
+				bool bang = false;
 				if (*ptr == '!') {
 					bang = true;
 					ptr++;
