@@ -79,16 +79,18 @@ RBinAddr *r_coff_get_entry(struct r_bin_coff_obj *obj) {
 	 * or 'main' if present */
 	if (obj->symbols) {
 		for (i = 0; i < obj->hdr.f_nsyms; i++) {
-			if ((!strcmp (obj->symbols[i].n_name, "_start") || 
-				 !strcmp (obj->symbols[i].n_name, "start")) &&
-				 r_coff_rebase_sym (obj, addr, &obj->symbols[i]))
+			if ((!strcmp (obj->symbols[i].n_name, "_start") ||
+				    !strcmp (obj->symbols[i].n_name, "start")) &&
+				r_coff_rebase_sym (obj, addr, &obj->symbols[i])) {
 				return addr;
+			}
 		}
 		for (i = 0; i < obj->hdr.f_nsyms; i++) {
-			if ((!strcmp (obj->symbols[i].n_name, "_main") || 
-				 !strcmp (obj->symbols[i].n_name, "main")) &&
-				 r_coff_rebase_sym (obj, addr, &obj->symbols[i]))
+			if ((!strcmp (obj->symbols[i].n_name, "_main") ||
+				    !strcmp (obj->symbols[i].n_name, "main")) &&
+				r_coff_rebase_sym (obj, addr, &obj->symbols[i])) {
 				return addr;
+			}
 		}
 	}
 	/* Still clueless ? Let's just use the address of .text */
