@@ -15,6 +15,9 @@ char* r_bin_dex_get_version(RBinDexObj *bin) {
 
 #define FAIL(x) { eprintf(x"\n"); goto fail; }
 RBinDexObj *r_bin_dex_new_buf(RBuffer *buf) {
+	if (!buf) {
+		return NULL;
+	}
 	RBinDexObj *bin = R_NEW0 (RBinDexObj);
 	int i;
 	ut8 *bufptr;
@@ -253,7 +256,7 @@ int dex_uleb128_len(const ut8 *ptr, int size) {
 	return i;
 }
 
-#define SIG_EXTEND(X,Y) X = (X << Y) >> Y
+#define SIG_EXTEND(X,Y) X = ((X) << (Y)) >> Y
 int dex_read_sleb128(const char *ptr, int size) {
 	int cur, result;
 	ut8 len = dex_uleb128_len ((const ut8*)ptr, size);
