@@ -134,12 +134,20 @@ ut64 Elf_(r_bin_elf_resize_section)(struct Elf_(r_bin_elf_obj_t) *bin, const cha
 #endif 
 		if (phdrp->p_offset >= rsz_offset + rsz_osize) {
 			phdrp->p_offset += delta;
-			if (phdrp->p_vaddr) phdrp->p_vaddr += delta;
-			if (phdrp->p_paddr) phdrp->p_paddr += delta;
-		}
+			if (phdrp->p_vaddr) {
+				phdrp->p_vaddr += delta;
+			}
+			if (phdrp->p_paddr) {
+				phdrp->p_paddr += delta;
+			}
+		} else if (phdrp->p_offset + phdrp->p_filesz >= rsz_offset + rsz_osize) {
+			phdrp->p_filesz += delta;
+			phdrp->p_memsz += delta;
+		} 
 		off = ehdr->e_phoff + i * sizeof (Elf_(Phdr));
-		if (r_buf_write_at (bin->b, off, (ut8*)phdrp, sizeof (Elf_(Phdr))) == -1)
+		if (r_buf_write_at (bin->b, off, (ut8 *)phdrp, sizeof (Elf_ (Phdr))) == -1) {
 			perror ("write (phdr)");
+		}
 		printf ("-> program header (0x%08"PFMT64x")\n", (ut64) phdrp->p_offset);
 	}
 
