@@ -12,11 +12,28 @@
 
 #define CSMAGIC_CODEDIRECTORY      0xfade0c02
 #define CSMAGIC_EMBEDDED_SIGNATURE 0xfade0cc0
+#define CSMAGIC_DETACHED_SIGNATURE 0xfade0cc1 /* multi-arch collection of embedded signatures */
 #define CSMAGIC_ENTITLEMENTS       0xfade7171
+#define CSMAGIC_REQUIREMENT        0xfade0c00 /* single Requirement blob */
+#define CSMAGIC_REQUIREMENTS       0xfade0c01 /* Requirements vector (internal requirements) */
+
+#define CS_PAGE_SIZE 4096
+
+#define CS_HASHTYPE_SHA1 1
+#define CS_HASHTYPE_SHA256 2
+#define CS_HASHTYPE_SHA256_TRUNCATED 3
+
+#define CS_HASH_SIZE_SHA1 20
+#define CS_HASH_SIZE_SHA256 32
+#define CS_HASH_SIZE_SHA256_TRUNCATED 20
 
 #define CSSLOT_CODEDIRECTORY 0
+#define CSSLOT_INFOSLOT 1
 #define CSSLOT_REQUIREMENTS  2
+#define CSSLOT_RESOURCEDIR 3
+#define CSSLOT_APPLICATION 4
 #define CSSLOT_ENTITLEMENTS  5
+#define CSSLOT_CMS_SIGNATURE 0x10000
 
 struct section_t {
 	ut64 offset;
@@ -25,7 +42,7 @@ struct section_t {
 	ut64 vsize;
 	ut32 align;
 	ut32 flags;
-	int srwx;
+	int perm;
 	char name[R_BIN_MACH0_STRING_LENGTH];
 	int last;
 };
@@ -134,6 +151,9 @@ struct MACH0_(obj_t) {
 	Sdb *kv;
 	int has_crypto;
 	int has_canary;
+	int has_retguard;
+	int has_sanitizers;
+	int has_blocks_ext;
 	int dbg_info;
 	const char *lang;
 	int uuidn;
@@ -150,6 +170,7 @@ struct MACH0_(obj_t)* MACH0_(new_buf)(RBuffer *buf, struct MACH0_(opts_t) *optio
 void* MACH0_(mach0_free)(struct MACH0_(obj_t)* bin);
 struct section_t* MACH0_(get_sections)(struct MACH0_(obj_t)* bin);
 struct symbol_t* MACH0_(get_symbols)(struct MACH0_(obj_t)* bin);
+void MACH0_(pull_symbols)(struct MACH0_(obj_t)* mo, RBinSymbolCallback cb, void *user);
 struct import_t* MACH0_(get_imports)(struct MACH0_(obj_t)* bin);
 struct reloc_t* MACH0_(get_relocs)(struct MACH0_(obj_t)* bin);
 struct addr_t* MACH0_(get_entrypoint)(struct MACH0_(obj_t)* bin);
@@ -158,8 +179,8 @@ ut64 MACH0_(get_baddr)(struct MACH0_(obj_t)* bin);
 char* MACH0_(get_class)(struct MACH0_(obj_t)* bin);
 int MACH0_(get_bits)(struct MACH0_(obj_t)* bin);
 bool MACH0_(is_big_endian)(struct MACH0_(obj_t)* bin);
-int MACH0_(is_pie)(struct MACH0_(obj_t)* bin);
-int MACH0_(has_nx)(struct MACH0_(obj_t)* bin);
+bool MACH0_(is_pie)(struct MACH0_(obj_t)* bin);
+bool MACH0_(has_nx)(struct MACH0_(obj_t)* bin);
 const char* MACH0_(get_intrp)(struct MACH0_(obj_t)* bin);
 const char* MACH0_(get_os)(struct MACH0_(obj_t)* bin);
 const char* MACH0_(get_cputype)(struct MACH0_(obj_t)* bin);
