@@ -133,6 +133,12 @@ typedef enum minidump_stream_type_t {
 	MEMORY_INFO_LIST_STREAM		= 16,
 	THREAD_INFO_LIST_STREAM		= 17,
 	HANDLE_OPERATION_LIST_STREAM	= 18,
+	TOKEN_STREAM	= 19,
+	JAVASCRIPT_DATA_STREAM	= 20,
+	SYSTEM_MEMORY_INFO_STREAM	= 21,
+	PROCESS_VM_COUNTERS_STREAM	= 22,
+	IPT_TRACE_STREAM	= 23,
+	THREAD_NAMES_STREAM	= 24,
 	LAST_RESERVED_STREAM		= 0xffff
 } minidump_stream_type;
 
@@ -475,24 +481,23 @@ struct minidump_module {
 /* Contains a list of modules. */
 R_PACKED (
 struct minidump_module_list {
-	ut32	number_of_modules;
-
+	ut32 number_of_modules;
 	struct minidump_module *modules;
 });
 
 /* Describes a string. */
 R_PACKED (
 struct minidump_string {
-	ut32	length;
-	ut16	*buffer;
+	ut32 length;
+	ut16 *buffer;
 });
 
 /* Contains processor and operating system information. */
 R_PACKED (
 struct minidump_system_info {
-	ut16	processor_architecture;
-	ut16	processor_level;
-	ut16	processor_revision;
+	ut16 processor_architecture;
+	ut16 processor_level;
+	ut16 processor_revision;
 
 	union {
 		ut16 reserved_0;
@@ -502,29 +507,29 @@ struct minidump_system_info {
 		};
 	};
 
-	ut32	major_version;
-	ut32	minor_version;
-	ut32	build_number;
-	ut32	platform_id;
-	rva_t	csd_version_rva;
+	ut32 major_version;
+	ut32 minor_version;
+	ut32 build_number;
+	ut32 platform_id;
+	rva_t csd_version_rva;
 
 	union {
-		ut32	reserved_1;
+		ut32 reserved_1;
 		struct {
-			ut16	suite_mask;
-			ut16	reserved_2;
+			ut16 suite_mask;
+			ut16 reserved_2;
 		};
 	};
 
 	union {
 		struct {
-			ut32	vendor_id[3];
-			ut32	version_information;
-			ut32	feature_information;
-			ut32	amd_extended_cpu_features;
+			ut32 vendor_id[3];
+			ut32 version_information;
+			ut32 feature_information;
+			ut32 amd_extended_cpu_features;
 		} x86_cpu_info;
 		struct {
-			ut64	processor_features[2];
+			ut64 processor_features[2];
 		} other_cpu_info;
 	} cpu;
 });
@@ -532,12 +537,11 @@ struct minidump_system_info {
 /* Contains information for a specific thread. */
 R_PACKED (
 struct minidump_thread {
-	ut32	thread_id;
-	ut32	suspend_count;
-	ut32	priority_class;
-	ut32	priority;
-	ut64	teb;
-
+	ut32 thread_id;
+	ut32 suspend_count;
+	ut32 priority_class;
+	ut32 priority;
+	ut64 teb;
 	struct minidump_memory_descriptor stack;
 	struct minidump_location_descriptor thread_context;
 });
@@ -545,8 +549,7 @@ struct minidump_thread {
 /* Contains a list of threads. */
 R_PACKED (
 struct minidump_thread_list {
-	ut32	number_of_threads;
-
+	ut32 number_of_threads;
 	struct minidump_thread *threads;
 });
 
@@ -595,6 +598,23 @@ struct minidump_thread_info_list {
 	ut32	number_of_entries;
 });
 
+/* Contains a token information. */
+R_PACKED (
+struct minidump_token_info {
+	ut32	token_size;
+	ut32	token_id;
+	ut64	token_handle;
+});
+
+/* Contains a list of token information. */
+R_PACKED (
+struct minidump_token_info_list {
+	ut32	size_of_list;
+	ut32	number_of_entries;
+	ut32	list_header_size;
+	ut32	element_header_size;
+});
+
 /* Contains information about a module that has been unloaded. This information
  * can help diagnose problems calling code that is no longer loaded. */
 R_PACKED (
