@@ -84,7 +84,9 @@ static const char *getnum(const char* n, int *num) {
 }
 
 static const char *numpos(const char* n) {
-	while (*n && (*n<'0' || *n>'9')) n++;
+	while (*n && (*n < '0' || *n > '9')) {
+		n++;
+	}
 	return n;
 }
 
@@ -105,7 +107,9 @@ static const char *resolve(struct Type *t, const char *foo, const char **bar) {
 	for (; t[0].code; t++) {
 		int len = strlen (t[0].code);
 		if (!strncmp (foo, t[0].code, len)) {
-			if (bar) *bar = t[0].name;
+			if (bar) {
+				*bar = t[0].name;
+			}
 			return foo + len;
 		}
 	}
@@ -151,21 +155,27 @@ static char *swift_demangle_cmd(const char *s) {
 	return NULL;
 }
 
-R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
-#define STRCAT_BOUNDS(x) if ((x + 2 + strlen (out)) > sizeof (out)) break;
+R_API char *r_bin_demangle_swift(const char *s, bool syscmd) {
+#define STRCAT_BOUNDS(x) if (((x) + 2 + strlen (out)) > sizeof (out)) break;
 	char out[1024];
 	int i, len, is_generic = 0;
 	int is_first = 1;
 	int is_last = 0;
 	int retmode = 0;
-	if (!strncmp (s, "imp.", 4)) s = s + 4;
-	if (!strncmp (s, "reloc.", 6)) s = s + 6;
+	if (!strncmp (s, "imp.", 4)) {
+		s = s + 4;
+	}
+	if (!strncmp (s, "reloc.", 6)) {
+		s = s + 6;
+	}
 
 	if (*s != 'T' && strncmp (s, "_T", 2) && strncmp (s, "__T", 3)) {
 		return NULL;
 	}
 
-	if (!strncmp (s, "__", 2)) s = s + 2;
+	if (!strncmp (s, "__", 2)) {
+		s = s + 2;
+	}
 #if 0
 	const char *element[] = {
 		"module", "class", "method", NULL
@@ -274,8 +284,9 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 				q++;
 			}
 			q = getnum (q, &len);
-			if (!len)
+			if (!len) {
 				break;
+			}
 			const char *str = getstring (q, len);
 			if (len == 2 && !strcmp (str, "ee")) {
 				strcat (out, "Swift");
@@ -285,7 +296,9 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 						len, getstring (q, len));
 #endif
 				// push string
-				if (i && *out) strcat (out, ".");
+				if (i && *out) {
+					strcat (out, ".");
+				}
 				STRCAT_BOUNDS (len);
 				len = R_MIN (len, strlen (q));
 				strcat (out, getstring (q, len));
@@ -359,7 +372,7 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 				switch (*q) {
 				case 's':
 					{
-						int n;
+						int n = 0;
 						const char *Q = getnum (q + 1, &n);
 						const char *res = getstring (Q, n);
 						if (res) {
@@ -372,7 +385,7 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 				case 'u':
 					if (!strncmp (q, "uRxs", 4)) {
 						strcat (out, "..");
-						int n;
+						int n = 0 ;
 						const char *Q = getnum (q + 4, &n);
 						strcat (out, getstring (Q, n));
 						q = Q + n + 1;
@@ -380,7 +393,9 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 					}
 					break;
 				case 'S': // "S0"
-					if (q[1]=='1') q++;
+					if (q[1] == '1') {
+						q++;
+					}
 					switch (q[1]) {
 					case '0':
 						strcat (out, " (self) -> ()");
@@ -399,7 +414,7 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 						// swift string
 						if (q[0] && q[1] && q[2]) {
 							strcat (out, "..");
-							int n;
+							int n = 0;
 							const char *Q = getnum (q + 2, &n);
 							strcat (out, getstring (Q, n));
 							q = Q + n + 1;
@@ -441,7 +456,6 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 				}
 
 				if (p) {
-					q = p;
 					q = getnum (p, &len);
 					if (attr && !strcmp (attr, "generic")) {
 						is_generic = 1;
@@ -536,8 +550,9 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 		//printf ("Unsupported type: %c\n", *p);
 	}
 	if (*out) {
-		if (tail)
+		if (tail) {
 			strcat (out, tail);
+		}
 #if 1
 		char *p, *outstr = strdup (out);
 		p = outstr;
@@ -547,7 +562,9 @@ R_API char *r_bin_demangle_swift(const char *s, int syscmd) {
 				p[0] = '_';
 				p[1] = '_';
 				p+=2;
-			} else break;
+			} else {
+				break;
+			}
 		}
 		return outstr;
 #endif
