@@ -19,8 +19,8 @@ static char *get_filetype(RBinFile *bf) {
 	if (ck && bf && bf->buf) {
 		const char *tmp = NULL;
 		// TODO: dir.magic not honored here
-		char *pfx = r_str_newf ("%s/%s", r_sys_prefix (NULL), R_MAGIC_PATH);
-		r_magic_load (ck, R_MAGIC_PATH);
+		char *pfx = r_str_newf (R_JOIN_2_PATHS ("%s", R2_SDB_MAGIC), r_sys_prefix (NULL));
+		r_magic_load (ck, R2_SDB_MAGIC);
 		r_buf_read_at (bf->buf, 0, buf, sizeof (buf));
 		tmp = r_magic_buffer (ck, buf, sizeof (buf));
 		if (tmp) {
@@ -42,6 +42,7 @@ static RBinInfo *info(RBinFile *bf) {
 	ret->type = get_filetype (bf);
 	ret->has_pi = 0;
 	ret->has_canary = 0;
+	ret->has_retguard = -1;
 	if (R_SYS_BITS & R_SYS_BITS_64) {
 		ret->bits = 64;
 	} else {
@@ -60,6 +61,10 @@ static bool load(RBinFile *bf) {
 	return true;
 }
 
+static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
+	return r_buf_new ();
+}
+
 static int destroy(RBinFile *bf) {
 	return true;
 }
@@ -73,6 +78,7 @@ RBinPlugin r_bin_plugin_any = {
 	.desc = "Dummy format r_bin plugin",
 	.license = "LGPL3",
 	.load = &load,
+	.load_buffer = &load_buffer,
 	.destroy = &destroy,
 	.baddr = &baddr,
 	.info = info,
@@ -80,7 +86,7 @@ RBinPlugin r_bin_plugin_any = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_any,
 	.version = R2_VERSION
