@@ -4,16 +4,16 @@
 
 #define CHECK4INSTR(b, instr, size) \
 	if (!instr (b) ||\
-		!instr (b + size) ||\
-		!instr (b + size * 2) ||\
-		!instr (b + size * 3)) {\
+		!instr ((b) + (size)) ||\
+		!instr ((b) + (size) * 2) ||\
+		!instr ((b) + (size) * 3)) {\
 		return false;\
 	}
 
 #define CHECK3INSTR(b, instr, size) \
-	if (!instr (b + size) ||\
-		!instr (b + size * 2) ||\
-		!instr (b + size * 3)) {\
+	if (!instr ((b) + (size)) ||\
+		!instr ((b) + (size) * 2) ||\
+		!instr ((b) + (size) * 3)) {\
 		return false;\
 	}
 
@@ -68,9 +68,8 @@ static bool check_bytes(const ut8 *b, ut64 length) {
 	return check_bytes_rjmp (b, length);
 }
 
-static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
-	check_bytes (buf, sz);
-	return R_NOTNULL;
+static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
+	return check_bytes (buf, sz);
 }
 
 static RBinInfo* info(RBinFile *bf) {
@@ -111,7 +110,9 @@ static RList* entries(RBinFile *bf) {
 
 static void addsym(RList *ret, const char *name, ut64 addr) {
 	RBinSymbol *ptr = R_NEW0 (RBinSymbol);
-	if (!ptr) return;
+	if (!ptr) {
+		return;
+	}
 	ptr->name = strdup (name? name: "");
 	ptr->paddr = ptr->vaddr = addr;
 	ptr->size = 0;
@@ -121,9 +122,9 @@ static void addsym(RList *ret, const char *name, ut64 addr) {
 
 static void addptr(RList *ret, const char *name, ut64 addr, const ut8 *b, int len) {
 	if (b && rjmp (b)) {
-		addsym (ret, sdb_fmt (0, "vector.%s", name), addr);
+		addsym (ret, sdb_fmt ("vector.%s", name), addr);
 		ut64 ptr_addr = rjmp_dest (addr, b + addr);
-		addsym (ret, sdb_fmt (0, "syscall.%s", name), ptr_addr);
+		addsym (ret, sdb_fmt ("syscall.%s", name), ptr_addr);
 	}
 }
 
@@ -167,7 +168,7 @@ RBinPlugin r_bin_plugin_avr = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_avr,
 	.version = R2_VERSION
