@@ -1,13 +1,7 @@
 /* radare - LGPL - Copyright 2009-2015 - ret2libc, pancake */
 
-#include <stdio.h>
-#include <r_types.h>
-#include <r_util.h>
-#include <r_lib.h>
-#include <r_bin.h>
-
 #define R_BIN_CGC 1
-#include "bin_elf.c"
+#include "bin_elf.inc"
 
 extern struct r_bin_dbginfo_t r_bin_dbginfo_elf;
 extern struct r_bin_write_t r_bin_write_elf;
@@ -16,7 +10,7 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return buf && length > 4 && !memcmp (buf, CGCMAG, SCGCMAG) && buf[4] != 2;
 }
 
-static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data, int datalen) {
+static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data, int datalen, RBinArchOptions *opt) {
 	ut32 filesize, code_va, code_pa, phoff;
 	ut32 p_start, p_phoff, p_phdr;
 	ut32 p_ehdrsz, p_phdrsz;
@@ -25,11 +19,11 @@ static RBuffer* create(RBin* bin, const ut8 *code, int codelen, const ut8 *data,
 	ut32 baddr = 0x8048000;
 	RBuffer *buf = r_buf_new ();
 
-#define B(x,y) r_buf_append_bytes(buf,(const ut8*)x,y)
+#define B(x,y) r_buf_append_bytes(buf,(const ut8*)(x),y)
 #define D(x) r_buf_append_ut32(buf,x)
 #define H(x) r_buf_append_ut16(buf,x)
 #define Z(x) r_buf_append_nbytes(buf,x)
-#define W(x,y,z) r_buf_write_at(buf,x,(const ut8*)y,z)
+#define W(x,y,z) r_buf_write_at(buf,x,(const ut8*)(y),z)
 #define WZ(x,y) p_tmp=buf->length;Z(x);W(p_tmp,y,strlen(y))
 
 	B ("\x7F" "CGC" "\x01\x01\x01\x43", 8);
@@ -104,6 +98,7 @@ RBinPlugin r_bin_plugin_cgc = {
 	.get_sdb = &get_sdb,
 	.load = &load,
 	.load_bytes = &load_bytes,
+	.load_buffer= load_buffer,
 	.destroy = &destroy,
 	.check_bytes = &check_bytes,
 	.baddr = &baddr,
@@ -123,4 +118,7 @@ RBinPlugin r_bin_plugin_cgc = {
 	.create = &create,
 	.patch_relocs = &patch_relocs,
 	.write = &r_bin_write_elf,
+	.file_type = get_file_type,
+	.regstate = regstate,
+	.maps = maps,
 };
