@@ -12,6 +12,18 @@
 #include "objc/mach0_classes.h"
 
 typedef struct {
+	ut8 version;
+	ut64 slide;
+	ut8 *one_page_buf;
+	ut32 page_size;
+	ut64 start_of_data;
+} RDyldRebaseInfo;
+
+typedef struct {
+	ut8 version;
+	ut64 slide;
+	ut8 *one_page_buf;
+	ut32 page_size;
 	ut64 start_of_data;
 	ut16 *page_starts;
 	ut32 page_starts_count;
@@ -21,10 +33,19 @@ typedef struct {
 	ut64 value_mask;
 	ut32 delta_shift;
 	ut64 value_add;
-	ut32 page_size;
+} RDyldRebaseInfo2;
+
+typedef struct {
+	ut8 version;
 	ut64 slide;
 	ut8 *one_page_buf;
-} RDyldRebaseInfo;
+	ut32 page_size;
+	ut64 start_of_data;
+	ut16 *toc;
+	ut32 toc_count;
+	ut8 *entries;
+	ut32 entries_size;
+} RDyldRebaseInfo1;
 
 typedef struct _r_dyldcache {
 	ut8 magic[8];
@@ -47,12 +68,28 @@ static void free_bin(RDyldBinImage *bin) {
 		return;
 	}
 
-	if (bin->file) {
-		free (bin->file);
-		bin->file = NULL;
+	R_FREE (bin->file);
+	R_FREE (bin);
+}
+
+static void rebase_info2_free(RDyldRebaseInfo2 *rebase_info) {
+	if (!rebase_info) {
+		return;
 	}
 
-	R_FREE (bin);
+	R_FREE (rebase_info->page_starts);
+	R_FREE (rebase_info->page_extras);
+	R_FREE (rebase_info);
+}
+
+static void rebase_info1_free(RDyldRebaseInfo1 *rebase_info) {
+	if (!rebase_info) {
+		return;
+	}
+
+	R_FREE (rebase_info->toc);
+	R_FREE (rebase_info->entries);
+	R_FREE (rebase_info);
 }
 
 static void rebase_info_free(RDyldRebaseInfo *rebase_info) {
@@ -60,22 +97,17 @@ static void rebase_info_free(RDyldRebaseInfo *rebase_info) {
 		return;
 	}
 
-	if (rebase_info->page_starts) {
-		free (rebase_info->page_starts);
-		rebase_info->page_starts = NULL;
-	}
+	R_FREE (rebase_info->one_page_buf);
 
-	if (rebase_info->page_extras) {
-		free (rebase_info->page_extras);
-		rebase_info->page_extras = NULL;
-	}
+	ut8 version = rebase_info->version;
 
-	if (rebase_info->one_page_buf) {
-		free (rebase_info->one_page_buf);
-		rebase_info->one_page_buf = NULL;
+	if (version == 1) {
+		rebase_info1_free ((RDyldRebaseInfo1*) rebase_info);
+	} else if (version == 2) {
+		rebase_info2_free ((RDyldRebaseInfo2*) rebase_info);
+	} else {
+		R_FREE (rebase_info);
 	}
-
-	R_FREE (rebase_info);
 }
 
 static void r_dyldcache_free(RDyldCache *cache) {
@@ -98,21 +130,9 @@ static void r_dyldcache_free(RDyldCache *cache) {
 		cache->rebase_info = NULL;
 	}
 
-	if (cache->hdr) {
-		free (cache->hdr);
-		cache->hdr = NULL;
-	}
-
-	if (cache->maps) {
-		free (cache->maps);
-		cache->maps = NULL;
-	}
-
-	if (cache->accel) {
-		free (cache->accel);
-		cache->accel = NULL;
-	}
-
+	R_FREE (cache->hdr);
+	R_FREE (cache->maps);
+	R_FREE (cache->accel);
 	R_FREE (cache);
 }
 
@@ -176,9 +196,15 @@ static struct MACH0_(obj_t) *bin_to_mach0(RBinFile *bf, RDyldBinImage *bin) {
 
 static int prot2perm(int x) {
 	int r = 0;
-	if (x&1) r |= 4;
-	if (x&2) r |= 2;
-	if (x&4) r |= 1;
+	if (x & 1) {
+		r |= 4;
+	}
+	if (x & 2) {
+		r |= 2;
+	}
+	if (x & 4) {
+		r |= 1;
+	}
 	return r;
 }
 
@@ -232,7 +258,7 @@ static ut64 estimate_slide(RBinFile *bf, RDyldCache *cache, ut64 value_mask) {
 
 		int i;
 		int incomplete = 2;
-		int classlist_idx, data_idx;
+		int classlist_idx = 0, data_idx = 0;
 		for (i = 0; !sections[i].last && incomplete; i++) {
 			if (sections[i].size == 0) {
 				continue;
@@ -284,8 +310,8 @@ beach:
 }
 
 static RDyldRebaseInfo *get_rebase_info(RBinFile *bf, RDyldCache *cache) {
-	ut16 *page_starts = NULL;
-	ut16 *page_extras = NULL;
+	ut8 *tmp_buf_1 = NULL;
+	ut8 *tmp_buf_2 = NULL;
 	ut8 *one_page_buf = NULL;
 	RBuffer *cache_buf = cache->buf;
 
@@ -294,7 +320,7 @@ static RDyldRebaseInfo *get_rebase_info(RBinFile *bf, RDyldCache *cache) {
 	int i;
 	for (i = 0; i < cache->hdr->mappingCount; ++i) {
 		int perm = prot2perm (cache->maps[i].initProt);
-		if (!(perm & R_BIN_SCN_EXECUTABLE)) {
+		if (!(perm & R_PERM_X)) {
 			start_of_data = cache->maps[i].fileOffset;// + bf->o->boffset;
 			break;
 		}
@@ -304,83 +330,159 @@ static RDyldRebaseInfo *get_rebase_info(RBinFile *bf, RDyldCache *cache) {
 		return NULL;
 	}
 
-	cache_slide2_t slide_info;
 	ut64 offset = cache->hdr->slideInfoOffset;
-	ut64 size = sizeof (cache_slide2_t);
-	if (r_buf_fread_at (cache_buf, offset, (ut8*) &slide_info, "6i2l", 1) != size) {
+	ut32 slide_info_version = 0;
+	if (r_buf_read_at (cache_buf, offset, (ut8*) &slide_info_version, 4) != 4) {
 		return NULL;
 	}
 
-	if (slide_info.version != 2) {
-		return NULL;
-	}
-
-	if (slide_info.page_starts_offset == 0 ||
-		slide_info.page_starts_offset > cache->hdr->slideInfoSize ||
-		slide_info.page_starts_offset + slide_info.page_starts_count * 2 > cache->hdr->slideInfoSize) {
-		return NULL;
-	}
-
-	if (slide_info.page_extras_offset == 0 ||
-		slide_info.page_extras_offset > cache->hdr->slideInfoSize ||
-		slide_info.page_extras_offset + slide_info.page_extras_count * 2 > cache->hdr->slideInfoSize) {
-		return NULL;
-	}
-
-	if (slide_info.page_starts_count > 0) {
-		ut64 size = slide_info.page_starts_count * 2;
-		ut64 at = cache->hdr->slideInfoOffset + slide_info.page_starts_offset;
-		page_starts = malloc (size);
-		if (r_buf_fread_at (cache_buf, at, (ut8*) page_starts, "s", slide_info.page_starts_count) != size) {
-			R_FREE (page_starts);
+	if (slide_info_version == 2) {
+		cache_slide2_t slide_info;
+		ut64 size = sizeof (cache_slide2_t);
+		if (r_buf_fread_at (cache_buf, offset, (ut8*) &slide_info, "6i2l", 1) != size) {
 			return NULL;
 		}
-	}
 
-	if (slide_info.page_extras_count > 0) {
-		ut64 size = slide_info.page_extras_count * 2;
-		ut64 at = cache->hdr->slideInfoOffset + slide_info.page_extras_offset;
-		page_extras = malloc (size);
-		if (r_buf_fread_at (cache_buf, at, (ut8*) page_extras, "s", slide_info.page_extras_count) != size) {
-			R_FREE (page_starts);
-			R_FREE (page_extras);
+		if (slide_info.page_starts_offset == 0 ||
+			slide_info.page_starts_offset > cache->hdr->slideInfoSize ||
+			slide_info.page_starts_offset + slide_info.page_starts_count * 2 > cache->hdr->slideInfoSize) {
 			return NULL;
 		}
-	}
 
-	if (slide_info.page_size > 0) {
-		one_page_buf = malloc (slide_info.page_size);
+		if (slide_info.page_extras_offset == 0 ||
+			slide_info.page_extras_offset > cache->hdr->slideInfoSize ||
+			slide_info.page_extras_offset + slide_info.page_extras_count * 2 > cache->hdr->slideInfoSize) {
+			return NULL;
+		}
+
+		if (slide_info.page_starts_count > 0) {
+			ut64 size = slide_info.page_starts_count * 2;
+			ut64 at = cache->hdr->slideInfoOffset + slide_info.page_starts_offset;
+			tmp_buf_1 = malloc (size);
+			if (!tmp_buf_1) {
+				goto beach;
+			}
+			if (r_buf_fread_at (cache_buf, at, tmp_buf_1, "s", slide_info.page_starts_count) != size) {
+				goto beach;
+			}
+		}
+
+		if (slide_info.page_extras_count > 0) {
+			ut64 size = slide_info.page_extras_count * 2;
+			ut64 at = cache->hdr->slideInfoOffset + slide_info.page_extras_offset;
+			tmp_buf_2 = malloc (size);
+			if (!tmp_buf_2) {
+				goto beach;
+			}
+			if (r_buf_fread_at (cache_buf, at, tmp_buf_2, "s", slide_info.page_extras_count) != size) {
+				goto beach;
+			}
+		}
+
+		if (slide_info.page_size > 0) {
+			one_page_buf = malloc (slide_info.page_size);
+			if (!one_page_buf) {
+				goto beach;
+			}
+		}
+
+		RDyldRebaseInfo2 *rebase_info = R_NEW0 (RDyldRebaseInfo2);
+		if (!rebase_info) {
+			goto beach;
+		}
+
+		rebase_info->version = 2;
+		rebase_info->start_of_data = start_of_data;
+		rebase_info->page_starts = (ut16*) tmp_buf_1;
+		rebase_info->page_starts_count = slide_info.page_starts_count;
+		rebase_info->page_extras = (ut16*) tmp_buf_2;
+		rebase_info->page_extras_count = slide_info.page_extras_count;
+		rebase_info->value_add = slide_info.value_add;
+		rebase_info->delta_mask = slide_info.delta_mask;
+		rebase_info->value_mask = ~rebase_info->delta_mask;
+		rebase_info->delta_shift = dumb_ctzll (rebase_info->delta_mask) - 2;
+		rebase_info->page_size = slide_info.page_size;
+		rebase_info->one_page_buf = one_page_buf;
+		rebase_info->slide = estimate_slide (bf, cache, rebase_info->value_mask);
+		if (rebase_info->slide) {
+			eprintf ("dyldcache is slid: 0x%"PFMT64x"\n", rebase_info->slide);
+		}
+
+		return (RDyldRebaseInfo*) rebase_info;
+	} else if (slide_info_version == 1) {
+		cache_slide1_t slide_info;
+		ut64 size = sizeof (cache_slide1_t);
+		if (r_buf_fread_at (cache_buf, offset, (ut8*) &slide_info, "6i", 1) != size) {
+			return NULL;
+		}
+
+		if (slide_info.toc_offset == 0 ||
+			slide_info.toc_offset > cache->hdr->slideInfoSize ||
+			slide_info.toc_offset + slide_info.toc_count * 2 > cache->hdr->slideInfoSize) {
+			return NULL;
+		}
+
+		if (slide_info.entries_offset == 0 ||
+			slide_info.entries_offset > cache->hdr->slideInfoSize ||
+			slide_info.entries_offset + slide_info.entries_count * slide_info.entries_size > cache->hdr->slideInfoSize) {
+			return NULL;
+		}
+
+		if (slide_info.toc_count > 0) {
+			ut64 size = slide_info.toc_count * 2;
+			ut64 at = cache->hdr->slideInfoOffset + slide_info.toc_offset;
+			tmp_buf_1 = malloc (size);
+			if (!tmp_buf_1) {
+				goto beach;
+			}
+			if (r_buf_fread_at (cache_buf, at, tmp_buf_1, "s", slide_info.toc_count) != size) {
+				goto beach;
+			}
+		}
+
+		if (slide_info.entries_count > 0) {
+			ut64 size = slide_info.entries_count * slide_info.entries_size;
+			ut64 at = cache->hdr->slideInfoOffset + slide_info.entries_offset;
+			tmp_buf_2 = malloc (size);
+			if (!tmp_buf_2) {
+				goto beach;
+			}
+			if (r_buf_read_at (cache_buf, at, tmp_buf_2, size) != size) {
+				goto beach;
+			}
+		}
+
+		one_page_buf = malloc (4096);
 		if (!one_page_buf) {
 			goto beach;
 		}
-	}
 
-	RDyldRebaseInfo *rebase_info = R_NEW0 (RDyldRebaseInfo);
-	if (!rebase_info) {
-		goto beach;
-	}
+		RDyldRebaseInfo1 *rebase_info = R_NEW0 (RDyldRebaseInfo1);
+		if (!rebase_info) {
+			goto beach;
+		}
 
-	rebase_info->start_of_data = start_of_data;
-	rebase_info->page_starts = page_starts;
-	rebase_info->page_starts_count = slide_info.page_starts_count;
-	rebase_info->page_extras = page_extras;
-	rebase_info->page_extras_count = slide_info.page_extras_count;
-	rebase_info->value_add = slide_info.value_add;
-	rebase_info->delta_mask = slide_info.delta_mask;
-	rebase_info->value_mask = ~rebase_info->delta_mask;
-	rebase_info->delta_shift = dumb_ctzll (rebase_info->delta_mask) - 2;
-	rebase_info->page_size = slide_info.page_size;
-	rebase_info->one_page_buf = one_page_buf;
-	rebase_info->slide = estimate_slide (bf, cache, rebase_info->value_mask);
-	if (rebase_info->slide) {
-		eprintf ("dyldcache is slid: 0x%"PFMT64x"\n", rebase_info->slide);
-	}
+		rebase_info->version = 1;
+		rebase_info->start_of_data = start_of_data;
+		rebase_info->one_page_buf = one_page_buf;
+		rebase_info->page_size = 4096;
+		rebase_info->slide = estimate_slide (bf, cache, 0xffffffffffffffffL);
+		rebase_info->toc = (ut16*) tmp_buf_1;
+		rebase_info->toc_count = slide_info.toc_count;
+		rebase_info->entries = tmp_buf_2;
+		rebase_info->entries_size = slide_info.entries_size;
+		if (rebase_info->slide) {
+			eprintf ("dyldcache is slid: 0x%"PFMT64x"\n", rebase_info->slide);
+		}
 
-	return rebase_info;
+		return (RDyldRebaseInfo*) rebase_info;
+	} else {
+		return NULL;
+	}
 
 beach:
-	R_FREE (page_starts);
-	R_FREE (page_extras);
+	R_FREE (tmp_buf_1);
+	R_FREE (tmp_buf_2);
 	R_FREE (one_page_buf);
 	return NULL;
 }
@@ -575,6 +677,7 @@ static RList *create_cache_bins(RBinFile *bf, RBuffer *cache_buf, cache_hdr_t *h
 			RDyldBinImage *bin = R_NEW0 (RDyldBinImage);
 			if (!bin) {
 				r_list_free (bins);
+				R_FREE (deps);
 				R_FREE (img);
 				return NULL;
 			}
@@ -614,11 +717,43 @@ static RList *create_cache_bins(RBinFile *bf, RBuffer *cache_buf, cache_hdr_t *h
 	return bins;
 }
 
-static void rebase_bytes(RDyldRebaseInfo *rebase_info, ut8 *buf, ut64 offset, int count, ut64 start_of_write) {
-	if (!rebase_info || !buf) {
-		return;
-	}
+static void rebase_bytes_v1(RDyldRebaseInfo1 *rebase_info, ut8 *buf, ut64 offset, int count, ut64 start_of_write) {
+	int in_buf = 0;
+	while (in_buf < count) {
+		ut64 offset_in_data = offset - rebase_info->start_of_data;
+		ut64 page_index = offset_in_data / rebase_info->page_size;
+		ut64 page_offset = offset_in_data % rebase_info->page_size;
+		ut64 to_next_page = rebase_info->page_size - page_offset;
+		ut64 entry_index = page_offset / 32;
+		ut64 offset_in_entry = (page_offset % 32) / 4;
 
+		if (entry_index >= rebase_info->entries_size) {
+			in_buf += to_next_page;
+			offset += to_next_page;
+			continue;
+		}
+
+		if (page_index >= rebase_info->toc_count) {
+			break;
+		}
+
+		ut8 *entry = &rebase_info->entries[rebase_info->toc[page_index] * rebase_info->entries_size];
+		ut8 b = entry[entry_index];
+
+		if (b & (1 << offset_in_entry)) {
+			ut64 value = r_read_le64 (buf + in_buf);
+			value += rebase_info->slide;
+			r_write_le64 (buf + in_buf, value);
+			in_buf += 8;
+			offset += 8;
+		} else {
+			in_buf += 4;
+			offset += 4;
+		}
+	}
+}
+
+static void rebase_bytes_v2(RDyldRebaseInfo2 *rebase_info, ut8 *buf, ut64 offset, int count, ut64 start_of_write) {
 	int in_buf = 0;
 	while (in_buf < count) {
 		ut64 offset_in_data = offset - rebase_info->start_of_data;
@@ -664,6 +799,18 @@ next_page:
 	}
 }
 
+static void rebase_bytes(RDyldRebaseInfo *rebase_info, ut8 *buf, ut64 offset, int count, ut64 start_of_write) {
+	if (!rebase_info || !buf) {
+		return;
+	}
+
+	if (rebase_info->version == 2) {
+		rebase_bytes_v2 ((RDyldRebaseInfo2*) rebase_info, buf, offset, count, start_of_write);
+	} else if (rebase_info->version == 1) {
+		rebase_bytes_v1 ((RDyldRebaseInfo1*) rebase_info, buf, offset, count, start_of_write);
+	}
+}
+
 static int dyldcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	if (!io) {
 		return -1;
@@ -709,6 +856,10 @@ static int dyldcache_io_read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 		ut8 *internal_buf = rebase_info->one_page_buf;
 		if (rounded_count > rebase_info->page_size) {
 			internal_buf = malloc (rounded_count);
+			if (!internal_buf) {
+				eprintf ("Cannot allocate memory for 'internal_buf'\n");
+				return -1;
+			}
 		}
 
 		ut64 original_off = io->off;
@@ -828,7 +979,7 @@ static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
 	}
 
 	RDyldCache *cache = R_NEW0 (RDyldCache);
-	strncpy ((char*) cache->magic, "dyldcac", 7);
+	memcpy (cache->magic, "dyldcac", 7);
 	cache->buf = fbuf;
 	cache->hdr = read_cache_header (fbuf);
 	if (!cache->hdr) {
@@ -866,15 +1017,15 @@ static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
 	return cache;
 }
 
-static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
-	return (void *) (size_t) check_bytes (buf, sz);
+static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
+	return check_bytes (buf, sz);
 }
 
 static bool load(RBinFile *bf) {
 	const ut8 *bytes = bf ? r_buf_buffer (bf->buf) : NULL;
 	ut64 sz = bf ? r_buf_size (bf->buf): 0;
 	ut64 la = (bf && bf->o) ? bf->o->loadaddr: 0;
-	return load_bytes (bf, bytes, sz, la, bf? bf->sdb: NULL) != NULL;
+	return load_bytes (bf, bf? &bf->o->bin_obj: NULL, bytes, sz, la, bf? bf->sdb: NULL);
 }
 
 static RList *entries(RBinFile *bf) {
@@ -1020,15 +1171,14 @@ static void sections_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 			break;
 		}
 		if (bin->file) {
-			r_snprintf (ptr->name, R_BIN_SIZEOF_STRINGS, "%s.%s", bin->file, (char*)sections[i].name);
+			ptr->name = r_str_newf ("%s.%s", bin->file, (char*)sections[i].name);
 		} else {
-			r_snprintf (ptr->name, R_BIN_SIZEOF_STRINGS, "%s", (char*)sections[i].name);
+			ptr->name = r_str_newf ("%s", (char*)sections[i].name);
 		}
 		if (strstr (ptr->name, "la_symbol_ptr")) {
 			int len = sections[i].size / 8;
 			ptr->format = r_str_newf ("Cd %d[%d]", 8, len);
 		}
-		ptr->name[R_BIN_SIZEOF_STRINGS] = 0;
 		handle_data_sections (ptr);
 		ptr->size = sections[i].size;
 		ptr->vsize = sections[i].vsize;
@@ -1038,7 +1188,7 @@ static void sections_from_bin(RList *ret, RBinFile *bf, RDyldBinImage *bin) {
 		if (!ptr->vaddr) {
 			ptr->vaddr = ptr->paddr;
 		}
-		ptr->srwx = sections[i].srwx;
+		ptr->perm = sections[i].perm;
 		r_list_append (ret, ptr);
 	}
 	free (sections);
@@ -1068,13 +1218,13 @@ static RList *sections(RBinFile *bf) {
 		if (!(ptr = R_NEW0 (RBinSection))) {
 			return NULL;
 		}
-		r_snprintf (ptr->name, R_BIN_SIZEOF_STRINGS, "cache_map.%d", i);
+		ptr->name = r_str_newf ("cache_map.%d", i);
 		ptr->size = cache->maps[i].size;
 		ptr->vsize = ptr->size;
 		ptr->paddr = cache->maps[i].fileOffset;// + bf->o->boffset;
 		ptr->vaddr = cache->maps[i].address;
 		ptr->add = true;
-		ptr->srwx = prot2perm (cache->maps[i].initProt);
+		ptr->perm = prot2perm (cache->maps[i].initProt);
 		r_list_append (ret, ptr);
 	}
 
@@ -1280,14 +1430,23 @@ static void header(RBinFile *bf) {
 	bin->cb_printf ("rangeTableCount: 0x%"PFMT64x"\n", cache->accel->rangeTableCount);
 	bin->cb_printf ("dyldSectionAddr: 0x%"PFMT64x"\n", cache->accel->dyldSectionAddr + slide);
 
-	bin->cb_printf ("\nslide info:\n");
-	bin->cb_printf ("page_starts_count: 0x%"PFMT64x"\n", cache->rebase_info->page_starts_count);
-	bin->cb_printf ("page_extras_count: 0x%"PFMT64x"\n", cache->rebase_info->page_extras_count);
-	bin->cb_printf ("delta_mask: 0x%"PFMT64x"\n", cache->rebase_info->delta_mask);
-	bin->cb_printf ("value_mask: 0x%"PFMT64x"\n", cache->rebase_info->value_mask);
-	bin->cb_printf ("delta_shift: 0x%"PFMT64x"\n", cache->rebase_info->delta_shift);
-	bin->cb_printf ("page_size: 0x%"PFMT64x"\n", cache->rebase_info->page_size);
+	ut8 version = cache->rebase_info->version;
+	bin->cb_printf ("\nslide info (v%d):\n", version);
 	bin->cb_printf ("slide: 0x%"PFMT64x"\n", slide);
+	if (version == 2) {
+		RDyldRebaseInfo2 * info2 = (RDyldRebaseInfo2*) cache->rebase_info;
+		bin->cb_printf ("page_starts_count: 0x%"PFMT64x"\n", info2->page_starts_count);
+		bin->cb_printf ("page_extras_count: 0x%"PFMT64x"\n", info2->page_extras_count);
+		bin->cb_printf ("delta_mask: 0x%"PFMT64x"\n", info2->delta_mask);
+		bin->cb_printf ("value_mask: 0x%"PFMT64x"\n", info2->value_mask);
+		bin->cb_printf ("delta_shift: 0x%"PFMT64x"\n", info2->delta_shift);
+		bin->cb_printf ("page_size: 0x%"PFMT64x"\n", info2->page_size);
+	} else if (version == 1) {
+		RDyldRebaseInfo1 * info1 = (RDyldRebaseInfo1*) cache->rebase_info;
+		bin->cb_printf ("toc_count: 0x%"PFMT64x"\n", info1->toc_count);
+		bin->cb_printf ("entries_size: 0x%"PFMT64x"\n", info1->entries_size);
+		bin->cb_printf ("page_size: 0x%"PFMT64x"\n", 4096);
+	}
 }
 
 RBinPlugin r_bin_plugin_dyldcache = {
@@ -1309,7 +1468,7 @@ RBinPlugin r_bin_plugin_dyldcache = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_dyldcache,
 	.version = R2_VERSION
