@@ -24,7 +24,7 @@ RBinLdrPlugin r_bin_ldr_plugin_ldr_linux = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN_LDR,
 	.data = &r_bin_ldr_plugin_ldr_linux,
 	.version = R2_VERSION
