@@ -11,12 +11,18 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return (!memcmp (buf, INES_MAGIC, 4));
 }
 
-static void * load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
-	check_bytes (buf, sz);
-	return R_NOTNULL;
+static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb){
+	return check_bytes (buf, sz);
 }
 
-static RBinInfo* info(RBinFile *bf) {
+static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
+	if (!check_bytes (r_buf_get_at (buf, 0, NULL), buf->length)) {
+		return NULL;
+	}
+	return r_buf_new ();
+}
+
+static RBinInfo *info(RBinFile *bf) {
 	RBinInfo *ret = NULL;
 	ines_hdr ihdr;
 	memset (&ihdr, 0, INES_HDR_SIZE);
@@ -40,7 +46,9 @@ static RBinInfo* info(RBinFile *bf) {
 
 static void addsym(RList *ret, const char *name, ut64 addr, ut32 size) {
 	RBinSymbol *ptr = R_NEW0 (RBinSymbol);
-	if (!ptr) return;
+	if (!ptr) {
+		return;
+	}
 	ptr->name = strdup (name? name: "");
 	ptr->paddr = ptr->vaddr = addr;
 	ptr->size = size;
@@ -94,21 +102,21 @@ static RList* sections(RBinFile *bf) {
 	if (!(ptr = R_NEW0 (RBinSection))) {
 		return ret;
 	}
-	strcpy (ptr->name, "ROM");
+	ptr->name = strdup ("ROM");
 	ptr->paddr = INES_HDR_SIZE;
 	ptr->size = ihdr.prg_page_count_16k * PRG_PAGE_SIZE;
 	ptr->vaddr = ROM_START_ADDRESS;
 	ptr->vsize = ROM_SIZE;
-	ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;
+	ptr->perm = R_PERM_RX;
 	ptr->add = true;
 	r_list_append (ret, ptr);
 	return ret;
 }
 
-static RList *mem (RBinFile *bf) {
+static RList *mem(RBinFile *bf) {
 	RList *ret;
 	RBinMem *m, *n;
-	if (!(ret = r_list_new())) {
+	if (!(ret = r_list_new ())) {
 		return NULL;
 	}
 	ret->free = free;
@@ -209,6 +217,7 @@ RBinPlugin r_bin_plugin_nes = {
 	.desc = "NES",
 	.license = "LGPL3",
 	.load_bytes = &load_bytes,
+	.load_buffer = &load_buffer,
 	.baddr = &baddr,
 	.check_bytes = &check_bytes,
 	.entries = &entries,
@@ -219,7 +228,7 @@ RBinPlugin r_bin_plugin_nes = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_nes,
 	.version = R2_VERSION
