@@ -44,7 +44,7 @@ static uint32_t decompress(const ut8 *cbuf, ut8 *obuf, int32_t csize, int32_t us
 }
 
 static ut64 baddr(RBinFile *bf) {
-	return 0;	// XXX
+	return 0x8000000;
 }
 
 static bool check_bytes(const ut8 *buf, ut64 length) {
@@ -54,9 +54,16 @@ static bool check_bytes(const ut8 *buf, ut64 length) {
 	return false;
 }
 
-static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
-	RBin *rbin = bf->rbin;
+static RBinNXOObj *nso_new () {
 	RBinNXOObj *bin = R_NEW0 (RBinNXOObj);
+	bin->methods_list = r_list_newf ((RListFree)free);
+	bin->imports_list = r_list_newf ((RListFree)free);
+	bin->classes_list = r_list_newf ((RListFree)free);
+	return bin;
+}
+
+static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
+	RBin *rbin = bf->rbin;
 	ut32 toff = readLE32 (bf->buf, NSO_OFF (text_memoffset));
 	ut32 tsize = readLE32 (bf->buf, NSO_OFF (text_size));
 	ut32 rooff = readLE32 (bf->buf, NSO_OFF (ro_memoffset));
@@ -67,44 +74,51 @@ static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sd
 	RBuffer *newbuf = r_buf_new_empty (total_size);
 	ut64 ba = baddr (bf);
 
-	if (rbin->iob.io && !(rbin->iob.io->cached & R_IO_WRITE)) {
-		eprintf ("Please add \'-e io.cache=true\' option to r2 command\n");
+	if (rbin->iob.io && !(rbin->iob.io->cached & R_PERM_W)) {
+		eprintf ("Please add \'-e io.cache=true\' option to r2 command. This is required to decompress the code.\n");
 		goto fail;
 	}
 	/* Decompress each sections */
 	if (decompress (buf + toff, r_buf_get_at (newbuf, 0, NULL), rooff - toff, tsize) != tsize) {
+		eprintf ("decompression failure\n");
 		goto fail;
 	}
 	if (decompress (buf + rooff, r_buf_get_at (newbuf, tsize, NULL), doff - rooff, rosize) != rosize) {
+		eprintf ("decompression2 failure\n");
 		goto fail;
 	}
 	if (decompress (buf + doff, r_buf_get_at (newbuf, tsize + rosize, NULL), r_buf_size (bf->buf) - doff, dsize) != dsize) {
+		eprintf ("decompression3 failure\n");
 		goto fail;
 	}
 	/* Load unpacked binary */
 	r_io_write_at (rbin->iob.io, ba, r_buf_get_at (newbuf, 0, NULL), total_size);
 	ut32 modoff = readLE32 (newbuf, NSO_OFFSET_MODMEMOFF);
-	bin->methods_list = r_list_newf ((RListFree)free);
-	bin->imports_list = r_list_newf ((RListFree)free);
-	bin->classes_list = r_list_newf ((RListFree)free);
+	RBinNXOObj *bin = nso_new ();
 	eprintf ("MOD Offset = 0x%"PFMT64x"\n", (ut64)modoff);
 	parseMod (newbuf, bin, modoff, ba);
 	r_buf_free (newbuf);
-	return (void *) bin;
+	*bin_obj = bin;
+	return true;
 fail:
 	r_buf_free (newbuf);
-	return NULL;
+	*bin_obj = NULL;
+	return false;
 }
 
-static bool load(RBinFile *bf) {
-	if (!bf || !bf->buf || !bf->o) {
-		return false;
+static void *load_buffer(RBinFile *bf, RBuffer *buf, ut64 loadaddr, Sdb *sdb) {
+	r_return_val_if_fail (bf && buf, NULL);
+	const ut64 sz = r_buf_size (buf);
+	ut8 *bytes = malloc (sz);
+	if (!bytes) {
+		return NULL;
 	}
-	const ut64 sz = r_buf_size (bf->buf);
-	const ut64 la = bf->o->loadaddr;
-	const ut8 *bytes = r_buf_buffer (bf->buf);
-	bf->o->bin_obj = load_bytes (bf, bytes, sz, la, bf->sdb);
-	return bf->o->bin_obj != NULL;
+	r_buf_read_at (buf, 0, bytes, sz);
+	void *ptr = NULL;
+	const ut64 la = bf->loadaddr;
+	(void)load_bytes (bf, &ptr, bytes, sz, la, bf->sdb);
+	free (bytes);
+	return ptr;
 }
 
 static int destroy(RBinFile *bf) {
@@ -112,7 +126,7 @@ static int destroy(RBinFile *bf) {
 }
 
 static RBinAddr *binsym(RBinFile *bf, int type) {
-	return NULL;	// TODO
+	return NULL; // TODO
 }
 
 static RList *entries(RBinFile *bf) {
@@ -160,12 +174,12 @@ static RList *sections(RBinFile *bf) {
 	if (!(ptr = R_NEW0 (RBinSection))) {
 		return ret;
 	}
-	strncpy (ptr->name, "header", R_BIN_SIZEOF_STRINGS);
+	ptr->name = strdup ("header");
 	ptr->size = readLE32 (b, NSO_OFF (text_memoffset));
 	ptr->vsize = readLE32 (b, NSO_OFF (text_memoffset));
 	ptr->paddr = 0;
 	ptr->vaddr = 0;
-	ptr->srwx = R_BIN_SCN_READABLE;
+	ptr->perm = R_PERM_R;
 	ptr->add = false;
 	r_list_append (ret, ptr);
 
@@ -173,12 +187,12 @@ static RList *sections(RBinFile *bf) {
 	if (!(ptr = R_NEW0 (RBinSection))) {
 		return ret;
 	}
-	strncpy (ptr->name, "text", R_BIN_SIZEOF_STRINGS);
+	ptr->name = strdup ("text");
 	ptr->vsize = readLE32 (b, NSO_OFF (text_size));
 	ptr->size = ptr->vsize;
 	ptr->paddr = readLE32 (b, NSO_OFF (text_memoffset));
 	ptr->vaddr = readLE32 (b, NSO_OFF (text_loc)) + ba;
-	ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE;	// r-x
+	ptr->perm = R_PERM_RX;	// r-x
 	ptr->add = true;
 	r_list_append (ret, ptr);
 
@@ -186,12 +200,12 @@ static RList *sections(RBinFile *bf) {
 	if (!(ptr = R_NEW0 (RBinSection))) {
 		return ret;
 	}
-	strncpy (ptr->name, "ro", R_BIN_SIZEOF_STRINGS);
+	ptr->name = strdup ("ro");
 	ptr->vsize = readLE32 (b, NSO_OFF (ro_size));
 	ptr->size = ptr->vsize;
 	ptr->paddr = readLE32 (b, NSO_OFF (ro_memoffset));
 	ptr->vaddr = readLE32 (b, NSO_OFF (ro_loc)) + ba;
-	ptr->srwx = R_BIN_SCN_READABLE;	// r--
+	ptr->perm = R_PERM_R;	// r--
 	ptr->add = true;
 	r_list_append (ret, ptr);
 
@@ -199,12 +213,12 @@ static RList *sections(RBinFile *bf) {
 	if (!(ptr = R_NEW0 (RBinSection))) {
 		return ret;
 	}
-	strncpy (ptr->name, "data", R_BIN_SIZEOF_STRINGS);
+	ptr->name = strdup ("data");
 	ptr->vsize = readLE32 (b, NSO_OFF (data_size));
 	ptr->size = ptr->vsize;
 	ptr->paddr = readLE32 (b, NSO_OFF (data_memoffset));
 	ptr->vaddr = readLE32 (b, NSO_OFF (data_loc)) + ba;
-	ptr->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_WRITABLE;	// rw-
+	ptr->perm = R_PERM_RW;
 	ptr->add = true;
 	eprintf ("BSS Size 0x%08"PFMT64x "\n", (ut64)
 		readLE32 (bf->buf, NSO_OFF (bss_size)));
@@ -212,34 +226,6 @@ static RList *sections(RBinFile *bf) {
 	return ret;
 }
 
-static RList *symbols(RBinFile *bf) {
-	RBinNXOObj *bin;
-	if (!bf || !bf->o || !bf->o->bin_obj) {
-		return NULL;
-	}
-	bin = (RBinNXOObj*) bf->o->bin_obj;
-	if (!bin) {
-		return NULL;
-	}
-	return bin->methods_list;
-}
-
-static RList *imports(RBinFile *bf) {
-	RBinNXOObj *bin;
-	if (!bf || !bf->o || !bf->o->bin_obj) {
-		return NULL;
-	}
-	bin = (RBinNXOObj*) bf->o->bin_obj;
-	if (!bin) {
-		return NULL;
-	}
-	return bin->imports_list;
-}
-
-static RList *libs(RBinFile *bf) {
-	return NULL;
-}
-
 static RBinInfo *info(RBinFile *bf) {
 	RBinInfo *ret = R_NEW0 (RBinInfo);
 	if (!ret) {
@@ -271,8 +257,7 @@ RBinPlugin r_bin_plugin_nso = {
 	.name = "nso",
 	.desc = "Nintendo Switch NSO0 binaries",
 	.license = "MIT",
-	.load = &load,
-	.load_bytes = &load_bytes,
+	.load_buffer = &load_buffer,
 	.destroy = &destroy,
 	.check_bytes = &check_bytes,
 	.baddr = &baddr,
@@ -280,14 +265,11 @@ RBinPlugin r_bin_plugin_nso = {
 	.entries = &entries,
 	.sections = &sections,
 	.get_sdb = &get_sdb,
-	.symbols = &symbols,
-	.imports = &imports,
 	.info = &info,
-	.libs = &libs,
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_nso,
 	.version = R2_VERSION
