@@ -4,11 +4,11 @@
 #include <r_bin.h>
 #include "pe/pe.h"
 
-static bool scn_perms(RBinFile *arch, const char *name, int perms) {
-	struct PE_(r_bin_pe_obj_t) *obj = arch->o->bin_obj;
-	bool ret = PE_(r_bin_pe_section_perms) (arch->o->bin_obj, name, perms);
-	r_buf_free (arch->buf);
-	arch->buf = obj->b;
+static bool scn_perms(RBinFile *bf, const char *name, int perms) {
+	struct PE_(r_bin_pe_obj_t) *obj = bf->o->bin_obj;
+	bool ret = PE_(r_bin_pe_section_perms) (bf->o->bin_obj, name, perms);
+	r_buf_free (bf->buf);
+	bf->buf = obj->b;
 	obj->b = NULL;
 	return ret;
 }
