@@ -77,11 +77,12 @@ static bool check_bytes (const ut8 *buf, ut64 length) {
 	return magic == r_read_be32 (buf);
 }
 
-static void *load_bytes(RBinFile *bf, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
+static bool load_bytes(RBinFile *bf, void **bin_obj, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb) {
 	if (check_bytes (r_buf_buffer (bf->buf), sz)) {
-		return memcpy (&n64_header, buf, sizeof (N64Header));
+		*bin_obj = memcpy (&n64_header, buf, sizeof (N64Header));
+		return true;
 	}
-	return NULL;
+	return false;
 }
 
 static bool load(RBinFile *bf) {
@@ -90,7 +91,7 @@ static bool load(RBinFile *bf) {
 	if (!bf || !bf->o) {
 		return false;
 	}
-	bf->o->bin_obj = load_bytes (bf, bytes, sz, bf->o->loadaddr, bf->sdb);
+	load_bytes (bf, &bf->o->bin_obj, bytes, sz, bf->o->loadaddr, bf->sdb);
 	return check_bytes (bytes, sz);
 }
 
@@ -122,12 +123,12 @@ static RList *sections(RBinFile *bf) {
 		r_list_free (ret);
 		return NULL;
 	}
-	strncpy (text->name, "text", R_BIN_SIZEOF_STRINGS);
+	text->name = strdup ("text");
 	text->size = bf->buf->length - N64_ROM_START;
 	text->vsize = text->size;
 	text->paddr = N64_ROM_START;
 	text->vaddr = baddr (bf);
-	text->srwx = R_BIN_SCN_READABLE | R_BIN_SCN_EXECUTABLE; // r-x
+	text->perm = R_PERM_RX;
 	text->add = true;
 	r_list_append (ret, text);
 	return ret;
@@ -149,7 +150,7 @@ static RBinInfo *info(RBinFile *bf) {
 	ret->arch = strdup ("mips");
 	ret->machine = strdup ("Nintendo 64");
 	ret->type = strdup ("ROM");
-	ret->bits = 32;
+	ret->bits = 64;
 	ret->has_va = true;
 	ret->big_endian = true;
 	return ret;
@@ -173,7 +174,7 @@ RBinPlugin r_bin_plugin_z64 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BIN,
 	.data = &r_bin_plugin_z64,
 	.version = R2_VERSION
