@@ -19,7 +19,7 @@ struct r_bp_plugin_t r_bp_plugin_mips = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_BP,
 	.data = &r_bp_plugin_mips,
 	.version = R2_VERSION
