@@ -1,16 +1,13 @@
-/* radare - LGPL - Copyright 2006-2016 - pancake */
+/* radare - LGPL - Copyright 2006-2018 - pancake */
 
 #include "r_config.h"
 #include "r_util.h" // r_str_hash, r_str_chop, ...
-#ifdef _MSC_VER
-#define strcasecmp stricmp
-#endif
+
 R_API RConfigNode* r_config_node_new(const char *name, const char *value) {
-	RConfigNode *node;
-	if (STRNULL (name)) {
+	if (IS_NULLSTR (name)) {
 		return NULL;
 	}
-	node = R_NEW0 (RConfigNode);
+	RConfigNode *node = R_NEW0 (RConfigNode);
 	if (!node) {
 		return NULL;
 	}
@@ -49,6 +46,19 @@ R_API void r_config_node_free(void *n) {
 	free (node);
 }
 
+static bool isBoolean(const char *val) {
+	if (!r_str_casecmp (val, "true") || !r_str_casecmp (val, "false")) {
+		return true;
+	}
+	if (!r_str_casecmp (val, "on") || !r_str_casecmp (val, "off")) {
+		return true;
+	}
+	if (!r_str_casecmp (val, "yes") || !r_str_casecmp (val, "no")) {
+		return true;
+	}
+	return false;
+}
+
 static void config_print_value_json(RConfig *cfg, RConfigNode *node) {
 	const char *val = node->value;
 	if (!val) {
@@ -59,7 +69,7 @@ static void config_print_value_json(RConfig *cfg, RConfigNode *node) {
 		if (!strncmp (val, "0x", 2)) {
 			ut64 n = r_num_get (NULL, val);
 			cfg->cb_printf ("%"PFMT64d, n);
-		} else if (r_str_isnumber (val) || !strcmp (val, "true") || !strcmp (val, "false")) {
+		} else if (r_str_isnumber (val) || isBoolean (val)) {
 			cfg->cb_printf ("%s", val);
 		} else {
 			cfg->cb_printf ("\"%s\"", sval);
@@ -148,7 +158,7 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 	bool json = false;
 	bool isFirst = false;
 
-	if (!STRNULL (str)) {
+	if (!IS_NULLSTR (str)) {
 		str = r_str_trim_ro (str);
 		len = strlen (str);
 		if (len > 0 && str[0] == 'j') {
@@ -271,10 +281,10 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 }
 
 R_API RConfigNode* r_config_node_get(RConfig *cfg, const char *name) {
-	if (STRNULL (name)) {
+	if (!cfg || IS_NULLSTR (name)) {
 		return NULL;
 	}
-	return ht_find (cfg->ht, name, NULL);
+	return ht_pp_find (cfg->ht, name, NULL);
 }
 
 R_API int r_config_set_getter(RConfig *cfg, const char *key, RConfigCallback cb) {
@@ -296,14 +306,17 @@ R_API int r_config_set_setter(RConfig *cfg, const char *key, RConfigCallback cb)
 }
 
 static bool is_true(const char *s) {
-	return !strcasecmp ("true", s) || !strcasecmp ("1", s);
+	return !r_str_casecmp ("yes", s) || !r_str_casecmp ("on", s) || !r_str_casecmp ("true", s) || !r_str_casecmp ("1", s);
 }
 
 static bool is_bool(const char *s) {
-	return !strcasecmp ("true", s) || !strcasecmp ("false", s);
+	return !r_str_casecmp ("true", s) || !r_str_casecmp ("false", s);
 }
 
 R_API const char* r_config_get(RConfig *cfg, const char *name) {
+	if (!cfg || !name) {
+		return NULL;
+	}
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
 		if (node->getter) {
@@ -339,6 +352,9 @@ R_API ut64 r_config_get_i(RConfig *cfg, const char *name) {
 		if (node->i_value || !strcmp (node->value, "false")) {
 			return node->i_value;
 		}
+		if (!strcmp (node->value, "true")) {
+			return 1;
+		}
 		return (ut64) r_num_math (cfg->num, node->value);
 	}
 	return (ut64) 0LL;
@@ -388,9 +404,10 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 	RConfigNode *node = NULL;
 	char *ov = NULL;
 	ut64 oi;
-	if (!cfg || STRNULL (name)) {
-		return NULL;
-	}
+
+	r_return_val_if_fail (cfg && cfg->ht, NULL);
+	r_return_val_if_fail (!IS_NULLSTR (name), NULL);
+
 	node = r_config_node_get (cfg, name);
 	if (node) {
 		if (node->flags & CN_RO) {
@@ -409,7 +426,7 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 		}
 		if (node->flags & CN_BOOL) {
 			bool b = is_true (value);
-			node->i_value = (ut64) b? 1: 0;
+			node->i_value = b? 1: 0;
 			char *value = strdup (r_str_bool (b));
 			if (value) {
 				free (node->value);
@@ -448,11 +465,9 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 					node->flags |= CN_BOOL;
 					node->i_value = is_true (value)? 1: 0;
 				}
-				if (cfg->ht) {
-					ht_insert (cfg->ht, node->name, node);
-					r_list_append (cfg->nodes, node);
-					cfg->n_nodes++;
-				}
+				ht_pp_insert (cfg->ht, node->name, node);
+				r_list_append (cfg->nodes, node);
+				cfg->n_nodes++;
 			} else {
 				eprintf ("r_config_set: unable to create a new RConfigNode\n");
 			}
@@ -469,6 +484,8 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 			}
 			free (node->value);
 			node->value = strdup (ov? ov: "");
+free (ov);
+			return NULL;
 		}
 	}
 beach:
@@ -498,7 +515,7 @@ R_API const char* r_config_node_desc(RConfigNode *node, const char *desc) {
 R_API int r_config_rm(RConfig *cfg, const char *name) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
-		ht_delete (cfg->ht, node->name);
+		ht_pp_delete (cfg->ht, node->name);
 		r_list_delete_data (cfg->nodes, node);
 		cfg->n_nodes--;
 		return true;
@@ -551,9 +568,7 @@ R_API RConfigNode* r_config_set_i(RConfig *cfg, const char *name, const ut64 i)
 			}
 			node->flags = CN_RW | CN_OFFT;
 			node->i_value = i;
-			if (cfg->ht) {
-				ht_insert (cfg->ht, node->name, node);
-			}
+			ht_pp_insert (cfg->ht, node->name, node);
 			if (cfg->nodes) {
 				r_list_append (cfg->nodes, node);
 				cfg->n_nodes++;
@@ -646,18 +661,12 @@ R_API int r_config_readonly(RConfig *cfg, const char *key) {
 	return true;
 }
 
-static void _ht_node_free_kv(HtKv *kv) {
-	free (kv->key);
-	//we do not free kv->value because there is other reference 
-	free (kv);
-}
-
 R_API RConfig* r_config_new(void *user) {
 	RConfig *cfg = R_NEW0 (RConfig);
 	if (!cfg) {
 		return NULL;
 	}
-	cfg->ht = ht_new (NULL, _ht_node_free_kv, NULL);
+	cfg->ht = ht_pp_new0 ();
 	cfg->nodes = r_list_newf ((RListFree)r_config_node_free);
 	if (!cfg->nodes) {
 		R_FREE (cfg);
@@ -680,7 +689,7 @@ R_API RConfig* r_config_clone(RConfig *cfg) {
 	}
 	r_list_foreach (cfg->nodes, iter, node) {
 		RConfigNode *nn = r_config_node_clone (node);
-		ht_insert (c->ht, node->name, nn);
+		ht_pp_insert (c->ht, node->name, nn);
 		r_list_append (c->nodes, nn);
 		c->n_nodes++;
 	}
@@ -695,7 +704,7 @@ R_API int r_config_free(RConfig *cfg) {
 	}
 	cfg->nodes->free = r_config_node_free; // damn
 	r_list_free (cfg->nodes);
-	ht_free (cfg->ht);
+	ht_pp_free (cfg->ht);
 	free (cfg);
 	return 0;
 }
@@ -739,6 +748,9 @@ R_API bool r_config_save_char(RConfigHold *h, ...) {
 R_API bool r_config_save_num(RConfigHold *h, ...) {
 	va_list ap;
 	char *key;
+	if (!h) {
+		return false;
+	}
 	if (!h->list_num) {
 		h->list_num = r_list_newf ((RListFree) free);
 		if (!h->list_num) {
