@@ -170,7 +170,7 @@ static int r_line_readchar_utf8(ut8 *s, int slen) {
 	} else {
 		return -1;
 	}
-	if (slen < len) {
+	if (len > slen) {
 		return -1;
 	}
 	for (i = 1; i < len; i++) {
@@ -253,7 +253,7 @@ R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb up, RLineHistor
 	line->cb_history_down = down;
 	line->offset_hist_index = 0;
 	line->file_hist_index = 0;
-	line->sdbshell_hist_iter = line->sdbshell_hist? r_list_head (line->sdbshell_hist): NULL;
+	line->sdbshell_hist_iter = r_list_head (line->sdbshell_hist);
 	return 1;
 }
 
@@ -372,8 +372,7 @@ R_API void r_line_hist_free() {
 	int i;
 	if (I.history.data) {
 		for (i = 0; i < I.history.size; i++) {
-			free (I.history.data[i]);
-			I.history.data[i] = NULL;
+			R_FREE (I.history.data[i]);
 		}
 	}
 	R_FREE (I.history.data);
@@ -557,26 +556,30 @@ static void selection_widget_update() {
 }
 
 R_API void r_line_autocomplete() {
-
-	int argc = 0;
 	char *p;
 	const char **argv = NULL;
-	int i, j, opt = 0, plen, len = 0;
+	int argc = 0, i, j, plen, len = 0;
+	bool opt = false;
 	int cols = (int)(r_cons_get_size (NULL) * 0.82);
 
 	/* prepare argc and argv */
 	if (I.completion.run) {
+		I.completion.opt = false;
 		I.completion.run (&I);
-		opt = argc = I.completion.argc;
+		argc = I.completion.argc;
 		argv = I.completion.argv;
+		opt = I.completion.opt;
 	}
-
 	if (I.sel_widget && !I.sel_widget->complete_common) {
 		selection_widget_update ();
 		return;
 	}
 
-	p = (char *) r_sub_str_lchr (I.buffer.data, 0, I.buffer.index, ' ');
+	if (opt) {
+		p = (char *) r_sub_str_lchr (I.buffer.data, 0, I.buffer.index, '=');
+	} else {
+		p = (char *) r_sub_str_lchr (I.buffer.data, 0, I.buffer.index, ' ');
+	}
 	if (!p) {
 		p = (char *) r_sub_str_lchr (I.buffer.data, 0, I.buffer.index, '@');	// HACK FOR r2
 	}
@@ -659,7 +662,7 @@ R_API void r_line_autocomplete() {
 	}
 
 	/* show options */
-	if (opt > 1 && I.echo) {
+	if (argc > 1 && I.echo) {
 		const int sep = 3;
 		int slen, col = 10;
 		printf ("%s%s\n", I.prompt, I.buffer.data);
@@ -725,8 +728,13 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 	r_cons_set_raw (1);
 
 	if (I.echo) {
-		r_cons_clear_line (0);
-		printf ("%s%s%s", Color_RESET, I.prompt, I.buffer.data);
+		if (I.ansicon) {
+			printf ("\r%s", R_CONS_CLEAR_LINE);
+			printf ("%s%s%s", Color_RESET, I.prompt, I.buffer.data);
+		} else {
+			r_cons_clear_line (0);
+			printf ("%s%s", I.prompt, I.buffer.data);
+		}
 		fflush (stdout);
 	}
 	r_cons_break_push (NULL, NULL);
@@ -743,17 +751,11 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 		}
 		buf[0] = ch;
 		if (I.echo) {
-			r_cons_clear_line (0);
-		}
-		columns = r_cons_get_size (NULL) - 2;
-		if (columns < 1) {
-			columns = 40;
-		}
-		if (I.echo) {
-			printf ("\r%*c\r", columns, ' ');
-		}
-		if (I.echo) {
-			printf ("\r\x1b[2K\r");	// %*c\r", columns, ' ');
+			if (I.ansicon) {
+				printf ("\r%s", R_CONS_CLEAR_LINE);
+			} else {
+				r_cons_clear_line (0);
+			}
 		}
 		/* process special at vch codes first*/
 		switch (vch) {
@@ -1079,7 +1081,11 @@ R_API const char *r_line_readline_cb_win(RLineReadCallback cb, void *user) {
 				int chars = R_MAX (1, strlen (I.buffer.data));	// wtf?
 				int len, cols = R_MAX (1, columns - r_str_ansi_len (I.prompt) - 2);
 				/* print line */
-				printf ("\r%s%s", Color_RESET, I.prompt);
+				if (I.ansicon) {
+					printf ("\r%s%s", Color_RESET, I.prompt);
+				} else {
+					printf ("\r%s", I.prompt);
+				}
 				fwrite (I.buffer.data, 1, R_MIN (cols, chars), stdout);
 				/* place cursor */
 				printf ("\r%s", I.prompt);
@@ -1166,9 +1172,12 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			break;
 		}
 		I.buffer.data[I.buffer.length] = '\0';
-		if (cb && !cb (user, I.buffer.data)) {
-			I.buffer.data[0] = 0;
-			I.buffer.length = 0;
+		if (cb) {
+			int cbret = cb (user, I.buffer.data);
+			if (cbret == 0) {
+				I.buffer.data[0] = 0;
+				I.buffer.length = 0;
+			}
 		}
 #if USE_UTF8
 		utflen = r_line_readchar_utf8 ((ut8 *) buf, sizeof (buf));
@@ -1200,7 +1209,6 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			printf ("\r\x1b[2K\r");	// %*c\r", columns, ' ');
 		}
 #endif
-
 		switch (*buf) {
 		case 0:	// control-space
 			/* ignore atm */
