@@ -14,6 +14,9 @@ R_API RLine *r_line_new() {
 	I.hist_down = NULL;
 	I.prompt = strdup ("> ");
 	I.contents = NULL;
+#if __WINDOWS__ && !__CYGWIN__
+	I.ansicon = r_sys_getenv ("ANSICON");
+#endif
 	if (!r_line_dietline_init ()) {
 		eprintf ("error: r_line_dietline_init\n");
 	}
