@@ -447,8 +447,7 @@ rep:
 			} else {
 				core->flags->base = r_num_math (core->num, input+1);
 			}
-			free (str);
-			str = NULL;
+			R_FREE (str);
 			break;
 		case '\0':
 			r_cons_printf ("%"PFMT64d" 0x%"PFMT64x"\n",
@@ -510,8 +509,7 @@ rep:
 			if (*flagname == '.') {
 				RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, off, 0);
 				if (fcn) {
-					eprintf ("TODO: local_del_name has been deprecated\n");
-					//;r_anal_fcn_local_del_name (core->anal, fcn, flagname+1);
+					r_anal_fcn_label_del (core->anal, fcn, flagname  +  1, off);
 				} else {
 					eprintf ("Cannot find function at 0x%08"PFMT64x"\n", off);
 				}
@@ -972,12 +970,13 @@ rep:
 		}
 		break;
 	case '?':
+	default:
 		if (input[1]) {
 			core->num->value = r_flag_get (core->flags, input + 1)? 1: 0;
 		} else {
-		r_core_cmd_help (core, help_msg_f);
-		break;
-	}
+			r_core_cmd_help (core, help_msg_f);
+			break;
+		}
 	}
 	free (str);
 	return 0;
