@@ -39,13 +39,14 @@ static const char *help_msg_i[] = {
 	"iM", "", "Show main address",
 	"io", " [file]", "Load info from file (or last opened) use bin.baddr",
 	"iO", "[?]", "Perform binary operation (dump, resize, change sections, ...)",
-	"ir", "", "Relocs",
-	"iR", "", "Resources",
-	"is", "", "Symbols",
+	"ir", "", "List the Relocations",
+	"iR", "", "List the Resources",
+	"is", "", "List the Symbols",
 	"is.", "", "Current symbol",
 	"iS ", "[entropy,sha1]", "Sections (choose which hash algorithm to use)",
 	"iS.", "", "Current section",
-	"iSS", " [entropy,sha1]", "Segments",
+	"iS=", "", "Show ascii-art color bars with the section ranges",
+	"iSS", "", "List memory segments (maps with om)",
 	"iV", "", "Display file version info",
 	"iX", "", "Display source files used (via dwarf)",
 	"iz|izj", "", "Strings in data sections (in JSON/Base64)",
@@ -56,13 +57,13 @@ static const char *help_msg_i[] = {
 	NULL
 };
 
+// TODO: this command needs a refactoring
 static const char *help_msg_id[] = {
-	"Usage: id", "", "Debug information",
-	"Output mode:", "", "",
-	"'*'", "", "Output in radare commands",
-	"id", "", "Source lines",
+	"Usage: idp", "", "Debug information",
+	"id", "", "Show DWARF source lines information",
 	"idp", " [file.pdb]", "Load pdb file information",
 	"idpi", " [file.pdb]", "Show pdb file information",
+	"idpi*", "", "Show symbols from pdb as flags (prefix with dot to import)",
 	"idpd", "", "Download pdb file on remote server",
 	NULL
 };
@@ -142,24 +143,24 @@ static void r_core_file_info(RCore *core, int mode) {
 	int fd = r_io_fd_get_current (core->io);
 	RIODesc *desc = r_io_desc_get (core->io, fd);
 	RBinPlugin *plugin = r_bin_file_cur_plugin (binfile);
-	if (mode == R_CORE_BIN_JSON) {
+	if (mode == R_MODE_JSON) {
 		r_cons_printf ("{");
 	}
-	if (mode == R_CORE_BIN_RADARE) {
+	if (mode == R_MODE_RADARE) {
 		return;
 	}
-	if (mode == R_CORE_BIN_SIMPLE) {
+	if (mode == R_MODE_SIMPLE) {
 		return;
 	}
 	if (info) {
 		fn = info->file;
-		if (mode == R_CORE_BIN_JSON) {
+		if (mode == R_MODE_JSON) {
 			r_cons_printf ("\"type\":\"%s\",", STR (info->type));
 		}
 	} else {
 		fn = desc ? desc->name: NULL;
 	}
-	if (desc && mode == R_CORE_BIN_JSON) {
+	if (desc && mode == R_MODE_JSON) {
 		const char *uri = fn;
 		if (!uri) {
 			if (desc && desc->uri && *desc->uri) {
@@ -169,28 +170,24 @@ static void r_core_file_info(RCore *core, int mode) {
 			}
 		}
 		{
-			char *escapedFile = r_str_utf16_encode (uri, -1);
+			char *escapedFile = r_str_escape_utf8_for_json (uri, -1);
 			r_cons_printf ("\"file\":\"%s\"", escapedFile);
 			free (escapedFile);
 		}
 		if (dbg) {
-			dbg = R_IO_WRITE | R_IO_EXEC;
+			dbg = R_PERM_WX;
 		}
 		if (desc) {
 			ut64 fsz = r_io_desc_size (desc);
 			r_cons_printf (",\"fd\":%d", desc->fd);
 			if (fsz != UT64_MAX) {
+				char humansz[8];
 				r_cons_printf (",\"size\":%"PFMT64d, fsz);
-				char *humansz = r_num_units (NULL, fsz);
-				if (humansz) {
-					r_cons_printf (",\"humansz\":\"%s\"", humansz);
-					free (humansz);
-				}
+				r_num_units (humansz, sizeof (humansz), fsz);
+				r_cons_printf (",\"humansz\":\"%s\"", humansz);
 			}
-			r_cons_printf (",\"iorw\":%s", r_str_bool ( io_cache ||\
-					desc->flags & R_IO_WRITE ));
-			r_cons_printf (",\"mode\":\"%s\"", r_str_rwx_i (
-					desc->flags & 7 ));
+			r_cons_printf (",\"iorw\":%s", r_str_bool ( io_cache || desc->perm & R_PERM_W));
+			r_cons_printf (",\"mode\":\"%s\"", r_str_rwx_i (desc->perm & R_PERM_RWX));
 			r_cons_printf (",\"obsz\":%"PFMT64d, (ut64) core->io->desc->obsz);
 			if (desc->referer && *desc->referer) {
 				r_cons_printf (",\"referer\":\"%s\"", desc->referer);
@@ -208,10 +205,10 @@ static void r_core_file_info(RCore *core, int mode) {
 			}
 		}
 		r_cons_printf ("}");
-	} else if (desc && mode != R_CORE_BIN_SIMPLE) {
+	} else if (desc && mode != R_MODE_SIMPLE) {
 		//r_cons_printf ("# Core file info\n");
 		if (dbg) {
-			dbg = R_IO_WRITE | R_IO_EXEC;
+			dbg = R_PERM_WX;
 		}
 		if (desc) {
 			pair ("blksz", sdb_fmt ("0x%"PFMT64x, (ut64) core->io->desc->obsz));
@@ -227,8 +224,8 @@ static void r_core_file_info(RCore *core, int mode) {
 			pair ("format", plugin->name);
 		}
 		if (desc) {
-			pair ("iorw", r_str_bool (io_cache || desc->flags & R_IO_WRITE ));
-			pair ("mode", r_str_rwx_i (desc->flags & 7));
+			pair ("iorw", r_str_bool (io_cache || desc->perm & R_PERM_W));
+			pair ("mode", r_str_rwx_i (desc->perm & R_PERM_RWX));
 		}
 		if (binfile && binfile->curxtr) {
 			pair ("packet", binfile->curxtr->name);
@@ -239,12 +236,10 @@ static void r_core_file_info(RCore *core, int mode) {
 		if (desc) {
 			ut64 fsz = r_io_desc_size (desc);
 			if (fsz != UT64_MAX) {
+				char humansz[8];
 				pair ("size", sdb_itoca (fsz));
-				char *humansz = r_num_units (NULL, fsz);
-				if (humansz) {
-					pair ("humansz", humansz);
-					free (humansz);
-				}
+				r_num_units (humansz, sizeof (humansz), fsz);
+				pair ("humansz", humansz);
 			}
 		}
 		if (info) {
@@ -261,7 +256,7 @@ static int bin_is_executable(RBinObject *obj){
 			return true;
 		}
 		r_list_foreach (obj->sections, it, sec){
-			if (R_BIN_SCN_EXECUTABLE & sec->srwx) {
+			if (sec->perm & R_PERM_X) {
 				return true;
 			}
 		}
@@ -273,23 +268,23 @@ static void cmd_info_bin(RCore *core, int va, int mode) {
 	RBinObject *obj = r_bin_cur_object (core->bin);
 	int array = 0;
 	if (core->file) {
-		if ((mode & R_CORE_BIN_JSON) && !(mode & R_CORE_BIN_ARRAY)) {
-			mode = R_CORE_BIN_JSON;
+		if ((mode & R_MODE_JSON) && !(mode & R_MODE_ARRAY)) {
+			mode = R_MODE_JSON;
 			r_cons_strcat ("{\"core\":");
 		}
-		if ((mode & R_CORE_BIN_JSON) && (mode & R_CORE_BIN_ARRAY)) {
-			mode = R_CORE_BIN_JSON;
+		if ((mode & R_MODE_JSON) && (mode & R_MODE_ARRAY)) {
+			mode = R_MODE_JSON;
 			array = 1;
 			r_cons_strcat (",\"core\":");
 		}
 		r_core_file_info (core, mode);
 		if (bin_is_executable (obj)) {
-			if ((mode & R_CORE_BIN_JSON)) {
+			if ((mode & R_MODE_JSON)) {
 				r_cons_strcat (",\"bin\":");
 			}
 			r_core_bin_info (core, R_CORE_BIN_ACC_INFO, mode, va, NULL, NULL);
 		}
-		if (mode == R_CORE_BIN_JSON && array == 0) {
+		if (mode == R_MODE_JSON && array == 0) {
 			r_cons_strcat ("}\n");
 		}
 	} else {
@@ -317,7 +312,7 @@ static int cmd_info(void *data, const char *input) {
 	int fd = r_io_fd_get_current (core->io);
 	RIODesc *desc = r_io_desc_get (core->io, fd);
 	int i, va = core->io->va || core->io->debug;
-	int mode = 0; //R_CORE_BIN_SIMPLE;
+	int mode = 0; //R_MODE_SIMPLE;
 	bool rdump = false;
 	int is_array = 0;
 	Sdb *db;
@@ -326,14 +321,15 @@ static int cmd_info(void *data, const char *input) {
 		;
 	if (i > 0) {
 		switch (input[i - 1]) {
-		case '*': mode = R_CORE_BIN_RADARE; break;
-		case 'j': mode = R_CORE_BIN_JSON; break;
-		case 'q': mode = R_CORE_BIN_SIMPLE; break;
+		case '*': mode = R_MODE_RADARE; break;
+		case 'j': mode = R_MODE_JSON; break;
+		case 'q': mode = R_MODE_SIMPLE; break;
 		}
 	}
-	if (mode == R_CORE_BIN_JSON) {
+	if (mode == R_MODE_JSON) {
 		int suffix_shift = 0;
-		if (!strncmp (input, "SS", 2) || !strncmp (input, "ee", 2)) {
+		if (!strncmp (input, "SS", 2) || !strncmp (input, "ee", 2)
+			|| !strncmp (input, "zz", 2)) {
 			suffix_shift = 1;
 		}
 		if (strlen (input + 1 + suffix_shift) > 1) {
@@ -350,6 +346,12 @@ static int cmd_info(void *data, const char *input) {
 	if (!strcmp (input, "*")) {
 		input = "I*";
 	}
+	char *question = strchr (input, '?');
+	if (question > input) {
+		question--;
+		r_core_cmdf (core, "i?~ i%c", *question);
+		goto done;
+	}
 	while (*input) {
 		switch (*input) {
 		case 'b': // "ib"
@@ -443,11 +445,11 @@ static int cmd_info(void *data, const char *input) {
 			break;
 		case 'E': // "iE"
 		{
-			if (input[1] == 'j' && input[2] == '.') {	
-				mode = R_CORE_BIN_JSON;
-				RBININFO ("exports", R_CORE_BIN_ACC_EXPORTS, input + 2, 0); 
+			if (input[1] == 'j' && input[2] == '.') {
+				mode = R_MODE_JSON;
+				RBININFO ("exports", R_CORE_BIN_ACC_EXPORTS, input + 2, 0);
 			} else {
-				RBININFO ("exports", R_CORE_BIN_ACC_EXPORTS, input + 1, 0); 
+				RBININFO ("exports", R_CORE_BIN_ACC_EXPORTS, input + 1, 0);
 			}
 			while (*(++input)) ;
 			input--;
@@ -481,17 +483,21 @@ static int cmd_info(void *data, const char *input) {
 					action = R_CORE_BIN_ACC_SEGMENTS;
 					param_shift = 1;
 				}
-				// case for iSj.
-				if (input[1] == 'j' && input[2] == '.') {
-					mode = R_CORE_BIN_JSON;
+				// case for iS=
+				if (input[1] == '=') {
+					mode = R_MODE_EQUAL;
+				} else if (input[1] == 'q' && input[2] == '.') {
+					mode = R_MODE_SIMPLE;
+				} else if (input[1] == 'j' && input[2] == '.') {
+					mode = R_MODE_JSON;
 				}
 				RBinObject *obj = r_bin_cur_object (core->bin);
-				if (mode == R_CORE_BIN_RADARE || mode == R_CORE_BIN_JSON || mode == R_CORE_BIN_SIMPLE) {
+				if (mode == R_MODE_RADARE || mode == R_MODE_JSON || mode == R_MODE_SIMPLE) {
 					RBININFO (name, action, input + 2 + param_shift,
-						obj? r_list_length (obj->sections): 0);
+						(obj && obj->sections)? r_list_length (obj->sections): 0);
 				} else {
 					RBININFO (name, action, input + 1 + param_shift,
-						obj? r_list_length (obj->sections): 0);
+						(obj && obj->sections)? r_list_length (obj->sections): 0);
 				}
 			}
 			//we move input until get '\0'
@@ -510,7 +516,7 @@ static int cmd_info(void *data, const char *input) {
 			break;
 		case 'l': { // "il"
 			RBinObject *obj = r_bin_cur_object (core->bin);
-			RBININFO ("libs", R_CORE_BIN_ACC_LIBS, NULL, obj? r_list_length (obj->libs): 0);
+			RBININFO ("libs", R_CORE_BIN_ACC_LIBS, NULL, (obj && obj->libs)? r_list_length (obj->libs): 0);
 			break;
 		}
 		case 'L': { // "iL"
@@ -533,10 +539,10 @@ static int cmd_info(void *data, const char *input) {
 			RBinObject *obj = r_bin_cur_object (core->bin);
 			// Case for isj.
 			if (input[1] == 'j' && input[2] == '.') {
-				mode = R_CORE_BIN_JSON;
-				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 2, obj? r_list_length (obj->symbols): 0);
+				mode = R_MODE_JSON;
+				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 2, (obj && obj->symbols)? r_list_length (obj->symbols): 0);
 			} else {
-				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 1, obj? r_list_length (obj->symbols): 0);
+				RBININFO ("symbols", R_CORE_BIN_ACC_SYMBOLS, input + 1, (obj && obj->symbols)? r_list_length (obj->symbols): 0);
 			}
 			while (*(++input)) ;
 			input--;
@@ -544,9 +550,9 @@ static int cmd_info(void *data, const char *input) {
 		}
 		case 'R': // "iR"
 			if  (input[1] == '*') {
-				mode = R_CORE_BIN_RADARE;
+				mode = R_MODE_RADARE;
 			} else if (input[1] == 'j') {
-				mode = R_CORE_BIN_JSON;
+				mode = R_MODE_JSON;
 			}
 			RBININFO ("resources", R_CORE_BIN_ACC_RESOURCES, NULL, 0);
 			break;
@@ -584,7 +590,6 @@ static int cmd_info(void *data, const char *input) {
 					break;
 				case 'i':
 					info = r_bin_get_info (core->bin);
-					file_found = false;
 					filename = strchr (input, ' ');
 					while (input[2]) input++;
 					if (filename) {
@@ -598,9 +603,9 @@ static int cmd_info(void *data, const char *input) {
 							break;
 						}
 						// Check raw path for debug filename
-						file_found = r_file_exists (info->debug_file_name);
+						file_found = r_file_exists (r_file_basename (info->debug_file_name));
 						if (file_found) {
-							filename = strdup (info->debug_file_name);
+							filename = strdup (r_file_basename (info->debug_file_name));
 						} else {
 							// Check debug filename basename in current directory
 							char* basename = (char*) r_file_basename (info->debug_file_name);
@@ -619,8 +624,8 @@ static int cmd_info(void *data, const char *input) {
 						if (!file_found) {
 							const char* symstore_path = r_config_get (core->config, "pdb.symstore");
 							char* pdb_path = r_str_newf ("%s" R_SYS_DIR "%s" R_SYS_DIR "%s" R_SYS_DIR "%s",
-										     symstore_path, info->debug_file_name,
-										     info->guid, info->debug_file_name);
+										     symstore_path, r_file_basename (info->debug_file_name),
+										     info->guid, r_file_basename (info->debug_file_name));
 							file_found = r_file_exists (pdb_path);
 							if (file_found) {
 								filename = pdb_path;
@@ -631,7 +636,7 @@ static int cmd_info(void *data, const char *input) {
 					}
 
 					if (!file_found) {
-						eprintf ("File '%s' not found in file directory or symbol store", info->debug_file_name);
+						eprintf ("File '%s' not found in file directory or symbol store", r_file_basename (info->debug_file_name));
 						free (filename);
 						break;
 					}
@@ -661,7 +666,7 @@ static int cmd_info(void *data, const char *input) {
 		case 'i': { // "ii"
 			RBinObject *obj = r_bin_cur_object (core->bin);
 			RBININFO ("imports", R_CORE_BIN_ACC_IMPORTS, NULL,
-				obj? r_list_length (obj->imports): 0);
+				(obj && obj->imports)? r_list_length (obj->imports): 0);
 			break;
 		}
 		case 'I': // "iI"
@@ -714,21 +719,21 @@ static int cmd_info(void *data, const char *input) {
 					rdump = true;
 					break;
 				case '*':
-					mode = R_CORE_BIN_RADARE;
+					mode = R_MODE_RADARE;
 					break;
 				case 'j':
-					mode = R_CORE_BIN_JSON;
+					mode = R_MODE_JSON;
 					break;
 				case 'q': //izzq
 					if (input[3] == 'q') { //izzqq
-						mode = R_CORE_BIN_SIMPLEST;
+						mode = R_MODE_SIMPLEST;
 						input++;
 					} else {
-						mode = R_CORE_BIN_SIMPLE;
+						mode = R_MODE_SIMPLE;
 					}
 					break;
 				default:
-					mode = R_CORE_BIN_PRINT;
+					mode = R_MODE_PRINT;
 					break;
 				}
 				input++;
@@ -746,13 +751,13 @@ static int cmd_info(void *data, const char *input) {
 				RBinObject *obj = r_bin_cur_object (core->bin);
 				if (input[1] == 'q') {
 					mode = (input[2] == 'q')
-					? R_CORE_BIN_SIMPLEST
-					: R_CORE_BIN_SIMPLE;
+					? R_MODE_SIMPLEST
+					: R_MODE_SIMPLE;
 					input++;
 				}
 				if (obj) {
 					RBININFO ("strings", R_CORE_BIN_ACC_STRINGS, NULL,
-						obj? r_list_length (obj->strings): 0);
+						(obj && obj->strings)? r_list_length (obj->strings): 0);
 				}
 			}
 			break;
@@ -807,7 +812,7 @@ static int cmd_info(void *data, const char *input) {
 									const char *comma = iter2->p? ",": "";
 
 									if (sym->method_flags) {
-										char *flags = r_core_bin_method_flags_str (sym->method_flags, R_CORE_BIN_JSON);
+										char *flags = r_core_bin_method_flags_str (sym->method_flags, R_MODE_JSON);
 										r_cons_printf ("%s{\"name\":\"%s\",\"flags\":%s,\"vaddr\":%"PFMT64d "}",
 											comma, sym->name, flags, sym->vaddr);
 										R_FREE (flags);
@@ -844,7 +849,7 @@ static int cmd_info(void *data, const char *input) {
 								}
 							}
 						} else if (input[1] == 'c' && obj) { // "icc"
-                					mode = R_CORE_BIN_CLASSDUMP;
+                					mode = R_MODE_CLASSDUMP;
 							RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, r_list_length (obj->classes));
 							input = " ";
 						} else {
@@ -854,8 +859,10 @@ static int cmd_info(void *data, const char *input) {
         			}
 			} else {
 				RBinObject *obj = r_bin_cur_object (core->bin);
-				int len = obj? r_list_length (obj->classes): 0;
-				RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, len);
+				if (obj && obj->classes) {
+					int len = r_list_length (obj->classes);
+					RBININFO ("classes", R_CORE_BIN_ACC_CLASSES, NULL, len);
+				}
 			}
 			break;
 		case 'D': // "iD"
@@ -865,9 +872,9 @@ static int cmd_info(void *data, const char *input) {
 			return 0;
 		case 'a': // "ia"
 			switch (mode) {
-			case R_CORE_BIN_RADARE: cmd_info (core, "IieEcsSmz*"); break;
-			case R_CORE_BIN_JSON: cmd_info (core, "IieEcsSmzj"); break;
-			case R_CORE_BIN_SIMPLE: cmd_info (core, "IieEcsSmzq"); break;
+			case R_MODE_RADARE: cmd_info (core, "IieEcsSmz*"); break;
+			case R_MODE_JSON: cmd_info (core, "IieEcsSmzj"); break;
+			case R_MODE_SIMPLE: cmd_info (core, "IieEcsSmzq"); break;
 			default: cmd_info (core, "IiEecsSmz"); break;
 			}
 			break;
@@ -875,16 +882,16 @@ static int cmd_info(void *data, const char *input) {
 			r_core_cmd_help (core, help_msg_i);
 			goto redone;
 		case '*': // "i*"
-			mode = R_CORE_BIN_RADARE;
+			mode = R_MODE_RADARE;
 			goto done;
 		case 'q': // "iq"
-			mode = R_CORE_BIN_SIMPLE;
+			mode = R_MODE_SIMPLE;
 			cmd_info_bin (core, va, mode);
 			goto done;
 		case 'j': // "ij"
-			mode = R_CORE_BIN_JSON;
+			mode = R_MODE_JSON;
 			if (is_array > 1) {
-				mode |= R_CORE_BIN_ARRAY;
+				mode |= R_MODE_ARRAY;
 			}
 			cmd_info_bin (core, va, mode);
 			goto done;
