@@ -11,6 +11,7 @@ static const char *help_msg_L[] = {
 	"L",  "", "show this help",
 	"L", " blah."R_LIB_EXT, "load plugin file",
 	"L-", "duk", "unload core plugin by name",
+	"LL", "", "lock screen",
 	"La", "", "list asm/anal plugins (aL, e asm.arch=" "??" ")",
 	"Lc", "", "list core plugins",
 	"Ld", "", "list debug plugins (same as dL)",
@@ -43,6 +44,51 @@ static void cmd_log_init(RCore *core) {
 	DEFINE_CMD_DESCRIPTOR (core, T);
 }
 
+static void screenlock(RCore *core) {
+	//  char *pass = r_cons_input ("Enter new password: ");
+	char *pass = r_cons_password (Color_INVERT "Enter new password:"Color_INVERT_RESET);
+	if (!pass || !*pass) {
+		return;
+	}
+	char *again = r_cons_password (Color_INVERT "Type it again:"Color_INVERT_RESET);
+	if (!again || !*again) {
+		return;
+	}
+	if (strcmp (pass, again)) {
+		eprintf ("Password mismatch!\n");
+		return;
+	}
+	bool running = true;
+	r_cons_clear_buffer ();
+	ut64 begin = r_sys_now ();
+	ut64 last = UT64_MAX;
+	ut64 tries = 0;
+	do {
+		r_cons_clear00 ();
+		r_cons_printf ("Retries: %d\n", tries);
+		r_cons_printf ("Locked ts: %s\n", r_time_to_string (begin));
+		if (last != UT64_MAX) {
+			r_cons_printf ("Last try: %s\n", r_time_to_string (last));
+		}
+		r_cons_newline ();
+		r_cons_flush ();
+		char *msg = r_cons_password ("radare2 password: ");
+		if (msg && !strcmp (msg, pass)) {
+			running = false;
+		} else {
+			eprintf ("\nInvalid password.\n");
+			last = r_sys_now ();
+			tries++;
+		}
+		free (msg);
+		int n = r_num_rand (10) + 1;
+		r_sys_usleep (n * 100000);
+	} while (running);
+	r_cons_set_cup (true);
+	free (pass);
+	eprintf ("Unlocked!\n");
+}
+
 static int textlog_chat(RCore *core) {
 	char prompt[64];
 	char buf[1024];
@@ -172,10 +218,10 @@ static int cmd_plugins(void *data, const char *input) {
 		// return r_core_cmd0 (core, "Lc");
 		break;
 	case '-':
-		r_lib_close (core->lib, input + 2);
+		r_lib_close (core->lib, r_str_trim_ro (input + 1));
 		break;
 	case ' ':
-		r_lib_open (core->lib, input + 2);
+		r_lib_open (core->lib, r_str_trim_ro (input + 1));
 		break;
 	case '?':
 		r_core_cmd_help (core, help_msg_L);
@@ -189,6 +235,9 @@ static int cmd_plugins(void *data, const char *input) {
 	case 'a': // "La"
 		r_core_cmd0 (core, "e asm.arch=??");
 		break;
+	case 'L': // "LL"
+		screenlock (core);
+		break;
 	case 'o': // "Lo"
 	case 'i': // "Li"
 		r_core_cmdf (core, "%cL", input[0]);
