@@ -537,9 +537,8 @@ static bool cmd_wff(RCore *core, const char *input) {
 		}
 	}
 	if ((buf = (ut8*) r_file_slurp (a, &size))) {
-		int u_size = size;
 		int u_offset = 0;
-		u_size = r_num_math (core->num, p);
+		int u_size = r_num_math (core->num, p);
 		if (u_size < 1) u_size = size;
 		if (p) {
 			*p++ = 0;
@@ -879,7 +878,7 @@ static int cmd_write(void *data, const char *input) {
 				if (len > 0){
 					ut64 cur_off = core->offset;
 					cmd_suc = r_core_extend_at (core, addr, len);
-					cmd_suc = r_core_seek (core, cur_off, 1);
+					r_core_seek (core, cur_off, 1);
 					core->offset = addr;
 					r_core_block_read (core);
 				}
@@ -906,15 +905,16 @@ static int cmd_write(void *data, const char *input) {
 			break;
 		case 's': // "wes"
 			input +=  3;
-			while (*input && *input == ' ') input++;
+			while (*input && *input == ' ') {
+				input++;
+			}
 			len = strlen (input);
-			input_shadow = len > 0? malloc (len+1): 0;
 
 			// since the distance can be negative,
 			// the r_num_math will perform an unwanted operation
 			// the solution is to tokenize the string :/
-			if (input_shadow) {
-				strncpy (input_shadow, input, len+1);
+			if (len > 0) {
+				input_shadow = strdup (input);
 				p = strtok (input_shadow, " ");
 				addr = p && *p ? r_num_math (core->num, p) : 0;
 
@@ -1108,17 +1108,19 @@ static int cmd_write(void *data, const char *input) {
 				r_io_cache_commit (core->io, core->offset, core->offset+1);
 			}
 			break;
-		case '-': // "wc-"
+		case '-': { // "wc-"
 			if (input[2]=='*') { // "wc-*"
 				r_io_cache_reset (core->io, true);
-			} else if (input[2]==' ') { // "wc- "
+				break;
+			}
+			ut64 from, to;
+			if (input[2]==' ') { // "wc- "
 				char *p = strchr (input+3, ' ');
-				ut64 to, from;
 				if (p) {
 					*p = 0;
 					from = r_num_math (core->num, input+3);
-					to = r_num_math (core->num, input+3);
-					if (to<from) {
+					to = r_num_math (core->num, p+1);
+					if (to < from) {
 						eprintf ("Invalid range (from>to)\n");
 						return 0;
 					}
@@ -1126,15 +1128,16 @@ static int cmd_write(void *data, const char *input) {
 					from = r_num_math (core->num, input+3);
 					to = from + core->blocksize;
 				}
-				r_io_cache_invalidate (core->io, from, to);
 			} else {
 				eprintf ("Invalidate write cache at 0x%08"PFMT64x"\n", core->offset);
-				r_io_cache_invalidate (core->io, core->offset, core->offset+core->blocksize);
+				from = core->offset;
+				to = core->offset + core->blocksize;
 			}
-			/* See 'r' above. */
-			memset (core->block, 0xff, core->blocksize);
+			eprintf("invalidated %d cache(s)\n",
+				r_io_cache_invalidate (core->io, from, to));
 			r_core_block_read (core);
 			break;
+		}
 		case 'i': // "wci"
 			r_io_cache_commit (core->io, 0, UT64_MAX);
 			r_core_block_read (core);
@@ -1242,6 +1245,10 @@ static int cmd_write(void *data, const char *input) {
 					return 0;
 				}
 				if (*str == '!') {
+					if (str[1] == '?') {
+						r_core_cmd_help (core, help_msg_wt);
+						return 0;
+					}
 					RIOSection *s = r_io_section_vget (core->io, poff);
 					toend = true;
 					//use physical address
@@ -1249,6 +1256,10 @@ static int cmd_write(void *data, const char *input) {
 					str++;
 				}
 				if (*str == 'f') {
+					if (str[1] == '?') {
+						r_core_cmd_help (core, help_msg_wt);
+						return 0;
+					}
 					const char *prefix = r_str_trim_ro (str + 1);
 					if (!*prefix) {
 						prefix = "dump";
@@ -1256,7 +1267,11 @@ static int cmd_write(void *data, const char *input) {
 					filename = r_str_newf ("%s-0x%08"PFMT64x, prefix, core->offset);
 				} else {
 					if (*str) {
-						filename = str + ((*str == ' ')? 1: 0);
+						if (str[1] == '?') {
+							r_core_cmd_help (core, help_msg_wt);
+							return 0;
+						}
+						filename = r_str_trim_ro (str);
 					} else {
 						filename = "";
 					}
@@ -1334,7 +1349,9 @@ static int cmd_write(void *data, const char *input) {
 				else tmp[i] = str[i>>1];
 			}
 			str = tmp;
-			r_io_use_fd (core->io, core->file->fd);
+			if (core->file) {
+				r_io_use_fd (core->io, core->file->fd);
+			}
 			r_io_write_at (core->io, core->offset, (const ut8*)str, len);
 			WSEEK (core, len);
 			r_core_block_read (core);
@@ -1405,21 +1422,23 @@ static int cmd_write(void *data, const char *input) {
 		case 'o': // "wao"
 			if (input[2] == ' ') {
 				char *op = r_str_trim (strdup (input + 3));
-				r_core_hack (core, op);
-				free (op);
+				if (op) {
+					r_core_hack (core, op);
+					free (op);
+				}
 			} else {
 				r_core_hack_help (core);
 			}
 			break;
 		case ' ':
 		case '*': {
-			const char *file = input[1]=='*'? input+2: input+1;
+			const char *file = input[1]=='*'? input + 2: input + 1;
 			RAsmCode *acode;
 			r_asm_set_pc (core->assembler, core->offset);
 			acode = r_asm_massemble (core->assembler, file);
 			if (acode) {
-				if (input[1]=='*') {
-					cmd_write_hexpair(core, acode->buf_hex);
+				if (input[1] == '*') {
+					cmd_write_hexpair (core, acode->buf_hex);
 				} else {
 					if (!r_core_write_at (core, core->offset, acode->buf, acode->len)) {
 						cmd_write_fail ();
