@@ -40,20 +40,20 @@ R_API void r_core_task_print (RCore *core, RCoreTask *task, int mode) {
 	case 'j':
 		r_cons_printf ("{\"id\":%d,\"state\":\"", task->id);
 		switch (task->state) {
-			case R_CORE_TASK_STATE_BEFORE_START:
-				r_cons_print ("before_start");
-				break;
-			case R_CORE_TASK_STATE_RUNNING:
-				r_cons_print ("running");
-				break;
-			case R_CORE_TASK_STATE_SLEEPING:
-				r_cons_print ("sleeping");
-				break;
-			case R_CORE_TASK_STATE_DONE:
-				r_cons_print ("done");
-				break;
+		case R_CORE_TASK_STATE_BEFORE_START:
+			r_cons_print ("before_start");
+			break;
+		case R_CORE_TASK_STATE_RUNNING:
+			r_cons_print ("running");
+			break;
+		case R_CORE_TASK_STATE_SLEEPING:
+			r_cons_print ("sleeping");
+			break;
+		case R_CORE_TASK_STATE_DONE:
+			r_cons_print ("done");
+			break;
 		}
-		r_cons_print ("\",\"cmd\":");
+		r_cons_printf ("\",\"transient\":%s,\"cmd\":", task->transient ? "true" : "false");
 		if (task->cmd) {
 			r_cons_printf ("\"%s\"}", task->cmd);
 		} else {
@@ -65,8 +65,9 @@ R_API void r_core_task_print (RCore *core, RCoreTask *task, int mode) {
 		if (task == core->main_task) {
 			info = "-- MAIN TASK --";
 		}
-		r_cons_printf ("%2d  %12s  %s\n",
+		r_cons_printf ("%3d %3s %12s  %s\n",
 					   task->id,
+					   task->transient ? "(t)" : "",
 					   r_core_task_status (task),
 					   info ? info : "");
 		break;
@@ -96,21 +97,40 @@ R_API void r_core_task_list(RCore *core, int mode) {
 	tasks_lock_leave (core, &old_sigset);
 }
 
+R_API int r_core_task_running_tasks_count(RCore *core) {
+	RListIter *iter;
+	RCoreTask *task;
+	int count = 0;
+	TASK_SIGSET_T old_sigset;
+	tasks_lock_enter (core, &old_sigset);
+	r_list_foreach (core->tasks, iter, task) {
+		if (task != core->main_task && task->state != R_CORE_TASK_STATE_DONE) {
+			count++;
+		}
+	}
+	tasks_lock_leave (core, &old_sigset);
+	return count;
+}
+
 static void task_join(RCoreTask *task) {
 	RThreadSemaphore *sem = task->running_sem;
 	if (!sem) {
 		return;
 	}
+
 	r_th_sem_wait (sem);
 	r_th_sem_post (sem);
 }
 
-R_API void r_core_task_join(RCore *core, RCoreTask *current, RCoreTask *task) {
-	RListIter *iter;
-	if (current && task == current) {
+R_API void r_core_task_join(RCore *core, RCoreTask *current, int id) {
+	if (current && id == current->id) {
 		return;
 	}
-	if (task) {
+	if (id >= 0) {
+		RCoreTask *task = r_core_task_get_incref (core, id);
+		if (!task) {
+			return;
+		}
 		if (current) {
 			r_core_task_sleep_begin (current);
 		}
@@ -118,8 +138,22 @@ R_API void r_core_task_join(RCore *core, RCoreTask *current, RCoreTask *task) {
 		if (current) {
 			r_core_task_sleep_end (current);
 		}
+		r_core_task_decref (task);
 	} else {
-		r_list_foreach_prev (core->tasks, iter, task) {
+		TASK_SIGSET_T old_sigset;
+		tasks_lock_enter (core, &old_sigset);
+		RList *tasks = r_list_clone (core->tasks);
+		RListIter *iter;
+		RCoreTask *task;
+		r_list_foreach (tasks, iter, task) {
+			if (current == task) {
+				continue;
+			}
+			r_core_task_incref (task);
+		}
+		tasks_lock_leave (core, &old_sigset);
+
+		r_list_foreach (tasks, iter, task) {
 			if (current == task) {
 				continue;
 			}
@@ -130,15 +164,27 @@ R_API void r_core_task_join(RCore *core, RCoreTask *current, RCoreTask *task) {
 			if (current) {
 				r_core_task_sleep_end (current);
 			}
+			r_core_task_decref (task);
 		}
+		r_list_free (tasks);
 	}
 }
 
-R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd, RCoreTaskCallback cb, void *user) {
-	if (cmd && *cmd == '=') {
-		eprintf ("=* commands disabled in tasks\n");
-		return NULL;
+static void task_free (RCoreTask *task) {
+	if (!task) {
+		return;
 	}
+	free (task->cmd);
+	free (task->res);
+	r_th_free (task->thread);
+	r_th_sem_free (task->running_sem);
+	r_th_cond_free (task->dispatch_cond);
+	r_th_lock_free (task->dispatch_lock);
+	r_cons_context_free (task->cons_context);
+	free (task);
+}
+
+R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd, RCoreTaskCallback cb, void *user) {
 	RCoreTask *task = R_NEW0 (RCoreTask);
 	if (!task) {
 		goto hell;
@@ -164,6 +210,8 @@ R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd,
 
 	task->id = core->task_id_next++;
 	task->state = R_CORE_TASK_STATE_BEFORE_START;
+	task->refcount = 1;
+	task->transient = false;
 	task->core = core;
 	task->user = user;
 	task->cb = cb;
@@ -171,22 +219,32 @@ R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd,
 	return task;
 
 hell:
-	r_core_task_free (task);
+	task_free (task);
 	return NULL;
 }
 
-R_API void r_core_task_free (RCoreTask *task) {
+R_API void r_core_task_incref (RCoreTask *task) {
 	if (!task) {
 		return;
 	}
-	free (task->cmd);
-	free (task->res);
-	r_th_free (task->thread);
-	r_th_sem_free (task->running_sem);
-	r_th_cond_free (task->dispatch_cond);
-	r_th_lock_free (task->dispatch_lock);
-	r_cons_context_free (task->cons_context);
-	free (task);
+	TASK_SIGSET_T old_sigset;
+	tasks_lock_enter (task->core, &old_sigset);
+	task->refcount++;
+	tasks_lock_leave (task->core, &old_sigset);
+}
+
+R_API void r_core_task_decref (RCoreTask *task) {
+	if (!task) {
+		return;
+	}
+	TASK_SIGSET_T old_sigset;
+	RCore *core = task->core;
+	tasks_lock_enter (core, &old_sigset);
+	task->refcount--;
+	if (task->refcount <= 0) {
+		task_free (task);
+	}
+	tasks_lock_leave (core, &old_sigset);
 }
 
 R_API void r_core_task_schedule(RCoreTask *current, RTaskState next_state) {
@@ -293,30 +351,35 @@ static void task_end(RCoreTask *t) {
 	r_core_task_schedule (t, R_CORE_TASK_STATE_DONE);
 }
 
-static int task_run(RCoreTask *task) {
+static RThreadFunctionRet task_run(RCoreTask *task) {
 	RCore *core = task->core;
 
 	task_wakeup (task);
 
+	if (task->cons_context && task->cons_context->breaked) {
+		// breaked in R_CORE_TASK_STATE_BEFORE_START
+		goto stillbirth;
+	}
+
 	char *res_str;
-	int res;
 	if (task == task->core->main_task) {
-		res = r_core_cmd (core, task->cmd, task->cmd_log);
+		r_core_cmd (core, task->cmd, task->cmd_log);
 		res_str = NULL;
 	} else {
-		res = 0;
 		res_str = r_core_cmd_str (core, task->cmd);
 	}
 
-	if (task->res) {
-		free (task->res);
-	}
+	free (task->res);
 	task->res = res_str;
 
 	if (task != core->main_task) {
 		eprintf ("\nTask %d finished\n", task->id);
 	}
 
+	TASK_SIGSET_T old_sigset;
+stillbirth:
+	tasks_lock_enter (core, &old_sigset);
+
 	task_end (task);
 
 	if (task->cb) {
@@ -327,18 +390,32 @@ static int task_run(RCoreTask *task) {
 		r_th_sem_post (task->running_sem);
 	}
 
-	return res;
-}
-
-static int task_run_thread(RThread *th) {
-	RCoreTask *task = (RCoreTask *)th->user;
-	int ret = task_run (task);
 	if (task->cons_context && task->cons_context->break_stack) {
 		r_cons_context_break_pop (task->cons_context, false);
 	}
+
+	int ret = R_TH_STOP;
+	if (task->transient) {
+		RCoreTask *ltask;
+		RListIter *iter;
+		r_list_foreach (core->tasks, iter, ltask) {
+			if (ltask == task) {
+				r_list_delete (core->tasks, iter);
+				ret = R_TH_FREED;
+				break;
+			}
+		}
+	}
+
+	tasks_lock_leave (core, &old_sigset);
 	return ret;
 }
 
+static RThreadFunctionRet task_run_thread(RThread *th) {
+	RCoreTask *task = (RCoreTask *)th->user;
+	return task_run (task);
+}
+
 R_API void r_core_task_enqueue(RCore *core, RCoreTask *task) {
 	if (!core || !task) {
 		return;
@@ -435,12 +512,34 @@ R_API RCoreTask *r_core_task_self (RCore *core) {
 	return core->current_task ? core->current_task : core->main_task;
 }
 
+static RCoreTask *task_get (RCore *core, int id) {
+	RCoreTask *task;
+	RListIter *iter;
+	r_list_foreach (core->tasks, iter, task) {
+		if (task->id == id) {
+			return task;
+		}
+	}
+	return NULL;
+}
+
+R_API RCoreTask *r_core_task_get_incref(RCore *core, int id) {
+	TASK_SIGSET_T old_sigset;
+	tasks_lock_enter (core, &old_sigset);
+	RCoreTask *task = task_get (core, id);
+	if (task) {
+		r_core_task_incref (task);
+	}
+	tasks_lock_leave (core, &old_sigset);
+	return task;
+}
+
 R_API void r_core_task_break(RCore *core, int id) {
 	TASK_SIGSET_T old_sigset;
 	tasks_lock_enter (core, &old_sigset);
-	RCoreTask *task = r_core_task_get (core, id);
+	RCoreTask *task = task_get (core, id);
 	if (!task || task->state == R_CORE_TASK_STATE_DONE) {
-		r_th_lock_leave (core->tasks_lock);
+		tasks_lock_leave (core, &old_sigset);
 		return;
 	}
 	if (task->cons_context) {
@@ -449,6 +548,19 @@ R_API void r_core_task_break(RCore *core, int id) {
 	tasks_lock_leave (core, &old_sigset);
 }
 
+R_API void r_core_task_break_all(RCore *core) {
+	TASK_SIGSET_T old_sigset;
+	tasks_lock_enter (core, &old_sigset);
+	RCoreTask *task;
+	RListIter *iter;
+	r_list_foreach (core->tasks, iter, task) {
+		if (task->state != R_CORE_TASK_STATE_DONE) {
+			r_cons_context_break (task->cons_context);
+		}
+	}
+	tasks_lock_leave (core, &old_sigset);
+}
+
 R_API int r_core_task_del (RCore *core, int id) {
 	RCoreTask *task;
 	RListIter *iter;
@@ -457,11 +569,14 @@ R_API int r_core_task_del (RCore *core, int id) {
 	tasks_lock_enter (core, &old_sigset);
 	r_list_foreach (core->tasks, iter, task) {
 		if (task->id == id) {
-			if (task == core->main_task
-				|| task->state != R_CORE_TASK_STATE_DONE) {
+			if (task == core->main_task) {
 				break;
 			}
-			r_list_delete (core->tasks, iter);
+			if (task->state == R_CORE_TASK_STATE_DONE) {
+				r_list_delete (core->tasks, iter);
+			} else {
+				task->transient = true;
+			}
 			ret = true;
 			break;
 		}
@@ -479,14 +594,3 @@ R_API void r_core_task_del_all_done (RCore *core) {
 		}
 	}
 }
-
-R_API RCoreTask *r_core_task_get (RCore *core, int id) {
-	RCoreTask *task;
-	RListIter *iter;
-	r_list_foreach (core->tasks, iter, task) {
-		if (task->id == id) {
-			return task;
-		}
-	}
-	return NULL;
-}
