@@ -34,8 +34,7 @@ static char *prompt(const char *str, const char *txt) {
 		free (r_cons_singleton ()->line->contents);
 		r_cons_singleton ()->line->contents = strdup (txt);
 	} else {
-		free (r_cons_singleton ()->line->contents);
-		r_cons_singleton ()->line->contents = NULL;
+		R_FREE (r_cons_singleton ()->line->contents);
 	}
 	*cmd = '\0';
 	r_line_set_prompt (str);
@@ -48,8 +47,7 @@ static char *prompt(const char *str, const char *txt) {
 	}
 	r_line_set_prompt (oprompt);
 	free (oprompt);
-	free (r_cons_singleton ()->line->contents);
-	r_cons_singleton ()->line->contents = NULL;
+	R_FREE (r_cons_singleton ()->line->contents);
 	return res;
 }
 
@@ -152,12 +150,12 @@ R_API bool r_core_visual_esil(RCore *core) {
 		r_cons_printf ("r2's esil debugger:\n\n");
 		r_cons_printf ("pos: %d\n", x);
 		{
-			char *res = r_print_hexpair (core->print, asmop.buf_hex, -1);
+			char *res = r_print_hexpair (core->print, r_asm_op_get_hex (&asmop), -1);
 			r_cons_printf ("hex: %s\n"Color_RESET, res);
 			free (res);
 		}
 		{
-			char *op = colorize_asm_string (core, asmop.buf_asm, analopType, core->offset);
+			char *op = colorize_asm_string (core, r_asm_op_get_asm (&asmop), analopType, core->offset);
 			r_cons_printf (Color_RESET"asm: %s\n"Color_RESET, op);
 			free (op);
 		}
@@ -280,7 +278,11 @@ static bool edit_bits (RCore *core) {
 	if (core->blocksize < sizeof (ut64)) {
 		return false;
 	}
-	memcpy (buf, core->block, sizeof (ut64));
+	int cur = 0;
+	if (core->print->cur != -1) {
+		cur = core->print->cur;
+	}
+	memcpy (buf, core->block + cur, sizeof (ut64));
 	for (;;) {
 		r_cons_clear00 ();
 		bool use_color = core->print->flags & R_PRINT_FLAGS_COLOR;
@@ -289,8 +291,9 @@ static bool edit_bits (RCore *core) {
 		(void)r_anal_op (core->anal, &analop, core->offset, buf, sizeof (ut64), R_ANAL_OP_MASK_ESIL);
 		analopType = analop.type & R_ANAL_OP_TYPE_MASK;
 		r_cons_printf ("r2's bit editor:\n\n");
+		r_cons_printf ("offset: 0x%08"PFMT64x"\n"Color_RESET, core->offset + cur);
 		{
-			char *res = r_print_hexpair (core->print, asmop.buf_hex, -1);
+			char *res = r_print_hexpair (core->print, r_asm_op_get_hex (&asmop), -1);
 			r_cons_printf ("hex: %s\n"Color_RESET, res);
 			free (res);
 		}
@@ -300,7 +303,7 @@ static bool edit_bits (RCore *core) {
 			r_cons_printf ("shift: >> %d << %d\n", word, (asmop.size * 8) - word - 1);
 		}
 		{
-			char *op = colorize_asm_string (core, asmop.buf_asm, analopType, core->offset);
+			char *op = colorize_asm_string (core, r_asm_op_get_asm (&asmop), analopType, core->offset);
 			r_cons_printf (Color_RESET"asm: %s\n"Color_RESET, op);
 			free (op);
 		}
@@ -381,13 +384,29 @@ static bool edit_bits (RCore *core) {
 		if (ch == -1 || ch == 4) {
 			break;
 		}
-		ch = r_cons_arrow_to_hjkl (ch); // get ESC+char, return 'hjkl' char
+		if (ch != 10) {
+			ch = r_cons_arrow_to_hjkl (ch); // get ESC+char, return 'hjkl' char
+		}
 		switch (ch) {
 		case 'Q':
 		case 'q':
 			return false;
+		case 'H':
+			{
+				int y = R_MAX (x - 8, 0);
+				x = y - y%8;
+			}
+			break;
+		case 'L':
+		case 9:
+			{
+				int y = R_MIN (x + 8, nbits - 8);
+				x = y - y%8;
+			}
+			break;
 		case 'j':
 		case 'k':
+		case 10:
 		case ' ':
 			//togglebit();
 			{
@@ -494,12 +513,13 @@ static int sdbforcb (void *p, const char *k, const char *v) {
 					vt->curfmt = strdup (v);
 					pre = ">";
 				}
-				if (use_color && *pre=='>')
+				if (use_color && *pre == '>') {
 					r_cons_printf ("%s %s %s  %s\n", color_sel,
 						Color_RESET, pre, k+strlen (s), v);
-				else
+				} else {
 					r_cons_printf (" %s %s  %s\n",
-						pre, k+strlen (s), v);
+						pre, k + strlen (s), v);
+				}
 				vt->t_ctr ++;
 			}
 			free (s);
@@ -871,7 +891,9 @@ static bool r_core_visual_config_hud(RCore *core) {
 	RListIter *iter;
 	RConfigNode *bt;
 	RList *list = r_list_new ();
-	if (!list) return false;
+	if (!list) {
+		return false;
+	}
 	char *res;
 	list->free = free;
 	r_list_foreach (core->config->nodes, iter, bt) {
@@ -882,15 +904,18 @@ static bool r_core_visual_config_hud(RCore *core) {
 		const char *oldvalue = NULL;
 		char cmd[512];
 		char *p = strchr (res, ' ');
-		if (p) *p = 0;
+		if (p) {
+			*p = 0;
+		}
 		oldvalue = r_config_get (core->config, res);
 		r_cons_show_cursor (true);
 		r_cons_set_raw (0);
 		cmd[0] = '\0';
 		eprintf ("set new value for %s (old=%s)\n", res, oldvalue);
 		r_line_set_prompt (":> ");
-		if (r_cons_fgets (cmd, sizeof (cmd) - 1, 0, NULL) < 0)
-			cmd[0]='\0';
+		if (r_cons_fgets (cmd, sizeof (cmd) - 1, 0, NULL) < 0) {
+			cmd[0] = '\0';
+		}
 		r_config_set (core->config, res, cmd);
 		r_cons_set_raw (1);
 		r_cons_show_cursor (false);
@@ -1246,7 +1271,9 @@ R_API int r_core_visual_trackflags(RCore *core) {
 				case 3: strcpy (cmd, "f="); break;
 				default: format = 0; continue;
 				}
-				if (*cmd) r_core_cmd (core, cmd, 0);
+				if (*cmd) {
+					r_core_cmd (core, cmd, 0);
+				}
 			} else {
 				r_cons_printf ("(no flags)\n");
 			}
@@ -1303,8 +1330,16 @@ R_API int r_core_visual_trackflags(RCore *core) {
 		case 'o': r_flag_sort (core->flags, 0); break;
 		case 'n': r_flag_sort (core->flags, 1); break;
 		case 'j': option++; break;
-		case 'k': if (--option<0) option = 0; break;
-		case 'K': option-=10; if (option<0) option = 0; break;
+		case 'k':
+			if (--option < 0) {
+				option = 0;
+			}
+			break;
+		case 'K': option-=10;
+			if (option < 0) {
+				option = 0;
+			}
+			break;
 		case 'h':
 		case 'b': // back
 		case 'Q':
@@ -1366,9 +1401,11 @@ R_API int r_core_visual_trackflags(RCore *core) {
 			r_core_block_size (core, core->blocksize-16);
 			break;
 		case '+':
-			if (menu==1)
+			if (menu == 1) {
 				r_core_cmdf (core, "f %s=%s+1", fs2, fs2);
-			else r_core_block_size (core, core->blocksize+1);
+			} else {
+				r_core_block_size (core, core->blocksize + 1);
+			}
 			break;
 		case '-':
 			if (menu == 1) {
@@ -1403,16 +1440,22 @@ R_API int r_core_visual_trackflags(RCore *core) {
 				eprintf ("Rename function '%s' as:\n", fs2);
 				r_line_set_prompt (":> ");
 				if (r_cons_fgets (line, sizeof (line), 0, NULL) < 0) {
-					cmd[0]='\0';
+					cmd[0] = '\0';
+				}
+				int res = snprintf (cmd, sizeof (cmd), "afr %s %s", line, fs2);
+				if (res < sizeof (cmd)) {
+					r_core_cmd (core, cmd, 0);
 				}
-				snprintf (cmd, sizeof (cmd), "afr %s %s", line, fs2);
-				r_core_cmd (core, cmd, 0);
 				r_cons_set_raw (1);
 				r_cons_show_cursor (false);
 			}
 			break;
-		case 'P': if (--format<0) format = MAX_FORMAT; break;
-// = (format<=0)? MAX_FORMAT: format-1; break;
+		case 'P':
+			if (--format < 0) {
+				format = MAX_FORMAT;
+			}
+			break;
+			// = (format<=0)? MAX_FORMAT: format-1; break;
 		case 'p': format++; break;
 		case 'l':
 		case ' ':
@@ -1505,23 +1548,21 @@ R_API int r_core_visual_comments (RCore *core) {
 	char *str;
 	char cmd[512], *p = NULL;
 	int ch, option = 0;
-	int format = 0, found = 0;
+	int format = 0, i = 0;
 	ut64 addr, from = 0, size = 0;
 
 	for (;;) {
 		r_cons_clear00 ();
 		r_cons_strcat ("Comments:\n");
-		found = 0;
 		RList *items = r_list_newf (free);
 		RAnalMetaItem *item;
 		RListIter *iter;
 		r_meta_list_cb (core->anal, R_META_TYPE_COMMENT, 0, meta_enumerate_cb, items, UT64_MAX);
-		int i = 0;
+		i = 0;
 		r_list_foreach (items, iter, item) {
 			str = item->str;
 			addr = item->from;
 			if (option==i) {
-				found = 1;
 				from = addr;
 				size = 1; // XXX: remove this thing size for comments is useless d->size;
 				free (p);
@@ -1532,9 +1573,8 @@ R_API int r_core_visual_comments (RCore *core) {
 				free (str);
 			}
 			i ++;
-			found = true;
 		}
-		if (!found) {
+		if (!i) {
 			if (--option < 0) {
 				r_cons_any_key ("No comments");
 				break;
@@ -1686,19 +1726,22 @@ R_API void r_core_visual_config(RCore *core) {
 			r_cons_printf ("[EvalSpace]\n\n");
 			hit = j = i = 0;
 			r_list_foreach (core->config->nodes, iter, bt) {
-				if (option==i) {
+				if (option == i) {
 					fs = bt->name;
 				}
-				if (old[0]=='\0') {
+				if (!old[0]) {
 					r_str_ccpy (old, bt->name, '.');
 					show = 1;
 				} else if (r_str_ccmp (old, bt->name, '.')) {
 					r_str_ccpy (old, bt->name, '.');
 					show = 1;
-				} else show = 0;
-
+				} else {
+					show = 0;
+				}
 				if (show) {
-					if (option == i) hit = 1;
+					if (option == i) {
+						hit = 1;
+					}
 					if ( (i >=option-delta) && ((i<option+delta)||((option<delta)&&(i<(delta<<1))))) {
 						r_cons_printf (" %c  %s\n", (option == i)?'>':' ', old);
 						j++;
@@ -1706,7 +1749,7 @@ R_API void r_core_visual_config(RCore *core) {
 					i++;
 				}
 			}
-			if (!hit && j>0) {
+			if (!hit && j > 0) {
 				option--;
 				continue;
 			}
@@ -1736,10 +1779,11 @@ R_API void r_core_visual_config(RCore *core) {
 				option = i-1;
 				continue;
 			}
-			if (fs2 != NULL)
+			if (fs2 != NULL) {
 				// TODO: Break long lines.
 				r_cons_printf ("\n Selected: %s (%s)\n\n",
-						fs2, desc);
+					fs2, desc);
+			}
 		}
 
 		if (fs && !strncmp (fs, "asm.", 4)) {
@@ -1747,8 +1791,9 @@ R_API void r_core_visual_config(RCore *core) {
 		}
 		r_cons_visual_flush ();
 		ch = r_cons_readchar ();
-		if (ch==4 || ch==-1)
+		if (ch == 4 || ch == -1) {
 			return;
+		}
 		ch = r_cons_arrow_to_hjkl (ch); // get ESC+char, return 'hjkl' char
 
 		switch (ch) {
@@ -1857,9 +1902,11 @@ R_API void r_core_visual_mounts(RCore *core) {
 			if (list) {
 				r_list_foreach (list, iter, part) {
 					if ((option-delta <= i) && (i <= option+delta)) {
-						if (option == i)
+						if (option == i) {
 							r_cons_printf (" > ");
-						else r_cons_printf ("   ");
+						} else {
+							r_cons_printf ("   ");
+						}
 						r_cons_printf ("%d %02x 0x%010"PFMT64x" 0x%010"PFMT64x"\n",
 								part->number, part->type,
 								part->start, part->start+part->length);
@@ -1868,12 +1915,16 @@ R_API void r_core_visual_mounts(RCore *core) {
 				}
 				r_list_free (list);
 				list = NULL;
-			} else r_cons_printf ("Cannot read partition\n");
+			} else {
+				r_cons_printf ("Cannot read partition\n");
+			}
 		} else if (mode == 1) {
 			r_cons_printf ("Types:\n\n");
 			for (i=0;;i++) {
 				n = r_fs_partition_type_get (i);
-				if (!n) break;
+				if (!n) {
+					break;
+				}
 				r_cons_printf ("%s%s\n", (i==partition)?" > ":"   ", n);
 			}
 		} else if (mode == 3) {
@@ -1902,8 +1953,12 @@ R_API void r_core_visual_mounts(RCore *core) {
 					r_cons_printf ("\n");
 					r_list_free (list);
 					list = NULL;
-				} else r_cons_printf ("Cannot open '%s' directory\n", root);
-			} else r_cons_printf ("Root undefined\n");
+				} else {
+					r_cons_printf ("Cannot open '%s' directory\n", root);
+				}
+			} else {
+				r_cons_printf ("Root undefined\n");
+			}
 		}
 		if (mode==2) {
 			r_str_trim_path (path);
@@ -1911,8 +1966,9 @@ R_API void r_core_visual_mounts(RCore *core) {
 			strncat (path, "/", sizeof (path)-strlen (path)-1);
 			list = r_fs_dir (core->fs, path);
 			file = r_list_get_n (list, dir);
-			if (file && file->type != 'd')
-				r_core_cmdf (core, "px @ 0x%"PFMT64x"!64", file->off);
+			if (file && file->type != 'd') {
+				r_core_cmdf (core, "px @ 0x%" PFMT64x "!64", file->off);
+			}
 			r_list_free (list);
 			list = NULL;
 			*str='\0';
@@ -1981,8 +2037,9 @@ R_API void r_core_visual_mounts(RCore *core) {
 						if (file->type == 'd') {
 							strncat (path, file->name, sizeof (path)-strlen (path)-1);
 							r_str_trim_path (path);
-							if (root && strncmp (root, path, strlen (root)-1))
-								strncpy (path, root, sizeof (path)-1);
+							if (root && strncmp (root, path, strlen (root) - 1)) {
+								strncpy (path, root, sizeof (path) - 1);
+							}
 						} else {
 							r_core_cmdf (core, "s 0x%"PFMT64x, file->off);
 							r_fs_umount (core->fs, root);
@@ -2007,32 +2064,39 @@ R_API void r_core_visual_mounts(RCore *core) {
 				break;
 			case 'k':
 				if (mode == 0 || mode == 3) {
-					if (option > 0)
+					if (option > 0) {
 						option--;
+					}
 				} else if (mode == 1) {
-					if (partition > 0)
+					if (partition > 0) {
 						partition--;
+					}
 				} else {
-					if (dir>0)
+					if (dir > 0) {
 						dir--;
+					}
 				}
 				break;
 			case 'j':
 				if (mode == 0) {
 					n = r_fs_partition_type_get (partition);
 					list = r_fs_partitions (core->fs, n, 0);
-					if (option < r_list_length (list)-1)
+					if (option < r_list_length (list) - 1) {
 						option++;
+					}
 				} else if (mode == 1) {
-					if (partition < r_fs_partition_get_size ()-1)
+					if (partition < r_fs_partition_get_size () - 1) {
 						partition++;
+					}
 				} else if (mode == 3) {
-					if (option < r_list_length (core->fs->roots)-1)
+					if (option < r_list_length (core->fs->roots) - 1) {
 						option++;
+					}
 				} else {
 					list = r_fs_dir (core->fs, path);
-					if (dir < r_list_length (list)-1)
+					if (dir < r_list_length (list) - 1) {
 						dir++;
+					}
 				}
 				break;
 			case 't':
@@ -2075,8 +2139,9 @@ R_API void r_core_visual_mounts(RCore *core) {
 					if (file && root) {
 						strncat (path, file->name, sizeof (path)-strlen (path)-1);
 						r_str_trim_path (path);
-						if (strncmp (root, path, strlen (root)-1))
-							strncpy (path, root, sizeof (path)-1);
+						if (strncmp (root, path, strlen (root) - 1)) {
+							strncpy (path, root, sizeof (path) - 1);
+						}
 						file = r_fs_open (core->fs, path);
 						if (file) {
 							r_fs_read (core->fs, file, 0, file->size);
@@ -2089,8 +2154,12 @@ R_API void r_core_visual_mounts(RCore *core) {
 							r_file_dump (buf, file->data, file->size, 0);
 							r_fs_close (core->fs, file);
 							r_cons_printf ("Done\n");
-						} else r_cons_printf ("Cannot dump file\n");
-					} else r_cons_printf ("Cannot dump file\n");
+						} else {
+							r_cons_printf ("Cannot dump file\n");
+						}
+					} else {
+						r_cons_printf ("Cannot dump file\n");
+					}
 					r_cons_flush ();
 					r_cons_any_key (NULL);
 					*str='\0';
@@ -2204,7 +2273,7 @@ static void variable_rename (RCore *core, ut64 addr, int vindex, const char *nam
 	r_list_foreach (list, iter, var) {
 		if (i == vindex) {
 			r_core_seek (core, addr, false);
-			r_core_cmd_strf (core, "afvn %s %s", var->name, name);
+			r_core_cmd_strf (core, "afvn %s %s", name, var->name);
 			r_core_seek (core, a_tmp, false);
 			break;
 		}
@@ -2289,10 +2358,10 @@ static ut64 var_variables_show(RCore* core, int idx, int *vindex, int show) {
 			if (show) {
 				r_cons_printf ("%s%s %s %s @ %s%s0x%x\n",
 						i == *vindex ? "* ":"  ",
-						var->kind=='v'?"var":"arg",
+						var->kind == 'v'?"var":"arg",
 						var->type, var->name,
 						core->anal->reg->name[R_REG_NAME_BP],
-						(var->kind=='v')?"-":"+",
+						(var->kind == 'v')?"-":"+",
 						var->delta);
 			}
 		}
@@ -2303,7 +2372,6 @@ static ut64 var_variables_show(RCore* core, int idx, int *vindex, int show) {
 }
 
 static int level = 0;
-static ut64 addr = 0;
 static st64 delta = 0;
 static int option = 0;
 static int variable_option = 0;
@@ -2343,7 +2411,6 @@ static void r_core_visual_anal_refresh_column (RCore *core, int colpos) {
 }
 
 static ut64 r_core_visual_anal_refresh (RCore *core) {
-	eprintf("r_core_visual_anal_refresh\n");
 	if (!core) {
 		return 0LL;
 	}
@@ -2399,12 +2466,8 @@ static ut64 r_core_visual_anal_refresh (RCore *core) {
 		sprintf (old, "afi @ 0x%08"PFMT64x, addr);
 		r_core_cmd0 (core, old);
 		break;
-	case 3:
-		r_cons_printf ("Press 'q' to view call refs\n");
-		r_cons_printf ("-[ xrefs ]----------------------- 0x%08"PFMT64x"\n", addr);
-		//sprintf (old, "axl~0x%08"PFMT64x, addr);
-		r_core_cmd0 (core, "pd 1");
-		//r_core_cmd0 (core, old);
+	default:
+		// assert
 		break;
 	}
 	r_cons_flush ();
@@ -2416,26 +2479,31 @@ static void r_core_visual_anal_refresh_oneshot (RCore *core) {
 }
 
 /* Like emenu but for real */
-R_API void r_core_visual_anal(RCore *core) {
+R_API void r_core_visual_anal(RCore *core, const char *input) {
 	char old[218];
 	int nfcns, ch, _option = 0;
 
 	RConsEvent olde = core->cons->event_resize;
 	void *olde_user = core->cons->event_data;
+	ut64 addr = core->offset;
 
 	core->cons->event_resize = NULL; // avoid running old event with new data
 	core->cons->event_data = core;
 	core->cons->event_resize = (RConsEvent) r_core_visual_anal_refresh_oneshot;
 
 	level = 0;
-	addr = core->offset;
 
 	int asmbytes = r_config_get_i (core->config, "asm.bytes");
 	r_config_set_i (core->config, "asm.bytes", 0);
 	for (;;) {
 		nfcns = r_list_length (core->anal->fcns);
 		addr = r_core_visual_anal_refresh (core);
-		ch = r_cons_readchar ();
+		if (input && *input) {
+			ch = *input;
+			input++;
+		} else {
+			ch = r_cons_readchar ();
+		}
 		if (ch == 4 || ch == -1) {
 			if (level == 0) {
 				goto beach;
@@ -2458,7 +2526,7 @@ R_API void r_core_visual_anal(RCore *core) {
 				" h,q     go back, quit\n"
 				" p,P     switch next/prev print mode\n"
 				" v       view selected function arguments and variables\n"
-				" x       see xrefs to the selected function\n"
+				" x,X     see xrefs to the selected function\n"
 				" tab     toggle disasm column selection (to scroll in code)\n"
 				" !       run 'afls' to sort all functions by address\n"
 				" .       seek to current function address\n"
@@ -2562,9 +2630,19 @@ R_API void r_core_visual_anal(RCore *core) {
 				break;
 			}
 			break;
-		case 'x': level = 3; break;
-		case 'c': level = 2; break;
-		case 'v': level = 1; variable_option = 0; break;
+		case 'x':
+			r_core_visual_refs (core, false, true);
+			break;
+		case 'X':
+			r_core_visual_refs (core, true, true);
+			break;
+		case 'c':
+			level = 2;
+			break;
+		case 'v':
+			level = 1;
+			variable_option = 0;
+			break;
 		case '_':
 			r_core_cmd0 (core, "s $(afl~...)");
 			break;
@@ -2580,7 +2658,9 @@ R_API void r_core_visual_anal(RCore *core) {
 					break;
 				default:
 					option++;
-					if (option >= nfcns) --option;
+					if (option >= nfcns) {
+						--option;
+					}
 					break;
 				}
 			}
@@ -2716,27 +2796,32 @@ R_API void r_core_seek_previous (RCore *core, const char *type) {
 	if (strstr (type, "fun")) {
 		RAnalFunction *fcni;
 		r_list_foreach (core->anal->fcns, iter, fcni) {
-			if (fcni->addr > next && fcni->addr < core->offset)
+			if (fcni->addr > next && fcni->addr < core->offset) {
 				next = fcni->addr;
+			}
 		}
 	} else
 	if (strstr (type, "hit")) {
 		RFlagItem *flag;
 		const char *pfx = r_config_get (core->config, "search.prefix");
 		r_list_foreach (core->flags->flags, iter, flag) {
-			if (!strncmp (flag->name, pfx, strlen (pfx)))
-				if (flag->offset > next && flag->offset< core->offset)
+			if (!strncmp (flag->name, pfx, strlen (pfx))) {
+				if (flag->offset > next && flag->offset < core->offset) {
 					next = flag->offset;
+				}
+			}
 		}
 	} else { // flags
 		RFlagItem *flag;
 		r_list_foreach (core->flags->flags, iter, flag) {
-			if (flag->offset > next && flag->offset < core->offset)
+			if (flag->offset > next && flag->offset < core->offset) {
 				next = flag->offset;
+			}
 		}
 	}
-	if (next!=0)
+	if (next != 0) {
 		r_core_seek (core, next, 1);
+	}
 }
 
 //define the data at offset according to the type (byte, word...) n times
@@ -2784,7 +2869,7 @@ static void handleHints(RCore *core) {
 	}
 }
 
-R_API void r_core_visual_define (RCore *core, const char *args) {
+R_API void r_core_visual_define(RCore *core, const char *args, int distance) {
 	int plen = core->blocksize;
 	ut64 off = core->offset;
 	int i, h = 0, n, ch, ntotal = 0;
@@ -2823,7 +2908,8 @@ R_API void r_core_visual_define (RCore *core, const char *args) {
 		," e    end of function"
 		," f    analyze function"
 		," F    format"
-		," i    immediate base (b(in), o(ct), d(ec), h(ex), s(tr))"
+		," i    (ahi) immediate base (b(in), o(ct), d(ec), h(ex), s(tr))"
+		," I    (ahi1) immediate base (b(in), o(ct), d(ec), h(ex), s(tr))"
 		," j    merge down (join this and next functions)"
 		," k    merge up (join this and previous function)"
 		," h    define anal hint"
@@ -2846,6 +2932,7 @@ R_API void r_core_visual_define (RCore *core, const char *args) {
 		r_cons_printf ("\r%s\n", lines[i]);
 	}
 	r_cons_flush ();
+	int wordsize = 0;
 	// get ESC+char, return 'hjkl' char
 repeat:
 	if (*args) {
@@ -2855,6 +2942,8 @@ repeat:
 		ch = r_cons_arrow_to_hjkl (r_cons_readchar ());
 	}
 
+onemoretime:
+	wordsize = 4;
 	switch (ch) {
 	case 'F':
 		{
@@ -2864,8 +2953,8 @@ repeat:
 			r_cons_flush ();
 			r_line_set_prompt ("format: ");
 			strcpy (cmd, "Cf 0 ");
-			if (r_cons_fgets (cmd+5, sizeof (cmd)-6, 0, NULL) > 0) {
-				r_core_cmd (core, cmd, 0);
+			if (r_cons_fgets (cmd + 5, sizeof (cmd) - 6, 0, NULL) > 0) {
+				r_core_cmdf (core, "%s @ 0x%08"PFMT64x, cmd, off);
 				r_cons_set_raw (1);
 				r_cons_show_cursor (false);
 			}
@@ -2887,29 +2976,43 @@ repeat:
 			}
 		}
 		break;
+	case 'I':
+		{
+			char str[128];
+			r_cons_show_cursor (true);
+			r_line_set_prompt ("immbase: ");
+			if (r_cons_fgets (str, sizeof (str), 0, NULL) > 0) {
+				r_core_cmdf (core, "ahi1 %s @ 0x%"PFMT64x, str, off);
+			}
+		}
+		break;
 	case 'b':
 		if (plen != core->blocksize) {
 			rep = plen / 2;
 		}
 		define_data_ntimes (core, off, rep, R_BYTE_DATA);
+		wordsize = 1;
 		break;
 	case 'B':
 		if (plen != core->blocksize) {
 			rep = plen;
 		}
 		define_data_ntimes (core, off, rep, R_WORD_DATA);
+		wordsize = 2;
 		break;
 	case 'w':
 		if (plen != core->blocksize) {
 			rep = plen / 4;
 		}
 		define_data_ntimes (core, off, rep, R_DWORD_DATA);
+		wordsize = 4;
 		break;
 	case 'W':
 		if (plen != core->blocksize) {
 			rep = plen / 8;
 		}
 		define_data_ntimes (core, off, rep, R_QWORD_DATA);
+		wordsize = 8;
 		break;
 	case 'm':
 		{
@@ -2921,15 +3024,18 @@ repeat:
 					RFlagItem *item = r_flag_get_i (core->flags, op->jump);
 					if (item) {
 						const char *ptr = r_str_lchr (item->name, '.');
-						if (ptr)
-							man = strdup (ptr+1);
+						if (ptr) {
+							man = strdup (ptr + 1);
+						}
 					}
 				}
 				r_anal_op_free (op);
 			}
 			if (man) {
 				char *p = strstr (man, "INODE");
-				if (p) *p = 0;
+				if (p) {
+					*p = 0;
+				}
 				r_cons_clear ();
 				r_cons_flush ();
 				r_sys_cmdf ("man %s", man);
@@ -2952,7 +3058,7 @@ repeat:
 
 		tgt_addr = op.jump != UT64_MAX ? op.jump : op.ptr;
 		if (op.var) {
-//			q = r_str_newf ("?i Rename variable %s to;afvn %s `?y`", op.var->name, op.var->name);
+//			q = r_str_newf ("?i Rename variable %s to;afvn %s `yp`", op.var->name, op.var->name);
 			RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, off, 0);
 			if (fcn) {
 				RAnalVar *bar = r_anal_var_get_byname (core->anal, fcn->addr, op.var->name);
@@ -2975,13 +3081,13 @@ repeat:
 			RAnalFunction *fcn = r_anal_get_fcn_at (core->anal, tgt_addr, R_ANAL_FCN_TYPE_NULL);
 			RFlagItem *f = r_flag_get_i (core->flags, tgt_addr);
 			if (fcn) {
-				q = r_str_newf ("?i Rename function %s to;afn `?y` 0x%"PFMT64x,
+				q = r_str_newf ("?i Rename function %s to;afn `yp` 0x%"PFMT64x,
 					fcn->name, tgt_addr);
 			} else if (f) {
-				q = r_str_newf ("?i Rename flag %s to;fr %s `?y`",
+				q = r_str_newf ("?i Rename flag %s to;fr %s `yp`",
 					f->name, f->name);
 			} else {
-				q = r_str_newf ("?i Create flag at 0x%"PFMT64x" named;f `?y` @ 0x%"PFMT64x,
+				q = r_str_newf ("?i Create flag at 0x%"PFMT64x" named;f `yp` @ 0x%"PFMT64x,
 					tgt_addr, tgt_addr);
 			}
 		}
@@ -3059,13 +3165,13 @@ repeat:
 		break;
 	case 'h': // "Vdh"
 		handleHints (core);
-		//r_core_cmdf (core, "?i highlight;e scr.highlight=`?y` @ 0x%08"PFMT64x, off);
+		//r_core_cmdf (core, "?i highlight;e scr.highlight=`yp` @ 0x%08"PFMT64x, off);
 		break;
 	case 'r': // "Vdr"
-		r_core_cmdf (core, "?i new function name;afn `?y` @ 0x%08"PFMT64x, off);
+		r_core_cmdf (core, "?i new function name;afn `yp` @ 0x%08"PFMT64x, off);
 		break;
 	case 'z': // "Vdz"
-		r_core_cmdf (core, "?i zone name;fz `?y` @ 0x%08"PFMT64x, off);
+		r_core_cmdf (core, "?i zone name;fz `yp` @ 0x%08"PFMT64x, off);
 		break;
 	case 'R': // "VdR"
 		eprintf ("Finding references to 0x%08"PFMT64x" ...\n", off);
@@ -3078,7 +3184,9 @@ repeat:
 		do {
 			n = r_str_nlen_w ((const char *)p + ntotal,
 					plen - ntotal) + 1;
-			if (n < 2) break;
+			if (n < 2) {
+				break;
+			}
 			name = malloc (n + 10);
 			strcpy (name, "str.");
 			for (i = 0, j = 0; i < n; i++, j++) {
@@ -3116,6 +3224,7 @@ repeat:
 				ntotal += n;
 			}
 		} while (ntotal < plen);
+		wordsize = ntotal;
 		}
 		break;
 	case 's':
@@ -3159,6 +3268,7 @@ repeat:
 		}
 		r_name_filter (name, n + 10);
 		r_flag_set (core->flags, name, off, n);
+		wordsize = n;
 		free (name);
 		}
 		break;
@@ -3167,8 +3277,8 @@ repeat:
 		r_meta_add (core->anal, R_META_TYPE_DATA, off, off+plen, "");
 		break;
 	case 'c': // TODO: check
-		r_meta_cleanup (core->anal, off, off+plen);
-		r_meta_add (core->anal, R_META_TYPE_CODE, off, off+plen, "");
+		r_meta_cleanup (core->anal, off, off + plen);
+		r_meta_add (core->anal, R_META_TYPE_CODE, off, off + plen, "");
 		break;
 	case 'u':
 		r_core_anal_undefine (core, off);
@@ -3269,7 +3379,7 @@ repeat:
 	case 'Q':
 	case 'q':
 	default:
-		if (IS_DIGIT(ch)) {
+		if (IS_DIGIT (ch)) {
 			if (rep < 0) {
 				rep = 0;
 			}
@@ -3278,53 +3388,97 @@ repeat:
 		}
 		break;
 	}
+	if (distance > 0) {
+		distance--;
+		off += wordsize;
+		goto onemoretime;
+	}
 }
 
 R_API void r_core_visual_colors(RCore *core) {
-	char color[32], cstr[32];
+	char *color = calloc (1, 64), cstr[32];
+	char preview_cmd[128] = "pd $r";
 	int ch, opt = 0, oopt = -1;
+	bool truecolor = r_cons_singleton ()->color == COLOR_MODE_16M;
+	char *rgb_xxx_fmt = truecolor ? "rgb:%2.2x%2.2x%2.2x ":"rgb:%x%x%x ";
 	const char *k;
 	RColor rcolor;
 
+	r_cons_show_cursor (false);
 	rcolor = r_cons_pal_get_i (opt);
 	for (;;) {
 		r_cons_clear ();
+		r_cons_gotoxy (0, 0);
 		k = r_cons_pal_get_name (opt);
 		if (!k) {
 			opt = 0;
 			k = r_cons_pal_get_name (opt);
 		}
-		r_cons_gotoxy (0, 0);
-		r_cons_rgb_str (cstr, sizeof (cstr), &rcolor);
-		if (r_cons_singleton ()->color < COLOR_MODE_16M) {
+		if (!truecolor) {
 			rcolor.r &= 0xf;
 			rcolor.g &= 0xf;
 			rcolor.b &= 0xf;
+			rcolor.r2 &= 0xf;
+			rcolor.g2 &= 0xf;
+			rcolor.b2 &= 0xf;
+		} else {
+			rcolor.r &= 0xff;
+			rcolor.g &= 0xff;
+			rcolor.b &= 0xff;
+			rcolor.r2 &= 0xff;
+			rcolor.g2 &= 0xff;
+			rcolor.b2 &= 0xff;
 		}
-		sprintf (color, "rgb:%x%x%x", rcolor.r, rcolor.g, rcolor.b);
-		r_cons_printf ("# Colorscheme %d - Use '.' to randomize current color and ':' to randomize palette\n"
-			"# Press 'rRgGbB', 'jk' or 'q'\nec %s %s   # %d (\\x1b%s)\n",
-			opt, k, color, atoi (cstr+7), cstr+1);
+		sprintf (color, rgb_xxx_fmt, rcolor.r, rcolor.g, rcolor.b);
+		if (rcolor.r2 || rcolor.g2 || rcolor.b2) {
+			color = r_str_appendf (color, rgb_xxx_fmt, rcolor.r2, rcolor.g2, rcolor.b2);
+			rcolor.a = ALPHA_FGBG;
+		} else {
+			rcolor.a = ALPHA_FG;
+		}
+		r_cons_rgb_str (cstr, sizeof (cstr), &rcolor);
+		char *esc = strchr (cstr + 1, '\x1b');
+		char *curtheme = r_core_get_theme ();
+
+		r_cons_printf ("# Use '.' to randomize current color and ':' to randomize palette\n");
+		r_cons_printf ("# Press '"Color_RED"rR"Color_GREEN"gG"Color_BLUE"bB"Color_RESET
+			"' or '"Color_BGRED"eE"Color_BGGREEN"fF"Color_BGBLUE"vV"Color_RESET
+			"' to change foreground/background color\n");
+		r_cons_printf ("# Export colorscheme with command 'ec* > filename'\n");
+		r_cons_printf ("# Preview command: '%s' - Press 'c' to change it\n", preview_cmd);
+		r_cons_printf ("# Selected colorscheme : %s  - Use 'hl' or left/right arrow keys to change colorscheme\n", curtheme ? curtheme : "default");
+		r_cons_printf ("# Selected element: %s  - Use 'jk' or up/down arrow keys to change element\n", k);
+		r_cons_printf ("# ec %s %s # %d (\\x1b%.*s)",
+			k, color, atoi (cstr+7), esc ? esc - cstr - 1 : strlen (cstr + 1), cstr+1);
+		if (esc) {
+			r_cons_printf (" (\\x1b%s)", esc + 1);
+		}
+		r_cons_newline ();
+
 		r_core_cmdf (core, "ec %s %s", k, color);
-		char * res = r_core_cmd_str (core, "pd $r");
+		char * res = r_core_cmd_str (core, preview_cmd);
 		int h, w = r_cons_get_size (&h);
-		char *body = r_str_ansi_crop (res, 0, 0, w, h - 4);
+		char *body = r_str_ansi_crop (res, 0, 0, w, h - 8);
 		if (body) {
-			r_cons_printf ("%s", body);
+			r_cons_printf ("\n%s", body);
 		}
 		r_cons_flush ();
 		ch = r_cons_readchar ();
 		ch = r_cons_arrow_to_hjkl (ch);
 		switch (ch) {
 #define CASE_RGB(x,X,y) \
-	case x:if (y > 0x00) { y--; } break;\
-	case X:if (y < 0xff) { y++; } break;
+	case x:if ((y) > 0x00) { (y)--; } break;\
+	case X:if ((y) < 0xff) { (y)++; } break;
 		CASE_RGB ('R','r',rcolor.r);
 		CASE_RGB ('G','g',rcolor.g);
 		CASE_RGB ('B','b',rcolor.b);
+		CASE_RGB ('E','e',rcolor.r2);
+		CASE_RGB ('F','f',rcolor.g2);
+		CASE_RGB ('V','v',rcolor.b2);
 		case 'Q':
 		case 'q':
 			free (body);
+			free (color);
 			return;
 		case 'k':
 			opt--;
@@ -3332,6 +3486,14 @@ R_API void r_core_visual_colors(RCore *core) {
 		case 'j':
 			opt++;
 			break;
+		case 'l':
+			r_core_cmd0 (core, "ecn");
+			oopt = -1;
+			break;
+		case 'h':
+			r_core_cmd0 (core, "ecp");
+			oopt = -1;
+			break;
 		case 'K':
 			opt = 0;
 			break;
@@ -3346,6 +3508,11 @@ R_API void r_core_visual_colors(RCore *core) {
 			rcolor.g = r_num_rand (0xff);
 			rcolor.b = r_num_rand (0xff);
 			break;
+		case 'c':
+			r_line_set_prompt ("Preview command> ");
+			r_cons_show_cursor (true);
+			r_cons_fgets (preview_cmd, sizeof preview_cmd, 0, NULL);
+			r_cons_show_cursor (false);
 		}
 		if (opt != oopt) {
 			rcolor = r_cons_pal_get_i (opt);
