@@ -52,7 +52,7 @@ RCryptoPlugin r_crypto_plugin_base91 = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_CRYPTO,
 	.data = &r_crypto_plugin_base91,
 	.version = R2_VERSION
