@@ -46,7 +46,7 @@ RCryptoPlugin r_crypto_plugin_punycode = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_CRYPTO,
 	.data = &r_crypto_plugin_punycode,
 	.version = R2_VERSION
