@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake, defragger */
+/* radare - LGPL - Copyright 2009-2018 - pancake, defragger */
 
 #include <r_asm.h>
 #include <r_debug.h>
@@ -9,7 +9,7 @@ typedef struct {
 	libgdbr_t desc;
 } RIOGdb;
 
-#define UNKNOWN -1
+#define UNKNOWN (-1)
 #define UNSUPPORTED 0
 #define SUPPORTED 1
 
@@ -28,6 +28,9 @@ static void check_connection (RDebug *dbg) {
 
 static int r_debug_gdb_step(RDebug *dbg) {
 	check_connection (dbg);
+	if (!desc) {
+		return R_DEBUG_REASON_UNKNOWN;
+	}
 	gdbr_step (desc, -1); // TODO handle thread specific step?
 	return true;
 }
@@ -44,6 +47,9 @@ static int r_debug_gdb_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	int copy_size;
 	int buflen = 0;
 	check_connection (dbg);
+	if (!desc) {
+		return R_DEBUG_REASON_UNKNOWN;
+	}
 	gdbr_read_registers (desc);
 	if (!desc || !desc->data) {
 		return -1;
@@ -92,7 +98,7 @@ static int r_debug_gdb_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 
 static RList *r_debug_gdb_map_get(RDebug* dbg) { //TODO
 	check_connection (dbg);
-	if (desc->pid <= 0) {
+	if (!desc || desc->pid <= 0) {
 		return NULL;
 	}
 	RList *retlist = NULL;
@@ -104,8 +110,7 @@ static RList *r_debug_gdb_map_get(RDebug* dbg) { //TODO
 				return NULL;
 			}
 			RDebugMap *map;
-			if (!(map = r_debug_map_new ("", baddr, baddr,
-						     R_IO_READ | R_IO_EXEC, 0))) {
+			if (!(map = r_debug_map_new ("", baddr, baddr, R_PERM_RX, 0))) {
 				r_list_free (retlist);
 				return NULL;
 			}
@@ -196,9 +201,9 @@ static RList *r_debug_gdb_map_get(RDebug* dbg) { //TODO
 		perm = 0;
 		for (i = 0; perms[i] && i < 5; i++) {
 			switch (perms[i]) {
-			case 'r': perm |= R_IO_READ; break;
-			case 'w': perm |= R_IO_WRITE; break;
-			case 'x': perm |= R_IO_EXEC; break;
+			case 'r': perm |= R_PERM_R; break;
+			case 'w': perm |= R_PERM_W; break;
+			case 'x': perm |= R_PERM_X; break;
 			case 'p': map_is_shared = false; break;
 			case 's': map_is_shared = true; break;
 			}
@@ -265,6 +270,9 @@ static RList* r_debug_gdb_modules_get(RDebug *dbg) {
 
 static int r_debug_gdb_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 	check_connection (dbg);
+	if (!desc) {
+		return R_DEBUG_REASON_UNKNOWN;
+	}
 	if (!reg_buf) {
 		// we cannot write registers before we once read them
 		return -1;
@@ -274,8 +282,9 @@ static int r_debug_gdb_reg_write(RDebug *dbg, int type, const ut8 *buf, int size
 	const char *pcname = r_reg_get_name (dbg->anal->reg, R_REG_NAME_PC);
 	RRegItem *reg = r_reg_get (dbg->anal->reg, pcname, 0);
 	if (reg) {
-		if (dbg->anal->bits != reg->size)
+		if (dbg->anal->bits != reg->size) {
 			bits = reg->size;
+		}
 	}
 	free (r_reg_get_bytes (dbg->reg, type, &buflen));
 	// some implementations of the gdb protocol are acting weird.
@@ -296,7 +305,9 @@ static int r_debug_gdb_reg_write(RDebug *dbg, int type, const ut8 *buf, int size
 	RRegItem* current = NULL;
 	for (;;) {
 		current = r_reg_next_diff (dbg->reg, type, reg_buf, buflen, current, bits);
-		if (!current) break;
+		if (!current) {
+			break;
+		}
 		ut64 val = r_reg_get_value (dbg->reg, current);
 		int bytes = bits / 8;
 		gdbr_write_reg (desc, current->name, (char*)&val, bytes);
@@ -306,6 +317,9 @@ static int r_debug_gdb_reg_write(RDebug *dbg, int type, const ut8 *buf, int size
 
 static int r_debug_gdb_continue(RDebug *dbg, int pid, int tid, int sig) {
 	check_connection (dbg);
+	if (!desc) {
+		return R_DEBUG_REASON_UNKNOWN;
+	}
 	gdbr_continue (desc, pid, -1, sig); // Continue all threads
 	if (desc->stop_reason.is_valid && desc->stop_reason.thread.present) {
 		//if (desc->tid != desc->stop_reason.thread.tid) {
@@ -318,6 +332,9 @@ static int r_debug_gdb_continue(RDebug *dbg, int pid, int tid, int sig) {
 
 static RDebugReasonType r_debug_gdb_wait(RDebug *dbg, int pid) {
 	check_connection (dbg);
+	if (!desc) {
+		return R_DEBUG_REASON_UNKNOWN;
+	}
 	if (!desc->stop_reason.is_valid) {
 		if (gdbr_stop_reason (desc) < 0) {
 			dbg->reason.type = R_DEBUG_REASON_UNKNOWN;
@@ -972,44 +989,46 @@ static const char *r_debug_gdb_reg_profile(RDebug *dbg) {
 }
 
 static int r_debug_gdb_breakpoint (RBreakpoint *bp, RBreakpointItem *b, bool set) {
-	int ret, bpsize;
+	int ret = 0, bpsize;
 	if (!b) {
 		return false;
 	}
 	bpsize = b->size;
         // TODO handle conditions
-	switch (b->rwx){
+	switch (b->perm) {
 	case R_BP_PROT_EXEC : {
-		if (set)
+		if (set) {
 			ret = b->hw?
 					gdbr_set_hwbp (desc, b->addr, "", bpsize):
 					gdbr_set_bp (desc, b->addr, "", bpsize);
-		else
-			ret = b->hw?
-					gdbr_remove_hwbp (desc, b->addr, bpsize):
-					gdbr_remove_bp (desc, b->addr, bpsize);
+		} else {
+			ret = b->hw ? gdbr_remove_hwbp (desc, b->addr, bpsize) : gdbr_remove_bp (desc, b->addr, bpsize);
+		}
 		break;
 	}
 	// TODO handle size (area of watch in upper layer and then bpsize. For the moment watches are set on exact on byte
-	case R_BP_PROT_WRITE : {
-		if (set)
+	case R_PERM_W: {
+		if (set) {
 			gdbr_set_hww (desc, b->addr, "", 1);
-		else
-			gdbr_remove_hwbp (desc, b->addr, 1);
+		} else {
+			gdbr_remove_hww (desc, b->addr, 1);
+		}
 		break;
 	}
-	case R_BP_PROT_READ : {
-		if (set)
+	case R_PERM_R: {
+		if (set) {
 			gdbr_set_hwr (desc, b->addr, "", 1);
-		else
+		} else {
 			gdbr_remove_hwr (desc, b->addr, 1);
+		}
 		break;
 	}
-	case R_BP_PROT_ACCESS : {
-		if (set)
+	case R_PERM_ACCESS: {
+		if (set) {
 			gdbr_set_hwa (desc, b->addr, "", 1);
-		else
-			gdbr_remove_hwa (desc, b->addr, 1);            
+		} else {
+			gdbr_remove_hwa (desc, b->addr, 1);
+		}
 		break;
 	}
 	}
@@ -1103,7 +1122,7 @@ RDebugPlugin r_debug_plugin_gdb = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_gdb,
 	.version = R2_VERSION
