@@ -11,7 +11,7 @@ RDebugPlugin r_debug_plugin_null = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_null,
 	.version = R2_VERSION
