@@ -7,7 +7,9 @@
 /* HACK_FOR_PLUGIN_LINKAGE */
 R_API RDebugPid *__r_debug_pid_new(const char *path, int pid, char status, ut64 pc) {
 	RDebugPid *p = R_NEW0 (RDebugPid);
-	if (!p) return NULL;
+	if (!p) {
+		return NULL;
+	}
 	p->path = strdup (path);
 	p->pid = pid;
 	p->status = status;
@@ -47,7 +49,9 @@ static RList *r_debug_qnx_tids (RDebug *dbg, int pid) {
 
 static RList *r_debug_qnx_pids (RDebug *dbg, int pid) {
 	RList *list = r_list_new ();
-	if (!list) return NULL;
+	if (!list) {
+		return NULL;
+	}
 	list->free = (RListFree)&__r_debug_pid_free;
 
 	/* TODO */
@@ -67,7 +71,9 @@ static int r_debug_qnx_reg_read (RDebug *dbg, int type, ut8 *buf, int size) {
 		return -1;
 	}
 	int len = qnxr_read_registers (desc);
-	if (len <= 0) return -1;
+	if (len <= 0) {
+		return -1;
+	}
 	// read the len of the current area
 	free (r_reg_get_bytes (dbg->reg, type, &buflen));
 	if (size < len) {
@@ -79,16 +85,18 @@ static int r_debug_qnx_reg_read (RDebug *dbg, int type, ut8 *buf, int size) {
 	if (reg_buf) {
 		if (buf_size < copy_size) {
 			ut8 *new_buf = realloc (reg_buf, copy_size);
-			if (!new_buf)
+			if (!new_buf) {
 				return -1;
+			}
 			reg_buf = new_buf;
 			buflen = copy_size;
 			buf_size = len;
 		}
 	} else {
 		reg_buf = calloc (buflen, 1);
-		if (!reg_buf)
+		if (!reg_buf) {
 			return -1;
+		}
 		buf_size = buflen;
 	}
 	memset ((void *)(volatile void *) buf, 0, size);
@@ -113,8 +121,9 @@ static int r_debug_qnx_reg_write (RDebug *dbg, int type, const ut8 *buf, int siz
 		return -1;
 	}
 	if (reg) {
-		if (dbg->anal->bits != reg->size)
+		if (dbg->anal->bits != reg->size) {
 			bits = reg->size;
+		}
 	}
 	free (r_reg_get_bytes (dbg->reg, type, &buflen));
 	// some implementations of the gdb protocol are acting weird.
@@ -135,7 +144,9 @@ static int r_debug_qnx_reg_write (RDebug *dbg, int type, const ut8 *buf, int siz
 	RRegItem *current = NULL;
 	for (;;) {
 		current = r_reg_next_diff (dbg->reg, type, reg_buf, buflen, current, bits);
-		if (!current) break;
+		if (!current) {
+			break;
+		}
 		ut64 val = r_reg_get_value (dbg->reg, current);
 		int bytes = bits / 8;
 		qnxr_write_reg (desc, current->name, (char *)&val, bytes);
@@ -176,7 +187,7 @@ static int r_debug_qnx_attach (RDebug *dbg, int pid) {
 			RIOQnx *g = d->data;
 			int arch = r_sys_arch_id (dbg->arch);
 			int bits = dbg->anal->bits;
-			if ((desc = &g->desc))
+			if ((desc = &g->desc)) {
 				switch (arch) {
 				case R_SYS_ARCH_X86:
 					if (bits == 16 || bits == 32) {
@@ -195,8 +206,10 @@ static int r_debug_qnx_attach (RDebug *dbg, int pid) {
 					}
 					break;
 				}
-			if (pid)
+			}
+			if (pid) {
 				qnxr_attach (desc, pid);
+			}
 		} else {
 			eprintf ("%s: error: underlying IO descriptor isn't a QNX one\n", __func__);
 			return false;
@@ -246,7 +259,7 @@ static const char *r_debug_qnx_reg_profile (RDebug *dbg) {
 #endif
 			);
 	case R_SYS_ARCH_ARM:
-		if (bits == 32)
+		if (bits == 32) {
 			return strdup (
 				"=PC	r15\n"
 				"=SP	r14\n" // XXX
@@ -307,7 +320,8 @@ static const char *r_debug_qnx_reg_profile (RDebug *dbg) {
 				"mmx	d30	.64	308	0\n" // neon
 				"mmx	d31	.64	316	0\n" // neon
 				"mmx	fpscr	.32	324	0\n" // neon
-				);
+			);
+		}
 	}
 	return NULL;
 }
@@ -349,7 +363,7 @@ RDebugPlugin r_debug_plugin_qnx = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_DBG,
 	.data = &r_debug_plugin_qnx,
 	.version = R2_VERSION};
