@@ -8,7 +8,9 @@
 
 R_API RDebugTrace *r_debug_trace_new () {
 	RDebugTrace *t = R_NEW0 (RDebugTrace);
-	if (!t) return NULL;
+	if (!t) {
+		return NULL;
+	}
 	t->tag = 1; // UT32_MAX;
 	t->addresses = NULL;
 	t->enabled = false;
@@ -27,12 +29,13 @@ R_API RDebugTrace *r_debug_trace_new () {
 }
 
 R_API void r_debug_trace_free (RDebugTrace *trace) {
-	if (!trace) return;
+	if (!trace) {
+		return;
+	}
 	r_list_purge (trace->traces);
 	free (trace->traces);
 	sdb_free (trace->db);
-	free (trace);
-	trace = NULL;
+	R_FREE (trace);
 }
 
 // TODO: added overlap/mask support here... wtf?
@@ -54,7 +57,7 @@ R_API int r_debug_trace_pc(RDebug *dbg, ut64 pc) {
 		return false;
 	}
 	(void)dbg->iob.read_at (dbg->iob.io, pc, buf, sizeof (buf));
-	if (r_anal_op (dbg->anal, &op, pc, buf, sizeof (buf)) < 1) {
+	if (r_anal_op (dbg->anal, &op, pc, buf, sizeof (buf), R_ANAL_OP_MASK_ESIL) < 1) {
 		eprintf ("trace_pc: cannot get opcode size at 0x%"PFMT64x"\n", pc);
 		return false;
 	}
@@ -81,7 +84,7 @@ R_API RDebugTracepoint *r_debug_trace_get (RDebug *dbg, ut64 addr) {
 	RDebugTracepoint *trace;
 #if R_DEBUG_SDB_TRACES
 	trace = (RDebugTracepoint*)(void*)(size_t)sdb_num_get (db,
-		sdb_fmt (0, "trace.%d.%"PFMT64x, tag, addr), NULL);
+		sdb_fmt ("trace.%d.%"PFMT64x, tag, addr), NULL);
 	return trace;
 #else
 	RListIter *iter;
@@ -120,8 +123,9 @@ static int r_debug_trace_is_traceable(RDebug *dbg, ut64 addr) {
 	if (dbg->trace->addresses) {
 		char addr_str[32];
 		snprintf (addr_str, sizeof (addr_str), "0x%08"PFMT64x, addr);
-		if (!strstr (dbg->trace->addresses, addr_str))
+		if (!strstr (dbg->trace->addresses, addr_str)) {
 			return false;
+		}
 	}
 	return true;
 }
@@ -129,13 +133,16 @@ static int r_debug_trace_is_traceable(RDebug *dbg, ut64 addr) {
 R_API RDebugTracepoint *r_debug_trace_add (RDebug *dbg, ut64 addr, int size) {
 	RDebugTracepoint *tp;
 	int tag = dbg->trace->tag;
-	if (!r_debug_trace_is_traceable (dbg, addr))
+	if (!r_debug_trace_is_traceable (dbg, addr)) {
 		return NULL;
+	}
 	r_anal_trace_bb (dbg->anal, addr);
 	tp = r_debug_trace_get (dbg, addr);
 	if (!tp) {
 		tp = R_NEW0 (RDebugTracepoint);
-		if (!tp) return NULL;
+		if (!tp) {
+			return NULL;
+		}
 		tp->stamp = r_sys_now ();
 		tp->addr = addr;
 		tp->tags = tag;
@@ -144,10 +151,12 @@ R_API RDebugTracepoint *r_debug_trace_add (RDebug *dbg, ut64 addr, int size) {
 		tp->times = 1;
 		r_list_append (dbg->trace->traces, tp);
 #if R_DEBUG_SDB_TRACES
-		sdb_num_set (dbg->trace->db, sdb_fmt (0, "trace.%d.%"PFMT64x, tag, addr),
+		sdb_num_set (dbg->trace->db, sdb_fmt ("trace.%d.%"PFMT64x, tag, addr),
 			(ut64)(size_t)tp, 0);
 #endif
-	} else tp->times++;
+	} else {
+		tp->times++;
+	}
 	return tp;
 }
 
