@@ -34,14 +34,18 @@ static char *regs[] = { "eax", "ebx", "ecx", "edx", "esi", "edi", "ebp" };
 
 static void emit_init (REgg *egg) {
 // TODO: add 'andb rsp, 0xf0'
-	if (attsyntax) r_egg_printf (egg, "mov %%"R_SP", %%"R_BP"\n");
-	else r_egg_printf (egg, "mov "R_BP", "R_SP"\n");
+if (attsyntax) {
+	r_egg_printf (egg, "mov %%" R_SP ", %%" R_BP "\n");
+} else {
+	r_egg_printf (egg, "mov " R_BP ", " R_SP "\n");
+}
 }
 
 static char *emit_syscall (REgg *egg, int nargs) {
 	char p[512];
-	if (attsyntax)
-		return strdup (": mov $`.arg`, %"R_AX"\n: "SYSCALL_ATT"\n");
+	if (attsyntax) {
+		return strdup (": mov $`.arg`, %" R_AX "\n: " SYSCALL_ATT "\n");
+	}
 	switch (egg->os) {
 	case R_EGG_OS_LINUX:
 		strcpy (p, "\n : mov "R_AX", `.arg`\n : "SYSCALL_INTEL "\n");
@@ -69,17 +73,21 @@ static char *emit_syscall (REgg *egg, int nargs) {
 }
 
 static void emit_frame (REgg *egg, int sz) {
-	if (sz<1)
+	if (sz < 1) {
 		return;
-	if (attsyntax)
+	}
+	if (attsyntax) {
 		r_egg_printf (egg,
 		"  push %%"R_BP"\n"
 		"  mov %%"R_SP", %%"R_BP"\n"
 		"  sub $%d, %%"R_SP"\n", sz);
-	else r_egg_printf (egg,
-		"  push "R_BP"\n"
-		"  mov "R_BP", "R_SP"\n"
-		"  sub "R_SP", %d\n", sz);
+	} else {
+		r_egg_printf (egg,
+			"  push " R_BP "\n"
+			"  mov " R_BP ", " R_SP "\n"
+			"  sub " R_SP ", %d\n",
+			sz);
+	}
 }
 
 static void emit_frame_end (REgg *egg, int sz, int ctx) {
@@ -92,8 +100,9 @@ static void emit_frame_end (REgg *egg, int sz, int ctx) {
 			r_egg_printf (egg, "  pop "R_BP"\n");
 		}
 	}
-	if (ctx>0)
+	if (ctx > 0) {
 		r_egg_printf (egg, "  ret\n");
+	}
 }
 
 static void emit_comment(REgg *egg, const char *fmt, ...) {
@@ -101,8 +110,11 @@ static void emit_comment(REgg *egg, const char *fmt, ...) {
 	char buf[1024];
 	va_start (ap, fmt);
 	vsnprintf (buf, sizeof (buf), fmt, ap);
-	if (attsyntax) r_egg_printf (egg, "  /* %s */\n", buf);
-	else r_egg_printf (egg, "# %s\n", buf);
+	if (attsyntax) {
+		r_egg_printf (egg, "  /* %s */\n", buf);
+	} else {
+		r_egg_printf (egg, "# %s\n", buf);
+	}
 	va_end (ap);
 }
 
@@ -141,25 +153,30 @@ static void emit_syscall_args(REgg *egg, int nargs) {
 }
 
 static void emit_string(REgg *egg, const char *dstvar, const char *str, int j) {
-	char *p, *s, str2[64];
-	int i, len, oj = j;
+	char *p, str2[64];
+	int i, oj = j;
 
-	len = strlen (str);
-	s = malloc (len+4);
-	if (!s) return;
+	int len = strlen (str);
+	char *s = calloc (1, len + 8);
+	if (!s) {
+		return;
+	}
 	memcpy (s, str, len);
-	memset (s+len, 0, 4);
+	memset (s + len, 0, 4);
 
 	/* XXX: Hack: Adjust offset in R_BP correctly for 64b addresses */
-#define BPOFF R_SZ-4
+#define BPOFF (R_SZ-4)
 #define M32(x) (unsigned int)((x) & 0xffffffff)
 	/* XXX: Assumes sizeof(ut32) == 4 */
 	for (i=4; i<=oj; i+=4) {
 		/* XXX endian issues (non-portable asm) */
 		ut32 *n = (ut32 *)(s+i-4);
 		p = r_egg_mkvar (egg, str2, dstvar, i+BPOFF);
-		if (attsyntax) r_egg_printf (egg, "  movl $0x%x, %s\n", M32(*n), p);
-		else r_egg_printf (egg, "  mov %s, 0x%x\n", p, M32(*n));
+		if (attsyntax) {
+			r_egg_printf (egg, "  movl $0x%x, %s\n", M32 (*n), p);
+		} else {
+			r_egg_printf (egg, "  mov dword %s, 0x%x\n", p, M32 (*n));
+		}
 		free (p);
 		j -= 4;
 	}
@@ -167,19 +184,28 @@ static void emit_string(REgg *egg, const char *dstvar, const char *str, int j) {
 
 	/* zero */
 	p = r_egg_mkvar (egg, str2, dstvar, i+BPOFF);
-	if (attsyntax) r_egg_printf (egg, "  movl $0, %s\n", p);
-	else r_egg_printf (egg, "  mov %s, 0\n", p);
+	if (attsyntax) {
+		r_egg_printf (egg, "  movl $0, %s\n", p);
+	} else {
+		r_egg_printf (egg, "  mov dword %s, 0\n", p);
+	}
 	free (p);
 
 	/* store pointer */
 	p = r_egg_mkvar (egg, str2, dstvar, j+4+BPOFF);
-	if (attsyntax) r_egg_printf (egg, "  lea %s, %%"R_AX"\n", p);
-	else r_egg_printf (egg, "  lea "R_AX", %s\n", p);
+	if (attsyntax) {
+		r_egg_printf (egg, "  lea %s, %%" R_AX "\n", p);
+	} else {
+		r_egg_printf (egg, "  lea " R_AX ", %s\n", p);
+	}
 	free (p);
 
 	p = r_egg_mkvar (egg, str2, dstvar, 0);
-	if (attsyntax) r_egg_printf (egg, "  mov %%"R_AX", %s\n", p);
-	else r_egg_printf (egg, "  mov %s, "R_AX"\n", p);
+	if (attsyntax) {
+		r_egg_printf (egg, "  mov %%" R_AX ", %s\n", p);
+	} else {
+		r_egg_printf (egg, "  mov %s, " R_AX "\n", p);
+	}
 	free (p);
 
 #undef BPOFF
@@ -206,16 +232,24 @@ static void emit_string(REgg *egg, const char *dstvar, const char *str, int j) {
 
 static void emit_call(REgg *egg, const char *str, int atr) {
 	if (atr) {
-		if (attsyntax) r_egg_printf (egg, "  call *%s\n", str);
-		else r_egg_printf (egg, "  call [%s]\n", str);
-	} else r_egg_printf (egg, "  call %s\n", str);
+		if (attsyntax) {
+			r_egg_printf (egg, "  call *%s\n", str);
+		} else {
+			r_egg_printf (egg, "  call [%s]\n", str);
+		}
+	} else {
+		r_egg_printf (egg, "  call %s\n", str);
+	}
 }
 
 static void emit_jmp(REgg *egg, const char *str, int atr) {
 	if (str) {
 		if (atr) {
-			if (attsyntax) r_egg_printf (egg, "  jmp *%s\n", str);
-			else r_egg_printf (egg, "  jmp [%s]\n", str);
+			if (attsyntax) {
+				r_egg_printf (egg, "  jmp *%s\n", str);
+			} else {
+				r_egg_printf (egg, "  jmp [%s]\n", str);
+			}
 		} else {
 			r_egg_printf (egg, "  jmp %s\n", str);
 		}
@@ -226,8 +260,9 @@ static void emit_jmp(REgg *egg, const char *str, int atr) {
 
 static void emit_arg (REgg *egg, int xs, int num, const char *str) {
 	int d = atoi (str);
-	if (!attsyntax && (*str=='$'))
-		str = str +1;
+	if (!attsyntax && (*str == '$')) {
+		str = str + 1;
+	}
 	switch (xs) {
 	case 0:
 #ifdef ARCH_X86_64
@@ -247,31 +282,48 @@ static void emit_arg (REgg *egg, int xs, int num, const char *str) {
 #endif
 		break;
 	case '*':
-		if (attsyntax) r_egg_printf (egg, "  push (%s)\n", str);
-		else r_egg_printf (egg, "  push [%s]\n", str);
+		if (attsyntax) {
+			r_egg_printf (egg, "  push (%s)\n", str);
+		} else {
+			r_egg_printf (egg, "  push [%s]\n", str);
+		}
 		break;
 	case '&':
 		if (attsyntax) {
-			if (d != 0) r_egg_printf (egg, "  addl $%d, %%"R_BP"\n", d);
+			if (d != 0) {
+				r_egg_printf (egg, "  addl $%d, %%" R_BP "\n", d);
+			}
 			r_egg_printf (egg, "  pushl %%"R_BP"\n");
-			if (d != 0) r_egg_printf (egg, "  subl $%d, %%"R_BP"\n", d);
+			if (d != 0) {
+				r_egg_printf (egg, "  subl $%d, %%" R_BP "\n", d);
+			}
 		} else {
-			if (d != 0) r_egg_printf (egg, "  add "R_BP", %d\n", d);
+			if (d != 0) {
+				r_egg_printf (egg, "  add " R_BP ", %d\n", d);
+			}
 			r_egg_printf (egg, "  push "R_BP"\n");
-			if (d != 0) r_egg_printf (egg, "  sub "R_BP", %d\n", d);
+			if (d != 0) {
+				r_egg_printf (egg, "  sub " R_BP ", %d\n", d);
+			}
 		}
 		break;
 	}
 }
 
 static void emit_get_result(REgg *egg, const char *ocn) {
-	if (attsyntax) r_egg_printf (egg, "  mov %%"R_AX", %s\n", ocn);
-	else r_egg_printf (egg, "  mov %s, "R_AX"\n", ocn);
+	if (attsyntax) {
+		r_egg_printf (egg, "  mov %%" R_AX ", %s\n", ocn);
+	} else {
+		r_egg_printf (egg, "  mov %s, " R_AX "\n", ocn);
+	}
 }
 
 static void emit_restore_stack (REgg *egg, int size) {
-	if (attsyntax) r_egg_printf (egg, "  add $%d, %%"R_SP" /* args */\n", size);
-	else r_egg_printf (egg, "  add "R_SP", %d\n", size);
+	if (attsyntax) {
+		r_egg_printf (egg, "  add $%d, %%" R_SP " /* args */\n", size);
+	} else {
+		r_egg_printf (egg, "  add " R_SP ", %d\n", size);
+	}
 }
 
 static void emit_get_while_end (REgg *egg, char *str, const char *ctxpush, const char *label) {
@@ -296,22 +348,34 @@ static void emit_while_end (REgg *egg, const char *labelback) {
 static void emit_get_var (REgg *egg, int type, char *out, int idx) {
 	switch (type) {
 	case 0:  /* variable */
-		if (idx>0) sprintf (out, "["R_BP"+%d]", idx);
-		else if (idx<0) sprintf (out, "["R_BP"%d]", idx);
-		else strcpy (out, "["R_BP"]");
+		if (idx > 0) {
+			sprintf (out, "[" R_BP "+%d]", idx);
+		} else if (idx < 0) {
+			sprintf (out, "[" R_BP "%d]", idx);
+		} else {
+			strcpy (out, "[" R_BP "]");
+		}
 		break;
 	case 1: /* argument */
 // OMG WE CANT stuff found in relative address in stack in the stack
 		eprintf ("WARNING: Using stack vars in naked functions\n");
 		idx = 8; // HACK to make arg0, arg4, ... work
-		if (idx>0) sprintf (out, "["R_SP"+%d]", idx);
-		else if (idx<0) sprintf (out, "["R_SP"%d]", idx);
-		else strcpy (out, "["R_SP"]");
+		if (idx > 0) {
+			sprintf (out, "[" R_SP "+%d]", idx);
+		} else if (idx < 0) {
+			sprintf (out, "[" R_SP "%d]", idx);
+		} else {
+			strcpy (out, "[" R_SP "]");
+		}
 		break;
 	case 2:
-		if (idx>0) sprintf (out, "["R_BP"+%d]", idx);
-		else if (idx<0) sprintf (out, "["R_BP"%d]", idx);
-		else strcpy (out, "["R_BP"]");
+		if (idx > 0) {
+			sprintf (out, "[" R_BP "+%d]", idx);
+		} else if (idx < 0) {
+			sprintf (out, "[" R_BP "%d]", idx);
+		} else {
+			strcpy (out, "[" R_BP "]");
+		}
 		break;
 	}
 }
@@ -324,7 +388,9 @@ static void emit_load_ptr(REgg *egg, const char *dst) {
 	int d = atoi (dst);
 	if (d == 0) { // hack to handle stackvarptrz
 		char *p = strchr (dst, '+');
-		if (p) d = atoi (p+1);
+		if (p) {
+			d = atoi (p + 1);
+		}
 	}
 	//eprintf ("emit_load_ptr: HACK\n");
 	// XXX: 32/64bit care
@@ -368,12 +434,17 @@ static void emit_branch(REgg *egg, char *b, char *g, char *e, char *n, int sz, c
 			op = "jne";
 		} else {
 			arg = attsyntax? "$0": "0";
-			if (n) op = "jnz";
-			else op ="jz";
+			if (n) {
+				op = "jnz";
+			} else {
+				op = "jz";
+			}
 		}
 	}
 
-	if (*arg=='=') arg++; /* for <=, >=, ... */
+	if (*arg == '=') {
+		arg++; /* for <=, >=, ... */
+	}
 	p = r_egg_mkvar (egg, str, arg, 0);
 	if (attsyntax) {
 		r_egg_printf (egg, "  pop %%"R_AX"\n"); /* TODO: add support for more than one arg get arg0 */
@@ -434,20 +505,31 @@ static void emit_mathop(REgg *egg, int ch, int vs, int type, const char *eq, con
 	default:  op = "mov"; break;
 	}
 	if (attsyntax) {
-		if (!eq) eq = "%"R_AX;
-		if (!p) p = "%"R_AX;
+		if (!eq) {
+			eq = "%" R_AX;
+		}
+		if (!p) {
+			p = "%" R_AX;
+		}
 		r_egg_printf (egg, "  %s%c %c%s, %s\n", op, vs, type, eq, p);
 	} else {
-		if (!eq) eq = R_AX;
-		if (!p) p = R_AX;
-	// TODO:
+		if (!eq) {
+			eq = R_AX;
+		}
+		if (!p) {
+			p = R_AX;
+		}
+		// TODO:
 #if 0
 		eprintf ("TYPE = %c\n", type);
 		eprintf ("  %s%c %c%s, %s\n", op, vs, type, eq, p);
 		eprintf ("  %s %s, [%s]\n", op, p, eq);
 #endif
-		if (type == '*') r_egg_printf (egg, "  %s %s, [%s]\n", op, p, eq);
-		else r_egg_printf (egg, "  %s %s, %s\n", op, p, eq);
+		if (type == '*') {
+			r_egg_printf (egg, "  %s %s, [%s]\n", op, p, eq);
+		} else {
+			r_egg_printf (egg, "  %s %s, %s\n", op, p, eq);
+		}
 	}
 }
 
@@ -458,7 +540,9 @@ static const char* emit_regs(REgg *egg, int idx) {
 static void emit_get_ar (REgg *egg, char *out, int idx) {
 	const char *reg = emit_regs (egg, R_REG_AR_OFF + idx);
 
-	if (reg) strcpy (out, reg);
+	if (reg) {
+		strcpy (out, reg);
+	}
 }
 
 REggEmit EMIT_NAME = {
