@@ -205,7 +205,7 @@ REggPlugin r_egg_plugin_bind = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_EGG,
 	.data = &r_egg_plugin_bind,
 	.version = R2_VERSION
