@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2017 - pancake */
+/* radare - LGPL - Copyright 2007-2018 - pancake */
 
 #include <r_flag.h>
 #include <r_util.h>
@@ -11,15 +11,13 @@ R_LIB_VERSION(r_flag);
 #define IS_IN_SPACE(f, i) ((f)->space_idx != -1 && (i)->space != (f)->space_idx)
 
 static const char *str_callback(RNum *user, ut64 off, int *ok) {
-	const RList *list;
 	RFlag *f = (RFlag*)user;
-	RFlagItem *item;
 	if (ok) {
 		*ok = 0;
 	}
 	if (f) {
-		list = r_flag_get_list (f, off);
-		item = r_list_get_top (list);
+		const RList *list = r_flag_get_list (f, off);
+		RFlagItem *item = r_list_get_top (list);
 		if (item) {
 			if (ok) {
 				*ok = true;
@@ -30,12 +28,6 @@ static const char *str_callback(RNum *user, ut64 off, int *ok) {
 	return NULL;
 }
 
-static void flag_free_kv(HtKv *kv) {
-	free (kv->key);
-	//we do not free kv->value since there is a reference in other list
-	free (kv);
-}
-
 static void flag_skiplist_free(void *data) {
 	RFlagsAtOffset *item = (RFlagsAtOffset *)data;
 	r_list_free (item->flags);
@@ -56,7 +48,7 @@ static ut64 num_callback(RNum *user, const char *name, int *ok) {
 	if (ok) {
 		*ok = 0;
 	}
-	item = ht_find (f->ht_name, name, NULL);
+	item = ht_pp_find (f->ht_name, name, NULL);
 	if (item) {
 		// NOTE: to avoid warning infinite loop here we avoid recursivity
 		if (item->alias) {
@@ -75,21 +67,15 @@ static ut64 num_callback(RNum *user, const char *name, int *ok) {
 	dir == 0 ->  result == off
 	dir == 1 ->  result >= off*/
 static RFlagsAtOffset* r_flag_get_nearest_list(RFlag *f, ut64 off, int dir) {
-	RFlagsAtOffset *flags = NULL;
-	RFlagsAtOffset key;
-	key.off = off;
-	if (dir >= 0) {
-		flags = r_skiplist_get_geq (f->by_off, &key);
-	} else {
-		flags = r_skiplist_get_leq (f->by_off, &key);
-	}
-	if (dir == 0 && flags && flags->off != off) {
-		return NULL;
-	}
-	return flags;
+	RFlagsAtOffset key = { .off = off };
+	RFlagsAtOffset *flags = (dir >= 0)
+		? r_skiplist_get_geq (f->by_off, &key)
+		: r_skiplist_get_leq (f->by_off, &key);
+	return (dir == 0 && flags && flags->off != off)? NULL: flags;
 }
 
 static void remove_offsetmap(RFlag *f, RFlagItem *item) {
+	r_return_if_fail (f && item);
 	RFlagsAtOffset *flags = r_flag_get_nearest_list (f, item->offset, 0);
 	if (flags) {
 		r_list_delete_data (flags->flags, item);
@@ -99,7 +85,8 @@ static void remove_offsetmap(RFlag *f, RFlagItem *item) {
 	}
 }
 
-static int set_name(RFlagItem *item, const char *name) {
+static bool set_name(RFlagItem *item, const char *name) {
+	r_return_val_if_fail (item && name, false);
 	if (item->name != item->realname) {
 		free (item->name);
 	}
@@ -117,7 +104,9 @@ static int set_name(RFlagItem *item, const char *name) {
 R_API RFlag * r_flag_new() {
 	int i;
 	RFlag *f = R_NEW0 (RFlag);
-	if (!f) return NULL;
+	if (!f) {
+		return NULL;
+	}
 	f->num = r_num_new (&num_callback, &str_callback, f);
 	if (!f->num) {
 		r_flag_free (f);
@@ -143,7 +132,7 @@ R_API RFlag * r_flag_new() {
 		r_flag_free (f);
 		return NULL;
 	}
-	f->ht_name = ht_new (NULL, flag_free_kv, NULL);
+	f->ht_name = ht_pp_new0 ();
 	f->by_off = r_skiplist_new (flag_skiplist_free, flag_skiplist_cmp);
 #if R_FLAG_ZONE_USE_SDB
 	sdb_free (f->zones);
@@ -157,15 +146,12 @@ R_API RFlag * r_flag_new() {
 }
 
 R_API RFlagItem *r_flag_item_clone(RFlagItem *item) {
-	if (!item) {
-		return NULL;
-	}
+	r_return_val_if_fail (item, NULL);
 
 	RFlagItem *n = R_NEW0 (RFlagItem);
 	if (!n) {
 		return NULL;
 	}
-
 	n->color = item->color ? strdup (item->color) : NULL;
 	n->comment = item->comment ? strdup (item->comment) : NULL;
 	n->alias = item->alias ? strdup (item->alias) : NULL;
@@ -178,26 +164,28 @@ R_API RFlagItem *r_flag_item_clone(RFlagItem *item) {
 }
 
 R_API void r_flag_item_free(RFlagItem *item) {
-	if (item) {
-		free (item->color);
-		free (item->comment);
-		free (item->alias);
-		/* release only one of the two pointers if they are the same */
-		if (item->name != item->realname) {
-			free (item->name);
-		}
-		free (item->realname);
-		free (item);
+	if (!item) {
+		return;
 	}
+	free (item->color);
+	free (item->comment);
+	free (item->alias);
+	/* release only one of the two pointers if they are the same */
+	if (item->name != item->realname) {
+		free (item->name);
+	}
+	free (item->realname);
+	free (item);
 }
 
 R_API RFlag *r_flag_free(RFlag *f) {
+	r_return_val_if_fail (f, NULL);
 	int i;
 	for (i = 0; i < R_FLAG_SPACES_MAX; i++) {
 		free (f->spaces[i]);
 	}
 	r_skiplist_free (f->by_off);
-	ht_free (f->ht_name);
+	ht_pp_free (f->ht_name);
 
 	r_list_free (f->flags);
 	sdb_free (f->tags);
@@ -263,9 +251,11 @@ R_API void r_flag_list(RFlag *f, int rad, const char *pfx) {
 			} else {
 				f->cb_printf ("\"offset\":%"PFMT64d, flag->offset);
 			}
-			if (flag->comment)
+			if (flag->comment) {
 				f->cb_printf (",\"comment\":\"}");
-			else f->cb_printf ("}");
+			} else {
+				f->cb_printf ("}");
+			}
 			first = 0;
 		}
 		f->cb_printf ("]\n");
@@ -281,18 +271,19 @@ R_API void r_flag_list(RFlag *f, int rad, const char *pfx) {
 				continue;
 			}
 			if (fs == -1 || flag->space != fs) {
-				const char *flagspace;
 				fs = flag->space;
-				flagspace = r_flag_space_get_i (f, fs);
-				if (!flagspace || !*flagspace)
+				const char *flagspace = r_flag_space_get_i (f, fs);
+				if (!flagspace || !*flagspace) {
 					flagspace = "*";
+				}
 				f->cb_printf ("fs %s\n", flagspace);
 			}
 			if (flag->alias) {
 				f->cb_printf ("fa %s %s\n", flag->name, flag->alias);
-				if (flag->comment && *flag->comment)
+				if (flag->comment && *flag->comment) {
 					f->cb_printf ("\"fC %s %s\"\n",
 						flag->name, flag->comment);
+				}
 			} else {
 				f->cb_printf ("f %s %"PFMT64d" 0x%08"PFMT64x"%s%s %s\n",
 					flag->name, flag->size, flag->offset,
@@ -339,7 +330,8 @@ R_API void r_flag_list(RFlag *f, int rad, const char *pfx) {
 }
 
 static RFlagItem *evalFlag(RFlag *f, RFlagItem *item) {
-	if (item && item->alias) {
+	r_return_val_if_fail (f && item, NULL);
+	if (item->alias) {
 		item->offset = r_num_math (f->num, item->alias);
 	}
 	return item;
@@ -348,11 +340,9 @@ static RFlagItem *evalFlag(RFlag *f, RFlagItem *item) {
 /* return true if flag.* exist at offset. Otherwise, false is returned.
  * For example (f, "sym", 3, 0x1000)*/
 R_API bool r_flag_exist_at(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64 off) {
+	r_return_val_if_fail (f && flag_prefix, NULL);
 	RListIter *iter = NULL;
 	RFlagItem *item = NULL;
-	if (!f) {
-		return false;
-	}
 	const RList *list = r_flag_get_list (f, off);
 	if (!list) {
 		return false;
@@ -368,21 +358,15 @@ R_API bool r_flag_exist_at(RFlag *f, const char *flag_prefix, ut16 fp_size, ut64
 /* return the flag item with name "name" in the RFlag "f", if it exists.
  * Otherwise, NULL is returned. */
 R_API RFlagItem *r_flag_get(RFlag *f, const char *name) {
-	RFlagItem *r;
-	if (!f) {
-		return NULL;
-	}
-	r = ht_find (f->ht_name, name, NULL);
-	return evalFlag (f, r);
+	r_return_val_if_fail (f, NULL);
+	RFlagItem *r = ht_pp_find (f->ht_name, name, NULL);
+	return r? evalFlag (f, r): NULL;
 }
 
 /* return the first flag item that can be found at offset "off", or NULL otherwise */
 R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off) {
-	const RList *list;
-	if (!f) {
-		return NULL;
-	}
-	list = r_flag_get_list (f, off);
+	r_return_val_if_fail (f, NULL);
+	const RList *list = r_flag_get_list (f, off);
 	return list ? evalFlag (f, r_list_get_top (list)) : NULL;
 }
 
@@ -391,6 +375,7 @@ R_API RFlagItem *r_flag_get_i(RFlag *f, ut64 off) {
  *
  * XXX: this function is buggy and it's not really clear what's the purpose */
 R_API RFlagItem *r_flag_get_i2(RFlag *f, ut64 off) {
+	r_return_val_if_fail (f, NULL);
 	RFlagItem *oitem = NULL, *item = NULL;
 	RListIter *iter;
 	const RList *list = r_flag_get_list (f, off);
@@ -420,10 +405,12 @@ R_API RFlagItem *r_flag_get_i2(RFlag *f, ut64 off) {
 			break;
 		}
 		oitem = item;
-		if (strlen (item->name) < 5 || item->name[3]!='.') continue;
+		if (strlen (item->name) < 5 || item->name[3] != '.') {
+			continue;
+		}
 		oitem = item;
 	}
-	return evalFlag (f, oitem);
+	return oitem? evalFlag (f, oitem): NULL;
 }
 
 static bool isFunctionFlag(const char *n) {
@@ -437,6 +424,8 @@ static bool isFunctionFlag(const char *n) {
 /* returns the last flag item defined before or at the given offset.
  * NULL is returned if such a item is not found. */
 R_API RFlagItem *r_flag_get_at(RFlag *f, ut64 off, bool closest) {
+	r_return_val_if_fail (f, NULL);
+
 	RFlagItem *item, *nice = NULL;
 	RListIter *iter;
 	const RFlagsAtOffset *flags_at = r_flag_get_nearest_list (f, off, -1);
@@ -480,7 +469,7 @@ R_API RFlagItem *r_flag_get_at(RFlag *f, ut64 off, bool closest) {
 			flags_at = NULL;
 		}
 	}
-	return evalFlag (f, nice);
+	return nice? evalFlag (f, nice): NULL;
 }
 
 /* return the list of flag items that are associated with a given offset */
@@ -502,11 +491,15 @@ R_API char *r_flag_get_liststr(RFlag *f, ut64 off) {
 }
 
 R_API RFlagItem *r_flag_set_next(RFlag *f, const char *name, ut64 off, ut32 size) {
+	r_return_val_if_fail (f && name, NULL);
 	if (!r_flag_get (f, name)) {
 		return r_flag_set (f, name, off, size);
 	}
 	int i, newNameSize = strlen (name);
 	char *newName = malloc (newNameSize + 16);
+	if (!newName) {
+		return NULL;
+	}
 	strcpy (newName, name);
 	for (i = 0; ; i++) {
 		snprintf (newName + newNameSize, 15, ".%d", i);
@@ -523,15 +516,9 @@ R_API RFlagItem *r_flag_set_next(RFlag *f, const char *name, ut64 off, ut32 size
  * The realname of the item will be the same as the name.
  * NULL is returned in case of any errors during the process. */
 R_API RFlagItem *r_flag_set(RFlag *f, const char *name, ut64 off, ut32 size) {
-	RFlagItem *item = NULL;
-	RList *list;
+	r_return_val_if_fail (f && name && *name, NULL);
 
-	/* contract fail */
-	if (!name || !*name) {
-		return NULL;
-	}
-
-	item = r_flag_get (f, name);
+	RFlagItem *item = r_flag_get (f, name);
 	if (item) {
 		if (item->offset == off) {
 			item->size = size;
@@ -550,7 +537,7 @@ R_API RFlagItem *r_flag_set(RFlag *f, const char *name, ut64 off, ut32 size) {
 		}
 		//item share ownership prone to uaf, that is why only
 		//f->flags has set up free pointer
-		ht_insert (f->ht_name, item->name, item);
+		ht_pp_insert (f->ht_name, item->name, item);
 		r_list_append (f->flags, item);
 	}
 
@@ -558,65 +545,61 @@ R_API RFlagItem *r_flag_set(RFlag *f, const char *name, ut64 off, ut32 size) {
 	item->offset = off + f->base;
 	item->size = size;
 
-	list = (RList *)r_flag_get_list (f, off);
+	RList *list = (RList *)r_flag_get_list (f, off);
 	if (!list) {
 		RFlagsAtOffset *flagsAtOffset = R_NEW (RFlagsAtOffset);
-		list = r_list_new ();
-		flagsAtOffset->flags = list;
-		flagsAtOffset->off = off;
-		// CID 1378268:  Resource leaks  (RESOURCE_LEAK)
-		// Ignoring storage allocated by "r_skiplist_insert(f->by_off, flagsAtOffset)" leaks it.
-		r_skiplist_insert (f->by_off, flagsAtOffset);
+		if (flagsAtOffset) {
+			list = r_list_new ();
+			flagsAtOffset->flags = list;
+			if (list) {
+				flagsAtOffset->off = off;
+				// CID 1378268:  Resource leaks  (RESOURCE_LEAK)
+				// Ignoring storage allocated by "r_skiplist_insert(f->by_off, flagsAtOffset)" leaks it.
+				r_skiplist_insert (f->by_off, flagsAtOffset);
+			} else {
+				flag_skiplist_free (flagsAtOffset);
+			}
+		}
+	}
+	if (list) {
+		r_list_append (list, item);
 	}
-	r_list_append (list, item);
 	return item;
 }
 
 /* add/replace/remove the alias of a flag item */
 R_API void r_flag_item_set_alias(RFlagItem *item, const char *alias) {
-	if (item) {
-		free (item->alias);
-		item->alias = ISNULLSTR (alias)? NULL: strdup (alias);
-	}
+	r_return_if_fail (item);
+	free (item->alias);
+	item->alias = ISNULLSTR (alias)? NULL: strdup (alias);
 }
 
 /* add/replace/remove the comment of a flag item */
 R_API void r_flag_item_set_comment(RFlagItem *item, const char *comment) {
-	if (item) {
-		free (item->comment);
-		item->comment = ISNULLSTR (comment) ? NULL : strdup (comment);
-	}
+	r_return_if_fail (item);
+	free (item->comment);
+	item->comment = ISNULLSTR (comment) ? NULL : strdup (comment);
 }
 
 /* add/replace/remove the realname of a flag item */
 R_API void r_flag_item_set_realname(RFlagItem *item, const char *realname) {
-	if (item) {
-		if (item->name != item->realname) {
-			free (item->realname);
-		}
-		item->realname = ISNULLSTR (realname) ? NULL : strdup (realname);
+	r_return_if_fail (item);
+	if (item->name != item->realname) {
+		free (item->realname);
 	}
+	item->realname = ISNULLSTR (realname) ? NULL : strdup (realname);
 }
 
 /* change the name of a flag item, if the new name is available.
  * true is returned if everything works well, false otherwise */
 R_API int r_flag_rename(RFlag *f, RFlagItem *item, const char *name) {
-	if (!f || !item || !name || !*name) {
-		return false;
-	}
-#if 0
-	ut64 off = item->offset;
-	int size = item->size;
-	r_flag_unset (f, item);
-	r_flag_set (f, name, off, size);
-	return true;
-#else
-	ht_delete (f->ht_name, item->name);
+	r_return_val_if_fail (f && item && name && *name, false);
+
+	ht_pp_delete (f->ht_name, item->name);
 	if (!set_name (item, name)) {
 		return false;
 	}
-	ht_insert (f->ht_name, item->name, item);
-#endif
+	ht_pp_insert (f->ht_name, item->name, item);
 	return true;
 }
 
@@ -624,16 +607,18 @@ R_API int r_flag_rename(RFlag *f, RFlagItem *item, const char *name) {
  * returns true if the item is successfully unset, false otherwise.
  *
  * NOTE: the item is freed. */
-R_API int r_flag_unset(RFlag *f, RFlagItem *item) {
+R_API bool r_flag_unset(RFlag *f, RFlagItem *item) {
+	r_return_val_if_fail (f && item, false);
 	remove_offsetmap (f, item);
-	ht_delete (f->ht_name, item->name);
+	ht_pp_delete (f->ht_name, item->name);
 	r_list_delete_data (f->flags, item);
 	return true;
 }
 
 /* unset the first flag item found at offset off.
  * return true if such a flag is found and unset, false otherwise. */
-R_API int r_flag_unset_off(RFlag *f, ut64 off) {
+R_API bool r_flag_unset_off(RFlag *f, ut64 off) {
+	r_return_val_if_fail (f, false);
 	RFlagItem *item = r_flag_get_i (f, off);
 	if (item && r_flag_unset (f, item)) {
 		return true;
@@ -642,9 +627,11 @@ R_API int r_flag_unset_off(RFlag *f, ut64 off) {
 }
 
 /* unset all the flag items that satisfy the given glob.
- * return the number of unset items. */
+ * return the number of unset items. -1 on error */
 // XXX This is O(n^n) because unset_globa iterates all flags and unset too.
 R_API int r_flag_unset_glob(RFlag *f, const char *glob) {
+	r_return_val_if_fail (f, -1);
+
 	RListIter *iter, *iter2;
 	RFlagItem *flag;
 	int n = 0;
@@ -653,7 +640,7 @@ R_API int r_flag_unset_glob(RFlag *f, const char *glob) {
 		if (IS_IN_SPACE (f, flag)) {
 			continue;
 		}
-		if (!glob || r_str_glob (flag->name, glob)) {
+		if (r_str_glob (flag->name, glob)) {
 			r_flag_unset (f, flag);
 			n++;
 		}
@@ -663,27 +650,27 @@ R_API int r_flag_unset_glob(RFlag *f, const char *glob) {
 
 /* unset the flag item with the given name.
  * returns true if the item is found and unset, false otherwise. */
-R_API int r_flag_unset_name(RFlag *f, const char *name) {
-	RFlagItem *item = ht_find (f->ht_name, name, NULL);
+R_API bool r_flag_unset_name(RFlag *f, const char *name) {
+	r_return_val_if_fail (f, false);
+	RFlagItem *item = ht_pp_find (f->ht_name, name, NULL);
 	return item && r_flag_unset (f, item);
 }
 
 /* unset all flag items in the RFlag f */
 R_API void r_flag_unset_all(RFlag *f) {
+	r_return_if_fail (f);
 	f->space_idx = -1;
 	r_list_free (f->flags);
 	f->flags = r_list_newf ((RListFree)r_flag_item_free);
-	if (!f->flags) {
-		return;
-	}
-	ht_free (f->ht_name);
+	ht_pp_free (f->ht_name);
 	//don't set free since f->flags will free up items when needed avoiding uaf
-	f->ht_name = ht_new (NULL, flag_free_kv, NULL);
+	f->ht_name = ht_pp_new0 ();
 	r_skiplist_purge (f->by_off);
 	r_flag_space_unset (f, NULL);
 }
 
 R_API int r_flag_relocate(RFlag *f, ut64 off, ut64 off_mask, ut64 to) {
+	r_return_val_if_fail (f, -1);
 	ut64 neg_mask = ~(off_mask);
 	RFlagItem *item;
 	RListIter *iter;
@@ -702,7 +689,8 @@ R_API int r_flag_relocate(RFlag *f, ut64 off, ut64 off_mask, ut64 to) {
 	return n;
 }
 
-R_API int r_flag_move(RFlag *f, ut64 at, ut64 to) {
+R_API bool r_flag_move(RFlag *f, ut64 at, ut64 to) {
+	r_return_val_if_fail (f, false);
 	RFlagItem *item = r_flag_get_i (f, at);
 	if (item) {
 		r_flag_set (f, item->name, to, item->size);
@@ -711,56 +699,41 @@ R_API int r_flag_move(RFlag *f, ut64 at, ut64 to) {
 	return false;
 }
 
-#ifdef MYTEST
-int main () {
-	RFlagItem *i;
-	RFlag *f = r_flag_new ();
-	r_flag_set (f, "rip", 0xfff333999000LL, 1);
-	r_flag_set (f, "rip", 0xfff333999002LL, 1);
-	r_flag_unset (f, "rip", NULL);
-	r_flag_set (f, "rip", 3, 4);
-	r_flag_set (f, "rip", 4, 4);
-	r_flag_set (f, "corwp", 300, 4);
-	r_flag_set (f, "barp", 300, 4);
-	r_flag_set (f, "rip", 3, 4);
-	r_flag_set (f, "rip", 4, 4);
-
-	i = r_flag_get (f, "rip");
-	if (i) printf ("nRIP: %p %llx\n", i, i->offset);
-	else printf ("nRIP: null\n");
-
-	i = r_flag_get_i (f, 0xfff333999000LL);
-	if (i) printf ("iRIP: %p %llx\n", i, i->offset);
-	else printf ("iRIP: null\n");
-}
-#endif
-
 R_API const char *r_flag_color(RFlag *f, RFlagItem *it, const char *color) {
-	if (!f || !it) return NULL;
-	if (!color) return it->color;
+	r_return_val_if_fail (f && it, NULL);
+	if (!color) {
+		return it->color;
+	}
 	free (it->color);
 	it->color = *color ? strdup (color) : NULL;
 	return it->color;
 }
 
 // BIND
-R_API int r_flag_bind(RFlag *f, RFlagBind *fb) {
+R_API void r_flag_bind(RFlag *f, RFlagBind *fb) {
+	r_return_if_fail (f && fb);
 	fb->f = f;
 	fb->exist_at = r_flag_exist_at;
 	fb->get = r_flag_get;
 	fb->get_at = r_flag_get_at;
 	fb->set = r_flag_set;
+	fb->unset = r_flag_unset;
+	fb->unset_name = r_flag_unset_name;
+	fb->unset_off = r_flag_unset_off;
 	fb->set_fs = r_flag_space_set;
-	return 0;
+	fb->push_fs = r_flag_space_push;
+	fb->pop_fs = r_flag_space_pop;
 }
 
 R_API int r_flag_count(RFlag *f, const char *glob) {
 	int count = 0;
 	RFlagItem *flag;
 	RListIter *iter;
+	r_return_val_if_fail (f, -1);
 	r_list_foreach (f->flags, iter, flag) {
-		if (r_str_glob (flag->name, glob))
-			count ++;
+		if (r_str_glob (flag->name, glob)) {
+			count++;
+		}
 	}
 	return count;
 }
