@@ -20,14 +20,10 @@ static RFlagZoneItem *r_flag_zone_get (RFlag *f, const char *name) {
 #endif
 
 R_API bool r_flag_zone_add(RFlag *f, const char *name, ut64 addr) {
-#if 1
-	if (!name || !f || !*name) {
-		return false;
-	}
-#endif
+	r_return_val_if_fail (f && name && *name, false);
 #if R_FLAG_ZONE_USE_SDB
 	RFlagZoneItem zi = { 0, 0, (const char *)name };
-	if (!f || !DB || !name) {
+	if (!DB) {
 		return false;
 	}
 	const char *bound = sdb_const_get (DB, name, NULL);
