@@ -1,14 +1,22 @@
 #ifndef R2_CONS_H
 #define R2_CONS_H
 
-#define HAVE_DIETLINE 1
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 #include <r_types.h>
-#include <r_util.h>
+#include <r_util/r_graph.h>
+#include <r_util/r_hex.h>
+#include <r_util/r_log.h>
+#include <r_util/r_num.h>
+#include <r_util/r_panels.h>
+#include <r_util/r_sandbox.h>
+#include <r_util/r_signal.h>
+#include <r_util/r_stack.h>
+#include <r_util/r_str.h>
+#include <r_util/r_sys.h>
+#include <r_util/r_file.h>
 #include <sdb.h>
 
 #include <stdio.h>
@@ -28,7 +36,6 @@ extern "C" {
 #include <unistd.h>
 
 /* constants */
-#define ENUM_FOR_PAL 0
 #define CONS_MAX_USER 102400
 #define CONS_BUFSZ 0x4f00
 #define STR_IS_NULL(x) (!x || !x[0])
@@ -76,14 +83,16 @@ typedef struct r_cons_grep_t {
 	int icase;
 } RConsGrep;
 
-#if ENUM_FOR_PAL
+#if 0
+// TODO Might be better than using r_cons_pal_get_i
+// And have smaller RConsPrintablePalette and RConsPalette
 enum {
 	R_CONS_PAL_0x00 = 0,
 	R_CONS_PAL_0x7f,
 	R_CONS_PAL_0xff,
-	R_CONS_PAL_args,
-	R_CONS_PAL_bin,
-	R_CONS_PAL_btext,
+	R_CONS_PAL_ARGS,
+	R_CONS_PAL_BIN,
+	R_CONS_PAL_BTEXT,
 	R_CONS_PAL_CALL,
 	R_CONS_PAL_CJMP,
 	R_CONS_PAL_CMP,
@@ -94,9 +103,9 @@ enum {
 	R_CONS_PAL_FLOC,
 	R_CONS_PAL_FLOW,
 	R_CONS_PAL_FLOW2,
-	R_CONS_PAL_fname,
-	R_CONS_PAL_help,
-	R_CONS_PAL_input,
+	R_CONS_PAL_FNAME,
+	R_CONS_PAL_HELP,
+	R_CONS_PAL_INPUT,
 	R_CONS_PAL_INVALID,
 	R_CONS_PAL_JMP,
 	R_CONS_PAL_LABEL,
@@ -120,11 +129,13 @@ enum {
 	R_CONS_PAL_AI_EXEC,
 	R_CONS_PAL_AI_SEQ,
 	R_CONS_PAL_AI_ASCII,
+	R_CONS_PAL_AI_UNMAP,
 	R_CONS_PAL_GUI_CFLOW,
 	R_CONS_PAL_GUI_DATAOFFSET,
 	R_CONS_PAL_GUI_BACKGROUND,
 	R_CONS_PAL_GUI_ALT_BACKGROUND,
 	R_CONS_PAL_GUI_BORDER,
+	R_CONS_PAL_LINEHL,
 	R_CONS_PAL_GRAPH_BOX,
 	R_CONS_PAL_GRAPH_BOX2,
 	R_CONS_PAL_GRAPH_BOX3,
@@ -135,14 +146,96 @@ enum {
 	R_CONS_PAL_GRAPH_TRACED,
 	R_CONS_PAL_GRAPH_CURRENT,
 	R_CONS_PAL_LAST
-}
+};
 #endif
 
+enum { COLOR_MODE_DISABLED = 0, COLOR_MODE_16, COLOR_MODE_256, COLOR_MODE_16M };
+
+enum { ALPHA_RESET = 0x00, ALPHA_FG = 0x01, ALPHA_BG = 0x02, ALPHA_FGBG = 0x03 };
+enum { R_CONS_ATTR_BOLD = 1 << 1 };
+
+typedef struct rcolor_t {
+	// bold, italic, underline, ...
+	ut8 attr;
+	ut8 a;
+	ut8 r;
+	ut8 g;
+	ut8 b;
+	ut8 r2; // Background color
+	ut8 g2; // Only used when a &= ALPHA_FGBG
+	ut8 b2;
+} RColor;
+
 typedef struct r_cons_palette_t {
-// TODO: this must be an array of char *
-#if ENUM_FOR_PAL
-	char *color[R_CONS_PAL_LAST];
-#else
+	RColor b0x00;
+	RColor b0x7f;
+	RColor b0xff;
+	RColor args;
+	RColor bin;
+	RColor btext;
+	RColor call;
+	RColor cjmp;
+	RColor cmp;
+	RColor comment;
+	RColor usercomment;
+	RColor creg;
+	RColor flag;
+	RColor fline;
+	RColor floc;
+	RColor flow;
+	RColor flow2;
+	RColor fname;
+	RColor help;
+	RColor input;
+	RColor invalid;
+	RColor jmp;
+	RColor label;
+	RColor math;
+	RColor mov;
+	RColor nop;
+	RColor num;
+	RColor offset;
+	RColor other;
+	RColor pop;
+	RColor prompt;
+	RColor push;
+	RColor crypto;
+	RColor reg;
+	RColor reset;
+	RColor ret;
+	RColor swi;
+	RColor trap;
+	RColor ai_read;
+	RColor ai_write;
+	RColor ai_exec;
+	RColor ai_seq;
+	RColor ai_ascii;
+	RColor gui_cflow;
+	RColor gui_dataoffset;
+	RColor gui_background;
+	RColor gui_alt_background;
+	RColor gui_border;
+	RColor wordhl;
+	RColor linehl;
+	RColor func_var;
+	RColor func_var_type;
+	RColor func_var_addr;
+	RColor widget_bg;
+	RColor widget_sel;
+
+	/* Graph colors */
+	RColor graph_box;
+	RColor graph_box2;
+	RColor graph_box3;
+	RColor graph_box4;
+	RColor graph_true;
+	RColor graph_false;
+	RColor graph_trufae;
+	RColor graph_traced;
+	RColor graph_current;
+} RConsPalette;
+
+typedef struct r_cons_printable_palette_t {
 	char *b0x00;
 	char *b0x7f;
 	char *b0xff;
@@ -186,12 +279,19 @@ typedef struct r_cons_palette_t {
 	char *ai_exec;
 	char *ai_seq;
 	char *ai_ascii;
+	char *ai_unmap;
 	char *gui_cflow;
 	char *gui_dataoffset;
 	char *gui_background;
 	char *gui_alt_background;
 	char *gui_border;
-	char *highlight;
+	char *wordhl;
+	char *linehl;
+	char *func_var;
+	char *func_var_type;
+	char *func_var_addr;
+	char *widget_bg;
+	char *widget_sel;
 
 	/* graph colors */
 	char *graph_box;
@@ -203,12 +303,9 @@ typedef struct r_cons_palette_t {
 	char *graph_trufae;
 	char *graph_traced;
 	char *graph_current;
-#endif
-#define R_CONS_PALETTE_LIST_SIZE 8
-	char *list[R_CONS_PALETTE_LIST_SIZE];
 	char **rainbow; // rainbow
 	int rainbow_sz; // size of rainbow
-} RConsPalette;
+} RConsPrintablePalette;
 
 R_API char *r_cons_rainbow_get(int idx, int last, bool bg);
 R_API void r_cons_rainbow_free(void);
@@ -216,7 +313,7 @@ R_API void r_cons_rainbow_new(int sz);
 
 typedef void (*RConsEvent)(void *);
 
-#define CONS_MAX_ATTR_SZ 15
+#define CONS_MAX_ATTR_SZ 16
 typedef struct r_cons_canvas_attr_t {
 	//TODO add support for 256 colors.
 	int loc;
@@ -228,8 +325,9 @@ typedef struct r_cons_canvas_t {
 	int h;
 	int x;
 	int y;
-	char *b;
-	int blen;
+	char **b;
+	int *blen;
+	int *bsize;
 	const char * attr;//The current attr (inserted on each write)
 	RConsCanvasAttr * attrs;// all the different attributes
 	int attrslen;
@@ -290,15 +388,35 @@ typedef struct r_cons_canvas_t {
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
+typedef void (*RConsBreakCallback)(void *core);
+typedef void *(*RConsSleepBeginCallback)(void *core);
+typedef void (*RConsSleepEndCallback)(void *core, void *user);
+typedef void (*RConsQueueTaskOneshot)(void *core, void *task, void *user);
 
-typedef struct r_cons_t {
+typedef struct r_cons_context_t {
 	RConsGrep grep;
 	RStack *cons_stack;
-	RStack *break_stack;
 	char *buffer;
-	//int line;
 	int buffer_len;
 	int buffer_sz;
+
+	bool breaked;
+	RStack *break_stack;
+	RConsEvent event_interrupt;
+	void *event_interrupt_data;
+
+	// Used for per-task logging redirection
+	RLogCallback log_callback; // TODO: RList of callbacks
+
+	char *lastOutput;
+	int lastLength;
+	bool lastMode;
+	bool lastEnabled;
+	bool pageable;
+} RConsContext;
+
+typedef struct r_cons_t {
+	RConsContext *context;
 	char *lastline;
 	int is_html;
 	int is_interactive;
@@ -311,20 +429,23 @@ typedef struct r_cons_t {
 	int force_columns;
 	int fix_rows;
 	int fix_columns;
-	bool breaked;
 	bool break_lines;
 	int noflush;
 	FILE *fdin; // FILE? and then int ??
 	int fdout; // only used in pipe.c :?? remove?
 	const char *teefile;
 	int (*user_fgets)(char *buf, int len);
-	RConsEvent event_interrupt;
 	RConsEvent event_resize;
-	void *data;
 	void *event_data;
 	int mouse_event;
 
-	RConsEditorCallback editor;
+	RConsEditorCallback cb_editor;
+	RConsBreakCallback cb_break;
+	RConsSleepBeginCallback cb_sleep_begin;
+	RConsSleepEndCallback cb_sleep_end;
+	RConsClickCallback cb_click;
+	RConsQueueTaskOneshot cb_task_oneshot;
+
 	void *user; // Used by <RCore*>
 #if __UNIX__ || __CYGWIN__ && !defined(MINGW32)
 	struct termios term_raw, term_buf;
@@ -336,16 +457,16 @@ typedef struct r_cons_t {
 	 * current window. If NULL or "" no pager is used. */
 	char *pager;
 	int blankline;
-	int truecolor; // 0 = ansi, 1 = rgb 256), 2 = truecolor (16M)
+	int color; // 0 = none, 1 = ansi (16), 2 = palette (256), 3 = truecolor (16M)
 	char *highlight;
 	int null; // if set, does not show anything
 	int mouse;
 	int is_wine;
-	RConsPalette pal;
+	RConsPalette cpal;
+	RConsPrintablePalette pal;
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
-	RConsClickCallback onclick;
 	bool newline;
 #if __WINDOWS__ && !__CYGWIN__
 	bool ansicon;
@@ -353,16 +474,18 @@ typedef struct r_cons_t {
 	bool flush;
 	bool use_utf8; // use utf8 features
 	bool use_utf8_curvy; // use utf8 curved corners
+	bool dotted_lines;
 	int linesleep;
 	int pagesize;
 	char *break_word;
 	int break_word_len;
 	ut64 timeout;
-	bool use_color;
 	bool grep_color;
+	bool grep_highlight;
 	bool use_tts;
 	bool filter;
-	char* (*rgbstr)(char *str, ut64 addr);
+	char* (*rgbstr)(char *str, size_t sz, ut64 addr);
+	// TODO: move into instance? + avoid unnecessary copies
 } RCons;
 
 // XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
@@ -391,13 +514,16 @@ typedef struct r_cons_t {
 
 #define R_CONS_KEY_ESC 0x1b
 
+#define R_CONS_CLEAR_LINE "\x1b[2K\r"
+#define R_CONS_CLEAR_SCREEN "\x1b[2J\r"
+
 #define Color_BLINK        "\x1b[5m"
 #define Color_INVERT       "\x1b[7m"
 #define Color_INVERT_RESET "\x1b[27m"
 /* plain colors */
-#define Color_RESET      "\x1b[0m"
-#define Color_NOBGRESET  "\x1b[22;24;25;27;28;39m"
-#define Color_BGRESET    "\x1b[49m"
+#define Color_RESET      "\x1b[0m" /* reset all */
+#define Color_RESET_NOBG "\x1b[22;24;25;27;28;39m"  /* Reset everything except background */
+#define Color_RESET_BG   "\x1b[49m"
 #define Color_BLACK      "\x1b[30m"
 #define Color_BGBLACK    "\x1b[40m"
 #define Color_RED        "\x1b[31m"
@@ -410,7 +536,6 @@ typedef struct r_cons_t {
 #define Color_BGMAGENTA  "\x1b[45m"
 #define Color_YELLOW     "\x1b[33m"
 #define Color_BGYELLOW   "\x1b[43m"
-#define Color_BGBYELLOW  "\x1b[1;43m"
 #define Color_CYAN       "\x1b[36m"
 #define Color_BGCYAN     "\x1b[46m"
 #define Color_BLUE       "\x1b[34m"
@@ -429,6 +554,30 @@ typedef struct r_cons_t {
 #define Color_BBLUE     "\x1b[1;34m"
 #define Color_BGRAY     "\x1b[1;38m"
 
+#ifdef _MSC_VER
+#define RCOLOR(a, r, g, b, bgr, bgg, bgb) {0, a, r, g, b, bgr, bgg, bgb}
+#else
+#define RCOLOR(a, r, g, b, bgr, bgg, bgb) (RColor) {0, a, r, g, b, bgr, bgg, bgb}
+#endif
+#define RColor_NULL      RCOLOR(0x00,     0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BLACK     RCOLOR(ALPHA_FG, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGBLACK   RCOLOR(ALPHA_BG, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_RED       RCOLOR(ALPHA_FG, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGRED     RCOLOR(ALPHA_BG, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00)
+#define RColor_WHITE     RCOLOR(ALPHA_FG, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGWHITE   RCOLOR(ALPHA_BG, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_GREEN     RCOLOR(ALPHA_FG, 0x00, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGGREEN   RCOLOR(ALPHA_BG, 0x00, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_MAGENTA   RCOLOR(ALPHA_FG, 0xff, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGMAGENTA RCOLOR(ALPHA_BG, 0xff, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_YELLOW    RCOLOR(ALPHA_FG, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_BGYELLOW  RCOLOR(ALPHA_BG, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00)
+#define RColor_CYAN      RCOLOR(ALPHA_FG, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGCYAN    RCOLOR(ALPHA_BG, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BLUE      RCOLOR(ALPHA_FG, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_BGBLUE    RCOLOR(ALPHA_BG, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00)
+#define RColor_GRAY      RCOLOR(ALPHA_FG, 0x7f, 0x7f, 0x7f, 0x00, 0x00, 0x00)
+#define RColor_BGGRAY    RCOLOR(ALPHA_BG, 0x7f, 0x7f, 0x7f, 0x00, 0x00, 0x00)
 
 #define Colors_PLAIN { \
 	Color_BLACK, Color_RED, Color_WHITE, \
@@ -468,9 +617,14 @@ enum {
 	LINE_NOSYM_HORIZ
 };
 
+#define DOT_STYLE_NORMAL 0
+#define DOT_STYLE_CONDITIONAL 1
+#define DOT_STYLE_BACKEDGE 2
+
 typedef struct r_cons_canvas_line_style_t {
 	int color;
 	int symbol;
+	int dot_style;
 } RCanvasLineStyle;
 
 // UTF-8 symbols indexes
@@ -516,7 +670,7 @@ R_API void r_cons_canvas_line(RConsCanvas *c, int x, int y, int x2, int y2, RCan
 R_API void r_cons_canvas_line_diagonal(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API void r_cons_canvas_line_square(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h);
-R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch, int replace);
+R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch);
 R_API void r_cons_canvas_line_square_defined (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int bendpoint, int isvert);
 R_API void r_cons_canvas_line_back_edge (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
 
@@ -524,12 +678,15 @@ R_API RCons *r_cons_new(void);
 R_API RCons *r_cons_singleton(void);
 R_API RCons *r_cons_free(void);
 R_API char *r_cons_lastline(int *size);
+R_API char *r_cons_lastline_utf8_ansi_len(int *len);
 
 typedef void (*RConsBreak)(void *);
 R_API void r_cons_break_end(void);
 R_API bool r_cons_is_breaked(void);
 R_API void r_cons_break_timeout(int timeout);
 R_API void r_cons_breakword(const char *s);
+R_API void *r_cons_sleep_begin(void);
+R_API void r_cons_sleep_end(void *user);
 
 /* pipe */
 R_API int r_cons_pipe_open(const char *file, int fdn, int append);
@@ -541,8 +698,16 @@ R_API int r_cons_w32_print(const ut8 *ptr, int len, int empty);
 
 R_API void r_cons_push(void);
 R_API void r_cons_pop(void);
+R_API RConsContext *r_cons_context_new(void);
+R_API void r_cons_context_free(RConsContext *context);
+R_API void r_cons_context_load(RConsContext *context);
+R_API void r_cons_context_reset(void);
+R_API bool r_cons_context_is_main(void);
+R_API void r_cons_context_break(RConsContext *context);
+R_API void r_cons_context_break_push(RConsContext *context, RConsBreak cb, void *user, bool sig);
+R_API void r_cons_context_break_pop(RConsContext *context, bool sig);
+R_API void r_cons_break_push(RConsBreak cb, void *user);
 R_API void r_cons_break_pop(void);
-R_API void r_cons_break_push(RConsBreak cb, void*user);
 R_API void r_cons_break_clear(void);
 
 /* control */
@@ -553,6 +718,7 @@ R_API void r_cons_print_clear(void);
 R_API void r_cons_zero(void);
 R_API void r_cons_highlight(const char *word);
 R_API void r_cons_clear(void);
+R_API void r_cons_clear_buffer(void);
 R_API void r_cons_clear00(void);
 R_API void r_cons_clear_line(int err);
 R_API void r_cons_fill_line(void);
@@ -568,16 +734,18 @@ R_API void r_cons_set_interactive(bool b);
 R_API void r_cons_set_last_interactive(void);
 
 /* output */
-R_API void r_cons_printf(const char *format, ...);
+R_API int r_cons_printf(const char *format, ...);
 R_API void r_cons_printf_list(const char *format, va_list ap);
 R_API void r_cons_strcat(const char *str);
 #define r_cons_print(x) r_cons_strcat (x)
 R_API void r_cons_println(const char* str);
+
 R_API void r_cons_strcat_justify(const char *str, int j, char c);
 R_API int r_cons_memcat(const char *str, int len);
 R_API void r_cons_newline(void);
 R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
+R_API void r_cons_last(void);
 R_API int r_cons_less_str(const char *str, const char *exitkeys);
 R_API void r_cons_less(void);
 R_API void r_cons_2048(bool color);
@@ -586,11 +754,16 @@ R_API void r_cons_visual_flush(void);
 R_API void r_cons_visual_write(char *buffer);
 R_API int r_cons_is_utf8(void);
 R_API void r_cons_cmd_help(const char * help[], bool use_color);
+R_API void r_cons_log_stub(const char *output, const char *funcname, const char *filename,
+ unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...);
+
 
 /* input */
-//R_API int  r_cons_fgets(char *buf, int len, int argc, const char **argv);
 R_API int r_cons_controlz(int ch);
 R_API int r_cons_readchar(void);
+R_API bool r_cons_readpush(const char *str, int len);
+R_API void r_cons_readflush(void);
+R_API void r_cons_switchbuf(bool active);
 R_API int r_cons_readchar_timeout(ut32 usec);
 R_API int r_cons_any_key(const char *msg);
 R_API int r_cons_eof(void);
@@ -599,13 +772,14 @@ R_API int r_cons_palette_init(const unsigned char *pal);
 R_API int r_cons_pal_set(const char *key, const char *val);
 R_API void r_cons_pal_update_event(void);
 R_API void r_cons_pal_free(void);
-R_API void r_cons_pal_init(const char *foo);
-R_API char *r_cons_pal_parse(const char *str);
+R_API void r_cons_pal_init(void);
+R_API char *r_cons_pal_parse(const char *str, RColor *outcol);
 R_API void r_cons_pal_random(void);
-R_API const char *r_cons_pal_get(const char *key);
-R_API const char *r_cons_pal_get_i(int n);
-R_API const char *r_cons_pal_get_color(int n);
-R_API int r_cons_rgb_parse(const char *p, ut8 *r, ut8 *g, ut8 *b, int *is_bg);
+R_API RColor r_cons_pal_get(const char *key);
+R_API RColor r_cons_pal_get_i(int index);
+R_API const char *r_cons_pal_get_name(int index);
+R_API int r_cons_pal_len(void);
+R_API int r_cons_rgb_parse(const char *p, ut8 *r, ut8 *g, ut8 *b, ut8 *a);
 R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b);
 R_API void r_cons_pal_list(int rad, const char *arg);
 R_API void r_cons_pal_show(void);
@@ -613,7 +787,6 @@ R_API int r_cons_get_size(int *rows);
 R_API bool r_cons_isatty(void);
 R_API int r_cons_get_cursor(int *rows);
 R_API int r_cons_arrow_to_hjkl(int ch);
-R_API int r_cons_html_print(const char *ptr);
 R_API char *r_cons_html_filter(const char *ptr, int *newlen);
 
 // TODO: use gets() .. MUST BE DEPRECATED
@@ -624,24 +797,26 @@ R_API char *r_cons_hud_string(const char *s);
 R_API char *r_cons_hud_file(const char *f);
 
 R_API const char *r_cons_get_buffer(void);
+R_API int r_cons_get_buffer_len();
 R_API void r_cons_grep_help(void);
 R_API void r_cons_grep_parsecmd(char *cmd, const char *quotestr);
 R_API char * r_cons_grep_strip(char *cmd, const char *quotestr);
 R_API void r_cons_grep_process(char * grep);
 R_API int r_cons_grep_line(char *buf, int len); // must be static
-R_API int r_cons_grepbuf(char *buf, int len);
+R_API void r_cons_grepbuf();
 
-R_API void r_cons_rgb(ut8 r, ut8 g, ut8 b, int is_bg);
+R_API void r_cons_rgb(ut8 r, ut8 g, ut8 b, ut8 a);
 R_API void r_cons_rgb_fgbg(ut8 r, ut8 g, ut8 b, ut8 R, ut8 G, ut8 B);
 R_API void r_cons_rgb_init(void);
-R_API char *r_cons_rgb_str(char *outstr, ut8 r, ut8 g, ut8 b, int is_bg);
-R_API char *r_cons_rgb_str_off(char *outstr, ut64 off);
+R_API char *r_cons_rgb_str(char *outstr, size_t sz, RColor *rcolor);
+R_API char *r_cons_rgb_str_off(char *outstr, size_t sz, ut64 off);
 R_API void r_cons_color(int fg, int r, int g, int b);
-R_API char *r_cons_color_random(int bg);
-R_API char *r_cons_color_random_string(int bg);
+
+R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
-R_API int r_cons_yesno(int def, const char *fmt, ...);
+R_API bool r_cons_yesno(int def, const char *fmt, ...);
 R_API char *r_cons_input(const char *msg);
+R_API char *r_cons_password(const char *msg);
 R_API void r_cons_set_cup(int enable);
 R_API void r_cons_column(int c);
 R_API int r_cons_get_column(void);
@@ -656,6 +831,20 @@ R_API const char* r_cons_get_rune(const ut8 ch);
 #define R_LINE_BUFSIZE 4096
 #define R_LINE_HISTSIZE 256
 
+#define R_EDGES_X_INC 4
+
+#define R_SELWIDGET_MAXH 15
+#define R_SELWIDGET_MAXW 30
+
+typedef struct r_selection_widget_t {
+	const char **options;
+	int options_len;
+	int selection;
+	int w, h;
+	int scroll;
+	bool complete_common;
+} RSelWidget;
+
 typedef struct r_line_hist_t {
 	char **data;
 	int size;
@@ -675,18 +864,26 @@ typedef struct r_line_t RLine; // forward declaration
 typedef int (*RLineCallback)(RLine *line);
 
 typedef struct r_line_comp_t {
+	bool opt;
 	int argc;
 	const char **argv;
 	RLineCallback run;
 } RLineCompletion;
 
 typedef char* (*RLineEditorCb)(void *core, const char *str);
+typedef int (*RLineHistoryUpCb)(RLine* line);
+typedef int (*RLineHistoryDownCb)(RLine* line);
 
 struct r_line_t {
 	RLineCompletion completion;
-	RLineHistory history;
 	RLineBuffer buffer;
-	RLineEditorCb editor_cb;
+	RLineHistory history;
+	RSelWidget *sel_widget;
+	/* callbacks */
+	RLineHistoryUpCb cb_history_up;
+	RLineHistoryDownCb cb_history_down;
+	RLineEditorCb cb_editor;
+	/* state , TODO: use more bool */
 	int echo;
 	int has_echo;
 	char *prompt;
@@ -697,6 +894,15 @@ struct r_line_t {
 	int (*hist_down)(void *user);
 	char *contents;
 	bool zerosep;
+	bool offset_prompt;
+	int offset_hist_index;
+	bool file_prompt;
+	int file_hist_index;
+	RList *sdbshell_hist;
+	RListIter *sdbshell_hist_iter;
+#if __WINDOWS__ && !__CYGWIN__
+	bool ansicon;
+#endif
 }; /* RLine */
 
 #ifdef R_API
@@ -721,6 +927,10 @@ R_API void r_line_label_show(void);
 R_API int r_line_hist_list(void);
 R_API const char *r_line_hist_get(int n);
 
+R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb cb_up, RLineHistoryDownCb cb_down);
+R_API int cmd_history_up(RLine *line);
+R_API int cmd_history_down(RLine *line);
+
 #define R_CONS_INVERT(x,y) (y? (x?Color_INVERT: Color_INVERT_RESET): (x?"[":"]"))
 
 #endif
@@ -772,8 +982,10 @@ typedef struct r_ascii_graph_t {
 	int edgemode;
 	int mode;
 	bool is_callgraph;
+	bool is_interactive;
 	int zoom;
 	int movspeed;
+	bool hints;
 
 	RANode *update_seek_on;
 	bool need_reload_nodes;
@@ -795,13 +1007,6 @@ typedef struct r_ascii_graph_t {
 	int n_layers;
 	RList *dists; /* RList<struct dist_t> */
 	RList *edges; /* RList<AEdge> */
-
-	/* colors */
-	const char *color_box;
-	const char *color_box2;
-	const char *color_box3;
-	const char *color_true;
-	const char *color_false;
 } RAGraph;
 
 #ifdef R_API
@@ -817,12 +1022,70 @@ R_API void r_agraph_add_edge(const RAGraph *g, RANode *a, RANode *b);
 R_API void r_agraph_add_edge_at(const RAGraph *g, RANode *a, RANode *b, int nth);
 R_API void r_agraph_del_edge(const RAGraph *g, RANode *a, RANode *b);
 R_API void r_agraph_print(RAGraph *g);
+R_API void r_agraph_print_json(RAGraph *g);
 R_API Sdb *r_agraph_get_sdb(RAGraph *g);
 R_API void r_agraph_foreach(RAGraph *g, RANodeCallback cb, void *user);
 R_API void r_agraph_foreach_edge(RAGraph *g, RAEdgeCallback cb, void *user);
 R_API void r_agraph_set_curnode(RAGraph *g, RANode *node);
 #endif
 
+typedef int (*RPanelsMenuCallback)(void *user);
+typedef struct r_panels_menu_item {
+	int n_sub, selectedIndex;
+	const char *name;
+	struct r_panels_menu_item **sub;
+	RPanelsMenuCallback cb;
+	RPanel *p;
+} RPanelsMenuItem;
+
+typedef struct r_panels_menu_t {
+	RPanelsMenuItem *root;
+	RPanelsMenuItem **history;
+	int depth;
+	int n_refresh;
+	RPanel **refreshPanels;
+} RPanelsMenu;
+
+typedef enum {
+	PANEL_MODE_DEFAULT,
+	PANEL_MODE_MENU,
+	PANEL_MODE_ZOOM,
+	PANEL_MODE_WINDOW,
+	PANEL_MODE_HELP
+} RPanelsMode;
+
+typedef enum {
+	PANEL_FUN_SNOW,
+	PANEL_FUN_NOFUN
+} RPanelsFun;
+
+typedef enum {
+	PANEL_LAYOUT_DEFAULT_STATIC = 0,
+	PANEL_LAYOUT_DEFAULT_DYNAMIC = 1
+} RPanelsLayout;
+
+typedef struct {
+	int x;
+	int y;
+} RPanelsSnow;
+
+typedef struct r_panels_t {
+	RConsCanvas *can;
+	RPanel *panel;
+	int n_panels;
+	int columnWidth;
+	int curnode;
+	bool isResizing;
+	RPanelsMenu *panelsMenu;
+	Sdb *db;
+	HtPP *mht;
+	RPanelsMode mode;
+	RPanelsFun fun;
+	RPanelsMode prevMode;
+	RPanelsLayout layout;
+	RList *snows;
+} RPanels;
+
 #ifdef __cplusplus
 }
 #endif
