@@ -6,10 +6,10 @@
 #include <libbochs.h>
 
 typedef struct {
-	libbochs_t desc;    
+	libbochs_t desc;
 } RIOBochs;
 
-static libbochs_t *desc = NULL; 
+static libbochs_t *desc = NULL;
 static RIODesc *riobochs = NULL;
 extern RIOPlugin r_io_plugin_bochs; // forward declaration
 
@@ -39,7 +39,6 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 	if (i) {
 		l = i - file - 8;
 		fileBochs = r_str_ndup (file + 8, l);
-		l = strlen (i + 1);
 		fileCfg = strdup (i + 1);
 	} else {
 		free (fileCfg);
@@ -77,9 +76,10 @@ static ut64 __lseek(RIO *io, RIODesc *fd, ut64 offset, int whence) {
 static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 	memset (buf, 0xff, count);
 	ut64 addr = io->off;
-	if (!desc || !desc->data) 
+	if (!desc || !desc->data) {
 		return -1;
-        lprintf ("io_read ofs= %016"PFMT64x" count= %x\n", io->off, count);
+	}
+	lprintf ("io_read ofs= %016"PFMT64x" count= %x\n", io->off, count);
 	bochs_read (desc,addr,count,buf);
 	return count;
 }
@@ -89,7 +89,7 @@ static int __close(RIODesc *fd) {
 	bochs_close (desc);
 	return true;
 }
-	
+
 static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
         lprintf ("system command (%s)\n", cmd);
         if (!strcmp (cmd, "help")) {
@@ -102,7 +102,7 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 	} else if (!strncmp (cmd, "dobreak", 7)) {
 		bochs_cmd_stop (desc);
 		io->cb_printf ("%s\n", desc->data);
-	}         
+	}
         return NULL;
 }
 
@@ -121,7 +121,7 @@ RIOPlugin r_io_plugin_bochs = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_bochs,
 	.version = R2_VERSION
