@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2008-2017 - pancake */
+/* radare - LGPL - Copyright 2008-2018 - pancake */
 
 #include <r_userconf.h>
 #include <r_io.h>
@@ -8,7 +8,7 @@
 typedef struct r_io_mmo_t {
 	char * filename;
 	int mode;
-	int flags;
+	int perm;
 	int fd;
 	int opened;
 	bool nocache;
@@ -18,7 +18,7 @@ typedef struct r_io_mmo_t {
 	int rawio;
 } RIOMMapFileObj;
 
-static int __io_posix_open(const char *file, int flags, int mode) {
+static int __io_posix_open(const char *file, int perm, int mode) {
 	int fd;
 	if (r_str_startswith (file, "file://")) {
 		file += strlen ("file://");
@@ -27,17 +27,20 @@ static int __io_posix_open(const char *file, int flags, int mode) {
 		return -1;
 	}
 #if __WINDOWS__
-	if (flags & R_IO_WRITE) {
+	// probably unnecessary to have this ifdef nowadays windows is posix enough
+	if (perm & R_PERM_W) {
 		fd = r_sandbox_open (file, O_BINARY | O_RDWR, 0);
-		if (fd == -1) {
+		if (fd == -1 && perm & R_PERM_CREAT) {
 			r_sandbox_creat (file, 0644);
-			fd = r_sandbox_open (file, O_BINARY | O_RDWR, 0);
+			fd = r_sandbox_open (file, O_BINARY | O_RDWR | O_CREAT, 0);
 		}
 	} else {
 		fd = r_sandbox_open (file, O_BINARY, 0);
 	}
 #else
-	fd = r_sandbox_open (file, (flags & R_IO_WRITE) ? (O_RDWR|O_CREAT) : O_RDONLY, mode);
+	const int posixFlags = (perm & R_PERM_W) ? (perm & R_PERM_CREAT)
+			? (O_RDWR | O_CREAT) : O_RDWR : O_RDONLY;
+	fd = r_sandbox_open (file, posixFlags, mode);
 #endif
 	return fd;
 }
@@ -87,7 +90,7 @@ static int r_io_def_mmap_refresh_def_mmap_buf(RIOMMapFileObj *mmo) {
 		mmo->rawio = 1;
 	}
 	if (mmo->rawio) {
-		mmo->fd = __io_posix_open (mmo->filename, mmo->flags, mmo->mode);
+		mmo->fd = __io_posix_open (mmo->filename, mmo->perm, mmo->mode);
 		if (mmo->nocache) {
 #ifdef F_NOCACHE
 			fcntl (mmo->fd, F_NOCACHE, 1);
@@ -95,13 +98,13 @@ static int r_io_def_mmap_refresh_def_mmap_buf(RIOMMapFileObj *mmo) {
 		}
 		return (mmo->fd != -1);
 	}
-	mmo->buf = r_buf_mmap (mmo->filename, mmo->flags);
+	mmo->buf = r_buf_mmap (mmo->filename, mmo->perm);
 	if (mmo->buf) {
 		r_io_def_mmap_seek (io, mmo, cur, SEEK_SET);
 		return true;
 	} else {
 		mmo->rawio = 1;
-		mmo->fd = __io_posix_open (mmo->filename, mmo->flags, mmo->mode);
+		mmo->fd = __io_posix_open (mmo->filename, mmo->perm, mmo->mode);
 		if (mmo->nocache) {
 #ifdef F_NOCACHE
 			fcntl (mmo->fd, F_NOCACHE, 1);
@@ -120,7 +123,7 @@ static void r_io_def_mmap_free (RIOMMapFileObj *mmo) {
 	free (mmo);
 }
 
-RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, int mode, int flags) {
+RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, int mode, int perm) {
 	if (!io) {
 		return NULL;
 	}
@@ -134,13 +137,11 @@ RIOMMapFileObj *r_io_def_mmap_create_new_file(RIO  *io, const char *filename, in
 	}
 	mmo->filename = strdup (filename);
 	mmo->mode = mode;
-	mmo->flags = flags;
+	mmo->perm = perm;
 	mmo->io_backref = io;
-	if (flags & R_IO_WRITE) {
-		mmo->fd = r_sandbox_open (filename, O_CREAT|O_RDWR, mode);
-	} else {
-		mmo->fd = r_sandbox_open (filename, O_RDONLY, mode);
-	}
+	const int posixFlags = (perm & R_PERM_W) ? (perm & R_PERM_CREAT)
+			? (O_RDWR | O_CREAT) : O_RDWR : O_RDONLY;
+	mmo->fd = r_sandbox_open (filename, posixFlags, mode);
 	if (mmo->fd == -1) {
 		free (mmo->filename);
 		free (mmo);
@@ -273,7 +274,11 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 				memcpy (a_buf+a_delta, buf, count);
 				for (i = 0; i < a_count; i += aligned) {
 					(void)lseek (mmo->fd, a_off + i, SEEK_SET);
-					(void)write (mmo->fd, a_buf + i, aligned);
+					len = write (mmo->fd, a_buf + i, aligned);
+					if (len != aligned) {
+						free (a_buf);
+						return len;
+					}
 				}
 			}
 			free (a_buf);
@@ -287,7 +292,9 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 	}
 
 	if (mmo && mmo->buf) {
-		if (!(mmo->flags & R_IO_WRITE)) return -1;
+		if (!(mmo->perm & R_PERM_W)) {
+			return -1;
+		}
 		if ( (count + addr > mmo->buf->length) || mmo->buf->empty) {
 			ut64 sz = count + addr;
 			r_file_truncate (mmo->filename, sz);
@@ -309,12 +316,12 @@ static int r_io_def_mmap_write(RIO *io, RIODesc *fd, const ut8 *buf, int count)
 	return len;
 }
 
-static RIODesc *r_io_def_mmap_open(RIO *io, const char *file, int flags, int mode) {
-	RIOMMapFileObj *mmo = r_io_def_mmap_create_new_file (io, file, mode, flags);
+static RIODesc *r_io_def_mmap_open(RIO *io, const char *file, int perm, int mode) {
+	RIOMMapFileObj *mmo = r_io_def_mmap_create_new_file (io, file, mode, perm);
 	if (!mmo) {
 		return NULL;
 	}
-	return r_io_desc_new (io, &r_io_plugin_default, mmo->filename, flags, mode, mmo);
+	return r_io_desc_new (io, &r_io_plugin_default, mmo->filename, perm, mode, mmo);
 }
 
 
@@ -340,17 +347,17 @@ static bool __plugin_open_default(RIO *io, const char *file, bool many) {
 }
 
 // default open should permit opening
-static RIODesc *__open_default(RIO *io, const char *file, int flags, int mode) {
+static RIODesc *__open_default(RIO *io, const char *file, int perm, int mode) {
 	if (r_str_startswith (file, "file://")) {
 		file += strlen ("file://");
 	}
 	if (!r_io_def_mmap_check_default (file)) {
 		return NULL;
 	}
-	RIODesc *iod = r_io_def_mmap_open (io, file, flags, mode);
+	RIODesc *iod = r_io_def_mmap_open (io, file, perm, mode);
 	return iod;
 // NTOE: uncomment this line to support loading files in ro as fallback is rw fails
-//	return iod? iod: r_io_def_mmap_open (io, file, R_IO_READ, mode);
+//	return iod? iod: r_io_def_mmap_open (io, file, R_PERM_R, mode);
 }
 
 static int __read(RIO *io, RIODesc *fd, ut8 *buf, int len) {
@@ -374,7 +381,7 @@ static bool __resize(RIO *io, RIODesc *fd, ut64 size) {
 		return false;
 	}
 	RIOMMapFileObj *mmo = fd->data;
-	if (!(mmo->flags & R_IO_WRITE)) {
+	if (!(mmo->perm & R_PERM_W)) {
 		return false;
 	}
 	return r_io_def_mmap_truncate (mmo, size);
@@ -417,7 +424,7 @@ RIOPlugin r_io_plugin_default = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_default,
 	.version = R2_VERSION
