@@ -2,7 +2,8 @@
 
 #include <r_io.h>
 #include <r_lib.h>
-#include <r_print.h>
+#include <r_util.h>
+#include <r_util/r_print.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -23,10 +24,14 @@
 #include <tchar.h>
 #include <windows.h>
 #else
-#if !__linux__ && !__APPLE__ && !__OpenBSD__ && !__FreeBSD__
+
+#if __linux__ ||  __APPLE__ || __OpenBSD__ || __FreeBSD__ || __NetBSD__ || __DragonFly__
+#include <sys/ioctl.h>
+#include <termios.h>
+#else
 #include <stropts.h>
 #endif
-#include <termios.h>
+
 #endif
 
 #define GPROBE_SIZE (1LL << 32)
@@ -80,8 +85,9 @@ static ut8 gprobe_checksum_i2c (const ut8 *p, unsigned int size, ut8 initial) {
 	ut8 res = initial;
 	unsigned int k;
 
-	for (k = 0; k < size; ++k)
+	for (k = 0; k < size; ++k) {
 		res ^= p[k];
+	}
 
 	return res;
 }
@@ -1088,16 +1094,13 @@ static ut64 __lseek (RIO *io, RIODesc *fd, ut64 offset, int whence) {
 			return gprobe->offset = GPROBE_SIZE - 1;
 		}
 		return gprobe->offset = offset;
-		break;
 	case SEEK_CUR:
 		if ((gprobe->offset + offset) >= GPROBE_SIZE) {
 			return gprobe->offset = GPROBE_SIZE - 1;
 		}
 		return gprobe->offset += offset;
-		break;
 	case SEEK_END:
 		return gprobe->offset = GPROBE_SIZE - 1;
-		break;
 	}
 	return offset;
 }
@@ -1229,7 +1232,7 @@ RIOPlugin r_io_plugin_gprobe = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_gprobe,
 	.version = R2_VERSION};
