@@ -3,7 +3,6 @@
 #include <r_io.h>
 #include <r_lib.h>
 #include <r_types.h>
-#include <r_print.h>
 #include <r_util.h>
 #include <sys/types.h>
 
@@ -102,8 +101,7 @@ static char *r2k__system(RIO *io, RIODesc *fd, const char *cmd) {
 
 static RIODesc *r2k__open(RIO *io, const char *pathname, int rw, int mode) {
 	if (!strncmp (pathname, "r2k://", 6)) {
-		rw |= R_IO_WRITE;
-		rw |= R_IO_EXEC;
+		rw |= R_PERM_WX;
 #if __WINDOWS__
 		RIOW32 *w32 = R_NEW0 (RIOW32);
 		if (Init (&pathname[6]) == FALSE) {
@@ -145,7 +143,7 @@ RIOPlugin r_io_plugin_r2k = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_r2k,
 	.version = R2_VERSION
