@@ -58,7 +58,7 @@ RIOPlugin r_io_plugin_rbuf = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_rbuf,
 	.version = R2_VERSION
