@@ -46,7 +46,7 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 		eprintf ("Failed to initialize windbg context\n");
 		return NULL;
 	}
-	return r_io_desc_new (io, &r_io_plugin_windbg, file, true, mode, ctx);
+	return r_io_desc_new (io, &r_io_plugin_windbg, file, rw, mode, ctx);
 }
 
 static int __write(RIO *io, RIODesc *fd, const ut8 *buf, int count) {
@@ -83,8 +83,9 @@ static int __read(RIO *io, RIODesc *fd, ut8 *buf, int count) {
 
 	if (windbg_get_target (fd->data)) {
 		ut64 va;
-		if (!windbg_va_to_pa (fd->data, io->off, &va))
+		if (!windbg_va_to_pa (fd->data, io->off, &va)) {
 			return -1;
+		}
 		return windbg_read_at_phys (fd->data, buf, va, count);
 	}
 
@@ -110,7 +111,7 @@ RIOPlugin r_io_plugin_windbg = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_windbg,
 	.version = R2_VERSION
