@@ -339,8 +339,10 @@ const char *msg =
 				}
 				char *row = r_str_newf ("0x%08"PFMT64x" - 0x%08" PFMT64x" %s %s\n", from, to, perm, "");
 				ptr = nl;
-				res = r_str_append (res, row);
-				free (row);
+				if (row) {
+					res = r_str_append (res, row);
+					free (row);
+				}
 			}
 			free (wineDbgMaps);
 			return res;
@@ -368,7 +370,7 @@ RIOPlugin r_io_plugin_winedbg = {
 };
 
 #ifndef CORELIB
-RLibStruct radare_plugin = {
+R_API RLibStruct radare_plugin = {
 	.type = R_LIB_TYPE_IO,
 	.data = &r_io_plugin_winedbg,
 	.version = R2_VERSION
