@@ -1,7 +1,8 @@
-/* radare - LGPL - Copyright 2006-2014 - esteve, pancake */
+/* radare - LGPL - Copyright 2006-2018 - esteve, pancake */
 
 #include "r_search.h"
-#include "r_print.h"
+#include "r_util.h"
+#include "r_util/r_print.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -19,7 +20,9 @@ typedef struct _fnditem {
 static fnditem* init_fi() {
 	fnditem* n;
 	n = (fnditem*) malloc (sizeof (fnditem));
-	if (!n) return NULL;
+	if (!n) {
+		return NULL;
+	}
 	n->next = NULL;
 	return n;
 }
@@ -38,7 +41,9 @@ static void fini_fi(fnditem* fi) {
 
 static void add_fi (fnditem* n, unsigned char* blk, int patlen) {
 	fnditem* p;
-	for (p=n; p->next!=NULL; p=p->next);
+	for (p = n; p->next != NULL; p = p->next) {
+		;
+	}
 	p->next = (fnditem*) malloc (sizeof (fnditem));
 	p = p->next;
 	memcpy (p->str, blk, patlen);
@@ -47,9 +52,11 @@ static void add_fi (fnditem* n, unsigned char* blk, int patlen) {
 
 static int is_fi_present(fnditem* n, unsigned char* blk , int patlen) {
 	fnditem* p;
-	for (p=n; p->next!=NULL; p=p->next)
-		if (!memcmp (blk, p->str, patlen))
+	for (p = n; p->next != NULL; p = p->next) {
+		if (!memcmp (blk, p->str, patlen)) {
 			return true;
+		}
+	}
 	return false;
 }
 
@@ -108,8 +115,9 @@ R_API int r_search_pattern(RSearch *s, ut64 from, ut64 to) {
 						add_fi (root, sblk, patlen);
 						pcnt++;
 						eprintf ("\nbytes: %d: ", pcnt);
-						for (k = 0; k<patlen; k++)
+						for (k = 0; k < patlen; k++) {
 							eprintf ("%02x", sblk[k]);
+						}
 						eprintf ("\nfound: %d: 0x%08"PFMT64x" ", pcnt, intaddr);
 					}
 					moar++;
@@ -117,8 +125,9 @@ R_API int r_search_pattern(RSearch *s, ut64 from, ut64 to) {
 					eprintf ("0x%08"PFMT64x" ", bproc+i);
 				}
 			}
-			if (moar>0)
-				eprintf ("\ncount: %d: %d\n", pcnt, moar+1);
+			if (moar > 0) {
+				eprintf ("\ncount: %d: %d\n", pcnt, moar + 1);
+			}
 			bproc += rb;
 		}
 		bact += (moar > 0)? patlen: 1;
