@@ -34,15 +34,41 @@ static inline RNumCalcValue Nmul(RNumCalcValue n, RNumCalcValue v) {
 
 static inline RNumCalcValue Nshl(RNumCalcValue n, RNumCalcValue v) { n.d += v.d; n.n <<= v.n; return n; }
 static inline RNumCalcValue Nshr(RNumCalcValue n, RNumCalcValue v) { n.d += v.d; n.n >>= v.n; return n; }
+static inline RNumCalcValue Nrol(RNumCalcValue n, RNumCalcValue v) {
+	n.d += v.d;
+	n.n = (n.n << v.n) | (n.n >> (sizeof(n.n) * 8 - v.n));
+	return n;
+}
+static inline RNumCalcValue Nror(RNumCalcValue n, RNumCalcValue v) {
+	n.d += v.d;
+	n.n = (n.n >> v.n) | (n.n << (sizeof(n.n) * 8 - v.n));
+	return n;
+}
 static inline RNumCalcValue Nmod(RNumCalcValue n, RNumCalcValue v) {
-	if (v.d) n.d = (n.d - (n.d/v.d)); else n.d = 0;
-	if (v.n) n.n %= v.n; else n.n = 0;
+	if (v.d) {
+		n.d = (n.d - (n.d / v.d));
+	} else {
+		n.d = 0;
+	}
+	if (v.n) {
+		n.n %= v.n;
+	} else {
+		n.n = 0;
+	}
 	return n;
 }
 
 static inline RNumCalcValue Ndiv(RNumCalcValue n, RNumCalcValue v) {
-	if (v.d) n.d /= v.d; else n.d = 0;
-	if (v.n) n.n /= v.n; else n.n = 0;
+	if (v.d) {
+		n.d /= v.d;
+	} else {
+		n.d = 0;
+	}
+	if (v.n) {
+		n.n /= v.n;
+	} else {
+		n.n = 0;
+	}
 	return n;
 }
 
@@ -64,6 +90,8 @@ static RNumCalcValue expr(RNum *num, RNumCalc *nc, int get) {
 		switch (nc->curr_tok) {
 		case RNCSHL: left = Nshl (left, term (num, nc, 1)); break;
 		case RNCSHR: left = Nshr (left, term (num, nc, 1)); break;
+		case RNCROL: left = Nrol (left, term (num, nc, 1)); break;
+		case RNCROR: left = Nror (left, term (num, nc, 1)); break;
 		case RNCPLUS: left = Nadd (left, term (num, nc, 1)); break;
 		case RNCMINUS: left = Nsub (left, term (num, nc, 1)); break;
 		case RNCXOR: left = Nxor (left, term (num, nc, 1)); break;
@@ -128,7 +156,6 @@ static RNumCalcValue prim(RNum *num, RNumCalc *nc, int get) {
 		}
 		return v;
 	case RNCNEG:
-		v = nc->number_value;
 		get_token (num, nc);
 		return Nneg (nc->number_value); //prim (num, nc, 1), 1);
 	case RNCINC:
@@ -158,6 +185,8 @@ static RNumCalcValue prim(RNum *num, RNumCalc *nc, int get) {
 	case RNCRIGHTP:
 	case RNCSHL:
 	case RNCSHR:
+	case RNCROL:
+	case RNCROR:
 		return v;
 	//default: error (num, nc, "primary expected");
 	}
@@ -200,7 +229,7 @@ static int cin_get(RNum *num, RNumCalc *nc, char *c) {
 
 static int cin_get_num(RNum *num, RNumCalc *nc, RNumCalcValue *n) {
 	double d;
-	char str[R_NUMCALC_STRSZ]; // TODO: move into the heap?
+	char str[R_NUMCALC_STRSZ + 1]; // TODO: move into the heap?
 	int i = 0;
 	char c;
 	str[0] = 0;
@@ -261,6 +290,26 @@ static RNumCalcToken get_token(RNum *num, RNumCalc *nc) {
 		}
 		cin_putback (num, nc, c);
 		return nc->curr_tok = (RNumCalcToken) ch;
+	case '<':
+		if (cin_get (num, nc, &c) && c == '<') {
+			if (cin_get (num, nc, &c) && c == '<') {
+				return nc->curr_tok = RNCROL;
+			}
+			cin_putback (num, nc, c);
+			return nc->curr_tok = RNCSHL;
+		}
+		cin_putback (num, nc, c);
+		return nc->curr_tok = RNCEND;
+	case '>':
+		if (cin_get (num, nc, &c) && c == '>') {
+			if (cin_get (num, nc, &c) && c == '>') {
+				return nc->curr_tok = RNCROR;
+			}
+			cin_putback (num, nc, c);
+			return nc->curr_tok = RNCSHR;
+		}
+		cin_putback (num, nc, c);
+		return nc->curr_tok = RNCEND;
 	case '^':
 	case '&':
 	case '|':
@@ -269,8 +318,6 @@ static RNumCalcToken get_token(RNum *num, RNumCalc *nc) {
 	case '/':
 	case '(':
 	case ')':
-	case '<':
-	case '>':
 	case '=':
 		return nc->curr_tok = (RNumCalcToken) ch;
 	case '0': case '1': case '2': case '3': case '4':
@@ -284,8 +331,8 @@ static RNumCalcToken get_token(RNum *num, RNumCalc *nc) {
 		return nc->curr_tok = RNCNUMBER;
 
 #define isvalidchar(x) \
-	(isalnum(x) || x==':' || x=='$' || x=='.' || x=='_' || x=='?' || x=='\\' \
-	|| x==' ' || x=='[' || x==']' || x=='}' || x=='{' || (x>='0'&&x<='9'))
+	(isalnum(x) || (x)==':' || (x)=='$' || (x)=='.' || (x)=='_' || (x)=='?' || (x)=='\\' \
+	|| (x)==' ' || (x)=='[' || (x)==']' || (x)=='}' || (x)=='{' || ((x)>='0'&&(x)<='9'))
 
 	default:
 		{
@@ -352,6 +399,7 @@ R_API ut64 r_num_calc(RNum *num, const char *str, const char **err) {
 	nc->calc_i = 0;
 	nc->calc_len = 0;
 	nc->calc_buf = NULL;
+	nc->under_calc = true;
 
 	load_token (num, nc, str);
 	get_token (num, nc);
@@ -362,6 +410,7 @@ R_API ut64 r_num_calc(RNum *num, const char *str, const char **err) {
 	if (num) {
 		num->fvalue = n.d;
 	}
+	nc->under_calc = false;
 	return n.n;
 }
 
