@@ -20,8 +20,9 @@ R_API int r_file_chmod (const char *file, const char *mod, int recursive) {
 #if __UNIX__
 	oper = '=';
 	mode = 0;
-	if (!parsemode (mod))
+	if (!parsemode (mod)) {
 		return false;
+	}
 	return chmodr (file, recursive);
 #else
 	return -1;
@@ -33,8 +34,9 @@ R_API int r_file_chmod (const char *file, const char *mod, int recursive) {
 int chmodr(const char *path, int rflag) {
 	struct stat st;
 
-	if (stat (path, &st) == -1)
+	if (stat (path, &st) == -1) {
 		return 0;
+	}
 
 	switch (oper) {
 	case '+':
@@ -51,8 +53,9 @@ int chmodr(const char *path, int rflag) {
 		eprintf ("chmod %s:", path);
 		return false;
 	}
-	if (rflag)
+	if (rflag) {
 		recurse (path, rflag, chmodr);
+	}
 	return true;
 }
 
@@ -64,20 +67,42 @@ int parsemode(const char *str) {
 
 	octal = strtol(str, &end, 8);
 	if (*end == '\0') {
-		if (octal & 04000) mode |= S_ISUID;
-		if (octal & 02000) mode |= S_ISGID;
-		if (octal & 00400) mode |= S_IRUSR;
-		if (octal & 00200) mode |= S_IWUSR;
-		if (octal & 00100) mode |= S_IXUSR;
-		if (octal & 00040) mode |= S_IRGRP;
-		if (octal & 00020) mode |= S_IWGRP;
-		if (octal & 00010) mode |= S_IXGRP;
-		if (octal & 00004) mode |= S_IROTH;
-		if (octal & 00002) mode |= S_IWOTH;
-		if (octal & 00001) mode |= S_IXOTH;
+		if (octal & 04000) {
+			mode |= S_ISUID;
+		}
+		if (octal & 02000) {
+			mode |= S_ISGID;
+		}
+		if (octal & 00400) {
+			mode |= S_IRUSR;
+		}
+		if (octal & 00200) {
+			mode |= S_IWUSR;
+		}
+		if (octal & 00100) {
+			mode |= S_IXUSR;
+		}
+		if (octal & 00040) {
+			mode |= S_IRGRP;
+		}
+		if (octal & 00020) {
+			mode |= S_IWGRP;
+		}
+		if (octal & 00010) {
+			mode |= S_IXGRP;
+		}
+		if (octal & 00004) {
+			mode |= S_IROTH;
+		}
+		if (octal & 00002) {
+			mode |= S_IWOTH;
+		}
+		if (octal & 00001) {
+			mode |= S_IXOTH;
+		}
 		return true;
 	}
-	for(p = str; *p; p++)
+	for (p = str; *p; p++) {
 		switch(*p) {
 		/* masks */
 		case 'u':
@@ -116,17 +141,22 @@ int parsemode(const char *str) {
 			eprintf ("%s: invalid mode\n", str);
 			return false;
 		}
-	if (mask)
+	}
+	if (mask) {
 		mode &= mask;
+	}
 	return true;
 }
 
 char * agetcwd(void) {
         char *buf = malloc (4096);
-	if (!buf) return NULL;
-        if(!getcwd(buf, 4096))
-                eprintf("getcwd:");
-        return buf;
+	if (!buf) {
+		return NULL;
+	}
+	if (!getcwd (buf, 4096)) {
+		eprintf ("getcwd:");
+	}
+	return buf;
 }
 
 static void recurse(const char *path, int rec, int (*fn)(const char *,int)) {
@@ -135,10 +165,10 @@ static void recurse(const char *path, int rec, int (*fn)(const char *,int)) {
         struct stat st;
         DIR *dp;
 
-        if (lstat (path, &st) == -1 || !S_ISDIR (st.st_mode))
-                return;
-        else if (!(dp = opendir (path))) {
-                eprintf ("opendir %s:", path);
+	if (lstat (path, &st) == -1 || !S_ISDIR (st.st_mode)) {
+		return;
+	} else if (!(dp = opendir (path))) {
+		eprintf ("opendir %s:", path);
 		return;
 	}
         cwd = agetcwd();
@@ -148,13 +178,16 @@ static void recurse(const char *path, int rec, int (*fn)(const char *,int)) {
 		free (cwd);
 		return;
 	}
-        while ((d = readdir (dp)))
-                if (strcmp (d->d_name, ".") && strcmp (d->d_name, ".."))
-                        fn (d->d_name, 1);
+	while ((d = readdir (dp))) {
+		if (strcmp (d->d_name, ".") && strcmp (d->d_name, "..")) {
+			fn (d->d_name, 1);
+		}
+	}
 
-        closedir (dp);
-        if (chdir (cwd) == -1)
-                eprintf ("chdir %s:", cwd);
-        free (cwd);
+	closedir (dp);
+	if (chdir (cwd) == -1) {
+		eprintf ("chdir %s:", cwd);
+	}
+	free (cwd);
 }
 #endif
