@@ -2,7 +2,7 @@
 
 #include "r_cons.h"
 #include "r_util.h"
-#include "r_print.h"
+#include "r_util/r_print.h"
 #include "r_reg.h"
 #ifdef _MSC_VER
 #include <time.h>
@@ -18,13 +18,14 @@
 #define ARRAYINDEX_COEF 10000
 
 #define MUSTSEE (mode & R_PRINT_MUSTSEE && mode & R_PRINT_ISFIELD && !(mode & R_PRINT_JSON))
+#define ISQUIET (mode & R_PRINT_QUIET)
 #define MUSTSET (mode & R_PRINT_MUSTSET && mode & R_PRINT_ISFIELD && setval)
 #define SEEVALUE (mode & R_PRINT_VALUE)
 #define MUSTSEEJSON (mode & R_PRINT_JSON && mode & R_PRINT_ISFIELD)
 
 //this define is used as a way to acknowledge when updateAddr should take len
 //as real len of the buffer
-#define THRESHOLD -4444
+#define THRESHOLD (-4444)
 
 //TODO REWRITE THIS IS BECOMING A NIGHTMARE
 
@@ -78,7 +79,7 @@ static void r_print_format_quadword(const RPrint* p, int endian, int mode,
 	if (MUSTSET) {
 		p->cb_printf ("wv8 %s @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem*8:0));
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) {
+		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = (qword)",
 				seeki + ((elem >= 0)? elem * 2: 0));
 		}
@@ -92,7 +93,9 @@ static void r_print_format_quadword(const RPrint* p, int endian, int mode,
 				updateAddr (buf + i, size - i, endian, NULL, &addr64);
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("0x%016"PFMT64x, addr64);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
@@ -102,7 +105,9 @@ static void r_print_format_quadword(const RPrint* p, int endian, int mode,
 				}
 				i += 8;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 	} else if (MUSTSEEJSON) {
 		if (size==-1) {
@@ -113,12 +118,16 @@ static void r_print_format_quadword(const RPrint* p, int endian, int mode,
 				updateAddr (buf + i, size - i, endian, NULL, &addr64);
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%"PFMT64d, addr64);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
 				}
-				if (elem > -1) elem--;
+				if (elem > -1) {
+					elem--;
+				}
 				i += 8;
 			}
 			p->cb_printf (" ]");
@@ -137,24 +146,33 @@ static void r_print_format_byte(const RPrint* p, int endian, int mode,
 	if (MUSTSET) {
 		p->cb_printf ("\"w %s\" @ 0x%08"PFMT64x"\n", setval, seeki + ((elem >= 0) ? elem : 0));
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) {
+		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki + ((elem >= 0) ? elem : 0));
 		}
 		if (size == -1) {
 			p->cb_printf ("0x%02x", buf[i]);
 		} else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("0x%02x", buf[i]);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i++;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 	} else if (MUSTSEEJSON) {
 		if (size==-1) {
@@ -164,7 +182,9 @@ static void r_print_format_byte(const RPrint* p, int endian, int mode,
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					p->cb_printf (", %d", buf[i]);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
@@ -204,25 +224,36 @@ static int r_print_format_uleb(const RPrint* p, int endian, int mode,
 		free (tmp);
 		free (nbr);
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki);
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki);
+		}
 		if (size==-1) {
 			r_uleb128_decode (buf+i, &offset, &value);
 			p->cb_printf ("%"PFMT64d, value);
 		} else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					r_uleb128_decode (buf+i, &s, &value);
 					i += s;
 					offset += s;
 					p->cb_printf ("%"PFMT64d, value);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
+			}
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
 		}
 	} else if (MUSTSEEJSON) {
 		if (size==-1) {
@@ -236,11 +267,16 @@ static int r_print_format_uleb(const RPrint* p, int endian, int mode,
 					i += s;
 					offset += s;
 					p->cb_printf ("\"%"PFMT64d"\"", value);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 			}
 			p->cb_printf (" ]");
 		}
@@ -259,36 +295,52 @@ static void r_print_format_char(const RPrint* p, int endian, int mode,
 	if (MUSTSET) {
 		p->cb_printf ("\"w %s\" @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem:0));
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*2:0)); //XXX:: shouldn't it be elem*1??
-		if (size==-1)
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 2 : 0)); //XXX:: shouldn't it be elem*1??
+		}
+		if (size == -1) {
 			p->cb_printf ("'%c'", IS_PRINTABLE (buf[i])?buf[i]:'.');
-		else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+		} else {
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("'%c'", IS_PRINTABLE (buf[i])?buf[i]:'.');
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i++;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 	} else if (MUSTSEEJSON) {
-		if (size==-1)
+		if (size == -1) {
 			p->cb_printf ("\"%c\"", IS_PRINTABLE (buf[i])?buf[i]:'.');
-		else {
+		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("\"%c\"", IS_PRINTABLE (buf[i])?buf[i]:'.');
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i++;
 			}
 			p->cb_printf (" ]");
@@ -307,36 +359,52 @@ static void r_print_format_decchar(const RPrint* p, int endian, int mode,
 	if (MUSTSET) {
 		p->cb_printf ("\"w %s\" @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem:0));
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem:0));
-		if (size==-1)
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem : 0));
+		}
+		if (size == -1) {
 			p->cb_printf ("%d", buf[i]);
-		else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+		} else {
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%d", buf[i]);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i++;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 	} else if (MUSTSEEJSON) {
-		if (size==-1)
+		if (size == -1) {
 			p->cb_printf ("\"%d\"", buf[i]);
-		else {
+		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("\"%d\"", buf[i]);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i++;
 			}
 			p->cb_printf (" ]");
@@ -362,8 +430,12 @@ static int r_print_format_string(const RPrint* p, ut64 seeki, ut64 addr64, ut64
 			free (encstr);
 		}
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki);
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" -> 0x%08"PFMT64x" ", seeki, addr);
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki);
+		}
+		if (!SEEVALUE) {
+			p->cb_printf ("0x%08" PFMT64x " -> 0x%08" PFMT64x " ", seeki, addr);
+		}
 		if (res && buffer[0] != 0xff && buffer[1] != 0xff) {
 			p->cb_printf ("%s", buffer);
 		}
@@ -374,6 +446,7 @@ static int r_print_format_string(const RPrint* p, ut64 seeki, ut64 addr64, ut64
 static void r_print_format_time(const RPrint* p, int endian, int mode,
 		const char* setval, ut64 seeki, ut8* buf, int i, int size) {
 	ut64 addr;
+	struct tm timestruct;
 	int elem = -1;
 	if (size >= ARRAYINDEX_COEF) {
 		elem = size/ARRAYINDEX_COEF-1;
@@ -383,17 +456,21 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 	if (MUSTSET) {
 		p->cb_printf ("wv4 %s @ 0x%08"PFMT64x"\n", setval, seeki+((elem>=0)?elem*4:0));
 	} else if (MUSTSEE) {
-		char *timestr = strdup(asctime (gmtime ((time_t*)&addr)));
+		char *timestr = strdup(asctime (gmtime_r ((time_t*)&addr, &timestruct)));
 		*(timestr+24) = '\0';
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*4:0));
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
+		}
 		if (size==-1) {
 			p->cb_printf ("%s", timestr);
 		} else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				free (timestr);
-				timestr = strdup (asctime (gmtime ((time_t*)&addr)));
+				timestr = strdup (asctime (gmtime_r ((time_t*)&addr, &timestruct)));
 				*(timestr+24) = '\0';
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%s", timestr);
@@ -409,11 +486,13 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 				}
 				i += 4;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 		free (timestr);
 	} else if (MUSTSEEJSON) {
-		char *timestr = strdup (asctime (gmtime ((time_t*)&addr)));
+		char *timestr = strdup (asctime (gmtime_r ((time_t*)&addr, &timestruct)));
 		*(timestr+24) = '\0';
 		if (size==-1) {
 			p->cb_printf ("\"%s\"", timestr);
@@ -422,11 +501,13 @@ static void r_print_format_time(const RPrint* p, int endian, int mode,
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				free (timestr);
-				timestr = strdup (asctime (gmtime ((time_t*)&addr)));
+				timestr = strdup (asctime (gmtime_r ((time_t*)&addr, &timestruct)));
 				*(timestr+24) = '\0';
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("\"%s\"", timestr);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
@@ -458,7 +539,9 @@ static void r_print_format_hex(const RPrint* p, int endian, int mode,
 	} else if (mode & R_PRINT_DOT) {
 		p->cb_printf ("%"PFMT64d, addr);
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*4:0));
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
+		}
 		if (size==-1) {
 			p->cb_printf ("%"PFMT64d, addr);
 		} else {
@@ -486,9 +569,9 @@ static void r_print_format_hex(const RPrint* p, int endian, int mode,
 			}
 		}
 	} else if (MUSTSEEJSON) {
-		if (size==-1)
+		if (size == -1) {
 			p->cb_printf ("%d", addr);
-		else {
+		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
@@ -526,7 +609,7 @@ static void r_print_format_int(const RPrint* p, int endian, int mode,
 	} else if (mode & R_PRINT_DOT) {
 		p->cb_printf ("0x%08"PFMT64x, addr);
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) {
+		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*4:0));
 		}
 		if (size == -1) {
@@ -556,9 +639,9 @@ static void r_print_format_int(const RPrint* p, int endian, int mode,
 			}
 		}
 	} else if (MUSTSEEJSON) {
-		if (size==-1)
+		if (size == -1) {
 			p->cb_printf ("%d", addr);
-		else {
+		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
@@ -613,42 +696,60 @@ static void r_print_format_octal (const RPrint* p, int endian, int mode,
 		p->cb_printf ("0%"PFMT64o, addr);
 	} else if (MUSTSEE) {
 		ut32 addr32 = (ut32)addr;
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*4:0));
-		if (!SEEVALUE) p->cb_printf ("(octal) ");
-		if (size==-1)
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
+		}
+		if (!SEEVALUE) {
+			p->cb_printf ("(octal) ");
+		}
+		if (size == -1) {
 			p->cb_printf (" 0%08"PFMT64o, addr32);
-		else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+		} else {
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("0%08"PFMT64o, addr32);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i+=4;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 	} else if (MUSTSEEJSON) {
 		ut32 addr32 = (ut32)addr;
-		if (size==-1)
+		if (size == -1) {
 			p->cb_printf ("%d", addr32);
-		else {
+		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf, i, endian, &addr, NULL);
 				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%d", addr32);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i+=4;
 			}
 			p->cb_printf (" ]");
@@ -672,37 +773,50 @@ static void r_print_format_hexflag(const RPrint* p, int endian, int mode,
 		p->cb_printf ("0x%08"PFMT64x, addr & UT32_MAX);
 	} else if (MUSTSEE) {
 		ut32 addr32 = (ut32)addr;
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*4:0));
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki + ((elem >= 0) ? elem * 4 : 0));
+		}
 		if (size==-1) {
 			p->cb_printf ("0x%08"PFMT64x, (ut64)addr32);
 		} else {
-			if (!SEEVALUE) p->cb_printf ("[ ");
+			if (!SEEVALUE) {
+				p->cb_printf ("[ ");
+			}
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("0x%08"PFMT64x, addr32);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
-				if (size != 0 && elem == -1)
+				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
-				if (elem > -1) elem--;
+				}
+				if (elem > -1) {
+					elem--;
+				}
 				i+=4;
 			}
-			if (!SEEVALUE) p->cb_printf (" ]");
+			if (!SEEVALUE) {
+				p->cb_printf (" ]");
+			}
 		}
 	} else if (MUSTSEEJSON) {
 		ut32 addr32 = (ut32)addr;
-		if (size==-1)
+		if (size == -1) {
 			p->cb_printf ("%d", addr32);
-		else {
+		} else {
 			p->cb_printf ("[ ");
 			while (size--) {
 				updateAddr (buf + i, size - i, endian, &addr, NULL);
 				addr32 = (ut32)addr;
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%d", addr32);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (",");
@@ -734,11 +848,15 @@ static int r_print_format_10bytes(const RPrint* p, int mode, const char* setval,
 			return -1;
 		}
 		p->iob.read_at (p->iob.io, (ut64)addr, buffer, 248);
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki);
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki);
+		}
 		for (j=0; j<10; j++) {
 			p->cb_printf ("%02x ", buf[j]);
 		}
-		if (!SEEVALUE) p->cb_printf (" ... (");
+		if (!SEEVALUE) {
+			p->cb_printf (" ... (");
+		}
 		for (j = 0; j < 10; j++) {
 			if (!SEEVALUE) {
 				if (IS_PRINTABLE (buf[j])) {
@@ -748,7 +866,9 @@ static int r_print_format_10bytes(const RPrint* p, int mode, const char* setval,
 				}
 			}
 		}
-		if (!SEEVALUE) p->cb_printf (")");
+		if (!SEEVALUE) {
+			p->cb_printf (")");
+		}
 	} else if (MUSTSEEJSON) {
 		if (!p->iob.read_at) {
 			printf ("(cannot read memory)\n");
@@ -779,17 +899,22 @@ static int r_print_format_hexpairs(const RPrint* p, int endian, int mode,
 		}
 	} else if (MUSTSEE) {
 		size = (size < 1) ? 1 : size;
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki);
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki);
+		}
 		for (j = 0; j < size; j++) {
 			p->cb_printf ("%02x ", buf[i + j]);
 		}
-		if (!SEEVALUE) p->cb_printf (" ... (");
+		if (!SEEVALUE) {
+			p->cb_printf (" ... (");
+		}
 		for (j=0; j<size; j++) {
 			if (!SEEVALUE) {
-				if (IS_PRINTABLE (buf[j]))
+				if (IS_PRINTABLE (buf[j])) {
 					p->cb_printf ("%c", buf[i+j]);
-				else
+				} else {
 					p->cb_printf (".");
+				}
 			}
 		}
 		p->cb_printf (")");
@@ -797,8 +922,9 @@ static int r_print_format_hexpairs(const RPrint* p, int endian, int mode,
 		size = (size < 1) ? 1 : size;
 		p->cb_printf ("[ %d", buf[0]);
 		j = 1;
-		for (; j < 10; j++)
+		for (; j < 10; j++) {
 			p->cb_printf (", %d", buf[j]);
+		}
 		p->cb_printf ("]}");
 		return size;
 	}
@@ -822,7 +948,7 @@ static void r_print_format_float(const RPrint* p, int endian, int mode,
 		p->cb_printf ("%f", val_f);
 	} else {
 		if (MUSTSEE) {
-			if (!SEEVALUE) {
+			if (!SEEVALUE && !ISQUIET) {
 				p->cb_printf ("0x%08"PFMT64x" = ",
 					seeki + ((elem >= 0) ? elem * 4 : 0));
 			}
@@ -878,7 +1004,7 @@ static void r_print_format_double(const RPrint* p, int endian, int mode,
 		p->cb_printf ("%f", val_f);
 	} else {
 		if (MUSTSEE) {
-			if (!SEEVALUE) {
+			if (!SEEVALUE && !ISQUIET) {
 				p->cb_printf ("0x%08"PFMT64x" = ",
 					seeki + ((elem >= 0) ? elem * 8 : 0));
 			}
@@ -940,7 +1066,9 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 				: (*(buf+i+1))<<8 | (*(buf+i));
 			if (elem == -1 || elem == 0) {
 				p->cb_printf ("%d", addr);
-				if (elem == 0) elem = -2;
+				if (elem == 0) {
+					elem = -2;
+				}
 			}
 			if (size != 0 && elem == -1) {
 				p->cb_printf (",");
@@ -951,7 +1079,7 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 			i += 2;
 		}
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) {
+		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki+((elem>=0)?elem*2:0));
 		}
 		if (size==-1) {
@@ -966,7 +1094,9 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 					: (*(buf+i+1))<<8 | (*(buf+i));
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("0x%04x", addr);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (", ");
@@ -991,12 +1121,16 @@ static void r_print_format_word(const RPrint* p, int endian, int mode,
 					: (*(buf+i+1))<<8 | (*(buf+i));
 				if (elem == -1 || elem == 0) {
 					p->cb_printf ("%d", addr);
-					if (elem == 0) elem = -2;
+					if (elem == 0) {
+						elem = -2;
+					}
 				}
 				if (size != 0 && elem == -1) {
 					p->cb_printf (",");
 				}
-				if (elem > -1) elem--;
+				if (elem > -1) {
+					elem--;
+				}
 				i += 2;
 			}
 			p->cb_printf (" ]");
@@ -1040,12 +1174,16 @@ static void r_print_format_nulltermstring(const RPrint* p, const int len, int en
 				p->cb_printf ("\\\"");
 			} else if (IS_PRINTABLE (ch)) {
 				p->cb_printf ("%c", ch);
-			} else p->cb_printf (".");
+			} else {
+				p->cb_printf (".");
+			}
 		}
 		p->cb_printf ("\\\"");
 	} else if (MUSTSEE) {
 		int j = i;
-		if (!SEEVALUE) p->cb_printf ("0x%08"PFMT64x" = ", seeki);
+		if (!SEEVALUE && !ISQUIET) {
+			p->cb_printf ("0x%08" PFMT64x " = ", seeki);
+		}
 		for (; j < len && ((size == -1 || size-- > 0) && buf[j]) ; j++) {
 			if (IS_PRINTABLE (buf[j])) {
 				p->cb_printf ("%c", buf[j]);
@@ -1077,14 +1215,14 @@ static void r_print_format_nulltermwidestring(const RPrint* p, const int len, in
 			newstring++;
 			vallen -= 2;
 		}
-		if ((size = strlen (setval)) > r_wstr_clen((char*)(buf+seeki))) {
+		if ((size = vallen) > r_wstr_clen((char*)(buf+seeki))) {
 			eprintf ("Warning: new string is longer than previous one\n");
 		}
 		p->cb_printf ("ww %s @ 0x%08"PFMT64x"\n", newstring, seeki);
 		free(ons);
 	} else if (MUSTSEE) {
 		int j = i;
-		if (!SEEVALUE) {
+		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki);
 		}
 		for (; j<len && ((size==-1 || size-->0) && buf[j]) ; j+=2) {
@@ -1094,6 +1232,17 @@ static void r_print_format_nulltermwidestring(const RPrint* p, const int len, in
 				p->cb_printf (".");
 			}
 		}
+	} else if (MUSTSEEJSON) {
+		int j = i;
+		p->cb_printf ("%d,\"string\":\"", seeki);
+		for (; j < len && ((size == -1 || size-- > 0) && buf[j]); j += 2) {
+			if (IS_PRINTABLE (buf[j])) {
+				p->cb_printf ("%c", buf[j]);
+			} else {
+				p->cb_printf (".");
+			}
+		}
+		p->cb_printf ("\"}");
 	}
 }
 
@@ -1110,12 +1259,18 @@ static void r_print_format_bitfield(const RPrint* p, ut64 seeki, char* fmtname,
 	}
 	bitfield = r_type_enum_getbitfield (p->sdb_types, fmtname, addr);
 	if (bitfield && *bitfield) {
-		if (MUSTSEEJSON) p->cb_printf ("\"%s\"}", bitfield);
-		else if (MUSTSEE) p->cb_printf (" %s (bitfield) = %s\n", fieldname, bitfield);
+		if (MUSTSEEJSON) {
+			p->cb_printf ("\"%s\"}", bitfield);
+		} else if (MUSTSEE) {
+			p->cb_printf (" %s (bitfield) = %s\n", fieldname, bitfield);
+		}
 	} else {
-		if (MUSTSEEJSON) p->cb_printf ("\"`tb %s 0x%x`\"}", fmtname, addr);
-		else if (MUSTSEE) p->cb_printf (" %s (bitfield) = `tb %s 0x%x`\n",
+		if (MUSTSEEJSON) {
+			p->cb_printf ("\"`tb %s 0x%x`\"}", fmtname, addr);
+		} else if (MUSTSEE) {
+			p->cb_printf (" %s (bitfield) = `tb %s 0x%x`\n",
 				fieldname, fmtname, addr);
+		}
 	}
 	free (bitfield);
 }
@@ -1162,8 +1317,11 @@ static void r_print_format_register (const RPrint* p, int mode,
 		if (MUSTSET) {
 			p->cb_printf ("dr %s=%s\n", name, setval);
 		} else if (MUSTSEE) {
-			if (!SEEVALUE) p->cb_printf("%s : 0x%08"PFMT64x"\n", ri->name, p->get_register_value (p->reg, ri));
-			else p->cb_printf("0x%08"PFMT64x"\n", p->get_register_value (p->reg, ri));
+			if (!SEEVALUE) {
+				p->cb_printf ("%s : 0x%08" PFMT64x "\n", ri->name, p->get_register_value (p->reg, ri));
+			} else {
+				p->cb_printf ("0x%08" PFMT64x "\n", p->get_register_value (p->reg, ri));
+			}
 		} else if (MUSTSEEJSON) {
 			p->cb_printf ("%d}", p->get_register_value (p->reg, ri));
 		}
@@ -1205,7 +1363,7 @@ static void r_print_format_num (const RPrint *p, int endian, int mode, const cha
 	} else if (mode & R_PRINT_DOT) {
 		p->cb_printf ("%"PFMT64u, addr);
 	} else if (MUSTSEE) {
-		if (!SEEVALUE) {
+		if (!SEEVALUE && !ISQUIET) {
 			p->cb_printf ("0x%08"PFMT64x" = ", seeki + ((elem >= 0)? elem * bytes: 0));
 		}
 		if (size == -1) {
@@ -1323,7 +1481,9 @@ int r_print_format_struct_size(const char *f, RPrint *p, int mode, int n) {
 			tabsize_set = true;
 			tabsize = r_num_math (NULL, fmt + i + 1);
 			*end = ']';
-			while (fmt[i++] != ']');
+			while (fmt[i++] != ']') {
+				;
+			}
 		} else {
 			tabsize = 1;
 		}
@@ -1414,6 +1574,8 @@ int r_print_format_struct_size(const char *f, RPrint *p, int mode, int n) {
 			} else {
 				format = sdb_get (p->formats, structname + 1, NULL);
 				if (format && !strncmp (format, f, strlen (format) - 1)) { // Avoid recursion here
+					free (o);
+					free (structname);
 					return -1;
 				}
 				if (!format) { // Fetch format from types db
@@ -1422,11 +1584,15 @@ int r_print_format_struct_size(const char *f, RPrint *p, int mode, int n) {
 			}
 			if (!format) {
 				eprintf ("Cannot find format for struct `%s'\n", structname + 1);
+				free (structname);
+				free (o);
 				return 0;
 			}
 			int newsize = r_print_format_struct_size (format, p, mode, n + 1);
 			if (newsize < 1) {
 				eprintf ("Cannot find size for `%s'\n", format);
+				free (structname);
+				free (o);
 				return 0;
 			}
 			if (format && newsize > 0) {
@@ -1565,7 +1731,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 	char *args = NULL, *bracket, tmp, last = 0;
 	ut64 addr = 0, addr64 = 0, seeki = 0;
 	static int slide = 0, oldslide = 0;
-	char namefmt[8], *field = NULL;
+	char namefmt[32], *field = NULL;
 	const char *arg = NULL;
 	const char *fmt = NULL;
 	const char *argend;
@@ -1580,7 +1746,9 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 	if (!fmt) {
 		fmt = formatname;
 	}
-	while (*fmt && IS_WHITECHAR (*fmt)) fmt++;
+	while (*fmt && IS_WHITECHAR (*fmt)) {
+		fmt++;
+	}
 	argend = fmt + strlen (fmt);
 	arg = fmt;
 
@@ -1652,7 +1820,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 		l++;
 		const char *ends = " "; // XXX trailing space warning
 		snprintf (namefmt, sizeof (namefmt), "%%%ds :%s",
-			((maxl + 1) * (1+slide)) % STRUCTPTR, ends);
+			((maxl + 1) * (1 + slide)) % STRUCTPTR, ends);
 	}
 #define ISPOINTED ((slide%STRUCTFLAG)/STRUCTPTR<=(oldslide%STRUCTFLAG)/STRUCTPTR)
 #define ISNESTED ((slide%STRUCTPTR)<=(oldslide%STRUCTPTR))
@@ -1695,7 +1863,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 				}
 				p->cb_printf ("[{\"index\":%d,\"offset\":%d},", otimes-times, seek+i);
 			} else if (mode) {
-				p->cb_printf ("0x%08"PFMT64x" [%d] {\n", seek+i, otimes-times);
+				p->cb_printf ("0x%08"PFMT64x" [%d] {\n", seek + i, otimes-times);
 			}
 		}
 		arg = orig;
@@ -1800,7 +1968,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					for ( ; bracket < end; bracket++) {
 						*bracket = '\0';
 					}
-					size += elem*ARRAYINDEX_COEF;
+					size += elem * ARRAYINDEX_COEF;
 				} else {
 					elem = -1;
 				}
@@ -1816,7 +1984,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 			switch (isptr) {
 			case PTRSEEK:
 				{
-				nexti = i + (p->bits/8);
+				nexti = i + (p->bits / 8);
 				i = 0;
 				if (tmp == '?' ) {
 					seeki = addr;
@@ -1832,7 +2000,6 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					if ( (i + 3) < len || (i + 7) < len) {
 						// XXX this breaks pf *D
 						if (tmp != 'D') {
-
 							updateAddr (buf + i, len - i, endian, &addr, &addr64);
 						}
 					} else {
@@ -1908,7 +2075,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 			if (mode & R_PRINT_SEEFLAGS && isptr != NULLPTR) {
 				char *newname = NULL;
 				if (!fieldname) {
-					newname = fieldname = r_str_newf ("pf.%d", seeki);
+					newname = fieldname = r_str_newf ("pf.%"PFMT64u, seeki);
 				}
 				if (mode & R_PRINT_UNIONMODE) {
 					p->cb_printf ("f %s=0x%08"PFMT64x"\n", formatname, seeki);
@@ -1919,10 +2086,12 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					p->cb_printf ("f %s=0x%08"PFMT64x"\n", fieldname, seeki);
 				} else if (slide/STRUCTFLAG>0 && idx==1) {
 					p->cb_printf ("%s=0x%08"PFMT64x"\n", fieldname, seeki);
-				} else p->cb_printf ("f %s=0x%08"PFMT64x"\n", fieldname , seeki);
+				} else {
+					p->cb_printf ("f %s=0x%08" PFMT64x "\n", fieldname, seeki);
+				}
 				if (newname) {
-					free (newname);
-					newname = fieldname = NULL;
+					R_FREE (newname);
+					fieldname = NULL;
 				}
 			}
 
@@ -1940,8 +2109,11 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 			/* json */
 			if (MUSTSEEJSON && mode & R_PRINT_JSON) {
 				if (oldslide <= slide) {
-					if (first) first = 0;
-					else p->cb_printf (",");
+					if (first) {
+						first = 0;
+					} else {
+						p->cb_printf (",");
+					}
 				} else if (oldslide) {
 					p->cb_printf ("]},");
 				}
@@ -1965,6 +2137,7 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 				p->cb_printf ("\",\"offset\":%d,\"value\":",
 					isptr? (seek + nexti - (p->bits / 8)) : seek + i);
 			}
+			bool noline = false;
 
 			if (isptr == NULLPTR) {
 				if (MUSTSEEJSON) {
@@ -1973,14 +2146,10 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					p->cb_printf (" NULL\n");
 				}
 				isptr = PTRBACK;
-			} else
-			/* format chars */
-			// before to enter in the switch statement check buf boundaries due to  updateAddr
-			// might go beyond its len and it's usually called in each of the following functions
-#if 0
-// those boundaries are wrong. the fix was not correct, we need a reproducer
-			if (((i+3)<len) || (i+7)<len) {
-#endif
+			} else {
+				/* format chars */
+				// before to enter in the switch statement check buf boundaries due to  updateAddr
+				// might go beyond its len and it's usually called in each of the following functions
 				switch (tmp) {
 				case 'u':
 					i += r_print_format_uleb (p, endian, mode, setval, seeki, buf, i, size);
@@ -2044,7 +2213,21 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					break;
 				case 'o':
 					r_print_format_octal (p, endian, mode, setval, seeki, buf, i, size);
-					i += (size==-1) ? 4 : 4*size;
+					i += (size==-1) ? 4 : 4 * size;
+					break;
+				case ';':
+					noline = true;
+					i -= (size==-1) ? 4 : 4 * size;
+					if (i < 0) {
+						i = 0;
+					}
+					break;
+				case ',':
+					noline = true;
+					i -= (size==-1) ? 1 : size;
+					if (i < 0) {
+						i = 0;
+					}
 					break;
 				case 'x':
 					r_print_format_hexflag (p, endian, mode, setval, seeki, buf, i, size);
@@ -2056,17 +2239,23 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					break;
 				case 'z': // zero terminated string
 					r_print_format_nulltermstring (p, len, endian, mode, setval, seeki, buf, i, size);
-					if (size == -1)
+					if (size == -1) {
 						i+=strlen((char*)buf+i)+1;
-					else
-						while (size--) i++;
+					} else {
+						while (size--) {
+							i++;
+						}
+					}
 					break;
 				case 'Z': // zero terminated wide string
 					r_print_format_nulltermwidestring (p, len, endian, mode, setval, seeki, buf, i, size);
-					if (size == -1)
+					if (size == -1) {
 						i += r_wstr_clen((char*)(buf+i))*2+2;
-					else
-						while (size--) i+=2;
+					} else {
+						while (size--) {
+							i += 2;
+						}
+					}
 					break;
 				case 's':
 					if (r_print_format_string (p, seeki, addr64, addr, 0, mode) == 0) {
@@ -2074,16 +2263,21 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					}
 					break;
 				case 'S':
-					if (r_print_format_string (p, seeki, addr64, addr, 1, mode) == 0)
-						i += (size==-1) ? 8 : 8*size;
+					if (r_print_format_string (p, seeki, addr64, addr, 1, mode) == 0) {
+						i += (size == -1) ? 8 : 8 * size;
+					}
 					break;
 				case 'B': // resolve bitfield
-					if (size >= ARRAYINDEX_COEF) size %= ARRAYINDEX_COEF;
+					if (size >= ARRAYINDEX_COEF) {
+						size %= ARRAYINDEX_COEF;
+					}
 					r_print_format_bitfield (p, seeki, fmtname, fieldname, addr, mode, size);
 					i+=(size==-1)?1:size;
 					break;
 				case 'E': // resolve enum
-					if (size >= ARRAYINDEX_COEF) size %= ARRAYINDEX_COEF;
+					if (size >= ARRAYINDEX_COEF) {
+						size %= ARRAYINDEX_COEF;
+					}
 					r_print_format_enum (p, seeki, fmtname, fieldname, addr, mode, size);
 					i += (size==-1)? 1: size;
 					break;
@@ -2110,7 +2304,9 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					}
 
 					if (MUSTSEE) {
-						if (!SEEVALUE) p->cb_printf ("\n");
+						if (!SEEVALUE) {
+							p->cb_printf ("\n");
+						}
 					}
 					if (MUSTSEEJSON) {
 						if (isptr) {
@@ -2221,22 +2417,21 @@ R_API int r_print_format(RPrint *p, ut64 seek, const ut8* b, const int len,
 					invalid = 1;
 					break;
 				} //switch
-#if 0
-			} else {
-				eprintf ("r_print_format: Likely a heap buffer overflow (%s)\n", buf);
-				goto beach;
 			}
-#endif
 			if (mode & R_PRINT_DOT) {
 				p->cb_printf ("}");
 			}
 			if (viewflags && p->offname) {
 				const char *s = p->offname (p->user, seeki);
-				if (s) p->cb_printf ("@(%s)", s);
+				if (s) {
+					p->cb_printf ("@(%s)", s);
+				}
 				s = p->offname (p->user, addr);
-				if (s) p->cb_printf ("*(%s)", s);
+				if (s) {
+					p->cb_printf ("*(%s)", s);
+				}
 			}
-			if (tmp != 'D' && !invalid && !fmtname && MUSTSEE) {
+			if (!noline && tmp != 'D' && !invalid && !fmtname && MUSTSEE) {
 				p->cb_printf ("\n");
 			}
 			last = tmp;
