@@ -1,10 +1,11 @@
-/* radare - LGPL - Copyright 2009-2015 - pancake */
+/* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #include <r_util.h>
 
 R_API int r_name_validate_char(const char ch) {
-	if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || (IS_DIGIT(ch)))
+	if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || (IS_DIGIT(ch))) {
 		return true;
+	}
 	switch (ch) {
 	case ':':
 	case '.':
@@ -15,32 +16,34 @@ R_API int r_name_validate_char(const char ch) {
 }
 
 R_API int r_name_check(const char *name) {
-	if (!name || !*name)
-		return false;
 	/* Cannot start by number */
-	if (IS_DIGIT(*name))
+	if (!name || !*name || IS_DIGIT (*name)) {
 		return false;
+	}
 	/* Cannot contain non-alphanumeric chars + [:._] */
-	for (; *name != '\0'; name++)
-		if (!r_name_validate_char (*name))
+	for (; *name != '\0'; name++) {
+		if (!r_name_validate_char (*name)) {
 			return false;
+		}
+	}
 	return true;
 }
 
 static inline bool is_special_char (char *name) {
 	const char n = *name;
-	return (n == 'b' || n == 'f' || n == 'n' || n == 'r' || n == 't' || n == 'v');
+	return (n == 'b' || n == 'f' || n == 'n' || n == 'r' || n == 't' || n == 'v' || n == 'a');
 }
 
 R_API int r_name_filter(char *name, int maxlen) {
 	int i;
 	size_t len;
-	char *oname;
-	if (!name) return 0;
+	if (!name) {
+		return 0;
+	}
 	if (maxlen < 0) {
 		maxlen = strlen (name);
 	}
-	name = oname = r_str_trim_head_tail (name);
+	char *oname = name = r_str_trim_head_tail (name);
 	for (i = 0; *name; name++, i++) {
 		if (maxlen && i > maxlen) {
 			*name = '\0';
@@ -68,11 +71,15 @@ R_API int r_name_filter(char *name, int maxlen) {
 	}
 	// trimming trailing and leading underscores
 	len = strlen (name);
-	for (; len > 0 && *(name + len - 1) == '_'; len--);
+	for (; len > 0 && *(name + len - 1) == '_'; len--) {
+		;
+	}
 	if (!len) { // name consists only of underscores
 		return r_name_check (oname);
 	}
-	for (i = 0; *(name + i) == '_'; i++, len--);
+	for (i = 0; *(name + i) == '_'; i++, len--) {
+		;
+	}
 	memmove (name, name + i, len);
 	*(name + len) = '\0';
 	return r_name_check (oname);
@@ -80,10 +87,10 @@ R_API int r_name_filter(char *name, int maxlen) {
 
 R_API char *r_name_filter2(const char *name) {
 	int i;
-	char *res;
-	while (!IS_PRINTABLE (*name))
+	while (!IS_PRINTABLE (*name)) {
 		name++;
-	res = strdup (name);
+	}
+	char *res = strdup (name);
 	for (i = 0; res[i]; i++) {
 		if (!r_name_validate_char (res[i])) {
 			res[i] = '_';
