@@ -2,19 +2,25 @@
 
 #include "r_anal.h"
 #include "r_cons.h"
-#include "r_print.h"
 #include "r_util.h"
+#include "r_util/r_print.h"
 
 #define DFLT_ROWS 16
 
 #define IS_ALPHA(C) (((C) >= 'a' && (C) <= 'z') || ((C) >= 'A' && (C) <= 'Z'))
 
-static void nullprinter(const char *a, ...) { }
-static void libc_printf(const char *format, ...) {
+static const char hex[16] = "0123456789ABCDEF";
+
+static int nullprinter(const char *a, ...) {
+	return 0;
+}
+
+static int libc_printf(const char *format, ...) {
 	va_list ap;
 	va_start (ap, format);
 	vprintf (format, ap);
 	va_end (ap);
+	return 0;
 }
 
 static RPrintIsInterruptedCallback is_interrupted_cb = NULL;
@@ -355,18 +361,27 @@ R_API void r_print_set_cursor(RPrint *p, int enable, int ocursor, int cursor) {
 	p->cur = cursor;
 }
 
-R_API void r_print_cursor(RPrint *p, int cur, int set) {
+R_API bool r_print_have_cursor(RPrint *p, int cur, int len) {
 	if (!p || !p->cur_enabled) {
-		return;
+		return false;
 	}
 	if (p->ocur != -1) {
 		int from = p->ocur;
 		int to = p->cur;
 		r_num_minmax_swap_i (&from, &to);
-		if (cur >= from && cur <= to) {
-			p->cb_printf ("%s", R_CONS_INVERT (set, 1));
-		}
-	} else if (cur == p->cur) {
+		do {
+			if (cur + len - 1 >= from && cur + len - 1 <= to) {
+				return true;
+			}
+		} while (--len);
+	} else if (p->cur >= cur && p->cur <= cur + len - 1) {
+		return true;
+	}
+	return false;
+}
+
+R_API void r_print_cursor(RPrint *p, int cur, int len, int set) {
+	if (r_print_have_cursor (p, cur, len)) {
 		p->cb_printf ("%s", R_CONS_INVERT (set, 1));
 	}
 }
@@ -375,7 +390,7 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 	char space[32] = {
 		0
 	};
-	const char *white;
+	const char *white = "";
 #define PREOFF(x) (p && p->cons && p->cons->pal.x)? p->cons->pal.x
 	PrintfCallback printfmt = (PrintfCallback) (p? p->cb_printf: libc_printf);
 	bool use_segoff = p? (p->flags & R_PRINT_FLAGS_SEGOFF): false;
@@ -389,7 +404,7 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 	if (use_segoff) {
 		ut32 s, a;
 		a = addr & 0xffff;
-		s = (addr - a) >> p->seggrn;
+		s = (addr - a) >> (p? p->seggrn: 0);
 		if (dec) {
 			snprintf (space, sizeof (space), "%d:%d", s & 0xffff, a & 0xffff);
 			white = r_str_pad (' ', 9 - strlen (space));
@@ -418,23 +433,33 @@ R_API void r_print_addr(RPrint *p, ut64 addr) {
 		if (use_color) {
 			const char *pre = PREOFF (offset): Color_GREEN;
 			const char *fin = Color_RESET;
-			if (p->flags & R_PRINT_FLAGS_RAINBOW) {
+			if (p && p->flags & R_PRINT_FLAGS_RAINBOW) {
 				// pre = r_cons_rgb_str_off (rgbstr, addr);
-				if (p && p->cons && p->cons->rgbstr) {
-					char rgbstr[32];
+				if (p->cons && p->cons->rgbstr) {
+					static char rgbstr[32];
 					pre = p->cons->rgbstr (rgbstr, sizeof (rgbstr), addr);
 				}
 			}
 			if (dec) {
 				printfmt ("%s%s%" PFMT64d "%s%c", pre, white, addr, fin, ch);
 			} else {
-				printfmt ("%s0x%08" PFMT64x "%s%c", pre, addr, fin, ch);
+				if (p && p->wide_offsets) {
+					// TODO: make %016 depend on asm.bits
+					printfmt ("%s0x%016" PFMT64x "%s%c", pre, addr, fin, ch);
+				} else {
+					printfmt ("%s0x%08" PFMT64x "%s%c", pre, addr, fin, ch);
+				}
 			}
 		} else {
 			if (dec) {
 				printfmt ("%s%" PFMT64d "%c", white, addr, ch);
 			} else {
-				printfmt ("0x%08" PFMT64x "%c", addr, ch);
+				if (p && p->wide_offsets) {
+					// TODO: make %016 depend on asm.bits
+					printfmt ("0x%016" PFMT64x "%c", addr, ch);
+				} else {
+					printfmt ("0x%08" PFMT64x "%c", addr, ch);
+				}
 			}
 		}
 	}
@@ -474,7 +499,7 @@ R_API char* r_print_hexpair(RPrint *p, const char *str, int n) {
 #define memcat(x, y)\
 	{ \
 		memcpy (x, y, strlen (y));\
-		x += strlen (y);\
+		(x) += strlen (y);\
 	}
 	for (s = str, i = 0; s[0]; i++) {
 		int d_inc = 2;
@@ -530,46 +555,48 @@ R_API char* r_print_hexpair(RPrint *p, const char *str, int n) {
 	return dst;
 }
 
+static char colorbuffer[64];
+#define P(x) (p->cons && p->cons->pal.x)? p->cons->pal.x
+R_API const char *r_print_byte_color(RPrint *p, int ch) {
+	if (p->flags & R_PRINT_FLAGS_RAINBOW) {
+		// EXPERIMENTAL
+		int bg = (p->flags & R_PRINT_FLAGS_NONHEX)? 48: 38;
+		snprintf (colorbuffer, sizeof (colorbuffer), "\033[%d;5;%dm", bg, ch);
+		return colorbuffer;
+	}
+	const bool use_color = p->flags & R_PRINT_FLAGS_COLOR;
+	if (!use_color) {
+		return NULL;
+	}
+	switch (ch) {
+	case 0x00: return P (b0x00): Color_GREEN;
+	case 0x7F: return P (b0x7f): Color_YELLOW;
+	case 0xFF: return P (b0xff): Color_RED;
+	default: return IS_PRINTABLE (ch)? P (btext): Color_MAGENTA: P (other): Color_WHITE;
+	}
+	return NULL;
+}
+
 R_API void r_print_byte(RPrint *p, const char *fmt, int idx, ut8 ch) {
 	PrintfCallback printfmt = (PrintfCallback) (p? p->cb_printf: libc_printf);
 	ut8 rch = ch;
 	if (!IS_PRINTABLE (ch) && fmt[0] == '%' && fmt[1] == 'c') {
 		rch = '.';
 	}
-	r_print_cursor (p, idx, 1);
+	r_print_cursor (p, idx, 1, 1);
 	if (p && p->flags & R_PRINT_FLAGS_COLOR) {
-#define P(x) (p->cons && p->cons->pal.x)? p->cons->pal.x
-		char *color_0x00 = P (b0x00): Color_GREEN;
-		char *color_0x7f = P (b0x7f): Color_YELLOW;
-		char *color_0xff = P (b0xff): Color_RED;
-		char *color_text = P (btext): Color_MAGENTA;
-		char *color_other = P (other): Color_WHITE;
-		char *pre = NULL;
-		switch (ch) {
-		case 0x00:
-			pre = color_0x00;
-			break;
-		case 0x7F:
-			pre = color_0x7f;
-			break;
-		case 0xFF:
-			pre = color_0xff;
-			break;
-		default:
-			pre = IS_PRINTABLE (ch)? color_text: color_other;
-			break;
-		}
-		if (pre) {
-			printfmt (pre);
+		const char *bytecolor = r_print_byte_color (p, ch);
+		if (bytecolor) {
+			printfmt (bytecolor);
 		}
 		printfmt (fmt, rch);
-		if (pre) {
+		if (bytecolor) {
 			printfmt (Color_RESET);
 		}
 	} else {
 		printfmt (fmt, rch);
 	}
-	r_print_cursor (p, idx, 0);
+	r_print_cursor (p, idx, 1, 0);
 }
 
 R_API int r_print_string(RPrint *p, ut64 seek, const ut8 *buf, int len, int options) {
@@ -594,7 +621,7 @@ R_API int r_print_string(RPrint *p, ut64 seek, const ut8 *buf, int len, int opti
 		if (zeroend && buf[i] == '\0') {
 			break;
 		}
-		r_print_cursor (p, i, 1);
+		r_print_cursor (p, i, 1, 1);
 		ut8 b = buf[i];
 		if (b == '\n') {
 			col = 0;
@@ -604,18 +631,18 @@ R_API int r_print_string(RPrint *p, ut64 seek, const ut8 *buf, int len, int opti
 			// TODO: some ascii can be bypassed here
 			p->cb_printf ("%%%02x", b);
 		} else {
-			if ((b == '\n' && !esc_nl) || IS_PRINTABLE (b)) {
+			if (b == '\\') {
+				p->cb_printf ("\\\\");
+			} else if ((b == '\n' && !esc_nl) || IS_PRINTABLE (b)) {
 				p->cb_printf ("%c", b);
 			} else {
 				p->cb_printf ("\\x%02x", b);
 			}
 		}
-		r_print_cursor (p, i, 0);
-		if (wrap) {
-			if (col + 1 >= p->width) {
-				p->cb_printf ("\n");
-				col = 0;
-			}
+		r_print_cursor (p, i, 1, 0);
+		if (wrap && col + 1 >= p->width) {
+			p->cb_printf ("\n");
+			col = 0;
 		}
 		if (wide) {
 			i++;
@@ -625,7 +652,6 @@ R_API int r_print_string(RPrint *p, ut64 seek, const ut8 *buf, int len, int opti
 	return i;
 }
 
-static const char hex[16] = "0123456789ABCDEF";
 R_API void r_print_hexpairs(RPrint *p, ut64 addr, const ut8 *buf, int len) {
 	int i;
 	for (i = 0; i < len; i++) {
@@ -657,13 +683,13 @@ static bool isAllZeros(const ut8 *buf, int len) {
 	return true;
 }
 
+#define Pal(x,y) (x->cons && x->cons->pal.y)? x->cons->pal.y
 R_API void r_print_hexii(RPrint *rp, ut64 addr, const ut8 *buf, int len, int step) {
-#define Pal(x) (rp->cons && rp->cons->pal.x)? rp->cons->pal.x
 	PrintfCallback p = (PrintfCallback) rp->cb_printf;
 	bool c = rp->flags & R_PRINT_FLAGS_COLOR;
-	const char *color_0xff = c? (Pal (b0xff): Color_RED): "";
-	const char *color_text = c? (Pal (btext): Color_MAGENTA): "";
-	const char *color_other = c? (Pal (other): Color_WHITE): "";
+	const char *color_0xff = c? (Pal (rp, b0xff): Color_RED): "";
+	const char *color_text = c? (Pal (rp, btext): Color_MAGENTA): "";
+	const char *color_other = c? (Pal (rp, other): Color_WHITE): "";
 	const char *color_reset = c? Color_RESET: "";
 	int i, j;
 
@@ -703,7 +729,9 @@ R_API void r_print_hexii(RPrint *rp, ut64 addr, const ut8 *buf, int len, int ste
 R_API void r_print_set_screenbounds(RPrint *p, ut64 addr) {
 	int r, rc;
 
-	if (!p || !p->screen_bounds) {
+	r_return_if_fail (p);
+
+	if (!p->screen_bounds) {
 		return;
 	}
 	if (!p->consbind.get_size) {
@@ -713,43 +741,52 @@ R_API void r_print_set_screenbounds(RPrint *p, ut64 addr) {
 		return;
 	}
 
-	(void) p->consbind.get_size (&r);
-	(void) p->consbind.get_cursor (&rc);
+	if (p->screen_bounds == 1) {
+		(void)p->consbind.get_size (&r);
+		(void)p->consbind.get_cursor (&rc);
 
-	if (rc > r - 1 && p->screen_bounds == 1) {
-		p->screen_bounds = addr;
+		if (rc > r - 1) {
+			p->screen_bounds = addr;
+		}
 	}
 }
 
 R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int base, int step, int zoomsz) {
 	PrintfCallback printfmt = (PrintfCallback) printf;
+	bool c = p->flags & R_PRINT_FLAGS_COLOR;
+	const char *color_title = c? (Pal (p, offset): Color_MAGENTA): "";
 	int i, j, k, inc = 16;
 	int sparse_char = 0;
 	int stride = 0;
 	int col = 0; // selected column (0=none, 1=hex, 2=ascii)
 	int use_sparse = 0;
-	int use_header = 1;
+	bool use_header = true;
+	bool use_hdroff = true;
+	int use_pair = 1;
 	int use_offset = 1;
 	bool compact = false;
 	int use_segoff = 0;
 	int pairs = 0;
-	const char *fmt = "%02x";
+	const char *bytefmt = "%02x";
 	const char *pre = "";
 	int last_sparse = 0;
 	bool use_hexa = true;
 	const char *a, *b;
-
+	int K = 0;
+	bool hex_style = false;
 
 	len = len - (len % step);
 	if (p) {
 		pairs = p->pairs;
 		use_sparse = p->flags & R_PRINT_FLAGS_SPARSE;
 		use_header = p->flags & R_PRINT_FLAGS_HEADER;
+		use_hdroff = p->flags & R_PRINT_FLAGS_HDROFF;
 		use_segoff = p->flags & R_PRINT_FLAGS_SEGOFF;
 		use_offset = p->flags & R_PRINT_FLAGS_OFFSET;
+		hex_style = p->flags & R_PRINT_FLAGS_STYLE;
 		use_hexa = !(p->flags & R_PRINT_FLAGS_NONHEX);
 		compact = p->flags & R_PRINT_FLAGS_COMPACT;
-		inc = p->cols;
+		inc = p->cols; // row width
 		col = p->col;
 		printfmt = (PrintfCallback) p->cb_printf;
 		stride = p->stride;
@@ -768,100 +805,127 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 	}
 	switch (base) {
 	case -10:
-		fmt = "0x%08x ";
+		bytefmt = "0x%08x ";
 		pre = " ";
 		if (inc < 4) {
 			inc = 4;
 		}
 		break;
 	case -1:
-		fmt = "0x%08x ";
+		bytefmt = "0x%08x ";
 		pre = "  ";
 		if (inc < 4) {
 			inc = 4;
 		}
 		break;
 	case 8:
-		fmt = "%03o";
+		bytefmt = "%03o";
 		pre = " ";
 		break;
 	case 10:
-		fmt = "%3d";
+		bytefmt = "%3d";
 		pre = " ";
 		break;
 	case 32:
-		fmt = "0x%08x ";
+		bytefmt = "0x%08x ";
 		pre = " ";
 		if (inc < 4) {
 			inc = 4;
 		}
 		break;
 	case 64:
-		fmt = "0x%016x ";
+		bytefmt = "0x%016x ";
 		pre = " ";
 		if (inc < 8) {
 			inc = 8;
 		}
 		break;
 	}
-
+	const char *space = hex_style? ".": " ";
 	// TODO: Use base to change %03o and so on
 	if (step == 1 && base < 0) {
 		use_header = false;
 	}
 	if (use_header) {
+		if (c) {
+			printfmt (color_title);
+		}
 		if (base < 32) {
 			ut32 opad = (ut32) (addr >> 32);
 			{ // XXX: use r_print_addr_header
 				int i, delta;
 				char soff[32];
+				if (hex_style) {
+					printfmt ("..offset..");
+				} else {
+					printfmt ("- offset -");
+					if (p->wide_offsets) {
+						printfmt ("       ");
+					}
+				}
 				if (use_segoff) {
 					ut32 s, a;
 					a = addr & 0xffff;
 					s = ((addr - a) >> p->seggrn) & 0xffff;
 					snprintf (soff, sizeof (soff), "%04x:%04x ", s, a);
-					printfmt ("- offset -");
+					delta = strlen (soff) - 10;
 				} else {
-					printfmt ("- offset - ");
 					snprintf (soff, sizeof (soff), "0x%08" PFMT64x, addr);
+					delta = strlen (soff) - 9;
 				}
-				delta = strlen (soff) - 10;
 				if (compact) {
 					delta--;
 				}
 				for (i = 0; i < delta; i++) {
-					printfmt (" ");
+					printfmt (space);
 				}
 			}
 			/* column after number, before hex data */
-			printfmt ((col == 1)? "|": " ");
+			printfmt ((col == 1)? "|": space);
 			opad >>= 4;
-			k = 0; // TODO: ??? SURE??? config.seek & 0xF;
+			if (use_hdroff)  {
+				k = addr & 0xf;
+				K = (addr >> 4) & 0xf;
+			} else {
+				k = 0; // TODO: ??? SURE??? config.seek & 0xF;
+			}
 			if (use_hexa) {
 				/* extra padding for offsets > 8 digits */
 				for (i = 0; i < inc; i++) {
 					printfmt (pre);
 					if (base < 0) {
 						if (i & 1) {
-							printfmt (" ");
+							printfmt (space);
 						}
 					}
-					printfmt (" %c", hex[(i + k) % 16]);
+					if  (use_hdroff) {
+						if (use_pair) {
+							printfmt ("%c%c",
+									hex[(((i+k) >> 4) + K) % 16],
+									hex[(i + k) % 16]);
+						} else {
+							printfmt (" %c", hex[(i + k) % 16]);
+						}
+					} else {
+						printfmt (" %c", hex[(i + k) % 16]);
+					}
 					if (i & 1 || !pairs) {
 						if (!compact) {
-							printfmt (col != 1? " ": ((i + 1) < inc)? " ": "|");
+							printfmt (col != 1? space: ((i + 1) < inc)? space: "|");
 						}
 					}
 				}
 			}
 			/* ascii column */
 			if (compact) {
-				printfmt (col > 0? "|": " ");
+				printfmt (col > 0? "|": space);
 			} else {
-				printfmt (col == 2? "|": " ");
+				printfmt (col == 2? "|": space);
 			}
-			for (i = 0; i < inc; i++) {
-				printfmt ("%c", hex[(i + k) % 16]);
+			if (!p || !(p->flags & R_PRINT_FLAGS_NONASCII)) {
+				for (i = 0; i < inc; i++) {
+					printfmt ("%c", hex[(i + k) % 16]);
+				}
 			}
 			if (col == 2) {
 				printfmt ("|");
@@ -871,10 +935,16 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 				if (col != 2) {
 					printfmt (" ");
 				}
-				printfmt (" comment ");
+				if (!hex_style) {
+					printfmt (" comment ");
+				}
 			}
 			printfmt ("\n");
 		}
+
+		if (c) {
+			printfmt (Color_RESET);
+		}
 	}
 
 	if (p) {
@@ -882,6 +952,7 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 	}
 	// is this necessary?
 	r_print_set_screenbounds (p, addr);
+	int rows = 0;
 	for (i = j = 0; i < len; i += (stride? stride: inc), j += (stride? stride: 0)) {
 		if (p && p->cons && p->cons->context && p->cons->context->breaked) {
 			break;
@@ -909,11 +980,19 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 		if (use_offset) {
 			r_print_addr (p, addr + j * zoomsz);
 		}
+		int row_have_cursor = -1;
+		ut64 row_have_addr = UT64_MAX;
 		if (use_hexa) {
 			if (!compact) {
 				printfmt ((col == 1)? "|": " ");
 			}
 			for (j = i; j < i + inc; j++) {
+				if (row_have_cursor == -1) {
+					if (r_print_have_cursor (p, j, 1)) {
+						row_have_cursor = j - i;
+						row_have_addr = addr + j;
+					}
+				}
 				if (!compact && j >= len) {
 					if (col == 1) {
 						if (j + 1 >= inc + i) {
@@ -932,6 +1011,10 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					}
 					continue;
 				}
+				const char *hl = (hex_style && p->offname (p->user, addr + j))? Color_INVERT: NULL;
+				if (hl) {
+					printfmt (hl);
+				}
 				if (p && (base == 32 || base == 64)) {
 					int left = len - i;
 					/* TODO: check step. it should be 2/4 for base(32) and 8 for
@@ -945,8 +1028,13 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 						sz_n = step == 2? sizeof (ut16): sizeof (ut32);
 					}
 					sz_n = R_MIN (left, sz_n);
+					if (j + sz_n > len) {
+						// oob
+						j += sz_n;
+						continue;
+					}
 					r_mem_swaporcopy ((ut8 *) &n, buf + j, sz_n, p && p->big_endian);
-					r_print_cursor (p, j, 1);
+					r_print_cursor (p, j, sz_n, 1);
 					// stub for colors
 					if (p && p->colorfor) {
 						if (!p->iob.addr_is_mapped (p->iob.io, addr + j)) {
@@ -969,29 +1057,42 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 					} else {
 						printfmt ("%s0x%08x%s ", a, (ut32) n, b);
 					}
-					r_print_cursor (p, j, 0);
+					r_print_cursor (p, j, sz_n, 0);
 					j += step - 1;
 				} else if (base == -8) {
 					long long w = r_read_ble64 (buf + j, p && p->big_endian);
+					r_print_cursor (p, j, 8, 1);
 					printfmt ("%23" PFMT64d " ", w);
+					r_print_cursor (p, j, 8, 0);
 					j += 7;
 				} else if (base == -1) {
 					st8 w = r_read_ble8 (buf + j);
+					r_print_cursor (p, j, 1, 1);
 					printfmt ("%4d ", w);
+					r_print_cursor (p, j, 1, 0);
 				} else if (base == -10) {
 					st16 w = r_read_ble16 (buf + j, p && p->big_endian);
+					r_print_cursor (p, j, 2, 1);
 					printfmt ("%7d ", w);
+					r_print_cursor (p, j, 2, 0);
 					j += 1;
-				} else if (base == 10) {
+				} else if (base == 10) { // "pxd"
 					int w = r_read_ble32 (buf + j, p && p->big_endian);
+					r_print_cursor (p, j, 4, 1);
 					printfmt ("%13d ", w);
+					r_print_cursor (p, j, 4, 0);
 					j += 3;
 				} else {
 					if (j >= len) {
 						break;
 					}
-					r_print_byte (p, fmt, j, buf[j]);
-					if (j % 2 || !pairs) {
+					r_print_byte (p, bytefmt, j, buf[j]);
+					if (pairs && !compact && (inc & 1)) {
+						bool mustspace = (rows % 2) ? !(j&1) : (j&1);
+						if (mustspace) {
+							printfmt (" ");
+						}
+					} else if (j % 2 || !pairs) {
 						if (col == 1) {
 							if (j + 1 < inc + i) {
 								if (!compact) {
@@ -1007,6 +1108,9 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 						}
 					}
 				}
+				if (hl) {
+					printfmt (Color_RESET);
+				}
 			}
 		}
 		if (compact) {
@@ -1020,11 +1124,13 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 		} else {
 			printfmt ((col == 2)? "|": " ");
 		}
-		for (j = i; j < i + inc; j++) {
-			if (j >= len) {
-				break;
+		if (!p || !(p->flags & R_PRINT_FLAGS_NONASCII)) {
+			for (j = i; j < i + inc; j++) {
+				if (j >= len) {
+					break;
+				}
+				r_print_byte (p, "%c", j, buf[j]);
 			}
-			r_print_byte (p, "%c", j, buf[j]);
 		}
 		/* ascii column */
 		if (col == 2) {
@@ -1037,14 +1143,16 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 				off &= UT32_MAX;
 			}
 			if (p->hasrefs) {
-				const char *rstr = p->hasrefs (p->user, addr + i, false);
+				char *rstr = p->hasrefs (p->user, addr + i, false);
 				if (rstr && *rstr) {
 					printfmt (" @%s", rstr);
 				}
+				free (rstr);
 				rstr = p->hasrefs (p->user, off, true);
 				if (rstr && *rstr) {
 					printfmt ("%s", rstr);
 				}
+				free (rstr);
 			}
 		}
 		if (p && p->use_comments) {
@@ -1052,6 +1160,13 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 				printfmt (" ");
 			}
 			for (j = i; j < i + inc; j++) {
+				if (p && p->offname) {
+					a = p->offname (p->user, addr + j);
+					if (p->colorfor && a && *a) {
+						const char *color = p->colorfor (p->user, addr + j, true);
+						printfmt ("%s  ; %s"Color_RESET, color?color: "", a);
+					}
+				}
 				char *comment = p->get_comments (p->user, addr + j);
 				if (comment) {
 					if (p && p->colorfor) {
@@ -1070,9 +1185,35 @@ R_API void r_print_hexdump(RPrint *p, ut64 addr, const ut8 *buf, int len, int ba
 			}
 		}
 		printfmt ("\n");
+		rows++;
+
+		if (p && p->cfmt && *p->cfmt) {
+			if (row_have_cursor != -1) {
+				int i=0;
+				printfmt (" _________");
+				if (!compact) {
+					printfmt ("_");
+				}
+				for (i = 0; i < row_have_cursor; i++) {
+					if (!pairs || (!compact && i % 2)) {
+						printfmt ("___");
+					} else {
+						printfmt ("__");
+					}
+				}
+				printfmt ("__|\n");
+				printfmt ("| cmd.hexcursor = %s\n", p->cfmt);
+				p->coreb.cmdf (p->coreb.core,
+						"%s @ 0x%08"PFMT64x, p->cfmt, row_have_addr);
+			}
+		}
 	}
 }
 
+R_API void r_print_hexdump_simple(const ut8 *buf, int len) {
+	r_print_hexdump (NULL, 0, buf, len, 16, 16, 0);
+}
+
 static const char* getbytediff(char *fmt, ut8 a, ut8 b) {
 	if (*fmt) {
 		if (a == b) {
@@ -1083,7 +1224,6 @@ static const char* getbytediff(char *fmt, ut8 a, ut8 b) {
 	} else {
 		sprintf (fmt, "%02x", a);
 	}
-	// else sprintf (fmt, "%02x", a);
 	return fmt;
 }
 
@@ -1102,8 +1242,8 @@ static const char* getchardiff(char *fmt, ut8 a, ut8 b) {
 	return fmt;
 }
 
-#define BD(a, b) getbytediff (fmt, a[i + j], b[i + j])
-#define CD(a, b) getchardiff (fmt, a[i + j], b[i + j])
+#define BD(a, b) getbytediff (fmt, (a)[i + j], (b)[i + j])
+#define CD(a, b) getchardiff (fmt, (a)[i + j], (b)[i + j])
 
 static ut8* M(const ut8 *b, int len) {
 	ut8 *r = malloc (len + 16);
@@ -1138,31 +1278,31 @@ R_API void r_print_hexdiff(RPrint *p, ut64 aa, const ut8 *_a, ut64 ba, const ut8
 		p->cb_printf ("0x%08" PFMT64x " ", aa + i);
 		for (j = 0; j < min; j++) {
 			*fmt = color;
-			r_print_cursor (p, i + j, 1);
+			r_print_cursor (p, i + j, 1, 1);
 			p->cb_printf (BD (a, b));
-			r_print_cursor (p, i + j, 0);
+			r_print_cursor (p, i + j, 1, 0);
 		}
 		p->cb_printf (" ");
 		for (j = 0; j < min; j++) {
 			*fmt = color;
-			r_print_cursor (p, i + j, 1);
+			r_print_cursor (p, i + j, 1, 1);
 			p->cb_printf ("%s", CD (a, b));
-			r_print_cursor (p, i + j, 0);
+			r_print_cursor (p, i + j, 1, 0);
 		}
 		if (scndcol) {
 			p->cb_printf (" %c 0x%08" PFMT64x " ", linediff, ba + i);
 			for (j = 0; j < min; j++) {
 				*fmt = color;
-				r_print_cursor (p, i + j, 1);
+				r_print_cursor (p, i + j, 1, 1);
 				p->cb_printf (BD (b, a));
-				r_print_cursor (p, i + j, 0);
+				r_print_cursor (p, i + j, 1, 0);
 			}
 			p->cb_printf (" ");
 			for (j = 0; j < min; j++) {
 				*fmt = color;
-				r_print_cursor (p, i + j, 1);
+				r_print_cursor (p, i + j, 1, 1);
 				p->cb_printf ("%s", CD (b, a));
-				r_print_cursor (p, i + j, 0);
+				r_print_cursor (p, i + j, 1, 0);
 			}
 			p->cb_printf ("\n");
 		} else {
@@ -1200,9 +1340,9 @@ R_API void r_print_raw(RPrint *p, ut64 addr, const ut8 *buf, int len, int offlin
 				}
 				ch = buf[i + j];
 				if (p->cur_enabled) {
-					r_print_cursor (p, i + j, 1);
+					r_print_cursor (p, i + j, 1, 1);
 					p->cb_printf ("%c", IS_PRINTABLE (ch)? ch: ' ');
-					r_print_cursor (p, i + j, 0);
+					r_print_cursor (p, i + j, 1, 0);
 				} else {
 					p->cb_printf ("%c", IS_PRINTABLE (ch)? ch: ' ');
 				}
@@ -1431,14 +1571,22 @@ R_API void r_print_fill(RPrint *p, const ut8 *arr, int size, ut64 addr, int step
 		if (next < arr[i]) {
 			//if (arr[i]>0 && i>0) p->cb_printf ("  ");
 			if (arr[i] > INC) {
-				for (j = 0; j < next + base; j += INC) p->cb_printf (i? " ": "'");
+				for (j = 0; j < next + base; j += INC) {
+					p->cb_printf (i ? " " : "'");
+				}
+			}
+			for (j = next + INC; j + base < arr[i]; j += INC) {
+				p->cb_printf ("_");
 			}
-			for (j = next + INC; j + base < arr[i]; j += INC) p->cb_printf ("_");
 		} else {
 			if (i == 0) {
-				for (j = INC; j < arr[i] + base; j += INC) p->cb_printf ("'");
+				for (j = INC; j < arr[i] + base; j += INC) {
+					p->cb_printf ("'");
+				}
 			} else {
-				for (j = INC; j < arr[i] + base; j += INC) p->cb_printf (" ");
+				for (j = INC; j < arr[i] + base; j += INC) {
+					p->cb_printf (" ");
+				}
 			}
 		}
 		//for (j=1;j<arr[i]; j+=INC) p->cb_printf (under);
@@ -1634,7 +1782,9 @@ static bool issymbol(char c) {
 static bool check_arg_name (RPrint *print, char *p, ut64 func_addr) {
 	if (func_addr && print->exists_var) {
 		int z;
-		for (z = 0; p[z] && (IS_ALPHA (p[z]) || IS_DIGIT (p[z]) || p[z] == '_'); z++);
+		for (z = 0; p[z] && (IS_ALPHA (p[z]) || IS_DIGIT (p[z]) || p[z] == '_'); z++) {
+			;
+		}
 		char tmp = p[z];
 		p[z] = '\0';
 		bool ret = print->exists_var (print, func_addr, p);
@@ -1670,7 +1820,8 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 	memset (o, 0, COLORIZE_BUFSIZE);
 	for (i = j = 0; p[i]; i++, j++) {
 		/* colorize numbers */
-		if ((ishexprefix (&p[i]) && previous != ':') || (isdigit (p[i]) && issymbol (previous))) {
+		if ((ishexprefix (&p[i]) && previous != ':') \
+		     || (isdigit ((ut8)p[i]) && issymbol (previous))) {
 			int nlen = strlen (num);
 			if (nlen + j >= sizeof (o)) {
 				eprintf ("Colorize buffer is too small\n");
@@ -1811,7 +1962,6 @@ R_API char* r_print_colorize_opcode(RPrint *print, char *p, const char *reg, con
 		char *t_o = o;
 		/* o = malloc (opcode_sz+21); */
 		memmove (o, t_o, opcode_sz);
-		opcode_sz += 21;
 		/* free (t_o); */
 	}
 	strcpy (o + j, reset);
@@ -1922,6 +2072,9 @@ R_API void r_print_hex_from_bin (RPrint *p, char *bin_str) {
 	int i, j, index;
 	RPrint myp = {.cb_printf = libc_printf};
 	const int len = strlen (bin_str);
+	if (!len) {
+		return;
+	}
 	ut64 n, *buf = malloc (sizeof (ut64) * ((len + 63) / 64));
 	if (buf == NULL) {
 		eprintf ("allocation failed\n");
@@ -1950,3 +2103,29 @@ R_API void r_print_hex_from_bin (RPrint *p, char *bin_str) {
 	p->cb_printf ("\n");
 	free (buf);
 }
+
+R_API const char* r_print_rowlog(RPrint *print, const char *str) {
+	int use_color = print->flags & R_PRINT_FLAGS_COLOR;
+	bool verbose = print->scr_prompt;
+	if (!verbose) {
+		return NULL;
+	}
+	if (use_color) {
+		eprintf ("[ ] "Color_YELLOW"%s\r["Color_RESET, str);
+	} else {
+		eprintf ("[ ] %s\r[", str);
+	}
+	return str;
+}
+
+R_API void r_print_rowlog_done(RPrint *print, const char *str) {
+	int use_color = print->flags & R_PRINT_FLAGS_COLOR;
+	bool verbose =  print->scr_prompt;
+	if (verbose) {
+		if (use_color) {
+			eprintf ("\r"Color_GREEN"[x]"Color_RESET" %s\n", str);
+		} else {
+			eprintf ("\r[x] %s\n", str);
+		}
+	}
+}
