@@ -66,3 +66,9 @@ R_API const char* r_signal_to_string (int code) {
 	}
 	return NULL;
 }
+
+#if HAVE_PTHREAD
+R_API void r_signal_sigmask(int how, const sigset_t *newmask, sigset_t *oldmask) {
+	pthread_sigmask (how, newmask, oldmask);
+}
+#endif
\ No newline at end of file
