@@ -86,7 +86,9 @@ R_API char *r_str_trim(char *str) {
 // TODO: rename to r_str_trim_head_ro()
 R_API const char *r_str_trim_ro(const char *str) {
 	if (str) {
-		for (; *str && IS_WHITECHAR (*str); str++);
+		for (; *str && IS_WHITECHAR (*str); str++) {
+			;
+		}
 	}
 	return str;
 }
@@ -95,7 +97,9 @@ R_API const char *r_str_trim_ro(const char *str) {
 // TODO: rename to r_str_trim_head_wp()
 R_API const char *r_str_trim_wp(const char *str) {
 	if (str) {
-		for (; *str && !IS_WHITESPACE (*str); str++);
+		for (; *str && !IS_WHITESPACE (*str); str++) {
+			;
+		}
 	}
 	return str;
 }
@@ -172,11 +176,10 @@ R_API int r_str_ansi_trim(char *str, int str_len, int n) {
 					i += 18;
 				}
 			} else if (ch2 == '[') {
-				for (++i; (i < str_len) && str[i]
-					&& str[i] != 'J'
-					&& str[i] != 'm'
-					&& str[i] != 'H';
-				     i++);
+				for (++i; (i < str_len) && str[i] && str[i] != 'J' && str[i] != 'm' && str[i] != 'H';
+					i++) {
+					;
+				}
 			}
 		} else if ((str[i] & 0xc0) != 0x80) {
 			len++;
@@ -187,4 +190,3 @@ R_API int r_str_ansi_trim(char *str, int str_len, int n) {
 	str[back] = 0;
 	return back;
 }
-
