@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2017 - pancake */
+/* radare - LGPL - Copyright 2009-2018 - pancake */
 
 #include <r_th.h>
 
@@ -21,12 +21,15 @@ static void *_r_th_launcher(void *_th) {
 		r_th_lock_wait (th->lock);
 	}
 #endif
+	r_th_lock_enter (th->lock);
 	do {
-		// CID 1378280:  API usage errors  (LOCK)
-		// "r_th_lock_leave" unlocks "th->lock->lock" while it is unlocked.
 		r_th_lock_leave (th->lock);
 		th->running = true;
 		ret = th->fun (th);
+		if (ret < 0) {
+			// th has been freed
+			return 0;
+		}
 		th->running = false;
 		r_th_lock_enter (th->lock);
 	} while (ret);
@@ -43,6 +46,17 @@ R_API int r_th_push_task(struct r_th_t *th, void *user) {
 	return ret;
 }
 
+R_API R_TH_TID r_th_self(void) {
+#if HAVE_PTHREAD
+	return pthread_self ();
+#elif __WINDOWS__
+	return (HANDLE)GetCurrentThreadId ();
+#else
+#pragma message("Not implemented on windows")
+	return (R_TH_TID)-1;
+#endif
+}
+
 R_API RThread *r_th_new(R_TH_FUNCTION(fun), void *user, int delay) {
 	RThread *th = R_NEW0 (RThread);
 	if (th) {
@@ -54,6 +68,8 @@ R_API RThread *r_th_new(R_TH_FUNCTION(fun), void *user, int delay) {
 		th->breaked = false;
 		th->ready = false;
 #if HAVE_PTHREAD
+		pthread_cond_init (&th->_cond, NULL);
+		pthread_mutex_init (&th->_mutex, NULL);
 		pthread_create (&th->tid, NULL, _r_th_launcher, th);
 #elif __WINDOWS__ && !defined(__CYGWIN__)
 		th->tid = CreateThread (NULL, 0, _r_th_launcher, th, 0, 0);
@@ -66,8 +82,8 @@ R_API void r_th_break(RThread *th) {
 	th->breaked = true;
 }
 
-R_API int r_th_kill(RThread *th, int force) {
-	if (!th) {
+R_API bool r_th_kill(RThread *th, bool force) {
+	if (!th || !th->tid) {
 		return false;
 	}
 	th->breaked = true;
@@ -85,6 +101,48 @@ R_API int r_th_kill(RThread *th, int force) {
 	return 0;
 }
 
+// running in parent
+R_API bool r_th_pause(RThread *th, bool enable) {
+	if (!th) {
+		return false;
+	}
+	if (enable) {
+#if HAVE_PTHREAD
+		pthread_mutex_trylock (&th->_mutex);
+#else
+#pragma message("warning r_th_pause not implemented")
+#endif
+	} else {
+#if HAVE_PTHREAD
+		// pthread_cond_signal (&th->_cond);
+		pthread_mutex_unlock (&th->_mutex);
+#else
+#pragma message("warning r_th_pause not implemented")
+#endif
+	}
+	return true;
+}
+
+// running in thread
+R_API bool r_th_try_pause(RThread *th) {
+	if (!th) {
+		return false;
+	}
+#if HAVE_PTHREAD
+	// pthread_mutex_lock (&th->_mutex);
+	// pthread_mutex_unlock (&th->_mutex);
+	if (pthread_mutex_trylock (&th->_mutex) != -1) {
+		pthread_mutex_unlock (&th->_mutex);
+	} else {
+		// oops
+	}
+	// pthread_cond_wait (&th->_cond, &th->_mutex);
+#else
+#pragma message("warning r_th_try_pause not implemented")
+#endif
+	return true;
+}
+
 R_API bool r_th_start(RThread *th, int enable) {
 	bool ret = true;
 	if (enable) {
@@ -98,7 +156,7 @@ R_API bool r_th_start(RThread *th, int enable) {
 	} else {
 		if (th->running) {
 			// stop thread
-			r_th_kill (th, 0);
+			//r_th_kill (th, 0);
 			r_th_lock_enter (th->lock); // deadlock?
 		}
 	}
@@ -125,7 +183,9 @@ R_API int r_th_wait_async(struct r_th_t *th) {
 }
 
 R_API void *r_th_free(struct r_th_t *th) {
-	r_th_kill (th, true);
+	if (!th) {
+		return NULL;
+	}
 #if __WINDOWS__ && !defined(__CYGWIN__)
 	CloseHandle (th->tid);
 #endif
@@ -134,6 +194,15 @@ R_API void *r_th_free(struct r_th_t *th) {
 	return NULL;
 }
 
+R_API void *r_th_kill_free(struct r_th_t *th) {
+	if (!th) {
+		return NULL;
+	}
+	r_th_kill (th, true);
+	r_th_free (th);
+	return NULL;
+}
+
 #if 0
 
 // Thread Pipes
