@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2014-2017 - thelemon, kazarmy, pancake */
+/* radare2 - LGPL - Copyright 2014-2018 - thelemon, kazarmy, pancake */
 
 #include <r_types.h>
 #include <r_util.h>
@@ -491,16 +491,24 @@ R_API int r_utf8_decode (const ut8 *ptr, int ptrlen, RRune *ch) {
 		return 0;
 	}
 	if (ptrlen > 0 && ptr[0] < 0x80) {
-		if (ch) *ch = (ut32)ptr[0];
+		if (ch) {
+			*ch = (ut32)ptr[0];
+		}
 		return 1;
 	} else if (ptrlen>1 && (ptr[0]&0xe0) == 0xc0 && (ptr[1]&0xc0) == 0x80) {
-		if (ch) *ch = (ptr[0] & 0x1f) << 6 | (ptr[1] & 0x3f);
+		if (ch) {
+			*ch = (ptr[0] & 0x1f) << 6 | (ptr[1] & 0x3f);
+		}
 		return 2;
 	} else if (ptrlen>2 && (ptr[0]&0xf0) == 0xe0 && (ptr[1]&0xc0) == 0x80 && (ptr[2]&0xc0) == 0x80) {
-		if (ch) *ch = (ptr[0] & 0xf) << 12 | (ptr[1] & 0x3f) << 6 | (ptr[2] & 0x3f);
+		if (ch) {
+			*ch = (ptr[0] & 0xf) << 12 | (ptr[1] & 0x3f) << 6 | (ptr[2] & 0x3f);
+		}
 		return 3;
 	} else if (ptrlen>3 && (ptr[0]&0xf8) == 0xf0 && (ptr[1]&0xc0) == 0x80 && (ptr[2]&0xc0) == 0x80 && (ptr[3]&0xc0) == 0x80) {
-		if (ch) *ch = (ptr[0] & 7) << 18 | (ptr[1] & 0x3f) << 12 | (ptr[2] & 0x3f) << 6 | (ptr[3] & 0x3f);
+		if (ch) {
+			*ch = (ptr[0] & 7) << 18 | (ptr[1] & 0x3f) << 12 | (ptr[2] & 0x3f) << 6 | (ptr[3] & 0x3f);
+		}
 		return 4;
 	}
 	return 0;
@@ -537,10 +545,13 @@ R_API int r_utf8_encode (ut8 *ptr, const RRune ch) {
 R_API int r_utf8_encode_str (const RRune *str, ut8 *dst, const int dst_length) {
 	int i, pos = 0;
 
-	if (!str || !dst) return -1;
+	if (!str || !dst) {
+		return -1;
+	}
 
-	for (i = 0; str[i] && pos < dst_length - 1; i++)
+	for (i = 0; i < sizeof (str) - 1 && str[i] && pos < dst_length - 1; i++) {
 		pos += r_utf8_encode (&dst[pos], str[i]);
+	}
 
 	dst[pos++] = '\0';
 	return pos;
@@ -565,8 +576,9 @@ R_API int r_utf8_strlen (const ut8 *str) {
 	int i, len = 0;
 
 	for (i = 0; str[i]; i++) {
-		if ((str[i] & 0xc0) != 0x80)
+		if ((str[i] & 0xc0) != 0x80) {
 			len++;
+		}
 	}
 
 	return len;
@@ -574,13 +586,12 @@ R_API int r_utf8_strlen (const ut8 *str) {
 
 R_API int r_isprint (const RRune c) {
 	const int last = nonprintable_ranges_count;
-	int low, hi, mid;
 
-	low = 0;
-	hi = last - 1;
+	int low = 0;
+	int hi = last - 1;
 
 	do {
-		mid = (low + hi) >> 1;
+		int mid = (low + hi) >> 1;
 		if (c >= nonprintable_ranges[mid].from && c <= nonprintable_ranges[mid].to) {
 			return false;
 		}
@@ -596,30 +607,66 @@ R_API int r_isprint (const RRune c) {
 }
 
 #if __WINDOWS__
-R_API char *r_utf16_to_utf8 (const wchar_t *wc) {
-	char *rutf8;
+R_API char *r_utf16_to_utf8_l (const wchar_t *wc, int len) {
+	if (!wc || !len || len < -1) {
+		return NULL;
+	}
+	char *rutf8 = NULL;
 	int csize;
 
-	if ((csize = WideCharToMultiByte (CP_UTF8, 0, wc, -1, NULL, 0, NULL, NULL))) {
+	if ((csize = WideCharToMultiByte (CP_UTF8, 0, wc, len, NULL, 0, NULL, NULL))) {
+		++csize;
 		if ((rutf8 = malloc (csize))) {
-			WideCharToMultiByte (CP_UTF8, 0, wc, -1, rutf8, csize, NULL, NULL);
+			WideCharToMultiByte (CP_UTF8, 0, wc, len, rutf8, csize, NULL, NULL);
+			if (len != -1) {
+				rutf8[csize - 1] = '\0';
+			}
 		}
 	}
 	return rutf8;
 }
 
-R_API wchar_t *r_utf8_to_utf16 (const char *cstring) {
-	wchar_t *rutf16;
+R_API wchar_t *r_utf8_to_utf16_l (const char *cstring, int len) {
+	if (!cstring || !len || len < -1) {
+		return NULL;
+	}
+	wchar_t *rutf16 = NULL;
 	int wcsize;
 
-	if ((wcsize = MultiByteToWideChar (CP_UTF8, 0, cstring, -1, NULL, 0))) {
+	if ((wcsize = MultiByteToWideChar (CP_UTF8, 0, cstring, len, NULL, 0))) {
+		++wcsize;
 		if ((rutf16 = (wchar_t *) calloc (wcsize, sizeof (wchar_t)))) {
-			MultiByteToWideChar (CP_UTF8, 0, cstring, -1, rutf16, wcsize);
+			MultiByteToWideChar (CP_UTF8, 0, cstring, len, rutf16, wcsize);
+			if (len != -1) {
+				rutf16[wcsize - 1] = L'\0';
+			}
 		}
 	}
 	return rutf16;
 }
-#endif
+
+R_API const char *r_acp_to_utf8_l (const ut8 *str, int len) {
+	if (!str || !len || len < -1) {
+		return NULL;
+	}
+	int wcsize;
+	if ((wcsize = MultiByteToWideChar (CP_ACP, 0, str, len, NULL, 0))) {
+		wchar_t *rutf16;
+		++wcsize;
+		if ((rutf16 = (wchar_t *) calloc (wcsize, sizeof (wchar_t)))) {
+			MultiByteToWideChar (CP_ACP, 0, str, len, rutf16, wcsize);
+			if (len != -1) {
+				rutf16[wcsize - 1] = L'\0';
+			}
+			const char *ret = r_utf16_to_utf8_l (rutf16, wcsize);
+			free (rutf16);
+			return ret;
+		}
+	}
+	return NULL;
+}
+
+#endif // __WINDOWS__
 
 R_API int r_utf_block_idx (RRune ch) {
 	const int last = r_utf_blocks_count;
@@ -644,12 +691,15 @@ R_API int r_utf_block_idx (RRune ch) {
 	return r_utf_blocks_count - 1; /* index for "No_Block" */
 }
 
-R_API int *r_utf_block_list (const ut8 *str) {
+/* str must be UTF8-encoded */
+R_API int *r_utf_block_list(const ut8 *str, int len) {
 	if (!str) {
 		return NULL;
 	}
+	if (len < 0) {
+		len = strlen ((const char *)str);
+	}
 	static bool has_block[r_utf_blocks_count] = {0};
-	int len = strlen ((const char *)str);
 	int *list = R_NEWS (int, len + 1);
 	if (!list) {
 		return NULL;
