@@ -108,7 +108,9 @@ R_API void r_vector_assign(RVector *vec, void *p, void *elem) {
 
 R_API void *r_vector_assign_at(RVector *vec, size_t index, void *elem) {
 	void *p = r_vector_index_ptr (vec, index);
-	r_vector_assign (vec, p, elem);
+	if (elem) {
+		r_vector_assign (vec, p, elem);
+	}
 	return p;
 }
 
@@ -134,7 +136,9 @@ R_API void *r_vector_insert(RVector *vec, size_t index, void *x) {
 		memmove ((char *)p + vec->elem_size, p, vec->elem_size * (vec->len - index));
 	}
 	vec->len++;
-	r_vector_assign (vec, p, x);
+	if (x) {
+		r_vector_assign (vec, p, x);
+	}
 	return p;
 }
 
@@ -148,7 +152,9 @@ R_API void *r_vector_insert_range(RVector *vec, size_t index, void *first, size_
 		memmove ((char *)p + sz, p, vec->elem_size * (vec->len - index));
 	}
 	vec->len += count;
-	memcpy (p, first, sz);
+	if (first) {
+		memcpy (p, first, sz);
+	}
 	return p;
 }
 
@@ -160,14 +166,18 @@ R_API void r_vector_pop(RVector *vec, void *into) {
 }
 
 R_API void r_vector_pop_front(RVector *vec, void *into) {
-	return r_vector_remove_at (vec, 0, into);
+	r_vector_remove_at (vec, 0, into);
 }
 
 R_API void *r_vector_push(RVector *vec, void *x) {
 	if (vec->len >= vec->capacity) {
 		RESIZE_OR_RETURN_NULL (NEXT_VECTOR_CAPACITY);
 	}
-	return r_vector_assign_at (vec, vec->len++, x);
+	void *p = r_vector_index_ptr (vec, vec->len++);
+	if (x) {
+		r_vector_assign (vec, p, x);
+	}
+	return p;
 }
 
 R_API void *r_vector_push_front(RVector *vec, void *x) {
@@ -240,6 +250,16 @@ R_API void *r_pvector_remove_at(RPVector *vec, size_t index) {
 	return r;
 }
 
+R_API void r_pvector_remove_data(RPVector *vec, void *x) {
+	void **el = r_pvector_contains (vec, x);
+	if (!el) {
+		return;
+	}
+
+	size_t index = el - (void **)vec->v.a;
+	r_vector_remove_at (&vec->v, index, NULL);
+}
+
 R_API void *r_pvector_pop(RPVector *vec) {
 	void *r = r_pvector_at (vec, vec->v.len - 1);
 	r_vector_pop (&vec->v, NULL);
@@ -254,17 +274,20 @@ R_API void *r_pvector_pop_front(RPVector *vec) {
 
 // CLRS Quicksort. It is slow, but simple.
 static void quick_sort(void **a, size_t n, RPVectorComparator cmp) {
-	if (n <= 1) return;
+	if (n <= 1) {
+		return;
+	}
 	int i = rand() % n, j = 0;
 	void *t, *pivot = a[i];
 	a[i] = a[n - 1];
-	for (i = 0; i < n - 1; i++)
+	for (i = 0; i < n - 1; i++) {
 		if (cmp (a[i], pivot) < 0) {
 			t = a[i];
 			a[i] = a[j];
 			a[j] = t;
 			j++;
 		}
+	}
 	a[n - 1] = a[j];
 	a[j] = pivot;
 	quick_sort (a, j, cmp);
