@@ -1,11 +1,12 @@
 /* radare2 - LGPL - Copyright 2017-2018 - wargio */
 
 #include <r_util.h>
+#include <r_cons.h>
 #include <stdlib.h>
 #include <string.h>
 #include "./x509.h"
 
-bool r_x509_parse_validity (RX509Validity *validity, RASN1Object *object) {
+static bool r_x509_parse_validity(RX509Validity *validity, RASN1Object *object) {
 	RASN1Object *o;
 	if (!validity || !object || object->list.length != 2) {
 		return false;
@@ -33,7 +34,7 @@ bool r_x509_parse_validity (RX509Validity *validity, RASN1Object *object) {
 	return true;
 }
 
-bool r_x509_parse_algorithmidentifier (RX509AlgorithmIdentifier *ai, RASN1Object * object) {
+bool r_x509_parse_algorithmidentifier(RX509AlgorithmIdentifier *ai, RASN1Object * object) {
 	if (!ai || !object || object->list.length < 1 || !object->list.objects) {
 		return false;
 	}
@@ -45,7 +46,7 @@ bool r_x509_parse_algorithmidentifier (RX509AlgorithmIdentifier *ai, RASN1Object
 	return true;
 }
 
-bool r_x509_parse_subjectpublickeyinfo (RX509SubjectPublicKeyInfo * spki, RASN1Object *object) {
+bool r_x509_parse_subjectpublickeyinfo(RX509SubjectPublicKeyInfo * spki, RASN1Object *object) {
 	RASN1Object *o;
 	if (!spki || !object || object->list.length != 2) {
 		return false;
@@ -82,8 +83,7 @@ bool r_x509_parse_name (RX509Name *name, RASN1Object * object) {
 		name->oids = (RASN1String**) calloc (name->length, sizeof (RASN1String*));
 		if (!name->oids) {
 			name->length = 0;
-			free (name->names);
-			name->names = NULL;
+			R_FREE (name->names);
 			return false;
 		}
 		for (i = 0; i < object->list.length; ++i) {
@@ -207,48 +207,35 @@ bool r_x509_parse_tbscertificate (RX509TBSCertificate *tbsc, RASN1Object * objec
 }
 
 RX509Certificate * r_x509_parse_certificate (RASN1Object *object) {
-	RX509Certificate *certificate;
-	RASN1Object *tmp;
 	if (!object) {
 		return NULL;
 	}
-	certificate = (RX509Certificate*) malloc (sizeof (RX509Certificate));
-	if (!certificate) {
-		r_asn1_free_object (object);
-		return NULL;
+	RX509Certificate *cert = R_NEW0 (RX509Certificate);
+	if (!cert) {
+		goto fail;
 	}
-	memset (certificate, 0, sizeof (RX509Certificate));
-
 	if (object->klass != CLASS_UNIVERSAL || object->form != FORM_CONSTRUCTED || object->list.length != 3) {
-		// Malformed certificate
-		// It needs to have tbsCertificate, algorithmIdentifier and a signature
-		r_asn1_free_object (object);
-		free (certificate);
-		return NULL;
+		R_FREE (cert);
+		goto fail;
 	}
-	tmp = object->list.objects[2];
+	RASN1Object *tmp = object->list.objects[2];
 	if (!tmp) {
-		r_asn1_free_object (object);
-		free (certificate);
-		return NULL;
+		R_FREE (cert);
+		goto fail;
 	}
 	if (tmp->klass != CLASS_UNIVERSAL || tmp->form != FORM_PRIMITIVE || tmp->tag != TAG_BITSTRING) {
-		r_asn1_free_object (object);
-		free (certificate);
-		return NULL;
+		R_FREE (cert);
+		goto fail;
 	}
+	cert->signature = r_asn1_create_binary (object->list.objects[2]->sector, object->list.objects[2]->length);
+	r_x509_parse_tbscertificate (&cert->tbsCertificate, object->list.objects[0]);
 
-	certificate->signature = r_asn1_create_binary (object->list.objects[2]->sector, object->list.objects[2]->length);
-
-	r_x509_parse_tbscertificate (&certificate->tbsCertificate, object->list.objects[0]);
-
-	if (!r_x509_parse_algorithmidentifier (&certificate->algorithmIdentifier, object->list.objects[1])) {
-		r_asn1_free_object (object);
-		free (certificate);
-		return NULL;
+	if (!r_x509_parse_algorithmidentifier (&cert->algorithmIdentifier, object->list.objects[1])) {
+		R_FREE (cert);
 	}
+fail:
 	r_asn1_free_object (object);
-	return certificate;
+	return cert;
 }
 
 RX509Certificate * r_x509_parse_certificate2 (const ut8 *buffer, ut32 length) {
@@ -257,7 +244,7 @@ RX509Certificate * r_x509_parse_certificate2 (const ut8 *buffer, ut32 length) {
 	if (!buffer || !length) {
 		return NULL;
 	}
-	object = r_asn1_create_object (buffer, length);
+	object = r_asn1_create_object (buffer, length, buffer);
 	certificate = r_x509_parse_certificate (object);
 	//object freed by r_x509_parse_certificate
 	return certificate;
@@ -277,7 +264,7 @@ RX509CRLEntry *r_x509_parse_crlentry (RASN1Object *object) {
 	return entry;
 }
 
-RX509CertificateRevocationList* r_x509_parse_crl (RASN1Object *object) {
+R_API RX509CertificateRevocationList* r_x509_parse_crl (RASN1Object *object) {
 	RX509CertificateRevocationList *crl;
 	RASN1Object **elems;
 	if (!object || object->list.length < 4) {
@@ -309,21 +296,19 @@ RX509CertificateRevocationList* r_x509_parse_crl (RASN1Object *object) {
 }
 
 void r_x509_free_algorithmidentifier (RX509AlgorithmIdentifier * ai) {
-	if (!ai) {
-		return;
+	if (ai) {
+		// no need to free ai, since this functions is used internally
+		r_asn1_free_string (ai->algorithm);
+		r_asn1_free_string (ai->parameters);
 	}
-	r_asn1_free_string (ai->algorithm);
-	r_asn1_free_string (ai->parameters);
-	//no need to free ai, since this functions is used internally
 }
 
-void r_x509_free_validity (RX509Validity * validity) {
-	if (!validity) {
-		return;
+static void r_x509_free_validity (RX509Validity * validity) {
+	if (validity) {
+		// not freeing validity since it's not allocated dinamically
+		r_asn1_free_string (validity->notAfter);
+		r_asn1_free_string (validity->notBefore);
 	}
-	r_asn1_free_string (validity->notAfter);
-	r_asn1_free_string (validity->notBefore);
-	// not freeing validity since it's not allocated dinamically
 }
 
 void r_x509_free_name (RX509Name * name) {
@@ -400,7 +385,7 @@ void r_x509_free_certificate (RX509Certificate * certificate) {
 	}
 }
 
-void r_x509_free_crlentry (RX509CRLEntry *entry) {
+static void r_x509_free_crlentry (RX509CRLEntry *entry) {
 	if (entry) {
 		r_asn1_free_binary (entry->userCertificate);
 		r_asn1_free_string (entry->revocationDate);
@@ -420,59 +405,44 @@ void r_x509_free_crl (RX509CertificateRevocationList *crl) {
 				r_x509_free_crlentry (crl->revokedCertificates[i]);
 				crl->revokedCertificates[i] = NULL;
 			}
-			free (crl->revokedCertificates);
-			crl->revokedCertificates = NULL;
+			R_FREE (crl->revokedCertificates);
 		}
 		free (crl);
 	}
 }
 
-char* r_x509_validity_dump (RX509Validity* validity, char* buffer, ut32 length, const char* pad) {
-	int p;
-	if (!validity || !buffer || !length) {
-		return NULL;
-	}
-	if (!pad)
-		pad = "";
-	const char* b = validity->notBefore ? validity->notBefore->string : "Missing";
-	const char* a = validity->notAfter ? validity->notAfter->string : "Missing";
-	p = snprintf (buffer, length, "%sNot Before: %s\n%sNot After: %s\n", pad, b, pad, a);
-	return p < 0 ? NULL : buffer + (ut32) p;
-}
-
-char* r_x509_name_dump (RX509Name* name, char* buffer, ut32 length, const char* pad) {
-	ut32 i, p, len;
-	int r;
-	char* c;
-	if (!name || !buffer || !length) {
-		return NULL;
+static void r_x509_validity_dump (RX509Validity* validity, const char* pad, RStrBuf *sb) {
+	if (!validity) {
+		return;
 	}
 	if (!pad) {
 		pad = "";
 	}
-	len = length;
-	c = buffer;
-	for (i = 0, p = 0; i < name->length; ++i) {
+	const char* b = validity->notBefore ? validity->notBefore->string : "Missing";
+	const char* a = validity->notAfter ? validity->notAfter->string : "Missing";
+	r_strbuf_appendf (sb, "%sNot Before: %s\n%sNot After: %s\n", pad, b, pad, a);
+}
+
+void r_x509_name_dump (RX509Name* name, const char* pad, RStrBuf *sb) {
+	ut32 i;
+	if (!name) {
+		return;
+	}
+	if (!pad) {
+		pad = "";
+	}
+	for (i = 0; i < name->length; ++i) {
 		if (!name->oids[i] || !name->names[i]) {
 			continue;
 		}
-		if (len <= p) {
-			return NULL;
-		}
-		r = snprintf (c + p, len - p, "%s%s: %s\n", pad, name->oids[i]->string, name->names[i]->string);
-		p += r;
-		if (r < 0 || len < p) {
-			return NULL;
-		}
+		r_strbuf_appendf (sb, "%s%s: %s\n", pad, name->oids[i]->string, name->names[i]->string);
 	}
-	return c + p;
 }
 
-char* r_x509_subjectpublickeyinfo_dump (RX509SubjectPublicKeyInfo* spki, char* buffer, ut32 length, const char* pad) {
-	int r;
+static void r_x509_subjectpublickeyinfo_dump (RX509SubjectPublicKeyInfo* spki, const char* pad, RStrBuf *sb) {
 	const char *a;
-	if (!spki || !buffer || !length) {
-		return NULL;
+	if (!spki) {
+		return;
 	}
 	if (!pad) {
 		pad = "";
@@ -485,202 +455,117 @@ char* r_x509_subjectpublickeyinfo_dump (RX509SubjectPublicKeyInfo* spki, char* b
 	//	RASN1String* e = r_asn1_stringify_bytes (spki->subjectPublicKeyExponent->sector, spki->subjectPublicKeyExponent->length);
 	//	r = snprintf (buffer, length, "%sAlgorithm: %s\n%sModule: %s\n%sExponent: %u bytes\n%s\n", pad, a, pad, m->string,
 	//				pad, spki->subjectPublicKeyExponent->length - 1, e->string);
-	r = snprintf (buffer, length, "%sAlgorithm: %s\n%sModule: %s\n%sExponent: %u bytes\n", pad, a, pad, m ? m->string : "Missing",
+	r_strbuf_appendf (sb, "%sAlgorithm: %s\n%sModule: %s\n%sExponent: %u bytes\n", pad, a, pad, m ? m->string : "Missing",
 				pad, spki->subjectPublicKeyExponent ? spki->subjectPublicKeyExponent->length - 1 : 0);
 	r_asn1_free_string (m);
 	//	r_asn1_free_string (e);
-	return r < 0 ? NULL : buffer + (ut32) r;
 }
 
-char* r_x509_extensions_dump (RX509Extensions* exts, char* buffer, ut32 length, const char* pad) {
-	ut32 i, p, len;
-	int r;
-	char* c;
-	if (!exts || !buffer || !length) {
-		return NULL;
+static void r_x509_extensions_dump (RX509Extensions* exts, const char* pad, RStrBuf *sb) {
+	ut32 i;
+	if (!exts) {
+		return;
 	}
 	if (!pad) {
 		pad = "";
 	}
-	len = length;
-	c = buffer;
-	for (i = 0, p = 0, r = 0; i < exts->length; ++i) {
-		//RASN1String *s;
+	for (i = 0; i < exts->length; ++i) {
 		RX509Extension *e = exts->extensions[i];
-		if (!e) continue;
+		if (!e) {
+			continue;
+		}
 		//TODO handle extensions..
 		//s = r_asn1_stringify_bytes (e->extnValue->sector, e->extnValue->length);
-		if (len < p) {
-			return NULL;
-		}
-		r = snprintf (c + p, len - p, "%s%s: %s\n%s%u bytes\n", pad,
-					e->extnID ? e->extnID->string : "Missing",
-					e->critical ? "critical" : "",
-					pad, e->extnValue ? e->extnValue->length : 0);
-		p += r;
+		r_strbuf_appendf (sb, "%s%s: %s\n%s%u bytes\n", pad,
+			e->extnID ? e->extnID->string : "Missing",
+			e->critical ? "critical" : "",
+			pad, e->extnValue ? e->extnValue->length : 0);
 		//r_asn1_free_string (s);
-		if (r < 0 || len <= p) {
-			return NULL;
-		}
 	}
-	return c + p;
 }
 
-char* r_x509_tbscertificate_dump (RX509TBSCertificate* tbsc, char* buffer, ut32 length, const char* pad) {
+static void r_x509_tbscertificate_dump (RX509TBSCertificate* tbsc, const char* pad, RStrBuf *sb) {
 	RASN1String *sid = NULL, *iid = NULL;
-	char *pad2, *tmp;
-	ut32 p;
-	int r;
-	if (!tbsc || !buffer || !length) {
-		return NULL;
+	if (!tbsc) {
+		return;
 	}
 	if (!pad) {
 		pad = "";
 	}
-	pad2 = r_str_newf ("%s  ", pad);
-	if (!pad2) return NULL;
-	r = snprintf (buffer, length, "%sVersion: v%u\n"
-				"%sSerial Number:\n%s  %s\n"
-				"%sSignature Algorithm:\n%s  %s\n"
-				"%sIssuer:\n",
-				pad, tbsc->version + 1,
-				pad, pad, tbsc->serialNumber ? tbsc->serialNumber->string : "Missing",
-				pad, pad, tbsc->signature.algorithm ? tbsc->signature.algorithm->string : "Missing",
-				pad);
-	p = (ut32) r;
-	if (r < 0 || length <= p || !(tmp = r_x509_name_dump (&tbsc->issuer, buffer + p, length - p, pad2))) {
-		free (pad2);
-		return NULL;
+	char *pad2 = r_str_newf ("%s  ", pad);
+	if (!pad2) {
+		return;
 	}
-	p = tmp - buffer;
-	if (length <= p) {
-		free (pad2);
-		return NULL;
-	}
-	r = snprintf (buffer + p, length - p, "%sValidity:\n", pad);
-	p += r;
-	if (r < 0 || length <= p || !(tmp = r_x509_validity_dump (&tbsc->validity, buffer + p, length - p, pad2))) {
-		free (pad2);
-		return NULL;
-	}
-	p = tmp - buffer;
-	if (r < 0 || length <= p) {
-		free (pad2);
-		return NULL;
-	}
-	r = snprintf (buffer + p, length - p, "%sSubject:\n", pad);
-	p += r;
-	if (r < 0 || length <= p || !(tmp = r_x509_name_dump (&tbsc->subject, buffer + p, length - p, pad2))) {
-		free (pad2);
-		return NULL;
-	}
-	p = tmp - buffer;
-	if (r < 0 || length <= p) {
-		free (pad2);
-		return NULL;
-	}
-	r = snprintf (buffer + p, length - p, "%sSubject Public Key Info:\n", pad);
-	p += r;
-	if (r < 0 || length <= p ||
-			!(tmp = r_x509_subjectpublickeyinfo_dump (&tbsc->subjectPublicKeyInfo, buffer + p, length - p, pad2))) {
-		free (pad2);
-		return NULL;
-	}
-	p = tmp - buffer;
+	r_strbuf_appendf (sb, "%sVersion: v%u\n"
+		"%sSerial Number:\n%s  %s\n"
+		"%sSignature Algorithm:\n%s  %s\n"
+		"%sIssuer:\n",
+		pad, tbsc->version + 1,
+		pad, pad, tbsc->serialNumber ? tbsc->serialNumber->string : "Missing",
+		pad, pad, tbsc->signature.algorithm ? tbsc->signature.algorithm->string : "Missing",
+		pad);
+	r_x509_name_dump (&tbsc->issuer, pad2, sb);
+
+	r_strbuf_appendf (sb, "%sValidity:\n", pad);
+	r_x509_validity_dump (&tbsc->validity, pad2, sb);
+
+	r_strbuf_appendf (sb, "%sSubject:\n", pad);
+	r_x509_name_dump (&tbsc->subject, pad2, sb);
+
+	r_strbuf_appendf (sb, "%sSubject Public Key Info:\n", pad);
+	r_x509_subjectpublickeyinfo_dump (&tbsc->subjectPublicKeyInfo, pad2, sb);
+
 	if (tbsc->issuerUniqueID) {
 		iid = r_asn1_stringify_integer (tbsc->issuerUniqueID->binary, tbsc->issuerUniqueID->length);
 		if (iid) {
-			if (length <= p) {
-				r_asn1_free_string (iid);
-				free (pad2);
-				return NULL;
-			}
-			r = snprintf (buffer + p, length - p, "%sIssuer Unique ID:\n%s  %s", pad, pad, iid->string);
-			p += r;
-		} else {
-			free (pad2);
-			return NULL;
+			r_strbuf_appendf (sb, "%sIssuer Unique ID:\n%s  %s", pad, pad, iid->string);
+			r_asn1_free_string (iid);
 		}
-		r_asn1_free_string (iid);
 	}
 	if (tbsc->subjectUniqueID) {
 		sid = r_asn1_stringify_integer (tbsc->subjectUniqueID->binary, tbsc->subjectUniqueID->length);
 		if (sid) {
-			if (length <= p) {
-				r_asn1_free_string (sid);
-				free (pad2);
-				return NULL;
-			}
-			r = snprintf (buffer + p, length - p, "%sSubject Unique ID:\n%s  %s", pad, pad, sid->string);
-			p += r;
-		} else {
-			free (pad2);
-			return NULL;
+			r_strbuf_appendf (sb, "%sSubject Unique ID:\n%s  %s", pad, pad, sid->string);
+			r_asn1_free_string (sid);
 		}
-		r_asn1_free_string (sid);
-	}
-	if (r < 0 || length <= p) {
-		free (pad2);
-		return NULL;
-	}
-	r = snprintf (buffer + p, length - p, "%sExtensions:\n", pad);
-	p += r;
-	if (r < 0 || length <= p || !(tmp = r_x509_extensions_dump (&tbsc->extensions, buffer + p, length - p, pad2))) {
-		free (pad2);
-		return NULL;
 	}
+
+	r_strbuf_appendf (sb, "%sExtensions:\n", pad);
+	r_x509_extensions_dump (&tbsc->extensions, pad2, sb);
 	free (pad2);
-	return buffer + p;
 }
 
-char* r_x509_certificate_dump (RX509Certificate* certificate, char* buffer, ut32 length, const char* pad) {
-	//	RASN1String *signature,
+void r_x509_certificate_dump (RX509Certificate* cert, const char* pad, RStrBuf *sb) {
 	RASN1String *algo = NULL;
-	ut32 p;
-	int r;
-	char *tbsc, *pad2;
-	if (!certificate || !buffer || !length) {
-		return NULL;
+	char *pad2;
+	if (!cert) {
+		return;
 	}
 	if (!pad) {
 		pad = "";
 	}
 	pad2 = r_str_newf ("%s  ", pad);
 	if (!pad2) {
-		return NULL;
+		return;
 	}
-	if ((r = snprintf (buffer, length, "%sTBSCertificate:\n", pad)) < 0) {
-		return NULL;
-	}
-	p = (ut32) r;
-	tbsc = r_x509_tbscertificate_dump (&certificate->tbsCertificate, buffer + p, length - p, pad2);
-	p = tbsc - buffer;
-	if (length <= p) {
-		free (pad2);
-		return NULL;
-	}
-	algo = certificate->algorithmIdentifier.algorithm;
+	r_strbuf_appendf (sb, "%sTBSCertificate:\n", pad);
+	r_x509_tbscertificate_dump (&cert->tbsCertificate, pad2, sb);
+
+	algo = cert->algorithmIdentifier.algorithm;
 	//	signature = r_asn1_stringify_bytes (certificate->signature->binary, certificate->signature->length);
-	//	r = snprintf (buffer + p, length - p, "%sAlgorithm:\n%s%s\n%sSignature: %u bytes\n%s\n",
+	//	eprintf ("%sAlgorithm:\n%s%s\n%sSignature: %u bytes\n%s\n",
 	//				pad, pad2, algo ? algo->string : "",
 	//				pad, certificate->signature->length, signature ? signature->string : "");
-	r = snprintf (buffer + p, length - p, "%sAlgorithm:\n%s%s\n%sSignature: %u bytes\n",
-				pad, pad2, algo ? algo->string : "", pad, certificate->signature->length);
-	if (r < 0) {
-		free (pad2);
-		return NULL;
-	}
-	p += (ut32) r;
+	r_strbuf_appendf (sb, "%sAlgorithm:\n%s%s\n%sSignature: %u bytes\n",
+		pad, pad2, algo ? algo->string : "", pad, cert->signature->length);
 	free (pad2);
 	//	r_asn1_free_string (signature);
-	return buffer + p;
 }
 
-char* r_x509_crlentry_dump (RX509CRLEntry *crle, char* buffer, ut32 length, const char* pad) {
+void r_x509_crlentry_dump (RX509CRLEntry *crle, const char* pad, RStrBuf *sb) {
 	RASN1String *id = NULL, *utc = NULL;
-	int r;
-	if (!crle || !buffer || !length) {
-		return NULL;
+	if (!crle) {
+		return;
 	}
 	if (!pad) {
 		pad = "";
@@ -689,63 +574,46 @@ char* r_x509_crlentry_dump (RX509CRLEntry *crle, char* buffer, ut32 length, cons
 	if (crle->userCertificate) {
 		id = r_asn1_stringify_integer (crle->userCertificate->binary, crle->userCertificate->length);
 	}
-
-	r = snprintf (buffer, length, "%sUser Certificate:\n%s  %s\n"
-				"%sRevocation Date:\n%s  %s\n",
-				pad, pad, id ? id->string : "Missing",
-				pad, pad, utc ? utc->string : "Missing");
-
+	r_strbuf_appendf (sb, "%sUser Certificate:\n%s  %s\n"
+		"%sRevocation Date:\n%s  %s\n",
+		pad, pad, id ? id->string : "Missing",
+		pad, pad, utc ? utc->string : "Missing");
 	r_asn1_free_string (id);
-	return r < 0 ? NULL : buffer + (ut32) r;
 }
 
-char* r_x509_crl_dump (RX509CertificateRevocationList *crl, char* buffer, ut32 length, const char* pad) {
+R_API char *r_x509_crl_to_string(RX509CertificateRevocationList *crl, const char* pad) {
 	RASN1String *algo = NULL, *last = NULL, *next = NULL;
-	ut32 i, p;
-	int r;
-	char *tmp, *pad2, *pad3;
-	if (!crl || !buffer || !length) {
+	ut32 i;
+	char *pad2, *pad3;
+	if (!crl) {
 		return NULL;
 	}
 	if (!pad) {
 		pad = "";
 	}
 	pad3 = r_str_newf ("%s    ", pad);
-	if (!pad3) return NULL;
+	if (!pad3) {
+		return NULL;
+	}
 	pad2 = pad3 + 2;
 	algo = crl->signature.algorithm;
 	last = crl->lastUpdate;
 	next = crl->nextUpdate;
-	r = snprintf (buffer, length, "%sCRL:\n%sSignature:\n%s%s\n%sIssuer\n",
-				pad, pad2, pad3, algo ? algo->string : "", pad2);
-	p = (ut32) r;
-	if (r < 0 || !(tmp = r_x509_name_dump (&crl->issuer, buffer + p, length - p, pad3))) {
-		free (pad3);
-		return NULL;
-	}
-	p = tmp - buffer;
-	if (length <= p) {
-		free (pad3);
-		return NULL;
-	}
-	r = snprintf (buffer + p, length - p, "%sLast Update: %s\n%sNext Update: %s\n%sRevoked Certificates:\n",
+	RStrBuf *sb = r_strbuf_new ("");
+	r_strbuf_appendf (sb, "%sCRL:\n%sSignature:\n%s%s\n%sIssuer\n", pad, pad2, pad3,
+			algo ? algo->string : "", pad2);
+	r_x509_name_dump (&crl->issuer, pad3, sb);
+
+	r_strbuf_appendf (sb, "%sLast Update: %s\n%sNext Update: %s\n%sRevoked Certificates:\n",
 				pad2, last ? last->string : "Missing",
 				pad2, next ? next->string : "Missing", pad2);
-	p += (ut32) r;
-	if (r < 0) {
-		free (pad3);
-		return NULL;
-	}
-	for (i = 0; i < crl->length; ++i) {
-		if (length <= p || !(tmp = r_x509_crlentry_dump (crl->revokedCertificates[i], buffer + p, length - p, pad3))) {
-			free (pad3);
-			return NULL;
-		}
-		p = tmp - buffer;
+
+	for (i = 0; i < crl->length; i++) {
+		r_x509_crlentry_dump (crl->revokedCertificates[i], pad3, sb);
 	}
 
 	free (pad3);
-	return buffer + p;
+	return r_strbuf_drain (sb);
 }
 
 RJSVar *r_x509_validity_json (RX509Validity* validity) {
@@ -874,7 +742,7 @@ RJSVar *r_x509_crlentry_json (RX509CRLEntry *crle) {
 	return obj;
 }
 
-RJSVar *r_x509_crl_json (RX509CertificateRevocationList *crl) {
+R_API RJSVar *r_x509_crl_json (RX509CertificateRevocationList *crl) {
 	ut32 i;
 	RJSVar* obj = r_json_object_new ();
 	RJSVar* array = NULL;
