@@ -184,6 +184,9 @@ int gdbr_connect(libgdbr_t *g, const char *host, int port) {
 	}
 	read_packet (g, false);
 	ret = send_ack (g);
+	if (ret < 0) {
+		return ret;
+	}
 	if (strcmp (g->data, "OK")) {
 		// return -1;
 	}
@@ -292,6 +295,10 @@ int gdbr_check_extended_mode(libgdbr_t *g) {
 	}
 	read_packet (g, false);
 	ret = send_ack (g);
+	if (ret < 0) {
+		g->stub_features.extended_mode = 0;
+		return ret;
+	}
 	if (strncmp (g->data, "OK", 2)) {
 		g->stub_features.extended_mode = 0;
 		return -1;
@@ -1186,7 +1193,7 @@ void gdbr_invalidate_reg_cache() {
 	reg_cache.valid = false;
 }
 
-int gdbr_send_qRcmd(libgdbr_t *g, const char *cmd, void (*cb_printf) (const char *fmt, ...)) {
+int gdbr_send_qRcmd(libgdbr_t *g, const char *cmd, PrintfCallback cb_printf) {
 	if (!g || !cmd) {
 		return -1;
 	}
