@@ -1,6 +1,6 @@
 #!/bin/sh
 # ASAN="address leak memory undefined"
-ASAN=${ASAN:="address"}
+ASAN=${ASAN:="address undefined signed-integer-overflow"}
 
 printf "\033[32m"
 echo "========================================================================="
@@ -40,4 +40,10 @@ if [ "$RET" != 0 ]; then
 	echo "Your compiler doesn't support ASAN."
 	exit 1
 fi
-exec sys/install.sh $*
+
+SCRIPT=install.sh
+if [ "$1" = "-u" ]; then
+	shift
+	SCRIPT=user.sh
+fi
+exec sys/${SCRIPT} $*
